/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * BILBM.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 05/08/90
 *   MODIFIED : 09/28/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Header file for big ReadILBM.C.
 *
 * HISTORY
 *		09/28/94 (dcc) - fix BigBitMap.xPlanes[] to use MAXSRCPLANES (not 8)
 *
*/

#ifndef ECHIDNA_BILBM_H
#define ECHIDNA_BILBM_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/bbytemap.h>
#include <echidna/readiff.h>
#include <echidna/listfunc.h>
#include <echidna/bitmap.h>
#include <echidna/xtramem.h>

/**************************** C O N S T A N T S ***************************/

#define MAXSRCPLANES	9

#define ILBM_FORM		MakeID ('I','L','B','M')
#define PBM_FORM		MakeID ('P','B','M',' ')

#define BMHD_CHUNK MakeID('B','M','H','D')
#define CMAP_CHUNK MakeID('C','M','A','P')
#define GRAB_CHUNK MakeID('G','R','A','B')
#define DEST_CHUNK MakeID('D','E','S','T')
#define SPRT_CHUNK MakeID('S','P','R','T')
#define CAMG_CHUNK MakeID('C','A','M','G')
#define BODY_CHUNK MakeID('B','O','D','Y')
#define CRNG_CHUNK MakeID('C','R','N','G')

#define	ILBM_CREATEBYTEMAP		(1L << 0)

/******************************** T Y P E S *******************************/

typedef struct BitMap  BitMap;

typedef struct BigBitMap
{
	UWORD BytesPerRow;
	UWORD Rows;
	UBYTE Flags;
	UBYTE Depth;
	UWORD pad;
	XTRAPntr xPlanes[MAXSRCPLANES];
} BigBitMap;

/* ---------- BitMapHeader ---------------------------------------------*/

typedef UBYTE Masking;		/* Choice of masking technique.*/
#define mskNone                 0
#define mskHasMask              1
#define mskHasTransparentColor  2
#define mskLasso                3

typedef UBYTE Compression;	/* Choice of compression algorithm applied to
     * each row of the source and mask planes. "cmpByteRun1" is the byte run
     * encoding generated by Mac's PackBits. See Packer.h . */
#define cmpNone      0
#define cmpByteRun1  1

/* Aspect ratios: The proper fraction xAspect/yAspect represents the pixel
 * aspect ratio pixel_width/pixel_height.
 *
 * For the 4 Amiga display modes:
 *   320 x 200: 10/11  (these pixels are taller than they are wide)
 *   320 x 400: 20/11
 *   640 x 200:  5/11
 *   640 x 400: 10/11		*/
#define x320x200Aspect 10
#define y320x200Aspect 11
#define x320x400Aspect 20
#define y320x400Aspect 11
#define x640x200Aspect  5
#define y640x200Aspect 11
#define x640x400Aspect 10
#define y640x400Aspect 11

/* A BitMapHeader is stored in a BMHD chunk. */
typedef struct {
    UWORD w, h;			/* raster width & height in pixels */
    WORD  x, y;			/* position for this image */
    UBYTE nPlanes;		/* # source bitplanes */
    Masking masking;		/* masking technique */
    Compression compression;	/* compression algorithm */
    UBYTE pad1;			/* UNUSED.  For consistency, put 0 here.*/
    UWORD transparentColor;	/* transparent "color number" */
    UBYTE xAspect, yAspect;	/* aspect ratio, a rational number x/y */
    WORD  pageWidth, pageHeight;  /* source "page" size in pixels */
    } BitMapHeader;

/* RowBytes computes the number of bytes in a row, from the width in pixels.*/
#define RowBytes(w)   ((((w) + 15) >> 4) << 1)


/* ---------- ColorRegister --------------------------------------------*/
/* A CMAP chunk is a packed array of ColorRegisters (3 bytes each). */
typedef struct {
    UBYTE red, green, blue;   /* MUST be UBYTEs so ">> 4" won't sign extend.*/
    } ColorRegister;

/* Use this constant instead of sizeof(ColorRegister). */
#define sizeofColorRegister  3

typedef WORD Color4;	/* Amiga RAM version of a color-register,
			 * with 4 bits each RGB in low 12 bits.*/

/* Maximum number of bitplanes in RAM. Current Amiga max w/dual playfield. */
#define MaxAmDepth 6

/* ---------- Point2D --------------------------------------------------*/
/* A Point2D is stored in a GRAB chunk. */
typedef struct {
    WORD x, y;		/* coordinates (pixels) */
    } Point2D;

/* ---------- DestMerge ------------------------------------------------*/
/* A DestMerge is stored in a DEST chunk. */
typedef struct {
    UBYTE depth;	/* # bitplanes in the original source */
    UBYTE pad1;		/* UNUSED; for consistency store 0 here */
    UWORD planePick;	/* how to scatter source bitplanes into destination */
    UWORD planeOnOff;	/* default bitplane data for planePick */
    UWORD planeMask;	/* selects which bitplanes to store into */
    } DestMerge;

/* ---------- SpritePrecedence -----------------------------------------*/
/* A SpritePrecedence is stored in a SPRT chunk. */
typedef UWORD SpritePrecedence;

/* ---------- Viewport Mode --------------------------------------------*/
/* A Commodore Amiga ViewPort->Modes is stored in a CAMG chunk. */
/* The chunk's content is declared as a LONG. */

/* ---------- CRange ---------------------------------------------------*/
/* A CRange is store in a CRNG chunk. */
typedef struct {
    WORD  pad1;		/* reserved for future use; store 0 here */
	WORD  rate;		/* 60/sec=16384, 30/sec=8192, 1/sec=16384/60=273 */
	WORD  active;	/* lo bit 0=no cycle, 1=yes; next bit 1=rvs */
	UBYTE low;		/* range lower */
	UBYTE high;		/* range upper */
    } CrngChunk;

typedef struct
{
	NodeType	nd;
	CrngChunk	Crng;
} CrngNode;

/* ---------- PixelData ------------------------------------------------*/

typedef union {
	BigBitMap		 BigBitMap;
	BigByteMap		 BigByteMap;
} PixelData;

/* ---------- ILBMInfo -------------------------------------------------*/
/* What Read IFF/ILBM returns form an ILBM FORM. */
typedef struct {
	BitMapHeader	 BM;
	UWORD			 NumberOfColors;
	ColorRegister	*Colors;
	ListType		 CrngListX;
	ListType		*CrngList;
	UBYTE			 pad0;
	UBYTE			 CAMGFlag;
	ULONG			 ViewModes;
	PixelData		 Pixels;
	} ILBMInfo;

typedef ILBMInfo BlockOPixels;
typedef ILBMInfo PlanesOPixels;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern void				 FreeBigByteMap (MPYPMPXPPIX mpYpMpXpPix, int height);
extern MPYPMPXPPIX		 AllocateBigByteMap(int width, int height);

extern long 			 ReadCMAP (IFFTracker *iff, UWORD *numcolors, ColorRegister **cr);
extern long 			 ILBM_ReadCMAP (IFFTracker *iff);
extern long 			 ReadILBMBODY (IFFTracker *iff, BitMapHeader *bmHdr, PixelData *pd);
extern long 			 ILBM_ReadBODY (IFFTracker *iff);
extern long 			 ReadPBMBODY (IFFTracker *iff, BitMapHeader *bmHdr, PixelData *pd);
extern long 			 PBM_ReadBODY (IFFTracker *iff);
extern long				 ReadCRNG (IFFTracker *iff, CrngChunk *cc);
extern long 			 ILBM_ReadCRNG (IFFTracker *iff);
extern long 			 ReadBMHD (IFFTracker *iff, BitMapHeader *bm);
extern long 			 ILBM_ReadBMHD (IFFTracker *iff);
extern long 			 ReadILBM (IFFTracker *iff);
extern long 			 ReadPBM (IFFTracker *iff);
extern void 			 FreeILBMInfo (ILBMInfo *ilbm);
extern ILBMInfo			*AllocILBMInfo (void);
extern BlockOPixels		*CreateBlockOPixels (char *filename);
extern void				 DestroyBlockOPixels (BlockOPixels *bop);
extern PlanesOPixels	*CreatePlanesOPixels (char *filename);
extern void				 DestroyPlanesOPixels (PlanesOPixels *bop);
extern ILBMInfo			*CreatePalette (char *filename);
extern void				 DestroyPalette (ILBMInfo *ilbm);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_BILBM_H */
