/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * BTUME.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 01/27/92
 *   MODIFIED : 09/21/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Routines for reading tUME map files.
 *
 * HISTORY
 *
*/

#ifndef ECHIDNA_BTUME_H
#define ECHIDNA_BTUME_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/listfunc.h>
#include <echidna/xtramem.h>
#include <echidna/readiff.h>
#include <echidna/bilbm.h>

/**************************** C O N S T A N T S ***************************/

#define MAXDIRECTIONS	(4)

#define	TUME_TMGC		(0x0001)

#define TUME_TILESET_COMPOSITE	0x0200

/******************************** T Y P E S *******************************/

typedef struct {
	UWORD	 Hue;
	UBYTE	 Saturation;
	UBYTE	 Value;
} HSVType;

typedef struct {
	NodeType		 Node;			/* Used for loading only		*/
	UWORD			 Speed;
	WORD			 Direction;
	UWORD			 Flags;
	UWORD			 NumColors;
	UWORD			 NumRegs;
	UWORD			*Registers;
	ColorRegister	*Colors;
	HSVType			*HSVs;
	UWORD			 CurrentColor;	/* Usable for runtime cycler	*/
} CycleInfo;

typedef struct {
	UWORD			 CycleFlag;
	UWORD			 NumColors;
	ColorRegister	*Colors;
	HSVType			*HSVs;
	UWORD			 NumCycles;
	CycleInfo		**Cycles;
} ColorInfo;

typedef struct {
	NodeType	 Node;
	UWORD		 NumTiles;
	XTRAPntr	 TileData;
} TileDataGroup;

typedef struct {
	UWORD		 TileCount;
	UWORD		 Width;
	UWORD		 Height;
	UWORD		 Depth;
	UWORD		 TransparentColor;
	ListType	 TileGroupListX;
	ListType	*TileGroupList;
} TMGType;

typedef struct {
	UBYTE	 PlotFlags;
	UBYTE	 TilesetID;
	UWORD	 TileID;
} PlotType;

typedef struct {
	NodeType	 Node;
	UWORD		 Flags;
	XTRAPntr	*Plot;
} LayerType;

typedef struct {
	NodeType	 Node;			/* Used for loading only */
	UWORD		 LoadFlags;
	WORD		 Flags;
	WORD		 ID;
	WORD		 Width;
	WORD		 Height;
	UWORD		 TileWidth;
	UWORD		 TileHeight;
	UWORD		 LayerCount;
	UWORD		 FloorNumber;
	WORD		 UserType;
	WORD		 UserNumber;
	UWORD		 pad0;
	UWORD		 pad1;
	WORD		 BackgroundColor;
	WORD		 Direction[MAXDIRECTIONS];
	UWORD		 GuideWidth;
	UWORD		 GuideHeight;
	UWORD		 GuideX;
	UWORD		 GuideY;
	UBYTE		 GuideOnOff;
	UBYTE		 pad2;
	UWORD		 GridWidth;
	UWORD		 GridHeight;
	UWORD		 GridX;
	UWORD		 GridY;
	UBYTE		 GridOnOff;
	UBYTE		 pad3;
	UWORD		 ZoomDstDup;
	UWORD		 ZoomSrcSkip;
	UBYTE		 ZoomOnOff;
	UBYTE		 pad4;
	ColorInfo	 ColorInfo;
	char		*Name;
	char		**Comments;
	struct Tileset	*CompositeTSet;
	LayerType	**Layers;
	void		*UserData;
} RoomType;

typedef struct Tileset {
	NodeType	 Node;			/* Used for Loading Only */
	WORD		 ID;
	UWORD		 Flags;
	WORD		 Width;
	WORD		 Height;
	WORD		 Depth;
	char		*FileSpec;
	char		**Comments;
	UWORD		 SaveFlags;
	WORD		 UserType;
	WORD		 UserNumber;
	WORD		 DisplayRoomID;	/* ??? */
	WORD		 TileCount;
	TMGType		 TMG;
	RoomType	*GridedTilesRoom;
	WORD		 TilesAcross;
	WORD		 TilesDown;
	UWORD		 GuideWidth;
	UWORD		 GuideHeight;
	UWORD		 GuideX;
	UWORD		 GuideY;
	UBYTE		 GuideOnOff;
	UBYTE		 pad2;
	UWORD		 GridWidth;
	UWORD		 GridHeight;
	UWORD		 GridX;
	UWORD		 GridY;
	UBYTE		 GridOnOff;
	UBYTE		 pad3;
	UWORD		 ZoomDstDup;
	UWORD		 ZoomSrcSkip;
	UBYTE		 ZoomOnOff;
	UBYTE		 pad4;
	void		*UserData;
} TilesetType;

typedef struct {
	UWORD		 NumRooms;
	RoomType	**Rooms;		/* Note: Starts at Rooms[1] */
	UWORD		 NumTilesets;
	TilesetType	**Tilesets;		/* Note: Starts at Tilesets[1] */
} MapType;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define AppendtUMEMapTMGX(map,name)	AppendtUMEMap((map),(name),NULL)
#define AppendtUMEMapTMGC(map,name)	AppendtUMEMap((map),(name),TUME_TMGC)
#define CreatetUMEMapTMGX(name)		AppendtUMEMap(NULL,(name),NULL)
#define CreatetUMEMapTMGC(name)		AppendtUMEMap(NULL,(name),TUME_TMGC)

#define IsCompositeTileset(ts)		((ts)->GridedTilesRoom != NULL)
#define IsCompositeRoom(rm)			((rm)->CompositeTSet != NULL)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern void		 DestroytUMETileset (TilesetType *ts);
extern void		 DestroytUMERoom (RoomType *room);
extern void		 DestroytUMEMap (MapType *map);
extern MapType	*AppendtUMEMap (MapType *map, char *filename, UWORD flags);

extern void		 ConvertTMGCtoTMGX (
							UBYTE	*src,
							int		 twidth,
							int		 theight,
							UBYTE	*dst
					);
extern BOOL		 ConvertTMGXtoTMGC (
							UBYTE	*src,
							int		 twidth,
							int		 theight,
							int		 tdepth,
							UBYTE	*dst
					);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_BTUME_H */

