/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * EDRAW.h
 *
 * PROGRAMMER : Gregg A. Tavares 
 *    VERSION : 00.000
 *    CREATED : 11/05/89 
 *   MODIFIED : 07/26/94
 *       TABS : 05 09
 *
 * DESCRIPTION
 *
 *
 * HISTORY
 *
*/
#ifndef ECHIDNA_ECHIDNADRAW_H 
#define ECHIDNA_ECHIDNADRAW_H 

#include <echidna/platform.h>
#include "switches.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/graphics.h>
#include <echidna/fonts.h>

/******************************* M A C R O S ******************************/
#define ED_End				 0
#define ED_FgColor(val)		 1, (val)
#define ED_BgColor(val)		 2, (val)
#define ED_DrawMode(val)	 3, (val)
#define ED_Plot(x,y)		 4, (x), (y)
#define ED_Draw(x,y)		 5, (x), (y)
#define ED_Move(x,y)		 6, (x), (y)
#define ED_Rect(x,y,w,h)	 7, (x), (y), (w), (h)
#define ED_Font(font)		 8, (font)
#define ED_Text(text)		 9, (text)
#define ED_Gosub(epic)		10, (epic)
#define ED_ForX(i,ilimit,istep,xstep,epic)	\
	11, (i), (ilimit), (istep), (xstep), (epic)
#define ED_ForY(i,ilimit,istep,ystep,epic)	\
	12, (i), (ilimit), (istep), (ystep), (epic)
#define ED_DrawShape(s)		13, (s)
#define ED_SetShapeTable(t)	14, (t)
#define ED_MoveRel(x,y)		15, (x), (y)
#define ED_FgColorTbl(val)	16, (val)
#define ED_BgColorTbl(val)	17, (val)

#define ED_Line(x1,y1,x2,y2)	ED_Move ((x1), (y1)), ED_Draw((x2), (y2))
#define ED_HLine(x1,x2,y)		ED_Rect ((x1), (y), ((x2) - (x1) + 1), 1)
#define ED_RelHLine(x,w,y)		ED_Rect ((x), (y), (w), 1)
#define ED_VLine(x,y1,y2)		ED_Rect ((x), (y1), 1, ((y2) - (y1) + 1))
#define ED_RelVLine(x,y,h)		ED_Rect ((x), (y), 1, (h))
#define ED_PosText(x,y,text)	ED_Move ((x),(y)), ED_Text((text))
#define ED_PosShape(s,x,y)		ED_Move ((x),(y)), ED_DrawShape((s))
#define	ED_ShadowRect(x,y,w,h)		\
	ED_Rect((x),(y),(w)-1,(h)-1),	\
	ED_RelHLine((x)+1,(w)-1,(y)+(h)-1),	\
	ED_RelVLine((x)+(w)-1,(y)+1,(h)-1)

/******************************** T Y P E S *******************************/
typedef short EPic;

typedef struct {
	EPic			*EPic;
	struct TextFont	***Fonts;
	char			***Strings;
	EPic			**EPics;
	GShape			***Shapes;
	UBYTE			*Colors;
	struct RastPort	*rp;
} EPicture;

/**************** F U N C T I O N A L   P R O T O T Y P E S ***************/

extern void StartEPic (
	struct RastPort	*rp,
	EPic			*epic,
	short			 x,
	short			 y,
	struct TextFont ***fonts,
	char			***strings,
	EPic			**epics,
	GShape			***shapes
);

extern void StartEPicture (
	struct RastPort	*rp,
	EPicture		*ep,
	short			 x,
	short			 y
);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_ECHIDNADRAW_H */

