/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * Eerrors.h
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 01.002
 *    CREATED : 12/12/89
 *   MODIFIED : 10/02/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Macros, Constants, Global Variables and Function to deal with
 *		Errors.
 *
 * HISTORY
 *	12/24/89 Sunday - Added ERR_GENERIC and PrintFreeMem. (GAT)
 *	12/24/89 Sunday - Added ERR_INVALID_DATA. (GAT)
 *	05/24/91 Friday (RGM) - Put an #if __MSDOS__ around IgnoreErrors().
 *	06/07/91 Friday (RGM) - SAS/Lattice stuff (non-support of __FILE__,
 *		__FUNC__).
 *
*/

#ifndef ECHIDNA_EERRORS_H
#define ECHIDNA_EERRORS_H

#include <echidna/platform.h>
#include "switches.h"

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

#if LATTICE
#include <error.h>
#endif

/**************************** C O N S T A N T S ***************************/

#define ERR_OUT_OF_MEMORY	1
#define ERR_FILE_NOT_FOUND	2
#define ERR_READING_FILE	3
#define ERR_REQUIREMENTS	4
#define ERR_INVALID_DATA	5
#define ERR_GENERIC			6

#if LATTICE
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS	(0)
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE	(20)
#endif
#endif

/******************************** T Y P E S *******************************/

typedef void (*SysErrorFuncPtr)(void);
#if __MSDOS16__
typedef void interrupt far32 INT(void);
#else
typedef void far32 INT(void);
#endif

/****************************** G L O B A L S *****************************/

extern int		 GlobalErr;
extern char		*GlobalErrMsg;
extern char		*GlobalErrStr[];
extern char		*CurrentFuncName;
extern short far32 ErrReturnCode;

extern int		ErrorCount;
extern int		WarnCount;

/******************************* M A C R O S ******************************/

#if ERRORS

#define SetErrMsg(msg)	strcpy (GlobalErrMsg,(msg))

#define GEprintf(fmt)								\
	__PrintError(0,fmt)
#define GEprintf1(fmt,a1)							\
	__PrintError(0,fmt,(a1))
#define GEprintf2(fmt,a1,a2)						\
	__PrintError(0,fmt,(a1),(a2))
#define GEprintf3(fmt,a1,a2,a3)						\
	__PrintError(0,fmt,(a1),(a2),(a3))
#define GEprintf4(fmt,a1,a2,a3,a4)					\
	__PrintError(0,fmt,(a1),(a2),(a3),(a4))
#define GEprintf5(fmt,a1,a2,a3,a4,a5)				\
	__PrintError(0,fmt,(a1),(a2),(a3),(a4),(a5))
#define GEprintf6(fmt,a1,a2,a3,a4,a5,a6)			\
	__PrintError(0,fmt,(a1),(a2),(a3),(a4),(a5),(a6))
#define GEprintf7(fmt,a1,a2,a3,a4,a5,a6,a7)			\
	__PrintError(0,fmt,(a1),(a2),(a3),(a4),(a5),(a6),(a7))
#define GEprintf8(fmt,a1,a2,a3,a4,a5,a6,a7,a8)		\
	__PrintError(0,fmt,(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8))
#define GEprintf9(fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)	\
	__PrintError(0,fmt,(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8),(a9))

#define GEcatf(fmt)									\
	__PrintError(1,(fmt))
#define GEcatf1(fmt,a1)								\
	__PrintError(1,(fmt),(a1))
#define GEcatf2(fmt,a1,a2)							\
	__PrintError(1,(fmt),(a1),(a2))
#define GEcatf3(fmt,a1,a2,a3)						\
	__PrintError(1,(fmt),(a1),(a2),(a3))
#define GEcatf4(fmt,a1,a2,a3,a4)					\
	__PrintError(1,(fmt),(a1),(a2),(a3),(a4))
#define GEcatf5(fmt,a1,a2,a3,a4,a5)					\
	__PrintError(1,(fmt),(a1),(a2),(a3),(a4),(a5))
#define GEcatf6(fmt,a1,a2,a3,a4,a5,a6)				\
	__PrintError(1,(fmt),(a1),(a2),(a3),(a4),(a5),(a6))
#define GEcatf7(fmt,a1,a2,a3,a4,a5,a6,a7)			\
	__PrintError(1,(fmt),(a1),(a2),(a3),(a4),(a5),(a6),(a7))
#define GEcatf8(fmt,a1,a2,a3,a4,a5,a6,a7,a8)		\
	__PrintError(1,(fmt),(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8))
#define GEcatf9(fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)		\
	__PrintError(1,(fmt),(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8),(a9))

#else

#define SetErrMsg(msg)

#define GEprintf(fmt)
#define GEprintf1(fmt,a1)
#define GEprintf2(fmt,a1,a2)
#define GEprintf3(fmt,a1,a2,a3)
#define GEprintf4(fmt,a1,a2,a3,a4)
#define GEprintf5(fmt,a1,a2,a3,a4,a5)
#define GEprintf6(fmt,a1,a2,a3,a4,a5,a6)
#define GEprintf7(fmt,a1,a2,a3,a4,a5,a6,a7)
#define GEprintf8(fmt,a1,a2,a3,a4,a5,a6,a7,a8)
#define GEprintf9fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)

#define GEcatf(fmt)
#define GEcatf1(fmt,a1)
#define GEcatf2(fmt,a1,a2)
#define GEcatf3(fmt,a1,a2,a3)
#define GEcatf4(fmt,a1,a2,a3,a4)
#define GEcatf5(fmt,a1,a2,a3,a4,a5)
#define GEcatf6(fmt,a1,a2,a3,a4,a5,a6)
#define GEcatf7(fmt,a1,a2,a3,a4,a5,a6,a7)
#define GEcatf8(fmt,a1,a2,a3,a4,a5,a6,a7,a8)
#define GEcatf9(fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)

#endif /* ERRORS */

#if REQUIRE

	#define reqif(v)			if (v)
	#define reqExit(v)			exit (v)
	#define reqPrintf(fmt)								\
		printf(fmt)
	#define reqPrintf1(fmt,a1)							\
		printf(fmt,(a1))
	#define reqPrintf2(fmt,a1,a2)						\
		printf(fmt,(a1),(a2))
	#define reqPrintf3(fmt,a1,a2,a3)   					\
		printf(fmt,(a1),(a2),(a3))
	#define reqPrintf4(fmt,a1,a2,a3,a4)					\
		printf(fmt,(a1),(a2),(a3),(a4))
	#define reqPrintf5(fmt,a1,a2,a3,a4,a5)				\
		printf(fmt,(a1),(a2),(a3),(a4),(a5))
	#define reqPrintf6(fmt,a1,a2,a3,a4,a5,a6)			\
		printf(fmt,(a1),(a2),(a3),(a4),(a5),(a6))
	#define reqPrintf7(fmt,a1,a2,a3,a4,a5,a6,a7)		\
		printf(fmt,(a1),(a2),(a3),(a4),(a5),(a6),(a7))
	#define reqPrintf8(fmt,a1,a2,a3,a4,a5,a6,a7,a8)		\
		printf(fmt,(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8))
	#define reqPrintf9(fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)	\
		printf(fmt,(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8),(a9))

#else

	#define reqif(v)
	#define reqExit(v)
	#define reqPrintf(fmt)
	#define reqPrintf1(fmt,a1)
	#define reqPrintf2(fmt,a1,a2)
	#define reqPrintf3(fmt,a1,a2,a3)
	#define reqPrintf4(fmt,a1,a2,a3,a4)
	#define reqPrintf5(fmt,a1,a2,a3,a4,a5)
	#define reqPrintf6(fmt,a1,a2,a3,a4,a5,a6)
	#define reqPrintf7(fmt,a1,a2,a3,a4,a5,a6,a7)
	#define reqPrintf8(fmt,a1,a2,a3,a4,a5,a6,a7,a8)
	#define reqPrintf9(fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)

#endif /* REQUIRE */


#if SHOW_MEM
#define PrintFreeMem()	__PrintFreeMem()
#else
#define PrintFreeMem()
#endif

#if NO_ERROR_NUMBERS
#define SetGlobalErr(val)
#define ClearGlobalErr()
#else
#define SetGlobalErr(val)	GlobalErr = (val)
#define ClearGlobalErr()	GlobalErr = 0
#endif

#if LATTICE
#define ERRINFO(fmt) "__FILE__?" ":%d:" "__FUNC__?" ":" fmt, __LINE__
#else
#define ERRINFO(fmt) __FILE__ ":%d:" __FUNC__ ":" fmt, __LINE__
#endif

#define GEreportf(ecode,fmt)									\
	SetGlobalErr(ecode)											\
	GEcatf((fmt))
#define GEreportf1(ecode,fmt,a1)								\
	SetGlobalErr(ecode)											\
	GEcatf1((fmt),(a1))
#define GEreportf2(ecode,fmt,a1,a2)								\
	SetGlobalErr(ecode)											\
	GEcatf2((fmt),(a1),(a2))
#define GEreportf3(ecode,fmt,a1,a2,a3)							\
	SetGlobalErr(ecode)											\
	GEcatf3((fmt),(a1),(a2),(a3))
#define GEreportf4(ecode,fmt,a1,a2,a3,a4)						\
	SetGlobalErr(ecode)											\
	GEcatf4((fmt),(a1),(a2),(a3),(a4))
#define GEreportf5(ecode,fmt,a1,a2,a3,a4,a5)					\
	SetGlobalErr(ecode)											\
	GEcatf5((fmt),(a1),(a2),(a3),(a4),(a5))
#define GEreportf6(ecode,fmt,a1,a2,a3,a4,a5,a6)					\
	SetGlobalErr(ecode)											\
	GEcatf6((fmt),(a1),(a2),(a3),(a4),(a5),(a6))
#define GEreportf7(ecode,fmt,a1,a2,a3,a4,a5,a6,a7)				\
	SetGlobalErr(ecode)											\
	GEcatf7((fmt),(a1),(a2),(a3),(a4),(a5),(a6),(a7))
#define GEreportf8(ecode,fmt,a1,a2,a3,a4,a5,a6,a7,a8)			\
	SetGlobalErr(ecode)											\
	GEcatf8((fmt),(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8))
#define GEreportf9(ecode,fmt,a1,a2,a3,a4,a5,a6,a7,a8,a9)		\
	SetGlobalErr(ecode)											\
	GEcatf9((fmt),(a1),(a2),(a3),(a4),(a5),(a6),(a7),(a8),(a9))

/****************** F U N C T I O N   P R O T O T Y P E S *****************/
#if __NOT_ANSI__		/** Non-ANSI Prototypes **/

extern void			__PrintError();
extern void			__PrintFreeMem();
extern void			FailMess	();
extern void			ErrMess		();
extern void			WarnMess	();
extern void			OutOfMemErr	();

#else					/** ANSI Prototypes **/

extern void			ClearGlobalError (void);
extern void			__PrintError(int append, char *fmt, ...);
extern void			__PrintFreeMem(void);
extern void			SetSysErrorFunc (SysErrorFuncPtr sefptr);
extern INT far32	*GetSysErrorVector(void);
extern void			SetSysErrorVector(INT far32 *pInt);
extern void			FailMess	(char *fmt, ...);
extern void			ErrMess		(char *fmt, ...);
extern void			WarnMess	(char *fmt, ...);
extern void			OutOfMemErr	(char *fmt, ...);

#if __MSDOS__
#if !__ZTC__
extern void	IgnoreErrors (void);
#endif
#endif

#if __MACSC__
extern char *MacSystemErrMsg (int err);
#endif

#endif /** Function Prototypes **/

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_EERRORS_H */

