/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * FPFLoad.H
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/13/90
 *   MODIFIED : 01/15/93
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Header for FPFLoad.C (the module that loads FilePack Files).
 *
 * HISTORY
 *	05/24/91 Friday (RGM) - Added include for <stddef.h>, fixed backslash
 *		on #include <echidna\eio.h> to forward slash.
 *
*/

#ifndef ECHIDNA_FPFLOAD_H
#define ECHIDNA_FPFLOAD_H

#include <echidna/platform.h>
#include "switches.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/eio.h>
#include <stddef.h>

/**************************** C O N S T A N T S ***************************/

/* Flags for FPF_open() and FPF_openf() */
#define IS_FPF		  		0
#define UNPACKED_NON_FPF	(-1)
#define PACKED_NON_FPF		(-2)

/* Decompression Method */
typedef enum {dcmLZSS, dcmLZDC1} DCM;

/******************************** T Y P E S *******************************/

typedef short	(*FileExistsFuncPtr)(char *filename);

struct FPF_tracker {
	short	CrntDirFileNum;	/* Directory index of file to examine next */
	short	FPF_IndexFound;				/* FilePack index of found file */
	char	NameFound[EIO_MAXPATH];		/* Name of found file */
	char	MatchPath[EIO_MAXPATH];		/* Path to match */
	char	FPF_Name[EIO_MAXPATH];		/* Name of FPF */
};

typedef struct FPF_Info {
	short	NumFiles;		/* Total files in FPF */
	short	NumDirFiles;	/* Number of files in file-name directory */
} FPF_InfoType;

/****************************** G L O B A L S *****************************/

extern FPF_InfoType	FPFInfo;
extern FileExistsFuncPtr	FPF_FileExistsFunc;

/******************************* M A C R O S ******************************/

#define FPF_FileCount(FPFName)	\
	( (FPF_Info ((FPFName), &FPFInfo)) ? (FPFInfo.NumFiles) : (-1) )

#define FPF_DirFileCount(FPFName)	\
	( (FPF_Info ((FPFName), &FPFInfo)) ? (FPFInfo.NumDirFiles) : (-1) )

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern int	FPF_open2	(	
							char	*filename,
							short	 selector,
							DCM		 dcm
						);
extern int	FPF_open	(	
							char	*filename,
							short	 selector
						);

extern int	FPF_openf	(
							char	*filename,
							char	*datafilename,
							short	 selector
						);

extern int	FPF_close	(void);

extern long	FPF_read	(
							void	*buffer, 
							long	 len
						);

extern long	FPF_skip	(long offset);

extern void FPF_SetFileExistsFunc (FileExistsFuncPtr fefptr);

extern long	FPF_BytesLeft (void);

extern FPF_InfoType	*FPF_Info	(
									char			*filename,
									FPF_InfoType	*fpf_info
								); 
extern short	FPF_findfirst	(
									char				*filename,
									char				*pathname,
									struct FPF_tracker	*fpftracker
								);
extern short	FPF_findnext 	(struct FPF_tracker *fpftracker);

extern int	FPF_openDeadOrAlive	(	
							char	*FPFfilename,
							short	 selector,
							char	*DataFilename,
							short	*pfUsedFPF
						);
extern short FPF_RecreateSourceFile (
	char	*pszFpfFilename, 
	short	 selector, 
	char	*pszDataFilename
);

#ifdef __cplusplus
}
#endif

#endif /* FPFLOAD_H */

/************************ D O C U M E N T A T I O N ***********************/
#if 0

/*********************************************************************
 *
 * FPF_open
 *
 * SYNOPSIS
 *		int FPF_open (
 *			char	*filename
 *			short	 selector
 *		)
 *
 * PURPOSE
 *		To setup access to a file containing data which may be compressed 
 *		or uncompressed and do whatever preparation is necessary for
 *		the rest of the FPF routines (FPF_read, FPF_skip, FPF_close)
 *		to function properly on the data and on the file.  
 *
 *		The FPF routines support 3 different mechanisms for accessing 
 *		data:
 *			Non-FPF
 *			FPF
 *			FPF-Indirect.  
 *
 *		Data files accessed through Non-FPF and FPF-Indirect methods are
 *		considered 'External', meaning that the actual data is not inside 
 *		an FPF file.
 *		In each mechanism the data may be compressed or
 *		uncompressed which means there are actually 6 different
 *		ways of accessing data.  Which mechanism is used and whether
 *		the data is treated as compressed or uncompressed depends
 *		upon the value of 'selector' or upon the contents of the file 
 *		named by 'filename'.  
 *
 *		Non-FPF:
 *		If 'selector' is negative, then 'filename' is not an FPF file 
 *		but is a data file containg compressed or uncompressed data as 
 *		the following table indicates:
 *
 *			selector	Compressed	
 *			----------	----------	
 *			-2			Yes			
 *							
 *			-1			No			
 *	
 *		Behavior is undefined for other negative values.
 *
 *		FPF and FPF-Indirect:
 *		If 'selector' is 0 or greater then 'filename' is an FPF file and 
 *		'selector' is used to index a table within the FPF that tells whether
 *		the data is compressed or uncompressed and where the data is (i.e.
 *		at what offset in the FPF or in what external file).  The contents
 *		of this table is determined by the instructions in the FPS file
 *		and any command line arguments to FilePack when the FPF was
 *		created.
 *
 * INPUT
 *		filename:	String containing name of a data file or of FPF.
 *		selector:	Must be >= -2.  See discussion under PURPOSE above.
 *
 * EFFECTS
 *		Memory will be allocated for a file buffer and decompression
 *		buffer if the data is compressed.  This memory will be freed
 *		when FPF_close() is called.
 *		
 *		WARNING: This routine is not re-entrant.  Only one file may be
 *			open at at time.  Once you open a file with FPF_open() or
 *			FPF_openf(), you must call FPF_close() before you can use
 *			either routine to open another file.
 *
 * RETURN VALUE read
 *		On success FPF_open returns a non-negative integer. 
 *		On error FPF_open returns -1 and errno has the value returned by 
 *		the UNIX function open(), which is used by FPF_open.
 *
 *		Call FPF_close() to close a file opened with this routine.
 *	
 * HISTORY
 *	08/10/90 Friday (JMA) - Modified to handle non FPF files, uncompressed
 *			FPF data and compressed indirect files.
 *		
 *	08/27/90 Monday (JMA) - Modified to handle new format containing 
 *			filename directory information.
 *
 * SEE ALSO
 *	FPF_openf(), FPF_read(), FPF_skip(), FPF_close()
 *
*/

/*********************************************************************
 *
 * FPF_openf
 *
 * SYNOPSIS
 *		int	FPF_openf (
 *			char	*filename,
 *			char	*datafilename,
 *			short	 selector
 *		)
 *
 * PURPOSE
 *		To get the FPF index for the given 'datafilename' from the
 *		given FPF file and then call FPF_open() to open it.
 *
 * INPUT
 *		filename:		FPF name.
 *		datafilename:	File name that appears in the directory table
 *						of the FPF file.
 *		
 *
 * EFFECTS
 *		WARNING: This routine is not re-entrant.  Only one file may be
 *			open at at time.  Once you open a file with FPF_open() or
 *			FPF_openf(), you must call FPF_close() before you can use
 *			either routine to open another file.
 *
 * RETURN VALUE
 *		On success FPF_open returns a non-negative integer. 
 *		On error FPF_openf returns -1 and errno has the value returned by 
 *		the UNIX function open(), which is used by FPF_openf.
 *
 *		Call FPF_close() to close a file opened with this routine.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		FPF_open(), FPF_read(), FPF_skip() FPF_close()
*/

/*********************************************************************
 *
 * FPF_close
 *
 * SYNOPSIS
 *		int	FPF_close (void)
 *
 * PURPOSE
 *		On success FPF_close returns 0.
 *		On error FPF_close returns -1 and errno will have the value 
 *		set by the function close, which is	used by FPF_close.
 *
 * INPUT
 *		None.
 *
 * EFFECTS
 *		None.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		FPF_open(), FPF_openf()		
*/

/*********************************************************************
 *
 * FPF_read
 *
 * SYNOPSIS
 *		size_t	FPF_read (
 *			void	*buffer, 
 *			size_t	 len
 *		)
 *
 * PURPOSE
 *		Read from an FPF file.
 *
 * INPUT
 *		buffer:	Where to put bytes.
 *		len:	How many to put.
 *
 * RETURN VALUE
 *		On successful completion , FPF_read returns the number of
 *		bytes placed in the buffer.
 *		Warning! Will read more bytes than are in file if you ask
 *		for them.
 *		On end-of-file, FPF_read returns 0.  
 *		On error, FPF_read returns -1 and errno has the value
 *		returned by the UNIX function read(), which is used by FPF_read.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		FPF_open(), FPF_openf(), FPF_skip(), FPF_close()
*/

/*********************************************************************
 *
 * FPF_skip
 *
 * SYNOPSIS
 *		long FPF_skip (
 *			long	offset
 *		)
 *
 * PURPOSE
 *		To skip 'offset' number of bytes forward in FPF data file.
 *
 * INPUT
 *		offset:	Number of bytes to skip from current position.
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		FPF_skip returns the offset of the pointer's new position
 *		measured in bytes from the file beginning.  FPF_skip returns
 *		-1L on error and errno has the value returned by the UNIX 
 *		function read(), which is used by FPF_skip.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		FPF_open(), FPF_openf(), FPF_read(), FPF_close()
 *		
*/

/*********************************************************************
 *
 * FPF_SetFileExistsFunc
 *
 * SYNOPSIS
 *		void FPF_SetFileExistsFunc (
 *			FileExistsFuncPtr	fefptr
 *		)
 *
 * PURPOSE
 *		To set the function used by the FPF routines when they
 *		check to see if a file exists before opening it.  If this
 *		function is not set then no check will be made before opening.
 *
 *		If your application has data on multiple disks, then you can
 *		write a 'FileExists' function that will make sure the right
 *		disk is in the drive (i.e. prompt the user to swap disks if
 *		necessary).
 *
 * INPUT
 *		fefptr:	Pointer to function which takes as an argument a 
 *				pointer to a file name string, and returns a short 
 *				int whose value has the meaing:
 *					0			File does NOT exit.
 *					non-zero	File Exists.
 *
 * EFFECTS
 *		None.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		'FileExistsFuncPtr' typedef in FPFLoad.H
*/

/*********************************************************************
 *
 * FPF_BytesLeft
 *
 * SYNOPSIS
 *		long FPF_BytesLeft (void)
 *
 * PURPOSE
 *		To determine the number of bytes remaining to be read in
 *		a data file opened by and FPF open routine.
 *
 * INPUT
 *		None.
 *
 * EFFECTS
 *		None.
 *
 * RETURN VALUE
 *		Returns Number of bytes remaining to be read in data file.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

/*********************************************************************
 *
 * FPF_openDeadOrAlive
 *
 * SYNOPSIS
 *		int FPF_openDeadOrAlive (
 *			char	*FPFfilename,
 *			short	 selector,
 *			char	*DataFilename,
 *			short   *pfUsedFPF
 *		)
 *
 * PURPOSE
 *		To see if DataFilename exists in the current directory and open
 *		it if it does for FPF reading. If it does not exist then open the
 *		file indexed by selector in the FPF file FPFfilename.  Fill in
 *		flag that tells if the FPF file was used.
 *		
 *
 * INPUT
 *		FPFfilename:	name of FPF file.
 *		selector:		FPF index.
 *		DataFilename:	File that you prefer to open if it exists.
 *		pfUsedFPF:		Pointer to flag to set if DataFilename was not found
 *						and the FPF file was used instead. May be NULL.
 *	
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		See FPF_open return values.
 *		*pfUsedFPF is not modified if the open fails both cases.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

/*********************************************************************
 *
 * FPF_RecreateSourceFile
 *
 * SYNOPSIS
 *		short FPF_RecreateSourceFile (
 *			char	*pszFpfFilename, 
 *			short	 selector, 
 *			char	*pszDataFilename
 *		)
 *
 * PURPOSE
 *		To recreate the source file from the FPF compressed file under
 *		the given name.
 *
 * INPUT
 *		pszFpfFilename:	FPF file name.	Non-NULL.
 *		selector:		FPF index of packed file for which to recreate the
 *						source file. 
 *		pszDataFilename: Name to give new source file. Non-NULL.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		Returns TRUE on success.
 *		Returns FALSE on error. 
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

/*********************************************************************
 *
 * FPF_Info
 *
 * SYNOPSIS
 *		FPF_InfoType *FPF_Info (
 *			char 			*filename,
 *			FPF_InfoType	*fpf_info
 *		) 
 *
 * PURPOSE
 *		To get information about an FPF. The information is put
 *		in the 'fpf_info' structure:
 *			typedef struct FPF_Info {
 *				short	NumFiles;		
 *				short	NumDirFiles;	
 *			} FPF_InfoType;
 *
 * INPUT
 *		filename:	FPF name.
 *		fpf_info:	Pointer to structure to fill out.
 *
 * EFFECTS
 *		Fills in values in 'fpf_info'.
 *
 *		This routine may be called on an FPF that is open or closed. 
 *		Note: An FPF does not actually remain open after the open call
 *			if the data file requested was 'External' to the FPF.
 *		Warning: If the FPF was already open and this routine fails,
 *			then it will close the FPF.
 *		
 *
 * RETURN VALUE
 *		On success returns 'fpf_info'. On error returns NULL.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *	
*/

/*********************************************************************
 *
 * FPF_findfirst
 *
 * SYNOPSIS
 *		short	FPF_findfirst (
 *			char				*filename,
 *			char				*pathname,
 *			struct FPF_tracker	*fpftracker
 *		)
 *
 * PURPOSE
 *		To find the first file in an FPF directory matching 'path'.
 *
 * INPUT
 *		filename:	Name of FPF.
 *		pathname:	File name to find. File name part may contain '*'
 *					and '?' wild cards.
 *		fpftracker:	Pointer to tracking structure:
 *					stuct FPF_tracker {
 *						short	CrntDirFileNum;
 *						short	FPF_IndexFound;
 *						char	NameFound;
 *						char	MatchPath[EIO_MAXPATH];
 *						char	FPF_Name[EIO_MAXPATH];
 *					}
 *
 * EFFECTS
 *		Fills *fpftracker with information to be used later by 
 *		FPF_findnext().
 *	
 *		If a match is found, then the 'NameFound' field and 'FPF_IndexFound'
 *		field of 'fpftracker' are filled in with the name and index of
 *		the matched file.
 *
 *		This routine may be called on an FPF that is open or closed. 
 *		Note: An FPF does not actually remain open after the open call
 *			if the data file requested was 'External' to the FPF.
 *		Warning: If the FPF was already open and this routine fails,
 *			then it will close the FPF.
 *
 * RETURN VALUE
 *		On Success returns index of file in FPF (index >= 0). 
 *		If fails to match or gets error then returns -1 and errno is
 *		set to -1.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		FPF_findnext()
 *
*/

/*********************************************************************
 *
 * FPF_findnext
 *
 * SYNOPSIS
 *		short FPF_findnext (
 *			struct FPF_tracker *fpftracker
 *		)
 *
 * PURPOSE
 *		To find subsequent files that match the 'pathname' given in 
 *		previous call to FPF_findfirst().  'fpftracker' is the same
 *		structure filled in by call to FPF_findfirst().  This structure
 *		contains necessary information for continuing the search.
 *		One FPF index for each call to FPF_findnext() will be returned
 *		until no more files are found in the FPF directory matching the
 *		'pathname'.
 *
 * INPUT
 *
 *
 * EFFECTS
 *		If a match is found, then the 'NameFound' field and 'FPF_IndexFound'
 *		field of 'fpftracker' are filled in with the name and index of
 *		the matched file.
 *
 *		This routine may be called on an FPF that is open or closed. 
 *		Note: An FPF does not actually remain open after the open call
 *			if the data file requested was 'External' to the FPF.
 *		Warning: If the FPF was already open and this routine fails,
 *			then it will close the FPF.
 *
 * RETURN VALUE
 *		On Success returns index of file in FPF (index >= 0). 
 *		If fails to match or gets error then returns -1 and errno is
 *		set to -1.
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		FPF_findfirst()
*/

#endif

