/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * RESOURCE.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 08/28/92
 *   MODIFIED : 07/26/94
 *       TABS : 05 09
 *
 * DESCRIPTION
 *	
 *		Note: If you don't want 64K taken from your memory to service
 *		XMS and you are not going to call any XTRAmem routines except
 *		AllocXTRA, FreeXTRA, ReadXTRA and WriteXTRA then at the very
 *		beginning of you program you should call
 *	
 *		SetMaxXMSWindows (0);
 *
 * HISTORY
 *
*/

#ifndef ECHIDNA_RESOURCE_H
#define ECHIDNA_RESOURCE_H

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <echidna/eerrors.h>
#include <echidna/listfunc.h>
#include <echidna/brushsup.h>
#include <echidna/xtramem.h>

/**************************** C O N S T A N T S ***************************/

#define RS_RELAVATEMODE_ABSOLUTE	0
#define RS_RELAVATEMODE_RELATIVE	1

#define RSTYPE_MEM		(-2)
#define RSTYPE_FREE		(-1)
#define RSTYPE_DATA		( 0)
#define RSTYPE_BPI		( 1)

#define RSF_RELATIVE	0x0001
#define RSF_CACHED		0x0002

/******************************** T Y P E S *******************************/

typedef struct {
	MinNode		 Node;
	long		 Size;
	long		 RealSize;
	short		 Type;
	void		*Mem;
} ResourceMem;

typedef struct {
	MinNode		 Node;
	ResourceMem	 MemTracker;
	UWORD		 Flags;
	ULONG		 CachePntr;
	short		 LockCount;
} Resource;

typedef struct {
	ListType	 MemOrderListX;
	ListType	*MemOrderList;
	ListType	 UseOrderListX;
	ListType	*UseOrderList;
	Resource	*FirstResource;
	short		 NumResources;
	long		 Size;
	long		*SizeTable;
	char		*FPF_Filename;
	short		 FPF_Index;
	long		 XtraMemSize;
	long		 XtraMemUsed;
	short		 XtraCount;
	XTRAPntr	 Xtra[1];
} ResourceTable;

typedef void *(* LoadResourceFunc) (char *fpf_filename, short fpf_index);
typedef void  (* RelavateResourceFunc) (void *data, short mode);

typedef struct {
	LoadResourceFunc		LoadFunc;
	RelavateResourceFunc	RelavateFunc;
} ResourceFunc;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define RSREL(pntr)	(((void *)(pntr))) = ((pntr) ? (void *)((((UBYTE huge *)(pntr)) - (basepntr))) : NULL)
#define RSABS(pntr)	(((void *)(pntr))) = ((pntr) ? (void *)((basepntr) + ((unsigned long)(pntr))) : NULL)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern void RSfree (void *mem);
extern void *RSmalloc (long size);
extern void *RScalloc (long count, long size);

extern void *LockLocalResource (ResourceTable *rt, short resID, short type);
extern void UnlockLocalResource (ResourceTable *rt, short resID);

extern void UnlockGlobalResource (short resID);
extern void *LockGlobalResource (short resID, short type);

extern void CloseGlobalResources (void);
extern BOOL OpenGlobalResources (
					char	*FPF_filename,
					short	 FPF_INDEX,
					long	 memSize,
					long	 xtraMemSize);

extern void CloseLocalResources (ResourceTable *rt);
extern ResourceTable *OpenLocalResources (
						char	*FPF_filename,
						short	 FPF_INDEX,
						long	 memSize,
						long	 xtraMemSize);

extern void *AllocateGlobalResourceMemory (long size);
extern void FreeGlobalResourceMemory (void *data);

extern void *AllocateLocalResourceMemory (ResourceTable *rt, long size);
extern void FreeLocalResourceMemory (ResourceTable *rt, void *data);

extern void CacheGlobalResource (short resID, short type);
extern void CacheLocalResource (ResourceTable *rt, short resID, short type);

#endif /* RESOURCE_H */

