/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * PCX.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 07/30/92
 *   MODIFIED : 09/28/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		09/21/94 (dcc) - compiles cleanly with 32-bit Borland C++ 4.02.
 *
*/

#include <echidna\platform.h>
#include "switches.h"

#include "pcx.h"

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * IsPCXFile
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
BOOL IsPCXFile (IFFTracker *iff)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "IsPCXFile";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		UBYTE	header = 0;

		EIO_Read (iff->fd, &header, 1);
		EIO_Seek (iff->fd, 0, EIO_SEEK_BEGINNING);

		return (header == 0x0A);
	}
} /* IsPCXFile */


/*********************************************************************
 *
 * Read256ColorPCX
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		08/23/94 (GAT) - added bytesPerLine argument
 *		09/28/94 (dcc) - set unused bits at end of line in bitplanes to 0
 *
 * SEE ALSO
 *
*/
static BOOL Read256ColorPCX (IFFTracker *iff, ILBMInfo *ilbm, int bytesPerLine)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "Read256ColorPCX";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		PixelData		*pd    = &ilbm->Pixels;
		BitMapHeader	*bmHdr = &ilbm->BM;
		int				 fd    = iff->fd;
		int				 nRows = bmHdr->h;
		int				 unpRowBytes = bmHdr->w;
		int				 srcRowBytes = bytesPerLine;
		int				 dstRowBytes;
		unsigned		 iPlane;
		int				 iRow;
		UBYTE			*workp[8]; /* array of ptrs to planes & mask */
		UBYTE			*buffer;
		BOOL			 result = FALSE;
		int				 createbmap = (int)(iff->Flags & ILBM_CREATEBYTEMAP);

		if (createbmap) {
			dstRowBytes = bmHdr->w;

			pd->BigByteMap.width  = bmHdr->w;
			pd->BigByteMap.height = (WORD) nRows;
			if ((pd->BigByteMap.mpYpMpXpPix = AllocateBigByteMap (bmHdr->w, bmHdr->h)) == NULL) {
/**/			goto RIBcleanup;
			}
		} else {
			dstRowBytes = RowBytes(srcRowBytes);
			for (iPlane = 0; iPlane < 8; iPlane++) {
				if ((pd->BigBitMap.xPlanes[iPlane] = AllocXTRA (dstRowBytes * nRows)) == NULL) {
					SetGlobalErr (ERR_OUT_OF_MEMORY);
					GEcatf ("\nRead256ColorPCX:OOXM bit planes");
/**/				goto RIBcleanup;
				}
			}
			for (iPlane = 0; iPlane < 8; iPlane++) {
				if ((workp[iPlane] = (UBYTE *) IFFcalloc(dstRowBytes, 1)) == NULL) {
					SetGlobalErr (ERR_OUT_OF_MEMORY);
					GEcatf ("\nRead256ColorPCX:OOM temp bit planes");
/**/				goto RIBcleanup;
				}
			}
			pd->BigBitMap.BytesPerRow = (UWORD) dstRowBytes;
			pd->BigBitMap.Rows        = (UWORD) nRows;
			pd->BigBitMap.Depth       = 8;
		}

		buffer = IFFcalloc (srcRowBytes + 8, 1);
		for (iRow = 0; iRow < nRows; iRow++) {
			UBYTE	*dst;
			UBYTE	 data;
			int		 len = bmHdr->w;

			dst = buffer;
			while (len > 0)
			{
				if (EIO_Read (fd, &data, 1) != 1) {
					SetGlobalErr (ERR_GENERIC);
					GEcatf ("\nRead256ColorPCX:Error reading file");
					goto RIBcleanup;
				}
				if ((data & 0xC0) == 0xC0) {
					UBYTE	count;

					count = (UBYTE) (data & 0x3F);
					if (EIO_Read (fd, &data, 1) != 1) {
						SetGlobalErr (ERR_GENERIC);
						GEcatf ("\nRead256ColorPCX:Error reading file");
						goto RIBcleanup;
					}
					memset (dst, data, count);
					len -= count;
					dst += count;
				} else {
					*dst++ = data;
					len--;
				}
			}
			#if 0
			if (len) {
				SetGlobalErr (ERR_GENERIC);
				GEcatf ("\nRead256ColorPCX:Error reading file (2)");
				goto RIBcleanup;
			}
			#endif

			if (createbmap) {
				WriteXTRA (buffer, unpRowBytes, pd->BigByteMap.mpYpMpXpPix[iRow], 0);
			} else {
				int		x;
				UBYTE	temps[8];
				UBYTE	plns[8];
				UBYTE	bit;
				UBYTE	*gDest;
				unsigned	p;

				gDest = buffer;
				for (x = 0; x < srcRowBytes; x += 8)
				{
					temps[0] = *gDest++;
					temps[1] = *gDest++;
					temps[2] = *gDest++;
					temps[3] = *gDest++;
					temps[4] = *gDest++;
					temps[5] = *gDest++;
					temps[6] = *gDest++;
					temps[7] = *gDest++;

					for (p = srcRowBytes - x; p < 8; p++)
						temps[p] = 0;

					bit      = 0x01;
					for (p = 0; p < 8; p++)
					{
						plns[p] = (UBYTE)
								  (
									((temps[0] & bit) ? 0x80 : 0x00) |
									((temps[1] & bit) ? 0x40 : 0x00) |
									((temps[2] & bit) ? 0x20 : 0x00) |
									((temps[3] & bit) ? 0x10 : 0x00) |
									((temps[4] & bit) ? 0x08 : 0x00) |
									((temps[5] & bit) ? 0x04 : 0x00) |
									((temps[6] & bit) ? 0x02 : 0x00) |
									((temps[7] & bit) ? 0x01 : 0x00)
								  );
						bit <<= 1;
					}
					workp[0][x/8] = plns[0];
					workp[1][x/8] = plns[1];
					workp[2][x/8] = plns[2];
					workp[3][x/8] = plns[3];
					workp[4][x/8] = plns[4];
					workp[5][x/8] = plns[5];
					workp[6][x/8] = plns[6];
					workp[7][x/8] = plns[7];
				}
				for (x = 0; x < 8; x++)
				{
					WriteXTRA(workp[x], dstRowBytes, pd->BigBitMap.xPlanes[x], dstRowBytes * iRow);
				}
			}
		}

		{
			UBYTE	haspalette;

			if (EIO_Seek (fd, -769L, EIO_SEEK_END) == -1) {
				SetGlobalErr (ERR_GENERIC);
				GEcatf ("\nRead256ColorPCX:Error reading palette (2)");
				goto RIBcleanup;
			}

			if (EIO_Read (fd, &haspalette, 1) != 1) {
				SetGlobalErr (ERR_GENERIC);
				GEcatf ("\nRead256ColorPCX:No palette");
				goto RIBcleanup;
			}

			if (haspalette != 12) {
				SetGlobalErr (ERR_GENERIC);
				GEcatf ("\nRead256ColorPCX:Error reading palette");
				goto RIBcleanup;
			}

			if (EIO_Read (fd, ilbm->Colors, 768) != 768) {
				SetGlobalErr (ERR_GENERIC);
				GEcatf ("\nRead256ColorPCX:Error reading palette (3)");
				goto RIBcleanup;
			}
			result = TRUE;
		}
RIBcleanup:
		if (!createbmap) {
			for (iPlane = 0; iPlane < 8; iPlane++) {
				if (workp[iPlane]) {
					IFFfree(workp[iPlane]);
				}
			}
		}
		if (buffer)
			IFFfree (buffer);
		return result;
	}
} /* Read256ColorPCX */


/*********************************************************************
 *
 * ReadPCX
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
BOOL ReadPCX (IFFTracker *iff)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadPCX";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		PCXHeader	 phead;
		int			 fd;
		ILBMInfo	*ilbm;

		fd = iff->fd;

		if ((ilbm = AllocILBMInfo ()) == NULL) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEcatf ("\nReadPCX:OOM ILBMInfo");
			return FALSE;
		}
		iff->IFFData.Data = ilbm;
		
		if (EIO_Read (fd, &phead, sizeof (phead)) != sizeof (phead)) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nReadPCX:Couldn't read header info");
			goto cleanup;
		}
		
		LilWord2Native (phead.XMin);
		LilWord2Native (phead.YMin);
		LilWord2Native (phead.XMax);
		LilWord2Native (phead.YMax);
		LilWord2Native (phead.HDPI);
		LilWord2Native (phead.VDPI);
		LilWord2Native (phead.BytesPerLine);
		LilWord2Native (phead.PaletteInfo);
		LilWord2Native (phead.HScreenSize);
		LilWord2Native (phead.VScreenSize);

		if (phead.Encoding != 1) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf1 ("\nReadPCX:Can't read PCX files with type %d encoding (yet)", phead.Encoding);
			goto cleanup;
		}

		if (phead.BitsPerPixel != 8 || phead.NPlanes != 1 || phead.Version < 5) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nReadPCX:Can only 256 color PCX files (currently)");
			goto cleanup;
		}

		ilbm->BM.w = (UWORD) (phead.XMax - phead.XMin + 1);
		ilbm->BM.h = (UWORD) (phead.YMax - phead.YMin + 1);
		ilbm->BM.x = phead.XMin;
		ilbm->BM.y = phead.YMin;
		ilbm->BM.nPlanes = 8;
		ilbm->BM.masking = mskNone;

		ilbm->NumberOfColors = 256;
		ilbm->Colors = IFFmalloc (256 * sizeof (ColorRegister));
		if (!ilbm->Colors) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEcatf ("\nReadPCX:OOM Color Palette");
			return FALSE;
		}

		if (!Read256ColorPCX (iff, ilbm, phead.BytesPerLine))
		{
			goto cleanup;
		}		

		return TRUE;

cleanup:
		FreeILBMInfo (ilbm);
		return FALSE;
	}
} /* ReadPCX */

