/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * READIFF.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 05/07/90
 *   MODIFIED : 09/21/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Routines to read an IFF file.
 *
 * HISTORY
 *
*/

#include <echidna\platform.h>
#include "switches.h"

#include <echidna\etypes.h>
#include <echidna\eerrors.h>
#include <echidna\sysextra.h>
#include <echidna\readiff.h>
#include <echidna\eio.h>
#include <echidna\emacros.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*------------------------------------------------------------------------*/
/**# MODULE:READIFF_IFFSkip                                               */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * IFFSkip
 *
 * SYNOPSIS
 *		long IFFRead (IFFTracker *iff, long size)
 *
 * PURPOSE
 *		skip bytes from file and update iff (IFFTracker.)
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long IFFSkip (IFFTracker *iff, long size)
{
	long	origsize;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "IFFRead";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	origsize   = size;
	iff->Size += size;

	EIO_Seek (iff->fd, size, EIO_SEEK_CURRENT);

	return origsize;

} /* IFFSkip */

/*------------------------------------------------------------------------*/
/**# MODULE:READIFF_IFFRead                                               */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * IFFRead
 *
 * SYNOPSIS
 *		long IFFRead (IFFTracker *iff, void *buf, long size)
 *
 * PURPOSE
 *		read bytes from file and update iff (IFFTracker.)
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long IFFRead (IFFTracker *iff, void *buf, long size)
{
	long	origsize;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "IFFRead";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	origsize   = size;
	iff->Size += size;

	if (size != EIO_Read (iff->fd, buf, size)) {
		SetGlobalErr (ERR_READING_FILE);
		GEcatf ("\nIFFRead:hit End-Of-File before reading all the data");
		return FALSE;
	}

	return origsize;

} /* IFFRead */

/*------------------------------------------------------------------------*/
/**# MODULE:READIFF_GetChunkHeader                                        */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * GetChunkHeader
 *
 * PURPOSE
 *		Read an IFF chunk header
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long GetChunkHeader (IFFTracker *iff, ChunkHeader *chunkhdr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GetChunkHeader";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (IFFRead (iff, chunkhdr, sizeof (ChunkHeader)) != 8) {
		GEcatf ("\nGetChunkHeader:couldn't read chunk header");
		return FALSE;
	}

	BigLong2Native (chunkhdr->Size);
	return TRUE;

} /* GetChunkHeader */

/*------------------------------------------------------------------------*/
/**# MODULE:READIFF_CallIDFunc                                            */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * SkipChunk
 *
 * SYNOPSIS
 *		long SkipChunk (IFFTracker *iff)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long SkipChunk (IFFTracker *iff)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SkipChunk";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	return IFFSkip (iff, iff->Size + (iff->Size & 0x01));

} /* SkipChunk */

/*********************************************************************
 *
 * CallIDFunc
 *
 * PURPOSE
 *		Call a function based on an IFF ID.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long CallIDFunc (long id, IFFRoutineID *ifffunc, IFFTracker *iff)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CallIDFunc";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while (ifffunc->ID) {
		if (id == ifffunc->ID) {
			return ifffunc->IFFFunc (iff);
		}
		ifffunc++;
	}

	return IFFSkip (iff, iff->Size);

} /* CallIDFunc */

/*------------------------------------------------------------------------*/
/**# MODULE:READIFF_ReadIFF                                               */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * ReadForm
 *
 * SYNOPSIS
 *		long ReadForm (IFFTracker *iff)
 *
 * PURPOSE
 *		Read a IFF FORM.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long ReadForm (IFFTracker *iff)
{
	long		 okay = TRUE;
	long		 size = iff->Size;
	long		 id;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadForm";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (IFFRead (iff, &id, sizeof (long)) != sizeof (long)) {
		GEcatf ("ReadForm:Reading FORM id");
		return FALSE;
	}

	iff->Size = size - 4;
	okay = CallIDFunc (id, iff->FORMTable, iff);
	if (!okay) {
		return FALSE;
	}

	return size;

} /* ReadForm */

/*********************************************************************
 *
 * ReadIFF
 *
 * PURPOSE
 *		Read an IFF file (find FORM/CATs/LISTs and parse one chunk.
 *		NOTE:
 *			CATs and LISTs not yet implemented.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long ReadIFF (IFFTracker *iff)
{
	static IFFRoutineID TopChunks[] = {
		{ FORM_CHUNK,	ReadForm,	},
		{ NULL,			NULL,		},
	};

	ChunkHeader		 chunkhdr;
	long			 okay;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadIFF";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (!GetChunkHeader (iff, &chunkhdr)) {
		return FALSE;
	}

	iff->Size = chunkhdr.Size;
	okay = CallIDFunc (chunkhdr.ID, TopChunks, iff);
	if (okay) {
		okay += 8;
	}

	return okay;

} /* ReadIFF */

