/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * EDIT.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 01/16/91
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Text Editing Routines?
 *
 * HISTORY
 *		10/05/92 Monday (dcc) - #if'ed out check for bad keys (CHECKBADKEYS)
 *
 *		12/04/92 Friday (GAT) - Added support for signalling that the string
 *								may have changes (for filereq.c)
 *
 *		03/02/93 Tuesday (dcc) - add support for editing strings using
 *								proportionally spaced fonts
 *
 *		03/20/93 Saturday (dcc) - fix support for proportionally spaced
 *								fonts breaking initial click in edit box
 *
 *		02/04/94 Friday (dcc) - fix [Del] & [<-] key press deleting too
 *								many highlighted letters
 *
 *		02/04/94 Friday (dcc) - allow [Shift] key to highlight letters
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdio.h>
#include <ctype.h>

#include <echidna/strings.h>
#include <echidna/listfunc.h>
#include <echidna/eerrors.h>
#include <echidna/grafx.h>
#include <echidna/fonts.h>
#include <echidna/hitareas.h>
#include <echidna/windows.h>
#include <echidna/eui.h>
#include <echidna/keyboard.h>

#include <echidna/emacros.h>

#define LOCAL static

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/

LOCAL Window		*EditWindow;
LOCAL Font			*EditFont;
LOCAL UWORD			 EditFlags;
LOCAL EditString	*CurrentString;
LOCAL EditString	*OldCurrentString;
LOCAL EditString	*OldHighlightString;
LOCAL char			*LocalString;
LOCAL short			 NMfirst;
LOCAL short			 NMlast;
LOCAL short			 NMcursor;

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * HiliteParts
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void HiliteParts (EditString *es, short first, short last)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HiliteParts";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	if (es && first >= 0) {
		short	wStart;
		short	width;

		if (first) {
			wStart = StringLenN (EditFont, es->String, first);
		} else {
			wStart = 0;
		}
		if (last - first) {
			width = StringLenN (EditFont, es->String + first, last - first);
		} else {
			width = 1;
		}
		SetPenColor (EditWindow->White ^ EditWindow->Black);
		XORRect (es->X + wStart, es->Y - 1, width, 9);
	}
	OldHighlightString = es;

} /* HiliteParts */


/*********************************************************************
 *
 * UpdateEditString4Reals
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void UpdateEditString4Reals (EditString *es)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateEditString4Reals";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short 	len;

		BeforeGraphics ();

		SetPenColor (EditWindow->White);

		DrawRect (es->X, es->Y - 1, es->MaxLen * 8, 9);
		len = strlen (es->String);
		len = min (len, es->MaxLen);
		DrawStringN (EditFont, es->X, es->Y, es->String, len);

		AfterGraphics ();
	}
} /* UpdateEditString4Reals */


/*********************************************************************
 *
 * UpdateEditString
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void UpdateEditString (EditString *es)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateEditString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (es == CurrentString) {
		SelectString (es);
	} else {
		UpdateEditString4Reals (es);
	}
} /* UpdateEditString */


/*********************************************************************
 *
 * UpdateString
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short UpdateString (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		BeforeGraphics ();
		UpdateEditString4Reals(CurrentString);
		HiliteParts (CurrentString, NMfirst, NMlast);
		AfterGraphics ();
	}

	return TRUE;
} /* UpdateString */


/*********************************************************************
 *
 * SelectString
 *
 * PURPOSE
 *		Select entire current string.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SelectString (EditString *es)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SelectString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (es != OldHighlightString) {
		HiliteParts (OldHighlightString, NMfirst, NMlast);
	}
	CurrentString = es;
	LocalString   = CurrentString->String;
	NMfirst = 0;
	NMlast  = strlen (LocalString);
	NMcursor= NMlast;
	UpdateString ();

} /* SelectString */


/*********************************************************************
 *
 * HighlightString
 *
 * PURPOSE
 *		Select Name Letters with mouse.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short HighlightString (HitInfo *hi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HighlightString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		static short	 first = TRUE;
		static short	 origfirst;
		short			 oldfirst;
		short			 oldlast;
		short			 len;
		short			 n;
		short			 x;
		short			 wDxLo;
		short			 wDxHi;
		short			 buttons;

		buttons       = hi->Buttons;
		CurrentString = hi->HitArea->UserData;
		LocalString   = CurrentString->String;

		n             = wFitString(EditFont, LocalString, hi->X - 2);
		wDxLo		  = StringLenN(EditFont, LocalString, n);
		wDxHi		  = *(LocalString+n) ?
							StringLenN(EditFont, LocalString, n+1) :
							wDxLo;
		if (abs((hi->X-1)-wDxLo) <= abs(wDxHi-(hi->X-1)))
			x = n;
		else
			x = n+1;

		oldfirst = NMfirst;
		oldlast  = NMlast;

		if (buttons) {
			if (first) {
				first     = FALSE;
				NMfirst   = x;
				NMlast    = NMfirst;
				origfirst = NMfirst;
			} else {
				NMlast  = x;
			}
		} else {
			first  = TRUE;
			NMlast = x;
		}

		if (NMlast < origfirst) {
			NMfirst = NMlast;
			NMlast  = origfirst;
			NMcursor= NMfirst;
		} else {
			NMfirst = origfirst;
			NMcursor= NMlast;
		}

		len = strlen (LocalString);
		NMfirst = min (len, NMfirst);
		NMfirst = max (0, NMfirst);
		NMlast  = min (len, NMlast);
		NMlast  = max (0, NMlast);
		NMcursor= min (len, NMcursor);
		NMcursor= max (0, NMcursor);
		
		if (NMfirst != oldfirst || NMlast != oldlast || OldCurrentString != CurrentString) {
			BeforeGraphics ();
			HiliteParts (OldHighlightString, oldfirst, oldlast);
			HiliteParts (CurrentString, NMfirst, NMlast);
			OldCurrentString = CurrentString;
			AfterGraphics ();
		}
	}
	return TRUE;

} /* HighlightString */


/*********************************************************************
 *
 * DeleteSelected
 *
 * PURPOSE
 *		Delete selected characters from NMfirst to NMlast
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DeleteSelected (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DeleteSelected";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		if (NMfirst < NMlast) {
			strcpy (&LocalString[NMfirst], &LocalString[NMlast]);
		}
		NMlast = NMfirst;
		NMcursor = NMfirst;
	}

} /* DeleteSelected */


/*********************************************************************
 *
 * InsertChar
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void InsertChar (short key)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InsertChar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		char	*s;
		char	*d;
		short	 len;

		DeleteSelected ();

		len = strlen (LocalString);
		if (len < CurrentString->MaxLen) {
			s   = LocalString + len;
			d   = s + 1;
			len = len - NMfirst + 1;

			while (len) {
				*d-- = *s--;
				len--;
			}

			LocalString[NMfirst] = key;
			NMfirst++;
			NMlast = NMfirst;
			NMcursor = NMfirst;
		}
		UpdateString ();

	}
} /* InsertChar */


/*********************************************************************
 *
 * CheckEditKeys
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		 0 = Ignore
 *		 1 = User Pressed Return;
 *		-1 = User Pressed Escape;
 *
 * HISTORY
 *		02/04/94 Friday (dcc) - fix [Del] & [<-] deleting an extra letter
 *		02/04/94 Friday (dcc) - change to use GetKeyNoWait()
 *		02/04/94 Friday (dcc) - add support for [Shift]-Arrows, [Shift]-[Home]
 *
 * SEE ALSO
 *
*/
short CheckEditKeys (short *outkey)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CheckEditKeys";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
#if CHECKBADKEYS
		static char badkeys[] = ";,=+/\"[]|<> *?\\";
#endif

		UWORD	 scancode;
		short	 key;
		short	 len;
		short	 status = 0;

		*outkey = 0;
		while ((scancode = GetKeyNoWait()) != 0)
		{
			key = ConvertKeyToAscii(scancode);
			if (key >= ' ' && key < 128)
			{
				if (EditFlags & EDIT_NUM_ONLY)
				{
					if (isdigit (key))
					{
						*outkey = key;
						InsertChar (key);
					}
				}
				else
				{
					if (isalnum (key))
					{
						*outkey = key;
						InsertChar (key);
					}
					else if (isprint (key))
					{
#if CHECKBADKEYS
						char	*s;
						short	 flag;

						flag = FALSE;
						s    = badkeys;
						while (*s)
						{
							if (key == *s)
							{
								flag = TRUE;
								break;
							}
							s++;
						}
						if (!flag)
						{
#endif
							*outkey = key;
							InsertChar (key);
#if CHECKBADKEYS
						}
#endif
					}
				}
			}
			else
			{
				len = strlen (LocalString);
				switch (key)
				{
				case ASCII_TAB:
					{
						EditString	*es;

						while ((es = CurrentString->Next) != NULL)
						{
							if (es && es->String)
							{
								*outkey = key;
								SelectString (es);
								break;
							}
							else if (!es)
							{
								break;
							}
						}
					}
					break;
				case ASCII_BKSP:
					if (NMfirst == NMlast && NMfirst)
					{
						NMfirst--;
					}
					*outkey = key;
					DeleteSelected ();
					UpdateString ();
					break;
				case EASCII_DELETE:
					if (NMfirst == NMlast && NMlast < len)
					{
						NMlast++;
					}
					*outkey = key;
					DeleteSelected ();
					UpdateString ();
					break;
				case ASCII_ESC:
					status = (-1);
					break;
				case ASCII_RETURN:
					status = 1;
					break;
				case EASCII_HOME:
					BeforeGraphics ();
					HiliteParts (CurrentString, NMfirst, NMlast);
					if (scancode & (QUAL_SHIFT << 8))
					{
						if (NMcursor == NMfirst)
						{
							NMfirst = 0;
						}
						else
						{
							NMlast  = NMfirst;
							NMfirst = 0;
						}
					}
					else
					{
						NMfirst = 0;
						NMlast  = 0;
					}
					NMcursor = NMfirst;

					HiliteParts (CurrentString, NMfirst, NMlast);
					AfterGraphics ();
					break;
				case EASCII_END:
					BeforeGraphics ();
					HiliteParts (CurrentString, NMfirst, NMlast);
					if (scancode & (QUAL_SHIFT << 8))
					{
						if (NMcursor == NMlast)
						{
							NMlast = len;
						}
						else
						{
							NMfirst = NMlast;
							NMlast  = len;
						}
					}
					else
					{
						NMfirst = len;
						NMlast  = len;
					}
					NMcursor = NMlast;
					HiliteParts (CurrentString, NMfirst, NMlast);
					AfterGraphics ();
					break;
				case EASCII_LEFT:
					BeforeGraphics ();
					HiliteParts (CurrentString, NMfirst, NMlast);
					if (scancode & (QUAL_SHIFT << 8))
					{
						if (NMcursor == NMfirst)
						{
							if (NMcursor)
							{
								NMcursor--;
							}
							NMfirst = NMcursor;
						}
						else
						{
							if (NMcursor)
							{
								NMcursor--;
							}
							NMlast = NMcursor;
						}
					}
					else
					{
						if (NMcursor)
						{
							NMcursor--;
						}
						NMfirst = NMcursor;
						NMlast  = NMcursor;
					}
					HiliteParts (CurrentString, NMfirst, NMlast);
					AfterGraphics ();
					break;
				case EASCII_RIGHT:
					BeforeGraphics ();
					HiliteParts (CurrentString, NMfirst, NMlast);
					if (scancode & (QUAL_SHIFT << 8))
					{
						if (NMcursor == NMlast)
						{
							if (NMcursor < len)
							{
								NMcursor++;
							}
							NMlast = NMcursor;
						}
						else
						{
							if (NMcursor < len)
							{
								NMcursor++;
							}
							NMfirst = NMcursor;
						}
					}
					else
					{
						if (NMcursor < len)
						{
							NMcursor++;
						}
						NMfirst = NMcursor;
						NMlast  = NMcursor;
					}
					HiliteParts (CurrentString, NMfirst, NMlast);
					AfterGraphics ();
					break;
				default:
					*outkey = key;
					return status;
				}
			}
		}
		return status;
	}

} /* CheckEditKeys */


/*********************************************************************
 *
 * InitStrings
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void InitStrings (Window *win)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitStrings";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	EditWindow         = win;
	EditFont           = win->Font;
	OldCurrentString   = NULL;
	OldHighlightString = NULL;

} /* InitStrings */


/*********************************************************************
 *
 * NewUpdateStrings
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void NewUpdateStrings (Window *win, EditString *es, short count, UWORD flags)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "NewUpdateStrings";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		InitStrings (win);
		EditFlags = flags;

		/*
		count--;
		es = es->Next;
		*/
		while (count) {
			CurrentString = es;
			LocalString   = es->String;
			NMfirst = -1;
			UpdateString ();
			es = es->Next;
			count--;
		}

	}

} /* NewUpdateStrings */


/*********************************************************************
 *
 * UpdateStrings
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void UpdateStrings (Window *win, EditString *es, short count, UWORD flags)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateStrings";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		NewUpdateStrings (win, es, count, flags);
		SelectString (es);
	}

} /* UpdateStrings */

