/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * MENUS.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 09/10/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Menu Routines.
 *
 * HISTORY
 *
 *
 * TODO
 *	* Make CURSOR Left and CURSOR Right go though columns.
 *		* Problem: Item in column has sub item?  Current code would exit
 *		  all the way back to menu bar and then to next menu instead of
 *		  exiting the sub item and continuing with the next COLUMN.
 *
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <echidna/strings.h>
#include <echidna/menus2.h>
#include <echidna/grafx.h>
#include <echidna/fonts.h>
#include <echidna/eerrors.h>
#include <echidna/listfunc.h>
#include <echidna/hitareas.h>
#include <echidna/windows.h>
#include <echidna/mouse.h>
#include <echidna/keyboard.h>

#include <stdio.h>
#include <ctype.h>
#include <conio.h>

#define LOCAL static

/**************************** C O N S T A N T S ***************************/

#define MAXCOLUMNS			20

#define MENU_BLACK			254
#define MENU_WHITE			255
#define MENU_XOR			(MenuBlack ^ MenuWhite)

#define	MENU_TEXT_HEIGHT	MenuTextHeight
#define MENU_TEXT_BREAK		MenuTextBreak
#define MENU_KEYQUAL_WIDTH	MenuKeyqualWidth
#define MENU_CHECK_WIDTH	MenuCheckWidth
#define MENU_POPUP_WIDTH	MenuPopupWidth

#define MENUITEM_HEIGHT		(MENU_TEXT_HEIGHT + 3)

#define TOP_MENU_SEP		8

#define MNU_RADIO_OFF_SHAPE	0
#define MNU_RADIO_ON_SHAPE	1
#define MNU_CHECK_OFF_SHAPE	2
#define MNU_CHECK_ON_SHAPE	3
#define MNU_SUBMENU_SHAPE	4
#define MNU_SHIFT_SHAPE		5
#define MNU_CTRL_SHAPE		6
#define MNU_ALT_SHAPE		7

#define NEWPOP_POPDOWN		3
#define NEWPOP_POPDOWNBACK	(-1)
#define NEWPOP_POPDOWNFORE	1
#define NEWPOP_ABORT		(-2)
#define NEWPOP_DOEVENT		2

#define MNU_BUTTONS			(mouse->Buttons & (MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON))
#define MNU_RBUTTON			(mouse->Buttons & MOUSE_RIGHTBUTTON)
#define MNU_LBUTTON			(mouse->Buttons & MOUSE_LEFTBUTTON)

/******************************** T Y P E S *******************************/

typedef struct {
	short			 Width;
	short			 Checked;
	short			 Popup;
} MenuColumnInfo;

/****************************** G L O B A L S *****************************/

short					 MenuSticky;
short					 MenuBarHeight;

LOCAL short				 MenuScreenWidth  = 320;
LOCAL short				 MenuScreenHeight = 200;
LOCAL short				 MenuBlack        = 254;
LOCAL short				 MenuWhite        = 255;

LOCAL short				 MenuTextHeight;
LOCAL short				 MenuTextBreak;
LOCAL short				 MenuKeyqualWidth;
LOCAL short				 MenuCheckWidth;
LOCAL short				 MenuPopupWidth;

LOCAL MenuColumnInfo	 ColumnInfo[MAXCOLUMNS];
LOCAL Font				*MenuFont;
LOCAL GShape			**MenuSymbols;

LOCAL short				 MenuNoSave;
LOCAL short				 MenuNoDraw;

LOCAL short				 MenuDepth;

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * MenuStringLen
 *
 * SYNOPSIS
 *		short MenuStringLen (char *str)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short MenuStringLen (char *str)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MenuStringLen";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		char	 work[80];
		char	*w;
		short	 flag;

		w    = work;
		flag = FALSE;
		while (*str) {
			if (!flag && *str == '&') {
				flag = TRUE;
			} else {
				flag = FALSE;
			}
			if (!flag) {
				*w++ = *str;
			}
			str++;
		}
		*w = '\0';
		return StringLen (MenuFont, work);
	}
} /* MenuStringLen */

/*********************************************************************
 *
 * DrawMenuString
 *
 * SYNOPSIS
 *		void DrawMenuString (short x, short y, char *str)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void DrawMenuString (short x, short y, char *str)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawMenuString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		char	 work[80];
		char	*w;
		short	 len;

		w = work;
		while (*str) {
			if (*str == '&') {
				*w  = '\0';
				w   = work;
				len = StringLen (MenuFont, work);
				DrawString (MenuFont, x, y, work);
				x += len;
				str++;
				if (*str) {
					work[0] = *str;
					work[1] = '\0';
					len = StringLen (MenuFont, work);
					SetPenColor (MENU_BLACK);
					DrawRect (x, y + MENU_TEXT_HEIGHT, len - 1, 1);
				}
			} else {
				*w++ = *str++;
			}
		}
		*w  = '\0';
		DrawString (MenuFont, x, y, work);
	}
} /* DrawMenuString */

/*********************************************************************
 *
 * HighlightMenuItem
 *
 * SYNOPSIS
 *		void HighlightMenuItem (MenuItem *mi)
 *
 * PURPOSE
 *		XOR MenuItem Area
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void HighlightMenuItem (PopupItem *pop, MenuItem *mi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HighlightMenuItem";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	BeforeGraphics ();

	SetPenColor (MENU_XOR);
	XORRect (pop->PopX + mi->X, pop->PopY + mi->Y, mi->Width, mi->Height);

	AfterGraphics ();
} /* HighlightMenuItem */


/*********************************************************************
 *
 * ComputeMenuColumn
 *
 * SYNOPSIS
 *		LOCAL short ComputeMenuColumn (
 *							MenuItem	**mi,
 *							short		 column,
 *							short		*height,
 *							short		 horiz)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL short ComputeMenuColumn (
					MenuItem	**miptr,
					short		 column,
					short		 y,
					short		*maxheight,
					short		 horiz)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ComputeMenuColumn";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MenuColumnInfo	*mci;
		EventState		*es;
		MenuItem		*mi;
		short			 width;
		short			 height;

		mi    = *miptr;
		mci   = &ColumnInfo[column];

		mci->Width   = 0;
		mci->Checked = FALSE;
		mci->Popup   = FALSE;
		height       = 0;

		while (!EndOfList (mi)) {
			width  = 0;
			es     = mi->Event;

			/*
			** If Separator add space for it
			*/
			if (!horiz) {
				if (mi->Flags & MNU_SEPARATOR) {
					height += 3;
				}
			}

			/*
			** Flag Checked and Popup;
			*/
			if (es && !horiz) {
				if (es->Flags & (MNU_HAS_CHECK | MNU_HAS_RADIO)) {
					mci->Checked = TRUE;
				}
			}

			if (!horiz) {
				if (mi->Flags & MNU_POPUP) {
					mci->Popup = TRUE;
				}
			}

			/*
			** Left Text Only Kludge
			*/
			if (!mci->Checked && !mci->Popup && !mi->RightText && !mi->KeyText) {
				width += 2;
			}

			/*
			** Compute Length of Menu Text
			*/
			if (mi->LeftText) {
				width += MenuStringLen (mi->LeftText);
			}

			if (mi->RightText) {
				width += MENU_TEXT_BREAK;
				width += MenuStringLen (mi->RightText);
			}

			if (mi->KeyText) {
				width += MENU_TEXT_BREAK;
				width += StringLen (MenuFont, mi->KeyText);
				if (mi->Flags & MNU_SHIFT) {
					width += MENU_KEYQUAL_WIDTH;
				}
				if (mi->Flags & MNU_CTRL) {
					width += MENU_KEYQUAL_WIDTH;
				}
				if (mi->Flags & MNU_ALT) {
					width += MENU_KEYQUAL_WIDTH;
				}
			}

			/*
			** Track MaxWidth
			*/
			if (width > mci->Width) {
				mci->Width = width;
			}

			/*
			** Track height of menu
			*/
			if (horiz) {
				mi->Width = width + TOP_MENU_SEP;
				height   += width + TOP_MENU_SEP;
				if (height >= MenuScreenWidth) {
					break;
				}
			} else {
				height += MENUITEM_HEIGHT;
				if (height > *maxheight) {
					*maxheight = height;
				}
			}

			mi = Next (mi);
			*miptr = mi;

			if (!EndOfList (mi))

				/*
				** If at bottom of screen, force end of column
				*/
				if (!horiz) {
					if (y + height + MENUITEM_HEIGHT >= MenuScreenHeight) {
						if (!(mi->Flags & (MNU_BREAK | MNU_BARBREAK))) {
							mi->Flags |= MNU_BREAK;
						break;
					}
				}
				if (mi->Flags & (MNU_BREAK | MNU_BARBREAK)) {
/**/				break;
				}

			}
		}

		if (mci->Checked) {
			mci->Width += MENU_CHECK_WIDTH;
		}
		if (mci->Popup) {
			mci->Width += MENU_POPUP_WIDTH;
		}

		return mci->Width;
	}
} /* ComputeMenuColumn */

/*********************************************************************
 *
 * DrawMenuColumn
 *
 * SYNOPSIS
 *		LOCAL void DrawMenuColumn (
 *							short	 		 x,
 *							short			 y,
 *							short			 tlx,
 *							short			 tly,
 *							MenuItem		**miptr,
 *							short			 column,
 *							short			 horiz)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void DrawMenuColumn (
					short	 		 x,
					short			 y,
					short	 		 tlx,
					short			 tly,
					MenuItem		**miptr,
					short			 column,
					short			 horiz)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawMenuColumn";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MenuColumnInfo	*mci;
		EventState		*es;
		MenuItem		*mi;
		short			 width;
		short			 height;
		short			 rx;
		short			 lx;
		short			 wy;
		short			 ty;

		mi     = *miptr;
		mci    = &ColumnInfo[column];
		width  = mci->Width;
		height = 0;
		wy     = y;

		while (!EndOfList (mi)) {
			ty = wy + 1;
			lx = x;
			if (horiz) {
				width = mi->Width;
				rx    = x + width;
			} else {
				rx = x + width;
			}

			if (!horiz) {
				if (mi->Flags & MNU_SEPARATOR) {
					if (!MenuNoDraw) {
						SetPenColor (MENU_BLACK);
						DrawRect (lx, wy + 1, width, 1);
					}
					wy     += 3;
					ty     += 3;
					height += 3;
				}
			}

			mi->X      = lx - tlx;
			mi->Y      = wy - tly;
			mi->Height = MENUITEM_HEIGHT;
			mi->Width  = width - (!horiz);

			#if 0
			SetPenColor (253);
			DrawRect (tlx + mi->X, tly + mi->Y, mi->Width, mi->Height);
			#endif

			if (horiz) {
				rx -= TOP_MENU_SEP / 2;
				lx += TOP_MENU_SEP / 2;
			}

			es = mi->Event;
			if (es && !horiz) {
				short	shape = (-1);

				if (es->Flags & MNU_HAS_CHECK) {
					if (es->StateVar && *es->StateVar) {
						shape = MNU_CHECK_ON_SHAPE;
					} else {
						shape = MNU_CHECK_OFF_SHAPE;
					}
				} else if (es->Flags & MNU_HAS_RADIO) {
					if (es->StateVar && *es->StateVar) {
						shape = MNU_RADIO_ON_SHAPE;
					} else {
						shape = MNU_RADIO_OFF_SHAPE;
					}
				}

				if (!MenuNoDraw && shape >= 0) {
					DrawGShape (MenuSymbols[shape], lx + 1, ty);
				}
			}

			if (mci->Checked) {
				lx += MENU_CHECK_WIDTH;
			} else {
				lx += 1;
			}

			if (mci->Popup) {
				rx -= MENU_POPUP_WIDTH;

				if (!MenuNoDraw && (mi->Flags & MNU_POPUP)) {
					DrawGShape (MenuSymbols[MNU_SUBMENU_SHAPE], rx + 1, ty + 1);
				}
			}

			if (!MenuNoDraw && mi->LeftText) {
				DrawMenuString (lx, ty, mi->LeftText);
			}

			if (mi->KeyText) {
				rx -= StringLen (MenuFont, mi->KeyText);
				if (!MenuNoDraw) {
					DrawString (MenuFont, rx, ty, mi->KeyText);
				}

				if (mi->Flags & MNU_CTRL) {
					rx -= MENU_KEYQUAL_WIDTH;
					if (!MenuNoDraw) {
						DrawGShape (MenuSymbols[MNU_CTRL_SHAPE], rx, ty);
					}
				}
				if (mi->Flags & MNU_SHIFT) {
					rx -= MENU_KEYQUAL_WIDTH;
					if (!MenuNoDraw) {
						DrawGShape (MenuSymbols[MNU_SHIFT_SHAPE], rx, ty);
					}
				}
				if (mi->Flags & MNU_ALT) {
					rx -= MENU_KEYQUAL_WIDTH;
					if (!MenuNoDraw) {
						DrawGShape (MenuSymbols[MNU_ALT_SHAPE], rx, ty);
					}
				}
				rx -= MENU_TEXT_BREAK;
			}

			if (mi->RightText) {
				rx -= MenuStringLen (mi->RightText);
				if (!MenuNoDraw) {
					DrawMenuString (rx, ty, mi->RightText);
				}
			}

			/*
			** Track height of menu
			*/
			if (horiz) {
				x      += width;
				height += width;
				if (height >= MenuScreenWidth) {
					break;
				}
			} else {
				wy     += MENUITEM_HEIGHT;
				height += MENUITEM_HEIGHT;
			}

			mi = Next (mi);
			*miptr = mi;

			if (!EndOfList (mi)) {

				#if 0
				/*
				** If at bottom of screen, force end of column
				*/
				if (!horiz) {
					if (y + height + MENUITEM_HEIGHT >= MenuScreenHeight) {
						break;
					}
				}
				#endif

				if (mi->Flags & (MNU_BREAK | MNU_BARBREAK)) {
/**/				break;
				}

			}
		}
	}
} /* DrawMenuColumn */

/*********************************************************************
 *
 * DrawPopup
 *
 * SYNOPSIS
 *		short DrawPopup (PopupItem *pop, short x, short y, short horiz)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawPopup (
			PopupItem	*pop,
			short		 x,
			short		 y,
			short		 x2,
			short		 horiz)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawPopup";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (!pop->PopCount) {
		MenuItem	*mi;
		short		 width;
		short		 height;
		short		 column;
		short		 cwidth;
		short		 ywork;
		short		 bf     = FALSE;
		short		 status = FALSE;
		UBYTE		 oldblack;
		UBYTE		 oldwhite;

		/*
		** Compute max width of items
		*/
		width  = 0;
		height = 0;
		column = 0;

		ywork = y;
		mi = Head (pop->MenuItemList);
		while (!EndOfList (mi)) {
			cwidth = ComputeMenuColumn (&mi, column, ywork, &height, horiz);
			if (!cwidth) {
				goto cleanup;
			}
			if (horiz) {
				ywork += MENUITEM_HEIGHT;
			}
			width += cwidth;
			column++;
		}
		
		if (!horiz) {
			width  += 3;
			height += 4;
		} else {
			width   = MenuScreenWidth;
			height += ywork + 1;
			MenuBarHeight = height;
		}

		/*
		** Compute Positions of columns
		*/
		if (!horiz) {
			if (x + width >= MenuScreenWidth) {
				if (MenuDepth <= 2) {
					x -= (x + width - MenuScreenWidth);
				} else {
					x  = x2 - width;
					if (x < 0) {
						x = MenuScreenWidth - width;
						if (x < x2 + 30) {
							x = 0;
						}
					}
				}
			}
			if (MenuDepth <= 2 && y + height >= MenuScreenHeight) {
				y -= (y + height - MenuScreenHeight);
			}
		}

		/*
		** Save Menu Area
		*/
		if (!MenuNoDraw) {
			BeforeGraphics ();
			bf = TRUE;
		}
		if (!MenuNoSave) {
			if (!PushSaveArea (x, y, width, height)) {
				goto cleanup;
			}
			pop->PopCount++;
		}
		pop->PopX      = x;
		pop->PopY      = y;
		pop->PopWidth  = width;
		pop->PopHeight = height;

		/*
		** Set Colors to Menu Colors
		*/
		oldblack = RemapTable[MENU_BLACK];
		oldwhite = RemapTable[MENU_WHITE];
		RemapTable[MENU_BLACK] = MenuBlack;
		RemapTable[MENU_WHITE] = MenuWhite;

		/*
		** DrawMenu Border
		*/
		if (!MenuNoDraw) {
			if (horiz) {
				SetPenColor (MENU_WHITE);
				DrawRect (0, 0, MenuScreenWidth, height - 1);
				SetPenColor (MENU_BLACK);
				DrawRect (0, height - 1, MenuScreenWidth, 1);
			} else {
				SetPenColor (MENU_WHITE);
				DrawRect (x + 1, y + 1, width - 3, height - 3);
				SetPenColor (MENU_BLACK);
				DrawRect (x, y, 1, height - 2);
				DrawRect (x, y, width - 2, 1);
				DrawRect (x + width - 3, y, 1, height - 2);
				DrawRect (x, y + height - 3, width - 2, 1);
				DrawRect (x + width - 2, y + 2, 2, height - 2);
				DrawRect (x + 2, y + height - 2, width - 2, 2);
			}
		}

		/*
		** DrawMenuColumns
		*/
		{
			short	wx;
			short	wy;

			wx     = x + (!horiz);
			wy     = y + (!horiz);
			column = 0;
			mi     = Head (pop->MenuItemList);
			while (!EndOfList (mi)) {
				DrawMenuColumn (wx, wy, x, y, &mi, column, horiz);
				if (horiz) {
					wy += MENUITEM_HEIGHT;
				} else {
					wx += ColumnInfo[column].Width;
					if (!MenuNoDraw) {
						SetPenColor (MENU_BLACK);
						DrawRect (wx - 1, y, 1, height);
					}
				}
				column++;
			}
		}
		
		RemapTable[MENU_BLACK] = oldblack;
		RemapTable[MENU_WHITE] = oldwhite;

		status = TRUE;

cleanup:
		if (bf) {
			AfterGraphics ();
		}
		return status;

	}
	pop->PopCount++;
	return TRUE;

} /* DrawPopup */

/*********************************************************************
 *
 * UndrawPopup
 *
 * SYNOPSIS
 *		LOCAL UndrawPopup (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void UndrawPopup (PopupItem *pop)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UndrawPopup";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (pop->PopCount) {
		pop->PopCount--;
		if (!pop->PopCount) {
			PopSaveArea ();
		}
	}
	
} /* UndrawPopup */

/**************************************************************************
 *
 * LastMenuEvent
 *
 * SYNOPSIS
 *		int LastMenuEvent (void)
 *
 * PURPOSE
 *		Trigger same event as was last chosen from menus.
 *
 * RETURN VALUE
 *
 *
 * BUGS
 *		What if Event is disabled or no longer exists?
*/
static EventState *LastMenuEventPtr = NULL;
short LastMenuEvent (EventInfo *ei)
{
	short flag = TRUE;

	/* So compiler doesn't complain */
	ei = ei;

	if (LastMenuEventPtr) {
		flag = TriggerEvent (LastMenuEventPtr, ETYPE_KEY, NULL);
	}

	return flag;

} /* LastMenuPick */

/*********************************************************************
 *
 * RealCheckMenuKeys
 *
 * SYNOPSIS
 *		MenuItem *RealCheckMenuKeys (PopupItem *pop, USHORT keyvalue, short alt)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
MenuItem *RealCheckMenuKeys (PopupItem *pop, USHORT keyvalue, short alt)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "RealCheckMenuKeys";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		USHORT		 key;
		USHORT		 qual;
		MenuItem	*mi;

		qual = keyvalue >> 8;
		if (!alt || qual & QUAL_ALT) {
			key = ConvertKeyToAscii (keyvalue);
			if (key) {
				key = toupper (key);
				mi  = Head (pop->MenuItemList);
				while (!EndOfList (mi)) {
					char	*s;

					if (mi->LeftText) {
						s = mi->LeftText;
						while (*s) {
							if (*s == '&' && toupper(*(s + 1)) == key) {
								return mi;
							}
							s++;
						}
					}
					if (mi->RightText) {
						s = mi->RightText;
						while (*s) {
							if (*s == '&' && toupper(*(s + 1)) == key) {
								return mi;
							}
							s++;
						}
					}
					mi = Next (mi);
				}
			}
		}

		return NULL;
	}
} /* RealCheckMenuKeys */

/*********************************************************************
 *
 * NextColumn
 *
 * SYNOPSIS
 *		LOCAL MenuItem *NextColumn (MenuItem *mi)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL MenuItem *NextColumn (MenuItem *mi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "NextColumn";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short count;

		count = 0;

		/*
		** Count lines to top of column
		*/
		while (!StartOfList (mi) && !(mi->Flags & (MNU_BREAK | MNU_BARBREAK))) {
			count++;
			mi = Prev (mi);
		}

		/*
		** We were already in the 1st column
		*/
		if (StartOfList (mi)) {
			mi = Next (mi);
			count--;
		}

		/*
		** Find Start Of Next column.
		*/
		mi = Next (mi);
		while (!EndOfList (mi) && !(mi->Flags & (MNU_BREAK | MNU_BARBREAK))) {
			mi = Next (mi);
		}

		/*
		** We are in last column.
		*/
		if (EndOfList (mi)) {
			return NULL;
		}

		/*
		** Now go down column at most 'count' times.
		*/
		while (count > 0) {
			mi = Next (mi);
			if (mi->Flags & (MNU_BREAK + MNU_BARBREAK)) {
				return Prev (mi);
			}
			count--;
		}

		return mi;
	}

} /* NextColumn */

/*********************************************************************
 *
 * PrevColumn
 *
 * SYNOPSIS
 *		LOCAL MenuItem *PrevColumn (MenuItem *mi)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL MenuItem *PrevColumn (MenuItem *mi)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrevColumn";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short count;

		count = 0;

		/*
		** Count lines to top of column
		*/
		while (!StartOfList (mi) && !(mi->Flags & (MNU_BREAK | MNU_BARBREAK))) {
			count++;
			mi = Prev (mi);
		}

		/*
		** We were already in the 1st column
		*/
		if (StartOfList (mi)) {
			return NULL;
		}

		/*
		** Find Start Of Prev column.
		*/
		mi = Prev (mi);
		while (!StartOfList (mi) && !(mi->Flags & (MNU_BREAK | MNU_BARBREAK))) {
			mi = Prev (mi);
		}

		if (StartOfList (mi)) {
			mi = Next (mi);
		}

		/*
		** Now go down column at most 'count' times.
		*/
		while (count > 0) {
			mi = Next (mi);
			if (mi->Flags & (MNU_BREAK + MNU_BARBREAK)) {
				return Prev (mi);
			}
			count--;
		}

		return mi;
	}

} /* PrevColumn */

/*********************************************************************
 *
 * FindMenuItem
 *
 * SYNOPSIS
 *		MenuItem *FindMenuItem (PopupItem *pop, MouseInfo *mi)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
MenuItem *FindMenuItem (PopupItem *pop, MouseInfo *mouse)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FindMenuItem";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MenuItem	*mi;
		short		 x;
		short		 y;

		x  = mouse->X - pop->PopX;
		y  = mouse->Y - pop->PopY;
		mi = Head (pop->MenuItemList);
		while (!EndOfList (mi)) {
			if (x >= mi->X && x < mi->X + mi->Width) {
				if (y >= mi->Y && y < mi->Y + mi->Height) {
					return mi;
				}
			}
			mi = Next (mi);
		}
		return NULL;
	}
} /* FindMenuItem */

/*********************************************************************
 *
 * ComputePopupPosition
 *
 * SYNOPSIS
 *		LOCAL void ComputePopupPosition (
 *						PopupItem	*pop,
 *						MenuItem	*mi,
 *						short		*x,
 *						short		*y,
 *						short		*x2)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL void ComputePopupPosition (
				PopupItem	*pop,
				MenuItem	*mi,
				short		*x,
				short		*y,
				short		*x2)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ComputePopupPosition";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (MenuDepth <= 1) {
		*x  = pop->PopX + mi->X;
		*x2 = 0;
		*y  = pop->PopY + mi->Y + mi->Height;
	} else {
		*x  = pop->PopX + mi->X + mi->Width - 3;
		*x2 = pop->PopX + mi->X + 3;
		*y  = pop->PopY + mi->Y;
	}
} /* ComputePopupPosition */

/*********************************************************************
 *
 * CheckOldPopups
 *
 * SYNOPSIS
 *		LOCAL PopupItem	*CheckOldPopups (PopupItem *pop, MouseInfo *mouse)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
LOCAL PopupItem	*CheckOldPopups (PopupItem *pop, MouseInfo *mouse)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CheckOldPopups";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		while (pop) {
			if (mouse->X >= pop->PopX && mouse->X < pop->PopX + pop->PopWidth) {
				if (mouse->Y >= pop->PopY && mouse->Y < pop->PopY + pop->PopHeight) {
					return pop;
				}
			}
			pop = pop->Parent;
		}
		return NULL;
	}
} /* CheckOldPopups */

/*********************************************************************
 *
 * HandlePopup
 *
 * SYNOPSIS
 *		short HandlePopup (PopupItem *pop, MenuItem *mi)
 *
 * PURPOSE
 *		Allow user to use the menus.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short HandlePopup (
			PopupItem	*pop,
			MenuItem	*mi,
			MouseInfo	*mouse,
			short		 x,
			short		 y,
			short		 x2)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HandlePopup";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		short		 bringdown = TRUE;
		short		 nomi      = FALSE;
		short		 justdn    = FALSE;
		short		 newgoin;
		short		 newpop;
		short		 goin;
		MenuItem	*newmi;
		PopupItem	*subpop;

		MenuDepth++;

		if (MNU_BUTTONS) {
			mi = FindMenuItem (pop, mouse);
			bringdown = FALSE;
		} else {
			if (!mi) {
				mi = Head (pop->MenuItemList);
				bringdown = FALSE;
			}
		}

		if (DrawPopup (pop, x, y, x2, MenuDepth == 1)) {
			newpop = 0;
			if (bringdown) {
				newgoin   = TRUE;
				bringdown = FALSE;
			} else {
				newgoin   = FALSE;
			}
			while (!newpop) {

				if (!justdn) {
					subpop = NULL;
					if (mi) {
						HighlightMenuItem (pop, mi);
						/*
						** If using mouse then if this item as a subitem (popup)
						** display it.
						*/
						if (MNU_BUTTONS) {
							if (mi->Flags & MNU_POPUP) {
								ComputePopupPosition (pop, mi, &x, &y, &x2);
								MenuDepth++;
								DrawPopup ((PopupItem *)mi, x, y, x2, FALSE);
								MenuDepth--;
								subpop = (PopupItem *)mi;
							}
						}
					}
				}

				justdn  = FALSE;
				newmi   = NULL;
				goin    = newgoin;
				newgoin = FALSE;
				while (!newpop && !goin && !newmi && !nomi) {

					/*
					** Using the mouse
					*/
					ReadMouse (mouse);
					if (MNU_BUTTONS) {

						/*
						** If a lower popup is showing then if the mouse is
						** inside it then switch to the new popup.
						*/
						if (subpop &&
							mouse->X >= subpop->PopX && mouse->X < subpop->PopX + subpop->PopWidth &&
							mouse->Y >= subpop->PopY && mouse->Y < subpop->PopY + subpop->PopHeight) {
							goin = TRUE;
						} else {
							/*
							** Is it in a new item in the current popup
							*/
							newmi = FindMenuItem (pop, mouse);
							if (!mi || (mi && !newmi)) {
								/*
								** No so is it in a lower popup?
								*/
								PopupItem	*ppop;

								if (pop->Parent && (ppop = CheckOldPopups (pop, mouse)) != 0 && pop != ppop) {
									newpop = NEWPOP_POPDOWN;
									newmi  = NULL;
								} else if (mi) {
									nomi = TRUE;
								}
							} else if (mi == newmi) {
								newmi = NULL;
							}
						}
					} else if (!MNU_BUTTONS && (mouse->OldButtons & (MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON))) {
						/*
						** Did user just let off button?
						**
						** Yes so if user was on event item return
						*/
						if (mi && mi->Event) {
							LastMenuEventPtr = mi->Event;
							newpop  = NEWPOP_DOEVENT;
						} else {
							if (MenuSticky) {
								/*
								** User was not on event item.  If user was
								** not on the menus at all then abort the
								** menus.
								*/
								if (!mi) {
									if (!CheckOldPopups (pop, mouse)) {
										newpop = NEWPOP_ABORT;
									} else {
										newmi = Head (pop->MenuItemList);
									}
								} else {
									if (mi->Flags & MNU_POPUP) {
										goin = TRUE;
									}
								}
							} else {
								newpop = NEWPOP_ABORT;
							}
						}
					} else {

					/*
					** Using Keys
					*/

						USHORT	key;
						USHORT	eascii;

						key = GetKeyNoWait ();
						if (key) {
							eascii = ConvertKeyToAscii (key);
							if (eascii) {
								switch (eascii) {
								case EASCII_UP:
									if (MenuDepth == 1) {
										goin = TRUE;
									} else {
										newmi = Prev (mi);
										if (StartOfList (newmi)) {
											newmi = Tail (pop->MenuItemList);
										}
									}
									break;
								case EASCII_DOWN:
									if (MenuDepth == 1) {
										goin = TRUE;
									} else {
										newmi = Next (mi);
										if (EndOfList (newmi)) {
											newmi = Head (pop->MenuItemList);
										}
									}
									break;
								case EASCII_LEFT:
									if (MenuDepth == 1) {
										newmi = Prev (mi);
										if (StartOfList (newmi)) {
											newmi = Tail (pop->MenuItemList);
										}
									} else {
										newmi = PrevColumn (mi);
										if (!newmi) {
											if (MenuDepth == 2) {
												newpop = NEWPOP_POPDOWNBACK;
											} else {
												newpop = NEWPOP_POPDOWN;
											}
										}
									}
									break;
								case EASCII_RIGHT:
									if (MenuDepth == 1) {
										newmi = Next (mi);
										if (EndOfList (newmi)) {
											newmi = Head (pop->MenuItemList);
										}
									} else {
										if (mi->Flags & MNU_POPUP) {
											goin = TRUE;
										} else {
											newmi = NextColumn (mi);
											if (!newmi) {
												newpop = NEWPOP_POPDOWNFORE;
											}
										}
									}
									break;
								case 0x1B:
									newpop = NEWPOP_POPDOWN;
									break;
								case EASCII_F10:
									newpop = NEWPOP_ABORT;
									break;
								case 0x0D:
									if (mi->Flags & MNU_POPUP) {
										goin = TRUE;
									} else if (mi->Event) {
										LastMenuEventPtr = mi->Event;
										newpop = NEWPOP_DOEVENT;
									}
									break;
								default:
									newmi = RealCheckMenuKeys (pop, key, FALSE);
									if (newmi) {
										if (newmi->Event) {
											LastMenuEventPtr = newmi->Event;
											newpop  = NEWPOP_DOEVENT;
										} else {
											newgoin = TRUE;
										}
									}
									break;
								}
							}
						}
					}
				}

				if (goin) {
					goin = FALSE;
					if (mi->Flags & MNU_POPUP) {
						ComputePopupPosition (pop, mi, &x, &y, &x2);
						newpop = HandlePopup ((PopupItem *)mi, NULL, mouse, x, y, x2);
						switch (newpop) {
						case NEWPOP_POPDOWN:
							newpop = 0;
							justdn = TRUE;
							break;
						case NEWPOP_POPDOWNBACK:
							if (MenuDepth < 2) {
								newgoin = TRUE;
								newpop  = 0;
								newmi   = Prev (mi);
								if (StartOfList (newmi)) {
									newmi = Tail (pop->MenuItemList);
								}
							}
							break;
						case NEWPOP_POPDOWNFORE:
							if (MenuDepth < 2) {
								newgoin = TRUE;
								newpop  = 0;
								newmi   = Next (mi);
								if (EndOfList (newmi)) {
									newmi = Head (pop->MenuItemList);
								}
							}
							break;
						case NEWPOP_DOEVENT:
						case NEWPOP_ABORT:
							break;
						}
					}
				}
			
				if (!justdn && mi) {
					if (subpop) {
						UndrawPopup ((PopupItem *)mi);
					}
					HighlightMenuItem (pop, mi);
				}

				if (newmi || nomi) {
					mi   = newmi;
					nomi = FALSE;
				}

			}

			UndrawPopup (pop);
		}
		MenuDepth--;

		return newpop;
	}
} /* HandlePopup */

/*********************************************************************
 *
 * DoLocalMenus
 *
 * SYNOPSIS
 *		short DoLocalMenus (MenuItem *mi)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DoLocalMenus (PopupItem *pop, MenuItem *mi, MouseInfo *mouse)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DoLocalMenus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short	 newpop;

		if (PushClipValues (0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)) {

			newpop = HandlePopup (pop, mi, mouse, 0, 0, 0);

			PopClipValues ();

			if (newpop == NEWPOP_DOEVENT && LastMenuEventPtr) {
				TriggerEvent (LastMenuEventPtr, ETYPE_KEY, NULL);
			}
		}

		return TRUE;
	}
} /* DoLocalMenus */

/*********************************************************************
 *
 * DoMenus
 *
 * SYNOPSIS
 *		short DoMenus (MenuItem *mi)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DoMenus (MenuItem *mi, MouseInfo *mouse)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DoMenus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		return DoLocalMenus (MainPopup, mi, mouse);
	}
} /* DoMenus */

/*********************************************************************
 *
 * CheckMenuKeys
 *
 * SYNOPSIS
 *		short CheckMenuKeys (USHORT keyvalue)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short CheckMenuKeys (USHORT keyvalue)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CheckMenuKeys";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		MenuItem	*mi;
		MouseInfo	 mouse = { 0, };

		mi = RealCheckMenuKeys (MainPopup, keyvalue, TRUE);
		if (mi) {
			DoMenus (mi, &mouse);
		}
		return TRUE;
	}

} /* CheckMenuKeys */

/*********************************************************************
 *
 * HandleMenus
 *
 * SYNOPSIS
 *		short HandleMenus (Window *win)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short HandleMenus (Window *win, MouseInfo *mouse)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HandleMenus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		win = win;

		DoMenus (NULL, mouse);
		return TRUE;
	}

} /* HandleMenus */

/*********************************************************************
 *
 * SetMenuInfo
 *
 * SYNOPSIS
 *		void SetMenuInfo (
 *					short	 swidth,
 *					short	 sheight,
 *					Font	*font,
 *					GShape	**symbols)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetMenuInfo (
			short	 swidth,
			short	 sheight,
			Font	*font,
			GShape	**symbols)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetMenuInfo";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	MenuScreenWidth  = swidth;
	MenuScreenHeight = sheight;
	MenuFont         = font;
	MenuSymbols      = symbols;

	MenuCheckWidth   = symbols[MNU_RADIO_OFF_SHAPE]->Width + 2;
	MenuPopupWidth   = symbols[MNU_SUBMENU_SHAPE]->Width + 2;
	MenuKeyqualWidth = symbols[MNU_SHIFT_SHAPE]->Width;
	MenuTextHeight   = font->Letters[0]->Height;
	MenuTextBreak    = font->Letters[0]->Width * 3 / 2;

} /* SetMenuInfo */

/*********************************************************************
 *
 * SetMenuColors
 *
 * SYNOPSIS
 *		void SetMenuColors (short white, short black)
 *
 * PURPOSE
 *		Set the colors used to draw the menus.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetMenuColors (short white, short black)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetMenuColors";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	MenuBlack = white;
	MenuWhite = black;

} /* SetMenuColors */

/*********************************************************************
 *
 * DrawMenuBar
 *
 * SYNOPSIS
 *		void DrawMenuBar (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DrawMenuBar (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawMenuBar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		MenuNoSave = TRUE;
		DrawPopup (MainPopup, 0, 0, 0, TRUE);
		MenuNoSave = FALSE;
	}
} /* DrawMenuBar */

/*********************************************************************
 *
 * ComputeMenuBarHeight
 *
 * SYNOPSIS
 *		short ComputeMenuBarHeight (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short ComputeMenuBarHeight (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ComputeMenuBarHeight";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	MenuNoDraw = TRUE;
	DrawMenuBar ();
	MenuNoDraw = FALSE;
	return MenuBarHeight;

} /* ComputeMenuBarHeight */

