/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * READMENU.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 02/28/92
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Read/Parse the menus from the .INI file/list
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/readini.h>
#include <echidna/listfunc.h>
#include <echidna/eerrors.h>
#include <echidna/argparse.h>
#include <echidna/strings.h>
#include <echidna/menus2.h>
#include <echidna/keyboard.h>
#include <echidna/events.h>

#include <stdio.h>

#define LOCAL static

/**************************** C O N S T A N T S ***************************/

#define MAX_MENU_LINE	256
#define	MAX_ARGS		8

/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/

LOCAL PopupItem			 MainPopupX;
PopupItem				*MainPopup = &MainPopupX;

LOCAL BOOL fShowKeyCmds = TRUE;

/******************************* M A C R O S ******************************/



/******************************* T A B L E S ******************************/

#define MARG_TEXT			(newargs[0])
#define MARG_EVENT			(newargs[1])
#define MARG_SEPARATOR		(newargs[2])
#define MARG_MENUBREAK		(newargs[3])
#define MARG_MENUBARBREAK	(newargs[4])

LOCAL ArgSpec MenuItemTemplate[] = {
	{	STANDARD_ARG,		"TEXT",			"", },
	{	STANDARD_ARG,		"EVENT",		"",	},
	{	SWITCH_ARG,			"SEPARATOR",	"",	},
	{	SWITCH_ARG,			"MENUBREAK",	"", },
	{	SWITCH_ARG,			"MENUBARBREAK",	"",	},
	{	0, NULL, NULL, },
};

/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * SetShowKeyCmds
 *
 * PURPOSE
 *		Set flag whether to show keyboard shortcuts on menus.
 *		This affects how the menus are read in (ie., if you read
 *		them w/out key cmds, you can't turn them back on and expect
 *		to see them on the menus).
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *		07/31/93 Saturday (dcc) - created.
 *
*/
void SetShowKeyCmds(BOOL f)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetShowKeyCmds";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	fShowKeyCmds = f;

} /* SetShowKeyCmds */


/*********************************************************************
 *
 * ParsePopup
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short ParsePopup (SectionTracker *sec, PopupItem *popup, short goodscan)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ParsePopup";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		ConfigLine		*cl;
		char			 line[MAX_MENU_LINE];
		short			 goodline;
		short			 LineNo;
		short			 p_argc;
		char			*p_argv[MAX_ARGS];
		int				 junkcount;
		char			**junkargs;
		char			**newargs;

		while ((cl = GetNextINILine (sec)) != NULL) {
			strncpy (line, NodeName (cl), MAX_MENU_LINE - 1);
			line[MAX_MENU_LINE - 1] = '\0';
			LineNo = GetConfigLineNo (cl);
			StripComment (line);

			goodline = TRUE;

			p_argc = argify (line, MAX_ARGS, p_argv);
			if (p_argc > 1) {
				junkcount = p_argc - 1;
				junkargs  = &p_argv[1];
				newargs   = argvark (&junkcount, &junkargs, MenuItemTemplate, FALSE);
				if (!newargs) {
					printf ("Error at line %d : %s\n", LineNo, GlobalErrMsg);
					ClearGlobalError ();
					goodscan = FALSE;
					goodline = FALSE;
				}
			}
			if (goodline) {
				if (!stricmp (p_argv[0], "STICKYMENUS")) {
					MenuSticky = TRUE;
				} else if (!stricmp (p_argv[0], "POPDOWN")) {
/**/				return goodscan;
				} else {
					MenuItem	 mi;

					/*
					** Clear out menu struct
					*/
					memset (&mi, 0, sizeof (mi));

					/*
					** Get Menu Flags
					*/
					mi.Flags |= (SWITCH_VALUE(MARG_SEPARATOR   )) ? MNU_SEPARATOR : 0;
					mi.Flags |= (SWITCH_VALUE(MARG_MENUBREAK   )) ? MNU_BREAK     : 0;
					mi.Flags |= (SWITCH_VALUE(MARG_MENUBARBREAK)) ? MNU_BARBREAK  : 0;
					
					/*
					** Separate Text into right and left text
					*/
					mi.LeftText  = NULL;
					mi.RightText = NULL;
					if (MARG_TEXT) {
						char	*s;
						char	*r;

						s   = MARG_TEXT;
						r   = NULL;

						while (*s) {
							if (*s == '\\') {
								if (*(s + 1) == 't') {
								   *s = '\0';
								   r  = s + 2;
								   s--;
								}
							}
							s++;
						}
						mi.LeftText = dupstr (MARG_TEXT);
						if (r) {
							mi.RightText = dupstr (r);
						}
						if (!mi.LeftText || (r && !mi.RightText)) {
							SetGlobalErr (ERR_OUT_OF_MEMORY);
							GEcatf1 ("\nERROR at line %d : OOM:Reading menu", LineNo);
							return FALSE;
						}
					}

					/*
					** Find Associated Event
					*/

					mi.Event = NULL;
					if (MARG_EVENT) {
						mi.Event = FindEvent (MARG_EVENT);
						if (!mi.Event) {
							printf ("Error at line %d : No such event '%s'\n", LineNo, MARG_EVENT);
							goodscan = FALSE;
						} else {
							USHORT	qualifier;

							/*
							** Add radio button or checkmark to menu
							** if event requires them.
							*/
							mi.Flags |= (USHORT) (mi.Event->Flags & (MNU_HAS_CHECK | MNU_HAS_RADIO));

							/*
							** Add First Event Key to RightText
							*/
							if (fShowKeyCmds && (mi.KeyText = FindFirstEventKey (MARG_EVENT, &qualifier)) != NULL) {
								mi.Flags |= (qualifier & QUAL_SHIFT ) ? MNU_SHIFT  : 0;
								mi.Flags |= (qualifier & QUAL_CTRL  ) ? MNU_CTRL   : 0;
								mi.Flags |= (qualifier & QUAL_ALT   ) ? MNU_ALT    : 0;
							}
						}
					}

					if (!stricmp (p_argv[0], "POPUP")) {
						PopupItem	*newpopup;

						if (MARG_EVENT) {
							printf ("Error at line %d : POPUPs can NOT have an EVENT!", LineNo);
							goodscan = FALSE;
						}
						newpopup = CreateNode (sizeof (PopupItem), NULL);
						if (!newpopup) {
							SetGlobalErr (ERR_OUT_OF_MEMORY);
							GEcatf1 ("\nERROR at line %d : OOM:Adding POPUP menu", LineNo);
							return FALSE;
						}
						
						newpopup->Item = mi;
						newpopup->Item.Flags |= MNU_POPUP;
						newpopup->Parent = popup;
						newpopup->MenuItemList = &newpopup->MenuItemListX;
						InitList (newpopup->MenuItemList);
						AddTail (popup->MenuItemList, newpopup);
						goodscan = ParsePopup (sec, newpopup, goodscan);
					} else if (!stricmp (p_argv[0], "MENUITEM")) {
						MenuItem	*newmenu;

						newmenu  = CreateNode (sizeof (MenuItem), NULL);
						*newmenu = mi;
						AddTail (popup->MenuItemList, newmenu);
					} else {
						printf ("Error at line %d : Unknown keyword '%s'\n", LineNo, p_argv[0]);
						goodscan = FALSE;
					}
				}
			}
		}
		return goodscan;
	}
} /* ParsePopup */


/*********************************************************************
 *
 * InitMenus
 *
 * SYNOPSIS
 *		short InitMenus (IniList *inilist)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short InitMenus (IniList *inilist)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitMenus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		SectionTracker	 secx;
		SectionTracker	*sec = &secx;

		sec = FindSection (sec, inilist, "[MENU]");
		if (!sec) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nNo [MENU] section found in .ini file");
			return FALSE;
		}

		MainPopup->MenuItemList = &MainPopup->MenuItemListX;
		InitList (MainPopup->MenuItemList);

		#if 1
		return ParsePopup (sec, MainPopup, TRUE);
		#else
		{
			short result;
			extern void PrintMenus (void);

			result = ParsePopup (sec, MainPopup, TRUE);
			if (result) {
				PrintMenus ();
			}
			return result;
		}
		#endif
	}
} /* InitMenus */

#if 0
static short tabs;

void StartLine (void) {
	short		 i;

	for (i = 1; i < tabs; i++) {
		printf ("   ");
	}
}

#define SAFESTR(str) ((str) ? (str) : "NULL")

void PrintPopup (PopupItem *pop)
{
	MenuItem	*mi;

	tabs++;

	mi = Head (pop->MenuItemList);
	while (!EndOfList (mi)) {
		if (mi->Flags & MNU_POPUP) {
			StartLine ();
			printf ("Popup LT='%s',KT='%s',RT='%s'\n",
				SAFESTR(mi->LeftText),
				SAFESTR(mi->KeyText),
				SAFESTR(mi->RightText));
			PrintPopup ((PopupItem *)mi);
		} else {
			StartLine ();
			printf ("MenuItem LT='%s',KT='%s',RT='%s',ES='%s'\n",
				SAFESTR(mi->LeftText),
				SAFESTR(mi->KeyText),
				SAFESTR(mi->RightText),
				mi->Event ? SAFESTR (NodeName (mi->Event)) : "NOEVENT");
			StartLine ();
			printf ("  %s|%s|%s|%s|%s|%s|%s|%s\n",
				((mi->Flags & MNU_HAS_CHECK) ? "CHECK" : "     "),
				((mi->Flags & MNU_HAS_RADIO) ? "RADIO" : "     "),
				((mi->Flags & MNU_SEPARATOR) ? "SEPARATOR" : "         "),
				((mi->Flags & MNU_BREAK) ? "BREAK" : "     "),
				((mi->Flags & MNU_BARBREAK) ? "BARBREAK" : "        "),
				((mi->Flags & MNU_SHIFT) ? "SHIFT" : "     "),
				((mi->Flags & MNU_CTRL) ? "CTRL" : "    "),
				((mi->Flags & MNU_ALT) ? "ALT" : "   "));
		}

		mi = Next (mi);
	}

	tabs--;
}
void PrintMenus (void)
{
	PrintPopup (MainPopup);
}
#endif

