/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * SCIC.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/05/92
 *   MODIFIED : 07/27/94
 *       TABS : 04 08
 *
 * DESCRIPTION
 *		Dimensioned-Image Object routines.
 *		
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>

#include <echidna/scic.h>
#include <echidna/grafx.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * DrawVerticalArrow
 *
 * SYNOPSIS
 *		short DrawVerticalArrow (
 *			short x, short y, 
 *			short cx, short cy, 
 *			short dyDir
 *		)
 *
 * PURPOSE
 *		To draw an arrow pointing up or down depending on the value of dyDir 
 *		(-1 for up, +1 for down)
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawVerticalArrow (
	short x, short y, 
	short cx, short cy, 
	short dyDir
)
{
	short xCenter = x + (cx / 2);
	short yCenter = y + (cy / 2);

	if (cx >= cxVArrowMin && cy >= cyVArrowMin) {
		short xHead, yHead, cxHead;
		short xShaft, yShaft;
		short cxFlap;	/* Width of arrow to either side of center pixel */
		short cyShaft, cyHead;
		short cp;
		
		cxHead = cx - !(cx&1); /* make sure it is odd for pointy tip */ 
		cyHead = (cxFlap = cxHead / 2) + 1;	
		cyShaft = cy / 2;
		cxFlap = (cp = min(cyHead, cyShaft)) - 1;

		xHead = xCenter;
		yHead = yCenter - ((dyDir > 0) && !(cy&1))  + cp * dyDir;
		xShaft = xCenter - (cxFlap / 2);
		yShaft = yCenter - ((dyDir > 0) ? (cp-(cy&1)) : 0);

		BeforeGraphics();
		/* Draw Shaft */
		DrawRect (xShaft, yShaft, 1 + 2 * (cxFlap/2), cp);

		/* Draw tip of head */
 		DrawPixel (xHead, yHead);
		--xHead; 
		yHead -= dyDir;
		cxHead = 3;
		--cp;
		/* Draw remainder of head */
		while (cp) {
			DrawRect (xHead, yHead, cxHead, 1);			
			--xHead;
			cxHead += 2;
			yHead -= dyDir;
			--cp;
		}
		AfterGraphics();

	} else {
		return FALSE;
	}

	return TRUE;
} /* DrawVerticalArrow */

/*********************************************************************
 *
 * DrawHorizontalArrow
 *
 * SYNOPSIS
 *		short DrawHorizontalArrow (
 *			short x, short y, 
 *			short cx, short cy, 
 *			short dxDir
 *		)
 *
 * PURPOSE
 *		To draw an arrow pointing left or right depending on the value of 
 *		dxDir (-1 for left, +1 for right)
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawHorizontalArrow (
	short x, short y, 
	short cx, short cy, 
	short dxDir
)
{
	short xCenter = x + (cx / 2);
	short yCenter = y + (cy / 2);

	if (cx >= cxHArrowMin && cy >= cyHArrowMin) {
		short xHead, yHead, cyHead;
		short xShaft, yShaft;
		short cyFlap;	/* Height of arrow to either side of center pixel */
		short cxShaft, cxHead;
		short cp;
		
		cyHead = cy - !(cy&1); /* make sure it is odd for pointy tip */ 
		cxHead = (cyFlap = cyHead / 2) + 1;	
		cxShaft = cx / 2;
		cyFlap = (cp = min(cxHead, cxShaft)) - 1;

		yHead = yCenter;
		xHead = xCenter - ((dxDir > 0) && !(cx&1))  + cp * dxDir;
		yShaft = yCenter - (cyFlap / 2);
		xShaft = xCenter - ((dxDir > 0) ? (cp-(cx&1)) : 0);

		BeforeGraphics();
		/* Draw Shaft */
		DrawRect (xShaft, yShaft, cp, 1 + 2 * (cyFlap/2));

		/* Draw tip of head */
 		DrawPixel (xHead, yHead);
		--yHead; 
		xHead -= dxDir;
		cyHead = 3;
		--cp;
		/* Draw remainder of head */
		while (cp) {
			DrawRect (xHead, yHead, 1, cyHead);			
			--yHead;
			cyHead += 2;
			xHead -= dxDir;
			--cp;
		}
		AfterGraphics();

	} else {
		return FALSE;
	}

	return TRUE;
} /* DrawHorizontalArrow */

short Draw_SCIC_UpArrow (short x, short y, SCIC *pscic) {
	return DrawVerticalArrow (x, y, pscic->cx, pscic->cy, -1);
}

short Draw_SCIC_DnArrow (short x, short y, SCIC *pscic) {
	return DrawVerticalArrow (x, y, pscic->cx, pscic->cy, +1);
}

short Draw_SCIC_LftArrow (short x, short y, SCIC *pscic) {
	return DrawHorizontalArrow (x, y, pscic->cx, pscic->cy, -1);
}

short Draw_SCIC_RgtArrow (short x, short y, SCIC *pscic) {
	return DrawHorizontalArrow (x, y, pscic->cx, pscic->cy, +1);
}

short Draw_SCIC_UpChevron (short x, short y, SCIC *pscic) {
	return DrawVerticalArrow (x, y, pscic->cx, pscic->cy, -1);
}

short Draw_SCIC_DnChevron (short x, short y, SCIC *pscic) {
	return DrawVerticalArrow (x, y, pscic->cx, pscic->cy, +1);
}

short Draw_SCIC_DnImpactArrow (short x, short y, SCIC *pscic) {
	return DrawVerticalArrow (x, y, pscic->cx, pscic->cy, +1);
}

