/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * ADBUF.c
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 01/27/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Amiga specific DBUF routines.
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#if LATTICE
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <stdio.h>
#endif
#include <hardware/blit.h>
#include <hardware/dmabits.h>
#include <hardware/custom.h>
#include <graphics/gfx.h>
#include <echidna/graphics.h>
#include <echidna/dbuf.h>
#if __AMIGAOS__
#if AZTEC_C
#include <functions.h>
#endif
#endif

#include "easydbuf.h"
#include "dbufarea.h"

#include <echidna/emacros.h>

/*************************** EXTERNAL VARAIBLES ***************************/
extern short ClipLeft;
extern short ClipTop;
extern short ClipWidth;
extern short ClipHeight;

/********************************* MACROS *********************************/
#define WAITFORBLITTER	while(hardware->dmaconr & DMAF_BLTDONE);


#undef DEBUG
/* 
#define DEBUG 1
*/

#if DEBUG
typedef struct {
	UWORD bltddat;
	UWORD bltcon0;
	UWORD bltcon1;
	UWORD bltafwm;
	UWORD bltalwm;
	APTR bltcpt;
	APTR bltbpt;
	APTR bltapt;
	APTR bltdpt;
	UWORD bltsize;
	UWORD bltcmod;
	UWORD bltbmod;
	UWORD bltamod;
	UWORD bltdmod;
	UWORD bltcdat;
	UWORD bltbdat;
	UWORD bltadat;
} BLITTER;
#else
typedef struct Custom BLITTER;
#endif

#if DEBUG
void dorealblit (hardware)
	BLITTER *hardware;
{
	OwnBlitter ();

	WAITFORBLITTER

	custom.bltddat = hardware->bltddat;
	custom.bltcon0 = hardware->bltcon0;
	custom.bltcon1 = hardware->bltcon1;
	custom.bltafwm = hardware->bltafwm;
	custom.bltalwm = hardware->bltalwm;
	custom.bltcpt  = hardware->bltcpt;
	custom.bltbpt  = hardware->bltbpt;
	custom.bltapt  = hardware->bltapt;
	custom.bltdpt  = hardware->bltdpt;
	custom.bltcmod = hardware->bltcmod;
	custom.bltbmod = hardware->bltbmod;
	custom.bltamod = hardware->bltamod;
	custom.bltdmod = hardware->bltdmod;
	custom.bltcdat = hardware->bltcdat;
	custom.bltbdat = hardware->bltbdat;
	custom.bltadat = hardware->bltadat;
	custom.bltsize = hardware->bltsize;

	DisownBlitter ();
}
#endif

/**************************************************************************
 *
 * CopyBitMap
 *
 * SYNOPSIS
 *    void CopyBitMap (
 *                struct BitMap *frombm,
 *                int            fromleft,  
 *                int            fromtop,
 *                struct BitMap *tobm,
 *                int            toleft,
 *                int            totop,
 *                int            width,
 *                int            height,
 *                UBYTE		 planemask,
 *                );
 *
 * FUNCTION
 *    Use the blitter to copy from one bitmap to another.  If the blit
 *    is word aligned (ie. First pixel is the first bit of the first
 *    word and last pixel is last bit of the last word) and the same is
 *    true for both the source and the destination then this routine will
 *    use a special blit (D=A) which is twice as fast as the normal
 *    'clipped' blit (D=AB+~AC).
 *
 * INPUTS
 *    frombm    = pointer to bitmap to copy from
 *    fromleft  = x position to start copying from
 *    fromtop   = y position to start copying from
 *    tobm      = pointer to bitmap to copy to
 *    toleft    = x position to start copying to
 *    totop     = y position to start copying to
 *    width     = width in pixels of area to copy
 *    height    = height in pixels of area to copy
 *    planemask = bitmask of planes to effect, bit0 = plane 0, bit1 = plane1...
 *
 * BUGS
 *   NOTE: If you copy from LEFT to RIGHT or TOP to BOTTOM from a bitmap
 *         to the SAME bitmap and the areas overlap the area will be trashed
 *         (or smeared)
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *   ShapeBlit (), CopyMaskBitMap ()
*/
void CopyBitMap (
	struct BitMap	*frombm,
	short			 fromleft,
	short			 fromtop,
	struct BitMap	*tobm,
	short			 toleft,
	short			 totop,
	short			 inwidth,
	short			 inheight
) {
	register ULONG		 fromoffset;
	register ULONG		 tooffset;
#if DEBUG
		 BLITTER	 h;
#endif
	register BLITTER	*hardware = 
#if DEBUG
						&h;
#else
						(struct Custom *)0x0DFF000;
#endif
	register ULONG		newoffset;
	UWORD		fsword;
	UWORD		tsword;
	UWORD		newwidth;
	UWORD		fend;
	UWORD		tend;
	UWORD		fsbit;
	UWORD		tsbit;
	UWORD		febit;
	UWORD		tebit;
	UWORD		frommod;
	UWORD		tomod;
	UWORD		shift;
	UWORD		bltshift;
	UWORD		ashift;
	UWORD		fbwidth;
	UWORD		tbwidth;
	UWORD		bltwidth;
	ULONG		toptr;
	UWORD		size;
	UWORD		firstmask;
	UWORD		lastmask;
	UWORD		ndx;
	UWORD		depth;
	UWORD		control;
	UWORD		temp;

/* --------------------------------------------------------------------- */
	int width  = inwidth; 
	int height = inheight;

	if (toleft < ClipLeft) {
		width	  = inwidth - (ClipLeft - toleft);
		toleft	  = ClipLeft;
		fromleft += inwidth - width;
	} else if (toleft > ClipLeft + ClipWidth - inwidth) {
		width	  = (ClipLeft + ClipWidth) - toleft;
	}

	if (totop < ClipTop) {
		height	 = inheight - (ClipTop - totop);
		totop	 = ClipTop;
		fromtop += inheight - height;
	} else  if (totop > ClipTop + ClipHeight - inheight) {
		height	 = (ClipTop + ClipHeight) - totop;
	}

	if (width <= 0 || height <= 0) {
		return;
	}
/* --------------------------------------------------------------------- */
	depth = min (frombm->Depth, tobm->Depth);

	fsword = fromleft >> 4;
	tsword = toleft   >> 4;

	newwidth = width - 1;
	fend   = fromleft + newwidth;
	tend   = toleft   + newwidth;

	fromoffset = fromtop * frombm->BytesPerRow + (fsword << 1);
	tooffset   = totop   * tobm->BytesPerRow   + (tsword << 1);

	fsbit	= fromleft & 0x0F;
	tsbit	= toleft   & 0x0F;

	febit	= fend & 0x0F;
	tebit	= tend & 0x0F;

	fbwidth = (fend >> 4) - fsword + 1;
	tbwidth = (tend >> 4) - tsword + 1;
	bltwidth  = fbwidth;

	if (!fsbit && !tsbit && febit == 15 && tebit == 15) {
		frommod = frombm->BytesPerRow - (bltwidth << 1);
		tomod   = tobm->BytesPerRow   - (bltwidth << 1);

		size = (height << 6) | bltwidth;

		if (fromoffset < tooffset) {
			control     = 0x0002;
			newoffset   = (bltwidth - 1) << 1;
			fromoffset += frombm->BytesPerRow * (height - 1) + newoffset;
			tooffset   += tobm->BytesPerRow * (height - 1) + newoffset;
		} else {
			control = 0x0000;
		}

#ifndef DEBUG
		OwnBlitter ();
		WAITFORBLITTER
#endif

		hardware->bltcon0 = 0x09F0;
		hardware->bltcon1 = control;
		hardware->bltafwm = 0xFFFF;
		hardware->bltalwm = 0xFFFF;
		hardware->bltamod = frommod;
		hardware->bltdmod = tomod;

		for (ndx = 0; ndx < depth; ndx++) {
			toptr = (ULONG)tobm->Planes[ndx] + (ULONG)tooffset;
#ifndef DEBUG
			WAITFORBLITTER
#endif

			hardware->bltapt  = (APTR)((ULONG)frombm->Planes[ndx] + fromoffset);
			hardware->bltdpt  = (APTR)toptr;
			hardware->bltsize = size;

#if DEBUG
	dorealblit (hardware);
#endif

		}
#ifndef DEBUG
		DisownBlitter ();
#endif

	} else {

#if 0
		BltBitMap (frombm, fromleft, fromtop,
			   tobm,   toleft,   totop,
			   width, height, 0xC0, 0xFF, NULL);
#endif
#if 1
		shift = (tsbit - fsbit);

		if (shift > 15) {
/** Backward blit **/
			control     = 0x0002;
			shift       = fsbit - tsbit;
			newoffset   = (bltwidth - 1) << 1;
			fromoffset += frombm->BytesPerRow * (height - 1) + newoffset;
			tooffset   += tobm->BytesPerRow   * (height - 1) + newoffset;
		} else {
/** Forward blit **/
			control = 0x0000;
		}

		bltshift = shift << 12;

		if (tbwidth > fbwidth) {
			firstmask = 0xFFFF >> tsbit;
			lastmask  = 0xFFFF << (15 - tebit);
			bltwidth  = tbwidth;
			ashift    = 0;
		} else {
			firstmask = 0xFFFF >> fsbit;
			lastmask  = 0xFFFF << (15 - febit);
			ashift    = bltshift;
		}

		if (control) {
			temp      = firstmask;
			firstmask = lastmask;
			lastmask  = temp;
		}

		frommod = frombm->BytesPerRow - (bltwidth << 1);
		tomod   = tobm->BytesPerRow   - (bltwidth << 1);

		size = (height << 6) | bltwidth;

#ifndef DEBUG
		OwnBlitter ();
		WAITFORBLITTER
#endif

		hardware->bltcon0 = ashift   | 0x07CA;
		hardware->bltcon1 = bltshift | control;
		hardware->bltafwm = firstmask;
		hardware->bltalwm = lastmask;
		hardware->bltadat = 0xFFFF;
		hardware->bltbmod = frommod;
		hardware->bltcmod = tomod;
		hardware->bltdmod = tomod;

		for (ndx = 0; ndx < depth; ndx++) {
			toptr = (ULONG)tobm->Planes[ndx] + (ULONG)tooffset;
#ifndef DEBUG
			WAITFORBLITTER
#endif

			hardware->bltbpt  = (APTR)((ULONG)frombm->Planes[ndx] + fromoffset);
			hardware->bltcpt  = (APTR)toptr;
			hardware->bltdpt  = (APTR)toptr;
			hardware->bltsize = size;

#if DEBUG
	dorealblit (hardware);
#endif

		}
#ifndef DEBUG
		DisownBlitter ();
#endif
#endif
	}
}

/**************************************************************************
 *
 * Restore
 *
 * SYNOPSIS
 *		void Restore (short x, short y, short width, short height)
 *
 * PURPOSE
 *		Copy area from BackBuffer to WorkBuffer
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void Restore (short x, short y, short width, short height)
{
	short	ex;

	ex  = (x + width + 15) & 0xFFF0;
	x  &= 0xFFF0;
	/* LATTICE wants the parenthesis & cast in 'ex - x', sheesh. . . */
	CopyBitMap (BackBuffer,	x, y,
				WorkBuffer, x, y, (short) (ex - x), height);

} /* Restore */

void UnRestore (short x, short y, short width, short height)
{
	short	ex;

	ex  = (x + width + 15) & 0xFFF0;
	x  &= 0xFFF0;
	/* LATTICE wants the parenthesis & cast in 'ex - x', sheesh. . . */
	CopyBitMap (WorkBuffer,	x, y,
				BackBuffer, x, y, (short) (ex - x), height);

} /* UnRestore */


/**************************************************************************
 *
 * ShowRect
 *
 * SYNOPSIS
 *		void ShowRect (short x, short y, short width, short height)
 *
 * PURPOSE
 *		Copy area from WorkBuffer to DisplayBuffer
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ShowRect (short x, short y, short width, short height)
{
	short	ex;

	ex  = (x + width + 15) & 0xFFF0;
	x  &= 0xFFF0;
	/* LATTICE wants the parenthesis & cast in 'ex - x', sheesh. . . */
	CopyBitMap (WorkBuffer,	x, y,
				DisplayBuffer, x, y, (short) (ex - x), height);

} /* ShowRect */

/*********************************************************************
 *
 * DBufPeepRect
 *
 * SYNOPSIS
 *		void DBufPeepRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DBufPeepRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= PEEP_FLAG_MASK;	/* Erase next frame */    

	DrawRect (x, y, width, height);

} /* DBufPeepRect */

/*********************************************************************
 *
 * DBufPeepGShape
 *
 * SYNOPSIS
 *		void DBufPeepGShape (
 *			GShape	*shapeptr,
 *			short x,
 *			short y
 *		) 
 *
 * PURPOSE
 *		To record the rectangular area where the shape will be drawn in the
 *		shape area draw list.  
 *		Then to draw the shape via call to 'DrawGShape'. This shape will
 *		stay on the display screen for only one frame.
 *
 * INPUT
 *		shapeptr: 	pointer to shape data.
 *		x, y:		Screen coordinates for drawing shape.	
 *		SAdrawPtr:	(GLOBAL) Pointer to first free record in Shape Area Draw
 *					List.  
 *  
 * EFFECTS
 *		SAdrawPtr:	(GLOBAL) Incremented to point to next free record.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DBufPeepGShape (
	GShape	*shapeptr,
	short x,
	short y
) 
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	WORD	*dataptr	= (WORD *)&shapeptr->Data.Amiga;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + *dataptr++;	/* Area->x = x + shape->xoffset */
	*(saptr++)	= *dataptr++;		/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;	/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;			/* Area->height = shape->height */
  	*saptr		= PEEP_FLAG_MASK;	/* Erase next frame */

	DrawGShape (shapeptr, x, y);

} /* DBufPeepGShape */

/*********************************************************************
 *
 * DBufStampGShape
 *
 * SYNOPSIS
 *		void DBufStampGShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a shape onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DBufStampGShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	WORD	*dataptr	= (WORD *)&shapeptr->Data.Amiga;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);	/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++);		/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;	/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;			/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	
	DrawGShape (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	
	DrawGShape (shapeptr, x, y);	/* in back */

} /* DBufStampGShape */

/*********************************************************************
 *
 * DBufStampGShapeNoTrans
 *
 * SYNOPSIS
 *		void DBufStampGShapeNoTrans (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *                   without transparency.
 *		To draw a shape ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DBufStampGShapeNoTrans (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= shapeptr->Width;	/* Area->width = shape->Width   */
	*(saptr++) 	= y;
	*(saptr++) 	= shapeptr->Height;	/* Area->height = shape->Height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	
	DrawGShapeNoTrans (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	
	DrawGShapeNoTrans (shapeptr, x, y);	/* in back */

} /* DBufStampGShapeNoTrans */

/*********************************************************************
 *
 * DBufStampClearGShape
 *
 * SYNOPSIS
 *		void DBufStampClearGShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DBufStampClearGShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	WORD	width;
	WORD	height;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= (width=shapeptr->Width); /* Area->width = shape->width */
	*(saptr++) 	= y;
	*(saptr++) 	= (height=shapeptr->Height);/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	
	SetPaperColor (shapeptr->TColor);
	DrawRect (x, y, width, height);
	
	SetWorkPage (WorkBuffer);
	
	DrawRect (x, y, width, height);

} /* DBufStampClearGShape */

/*********************************************************************
 *
 * DBufStampRect
 *
 * SYNOPSIS
 *		void DBufStampRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DBufStampRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);

	DrawRect (x, y, width, height);

	SetWorkPage (WorkBuffer);

	DrawRect (x, y, width, height);

} /* DBufStampRect */

/*********************************************************************
 *
 * RestorePeepAreas
 *
 * SYNOPSIS
 *		void RestorePeepAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular peep areas in 'arealist' from the Back screen
 *		to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *		
 * EFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/

static void RestorePeepAreas (
	AreaRecType	*arealist
) 
{
	WORD al_height;

	for ( ; (al_height = arealist->height); arealist++) {
			if (arealist->flags & PEEP_FLAG_MASK) {
				Restore (arealist->x, arealist->y, 
							arealist->width, al_height);		
			}
	}

} /* RestorePeepAreas */
	
/*********************************************************************
 *
 * RestoreStampAreas
 *
 * SYNOPSIS
 *		void RestoreStampAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular stamp areas in 'arealist' from the Background
 *		screen to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *
 * EFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/
static void RestoreStampAreas (
	AreaRecType	*arealist
) {
	WORD al_height;

	for ( ; (al_height = arealist->height); arealist++) {
		if (arealist->flags & STAMP_FLAG_MASK) {
			Restore (arealist->x, arealist->y, 
						arealist->width, al_height);		
		}
	}

} /* RestoreStampAreas */
	
/*********************************************************************
 *
 * DBufFlipScreen
 *
 * SYNOPSIS
 *		void DBufFlipScreen (void)
 *
 * PURPOSE
 *		To ShapeFlip double buffering screens and update the display.
 *
 * INPUT
 *		(GLOBALS)
 *		SAList1, SAList2:	Shape Area lists (arrays).
 *		SAeraseList:		List from which to restore areas of Display  
 *							screen where old shapes are.
 *		SAdrawList:			List from which to update areas of Display
 *							screen where new shapes are. List from which
 *							to restore areas of Work screen where new shapes
 *							were drawn.
 *		SAerasePtr:			Pointer to end of SAerasList.
 *		SAdrawPtr:			Pointer to end of SAdrawList.
 *
 * EFFECTS
 *		(GLOBALS)
 *		'SAeraseList' and 'SAdrawList' are swapped.
 *		'SAerasePtr' is set to final value of 'SAdrawPtr'.
 *		'SAdrawPtr' is set to start of new 'SAdrawList'.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		DisplayWorkAreas (), RestoreBackAreas (),
 *		SWAP_RESET_SALISTS ()
 *
 */
void DBufFlipScreen (void)
{
	/* Mark end of area lists */
		SAerasePtr->height = 0;
		SAdrawPtr->height = 0; 

	/* Flip Amiga Screens */
	FlipDBufScreen ();
	WaitDBufScreen ();

	/* Swap DiplayBuffer & WorkBuffer */
	DisplayBuffer = FrontRp->BitMap;
	WorkBuffer    = BackRp->BitMap;
	SetWorkPage (WorkBuffer);

	/* Update Areas on 'work' that were stamped on 'display' */ 
	RestoreStampAreas (SAdrawList); 

	/* Update Areas on 'work' that were peeped when it was the 'display' */ 
	RestorePeepAreas (SAeraseList); 

	/* Reset shape save areas */
	SWAP_RESET_SALISTS ();

} /* DBufFlipScreen */

