/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * CDBUF.c
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 03/05/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		CGA specific DBUF routines.
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/dbuf.h>
#include <echidna/memory.h>
#include <echidna/eerrors.h>
#include "dbufarea.h"

/**************************** C O N S T A N T S ***************************/
#define GHEADERSIZE	(GSHAPESIZE + CGASDATASIZE)

/*********************************************************************
 *
 * CGA_CloseDBufGraphics
 *
 * SYNOPSIS
 *		void CGA_CloseDBufGraphics (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_CloseDBufGraphics (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CGA_CloseDBufGraphics";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (*BackBuffer)	FreeMemOnPara (MK_FP(*BackBuffer,0));
	if (*WorkBuffer)	FreeMemOnPara (MK_FP(*WorkBuffer,0));

} /* CGA_CloseDBufGraphics */


/*********************************************************************
 *
 * CGA_OpenDBufGraphics
 *
 * SYNOPSIS
 *		CGA_OpenDBufGraphics (
 *			short	 mode, 
 *			size_t	 max_brush_size,
 *		)
 *
 * PURPOSE
 *		Allocate CGA buffers.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
CGA_OpenDBufGraphics (
	short	 mode, 
	size_t	 max_brush_size
)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CGA_OpenDBufGraphics";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	mode = mode;

	/** This really SUCKS! **/
	*WorkBuffer = FP_SEG(AllocMemOnPara (8000U + 192U + 8000U));
	if (!*WorkBuffer) {
		SetGlobalErr (ERR_OUT_OF_MEMORY);
		GEprintf ("OOM: allocating CGA workbuffer");
		return FALSE;
	}
	*BackBuffer = FP_SEG(AllocMemOnPara (max (max_brush_size, 8000U * 2U + 192U)));	/* work space too :-) */
	if (!*BackBuffer) {
		SetGlobalErr (ERR_OUT_OF_MEMORY);
		GEprintf ("OOM: allocating CGA backbuffer");
		return FALSE;
	}
	return TRUE;
} /* CGA_OpenDBufGraphics */


/*********************************************************************
 *
 * CGA_DBufPeepRect
 *
 * SYNOPSIS
 *		void CGA_DBufPeepRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_DBufPeepRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= PEEP_FLAG_MASK;	/* Erase next frame */    

	DrawRect (x, y, width, height);

} /* CGA_DBufPeepRect */

/*********************************************************************
 *
 * CGA_DBufPeepShape
 *
 * SYNOPSIS
 *		void CGA_DBufPeepShape (
 *			GShape	*shapeptr,
 *			short x,
 *			short y
 *		) 
 *
 * PURPOSE
 *		To record the rectangular area where the shape will be drawn in the
 *		shape area draw list.  
 *		Then to draw the shape via call to 'DrawGShape'. This shape will
 *		stay on the display screen for only one frame.
 *
 * INPUT
 *		shapeptr: 	Far pointer to shape data.
 *		x, y:		Screen coordinates for drawing shape.	
 *		SAdrawPtr:	(GLOBAL) Pointer to first free record in Shape Area Draw
 *					List.  
 *  
 * AFFECTS
 *		SAdrawPtr:	(GLOBAL) Incremented to point to next free record.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_DBufPeepShape (
	GShape	*shapeptr,
	short x,
	short y
) 
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/
	UWORD	*saptr		= (UWORD *)SAdrawPtr++;
	UWORD	*dataptr	= (UWORD *)shapeptr->Data.CGA.cs[x & 0x03];

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);		/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++) << 2;	/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;		/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;				/* Area->height = shape->height */
  	*saptr		= PEEP_FLAG_MASK;		/* Erase next frame */

	DrawGShape (shapeptr, x, y);

} /* CGA_DBufPeepShape */

/*********************************************************************
 *
 * CGA_DBufStampShape
 *
 * SYNOPSIS
 *		void CGA_DBufStampShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a shape onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_DBufStampShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	UWORD	*saptr		= (UWORD *)SAdrawPtr++;
	UWORD	*dataptr	= (UWORD *)shapeptr->Data.CGA.cs[x & 0x03];

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);		/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++) << 2;	/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;		/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;				/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;		/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawGShape (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	DrawGShape (shapeptr, x, y);	/* in work */

} /* CGA_DBufStampShape */

/*********************************************************************
 *
 * CGA_DBufStampShapeNoTrans
 *
 * SYNOPSIS
 *		void CGA_DBufStampShapeNoTrans (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *                   without transparency.
 *		To draw a shape ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_DBufStampShapeNoTrans (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	UWORD	 width		= shapeptr->Width;
	UWORD	 height		= shapeptr->Height;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;			/* Area->width = shape->Width   */
	*(saptr++) 	= y;
	*(saptr++) 	= height;			/* Area->height = shape->Height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawGShapeNoTrans (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	DrawGShapeNoTrans (shapeptr, x, y);	/* in work */

} /* CGA_DBufStampShapeNoTrans */

/*********************************************************************
 *
 * CGA_DBufStampClearShape
 *
 * SYNOPSIS
 *		void CGA_DBufStampClearShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_DBufStampClearShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
) {
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	WORD	width		= shapeptr->Width, 
			height		= shapeptr->Height;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;			/* Area->width = shape->width   */
	*(saptr++) 	= y;
	*(saptr++) 	= height;			/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	
	SetPaperColor (255);
#if 0
	EvenPatternTbl[255] = shapeptr->TColor;
	OddPatternTbl[255]  = shapeptr->TColor >> 8;
#else
	EvenPatternTbl[255] = shapeptr->Data.CGA.TPatEven;
	OddPatternTbl[255]  = shapeptr->Data.CGA.TPatOdd;
#endif
	DrawRect (x, y, shapeptr->Width, shapeptr->Height);
	
	SetWorkPage (WorkBuffer);
	DrawRect (x, y, shapeptr->Width, shapeptr->Height);

} /* CGA_DBufStampClearShape */

/*********************************************************************
 *
 * CGA_DBufStampRect
 *
 * SYNOPSIS
 *		void CGA_DBufStampRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CGA_DBufStampRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawRect (x, y, width, height);

	SetWorkPage (WorkBuffer);
	DrawRect (x, y, width, height);

} /* CGA_DBufStampRect */


/*********************************************************************
 *
 * DisplayWorkAreas
 *
 * SYNOPSIS
 *		void DisplayWorkAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular areas in 'arealist' from the Work screen to
 *		the Display screen via call to 'ShowRect'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *		
 * AFFECTS
 *		Copies areas from work buffer to display buffer.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		ShowRect()
 *
*/
static void DisplayWorkAreas (
	AreaRecType	*arealist
)
{
	WORD al_height;

	for ( ; (al_height = arealist->height) != 0; arealist++) {
		ShowRect (arealist->x, arealist->y, 
				arealist->width, al_height);		
	}
} /* DisplayWorkAreas */

/*********************************************************************
 *
 * RestoreBackAreas
 *
 * SYNOPSIS
 *		void RestoreBackAreas (
 *			AreaRecType	*arealist,
 *			AreaRecType	*endarea
 *		)
 *
 * PURPOSE
 *		To copy the rectangular areas in 'arealist' from the Background
 *		screen to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *
 * AFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/
static void RestoreBackAreas (
	AreaRecType	*arealist
)
{
	WORD al_height;

	for ( ; (al_height = arealist->height) != 0; arealist++) {
		if (arealist->flags & PEEP_FLAG_MASK) {
			Restore (arealist->x, arealist->y, 
						arealist->width, al_height);		
		}
	}

} /* RestoreBackAreas */
	
/*********************************************************************
 *
 * CGA_DBufFlipScreen
 *
 * SYNOPSIS
 *		void CGA_DBufFlipScreen (void)
 *
 * PURPOSE
 *		To ShapeFlip double buffering screens and update the display.
 *
 * INPUT
 *		(GLOBALS)
 *		SAList1, SAList2:	Shape Area lists (arrays).
 *		SAeraseList:		List from which to restore areas of Display  
 *							screen where old shapes are.
 *		SAdrawList:			List from which to update areas of Display
 *							screen where new shapes are. List from which
 *							to restore areas of Work screen where new shapes
 *							were drawn.
 *		SAerasePtr:			Pointer to end of SAerasList.
 *		SAdrawPtr:			Pointer to end of SAdrawList.
 *
 * AFFECTS
 *		(GLOBALS)
 *		'SAeraseList' and 'SAdrawList' are swapped.
 *		'SAerasePtr' is set to final value of 'SAdrawPtr'.
 *		'SAdrawPtr' is set to start of new 'SAdrawList'.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		DisplayWorkAreas (), RestoreBackAreas (),
 *		SWAP_RESET_SALISTS ()
 *
*/
void CGA_DBufFlipScreen (void)
{
	/* Mark end of area lists */
		SAerasePtr->height = 0;
		SAdrawPtr->height = 0; 

	/* Copy Old Shape areas from Work to Display */
		DisplayWorkAreas (SAeraseList); 

	/* Copy New Shape areas from Work to Display */
		DisplayWorkAreas (SAdrawList); 

	/* Copy New Shape areas from Background to Work */
		RestoreBackAreas (SAdrawList);

	/* Reset shape save areas */
		SWAP_RESET_SALISTS ();
} /* CGA_DBufFlipScreen */

typedef void (far *RoutinePtr)();

static RoutinePtr CGADBufRoutineTable[] = {
	(RoutinePtr)CGA_OpenDBufGraphics,
	(RoutinePtr)CGA_CloseDBufGraphics,
	(RoutinePtr)CGA_DBufPeepShape,
	(RoutinePtr)CGA_DBufStampShape,
	(RoutinePtr)CGA_DBufStampShapeNoTrans,
	(RoutinePtr)CGA_DBufStampClearShape, 
	(RoutinePtr)CGA_DBufStampRect,
	(RoutinePtr)CGA_DBufPeepRect,
	(RoutinePtr)CGA_DBufFlipScreen,
};


/*********************************************************************
 *
 * GetCGADBufRoutineTable
 *
 * SYNOPSIS
 *		RoutinePtr *GetCGADBufRoutineTable (void)
 *
 * PURPOSE
 *		Returns pointer to CGADBufRoutineTable.  Calling this
 *		routine should force loading if CGA_DBufRoutines as an
 *		overlay.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
RoutinePtr *GetCGADBufRoutineTable (void)
{
	return CGADBufRoutineTable;

} /* GetCGADBufRoutineTable */

