/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * CIRCLE.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 03/06/92
 *   MODIFIED : 07/27/94
 *       TABS : 04 08
 *
 * DESCRIPTION
 *		Circle drawing routines. Line and filled circles.
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/graphics.h>
#include <echidna/circle.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/

static short xbase, ybase;
static unsigned xAspect = 0;
static unsigned yAspect = 0;

/******************************* M A C R O S ******************************/

#define _setpixel(x,y)	DrawPixel ((x),(y))

/***************************** R O U T I N E S ****************************/

/* You pass the aspect ratio times 256 to this routine.
 * Reason: Avoid floating point routines. 
 */
void SetAspectMul256 (unsigned long aspectmul256)
{
	xAspect = 0;
	yAspect = 0;
	if (aspectmul256 != 256) {
		if (aspectmul256 > 256) {
			yAspect = (unsigned) ((65536UL * 256UL) / aspectmul256);
		} else {
			xAspect = (unsigned) ((65536UL * aspectmul256) / 256UL);
		}
	}
}

void plot (int x, int y)
{
   if (xAspect == 0)
       if (yAspect == 0)
           _setpixel (x+xbase, y+ybase);
       else
           _setpixel (x+xbase, (int) (ybase + (((long) y * (long) yAspect) >> 16)));
   else
       _setpixel ((int) (xbase + (((long) x * (long) xAspect) >> 16)), y+ybase);
}

void plot8 (int x, int y)
{
   plot(x,y);
   plot(-x,y);
   plot(x,-y);
   plot(-x,-y);
   plot(y,x);
   plot(-y,x);
   plot(y,-x);
   plot(-y,-x);
}


/* Dr. Dobb's #169 October 1990 pg 8
 * Letter from Michael P. Lindner.
 * Modified by Juan M. Alvarado: (e > 0) ==> (e >= 0) for rounder circle.
 */
void LindnerJMA_Circle (short radius)
{
	short x, y, e;
	y = radius;
	x = 0;
	e = -y;
	while (x <= y) {
		plot8(x,y);
		e += ( x << 1) + 1;
		if (e >= 0) {
			y--;
			e -= (y << 1);
		}
		x++;
	}
}

void fplot4 (short x, short y) 
{
	short len = 1 + (x << 1);
	DrawHLine (xbase - x, ybase - y, len);
	DrawHLine (xbase - x, ybase + y, len);
	DrawVLine (xbase - y, ybase - x, len);
	DrawVLine (xbase + y, ybase - x, len);

}

/* Dr. Dobb's #169 October 1990 pg 8
 * Letter from Michael P. Lindner.
 * Modified by Juan M. Alvarado: (e > 0) ==> (e >= 0) for rounder circle.
 * And made to do filled circle.
 */
void FLindnerJMA_Circle (short radius)
{
	short x, y, e;
	y = radius;
	x = 0;
	e = -y;
	while (x < y) {
		e += ( x << 1) + 1;
		if (e >= 0) {
			fplot4(x,y);
			y--;
			e -= (y << 1);
		}
		x++;
	}
	x = min (x,y);
	{	short len;
		len = 1 + (x<<1);
		DrawRect (xbase - x, ybase - x, len, len);
	}
}

void DrawCircle (short x, short y, short radius) {
	xbase = x;
	ybase = y;
	LindnerJMA_Circle (radius);
}

void DrawFCircle (short x, short y, short radius) {
	xbase = x;
	ybase = y;
	FLindnerJMA_Circle (radius);
}

