/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * CLIPPUSH.C
 *
 * PROGRAMMER : Dan Chang
 *    VERSION : 00.000
 *    CREATED : 02/27/94
 *   MODIFIED : 07/27/94
 *       TABS : 04 07
 *
 * DESCRIPTION
 *		ClipPushClipValues() routine.
 *
 * HISTORY
 *		02/27/94 Sunday (dcc) - created.
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/grafx.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * ClipPushClipValues
 *
 * PURPOSE
 *		Save current clip values and use new ones. This routine
 *		also clips the new clip values to the existing clip values.
 *
 *		This means that the new clip region will not extend the
 *		existing clip region.
 *
 * INPUT
 *		int x			: new x clip value to use
 *		int y			: new y clip value to use
 *		int w			: new clip width to use
 *		int h			: new clip height to use
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		TRUE if successful.
 *
 * HISTORY
 *		02/27/94 Sunday (dcc) - created.
 *		03/02/94 Wednesday (dcc) - fix clipping w & h.
 *
*/
BOOL ClipPushClipValues (int x, int y, int w, int h)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ClipPushClipValues";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (x < GetClipLeft())
	{
		int dx = GetClipLeft() - x;
		x += dx;
		w -= dx;
	}
	if (y < GetClipTop())
	{
		int dy = GetClipTop() - y;
		y += dy;
		h -= dy;
	}
	if (x+w > GetClipLeft()+GetClipWidth())
	{
		w -= (x+w) - (GetClipLeft()+GetClipWidth());
	}
	if (y+h > GetClipTop()+GetClipHeight())
	{
		h -= (y+h) - (GetClipTop()+GetClipHeight());
	}
	if (w < 0)
		w = 0;
	if (h < 0)
		h = 0;
	return PushClipValues(x, y, w, h);
} /* ClipPushClipValues */

