/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * dbuf.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 12/13/89
 *   MODIFIED : 10/08/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		All double buffered graphics related routines.
 *
 * HISTORY
 *		10/08/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/

/***************************** I N C L U D E S ****************************/

#include <echidna/platform.h>
#include "switches.h"
#include "eswitchs.h"

#include <stdlib.h>
#include <echidna/etypes.h>
#include <echidna/alloc.h>
#if __TURBOC__
#include <alloc.h>
#include <mem.h>
#include <dos.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <echidna/eerrors.h>
#include <echidna/memory.h>
#include <echidna/graphics.h>
#include <echidna/dbuf.h>
#include "dbufarea.h"

#ifndef USE_CGA
#define USE_CGA 0
#endif

#ifndef USE_EGA
#define USE_EGA 0
#endif

#ifndef USE_TANDY
#define USE_TANDY 0
#endif

#ifndef USE_MCGA
#define USE_MCGA 0
#endif

/****************************** G L O B A L S *****************************/

AreaRecType	*SAList1 = NULL; /* Shape Area List 1 */
AreaRecType	*SAList2 = NULL; /* Shape Area List 2 */

/* SAdrawList:		Toggles between SAList1 and SAList2.
 * SAeraseList:		Toggles between SAList1 and SAList2.
 * SAdrawPtr:		Steps through list pointed to by SAdrawList.
 * SAerasePtr:		Steps through list pointed to by SAeraseList.
*/
AreaRecType	*SAdrawList, 
			*SAeraseList;
AreaRecType	*SAdrawPtr,
			*SAerasePtr; 

/**************************** ROUTINE POINTERS ****************************/

#if __MSDOS__

short		 GRoutine(C_OpenDBufGraphics) (
					short mode, 
					size_t max_brush_size
				);
void		 GRoutine(C_CloseDBufGraphics) (void);
void		 GRoutine(DBufPeepGShape) (GShape *shapeptr, short x, short y);
void		 GRoutine(DBufStampGShape) (GShape *shapeptr, short x, short y);
void		 GRoutine(DBufStampGShapeNoTrans) (GShape *shapeptr, short x, short y);
void		 GRoutine(DBufStampClearGShape) (GShape *shapeptr, short x, short y);
void		 GRoutine(DBufStampRect) (short x, short y, short w, short h);
void		 GRoutine(DBufPeepRect) (short x, short y, short w, short h);
void		 GRoutine(DBufFlipScreen) (void);

static GfxRoutinePtr *Routines[] = {
	(GfxRoutinePtr *)&C_OpenDBufGraphics,
	(GfxRoutinePtr *)&C_CloseDBufGraphics,
	(GfxRoutinePtr *)&DBufPeepGShape,
	(GfxRoutinePtr *)&DBufStampGShape,
	(GfxRoutinePtr *)&DBufStampGShapeNoTrans,
	(GfxRoutinePtr *)&DBufStampClearGShape, 
	(GfxRoutinePtr *)&DBufStampRect,
	(GfxRoutinePtr *)&DBufPeepRect,
	(GfxRoutinePtr *)&DBufFlipScreen,
	NULL,
};

/**************************** EXTERNAL ROUTINES ***************************/

extern GfxRoutinePtr *GetMCGADBufRoutineTable (void);
extern GfxRoutinePtr *GetCGADBufRoutineTable (void);
extern GfxRoutinePtr *GetEGADBufRoutineTable (void);
extern GfxRoutinePtr *GetTGADBufRoutineTable (void);
#endif

/******************************* M A C R O S ******************************/

/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * ResetSALists
 *
 * SYNOPSIS
 *		void	ResetSALists	(void)
 *
 * PURPOSE
 *		Set up global pointers to area list arrays.
 *
 * INPUT
 *		(GLOBAL)
 *		SAList1, SAList2:	Shape Area Arrays.
 *		
 * EFFECTS
 *		(GLOBALS)
 *		SAdrawList, SAeraseList: Pointers to Shape Area Arrays.
 *		SAdrawPtr, SAerasePtr:	Index pointers for Shape Area Arrays.
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void	ResetSALists	(void)
{
	SAdrawList	= SAList1;
	SAeraseList	= SAList2;
 	SAdrawPtr	= SAdrawList;
	SAerasePtr	= SAeraseList;
} /* ResetSALists */

/*********************************************************************
 *
 * AddDBufStampArea
 *
 * SYNOPSIS
 *		void AddDBufStampArea (
 *			short	x,
 *			short	y,
 *			short	width,
 *			short	height
 *		)
 *
 * PURPOSE
 *		To add an area to the Duouble Buffering Shape Area List.  This
 *		area will not be restored from background screen.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void AddDBufStampArea (
	short	x,
	short	y,
	short	width,
	short	height
)
{
	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	*(saptr++) 	= x;				/* Area->x = x + shape->xoffset */
	*(saptr++)	= width;			/* Area->width = shape->width   */
	*(saptr++) 	= y;				/* Area->y = y + shape->yoffset */
	*(saptr++) 	= height;			/* Area->height = shape->height */
  	*saptr 		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

} /* AddDBufStampArea */

/*********************************************************************
 *
 * AddDBufPeepArea
 *
 * SYNOPSIS
 *		void AddDBufPeepArea (
 *			short	x,
 *			short	y,
 *			short	width,
 *			short	height
 *		)
 *
 * PURPOSE
 *		To add an area to the Duouble Buffering Shape Area List. This area
 *		will be restored from background.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void AddDBufPeepArea (
	short	x,
	short	y,
	short	width,
	short	height
)
{
	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	*(saptr++) 	= x;				/* Area->x = x + shape->xoffset */
	*(saptr++)	= width;			/* Area->width = shape->width   */
	*(saptr++) 	= y;				/* Area->y = y + shape->yoffset */
	*(saptr++) 	= height;			/* Area->height = shape->height */
  	*saptr		= PEEP_FLAG_MASK;	/* Do not Erase next frame */    

} /* AddDBufPeepArea */

/*********************************************************************
 *
 * AddDBufUpdateArea
 *
 * SYNOPSIS
 *		void AddDBufUpdateArea (
 *			short	x,
 *			short	y,
 *			short	width,
 *			short	height
 *		)
 *
 * PURPOSE
 *		To add an area to the Double Buffering Shape Area List.  This
 *		area will not be erased.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void AddDBufUpdateArea (
	short	x,
	short	y,
	short	width,
	short	height
)
{
	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	*(saptr++) 	= x;				/* Area->x = x + shape->xoffset */
	*(saptr++)	= width;			/* Area->width = shape->width   */
	*(saptr++) 	= y;				/* Area->y = y + shape->yoffset */
	*(saptr++) 	= height;			/* Area->height = shape->height */
  	*saptr 		= 0;				/* Do not Erase next frame */    

} /* AddDBufUpdateArea */

/*********************************************************************
 *
 * OpenDBufGraphics
 *
 * SYNOPSIS
 *		short OpenDBufGraphics (
 *				short	mode,
 *				short	max_shapes,
 *				size_t	max_brush_size,
 *				short	max_palette_colors, 
 *				short	max_cycle_colors, 
 *				short	palette_update_hertz
 *		)
 *
 * PURPOSE
 *		Init graphics routines for specific mode.
 *
 * INPUT
 *		mode:	One of MODE_MCGA, MODE_EGA, MODE_CGA, MODE_TANDY.
 *	
 *		max_shapes: Max number of shapes that will be drawn to screen
 *				in one frame. If you draw more than this max in one frame
 *				then you then the drawing routines will trash memory by
 *				overwriting there shape tracking tables.
 *		max_brush_size:	How much space to allocate for brush loading buffer.
 *	
 *		max_colors: Number of colors in largest palette or cycle range used.
 *	
 *		max_cycle_colors: Total number of colors in largest cycle set used.
 *				For example if you use only one cycle set, and it has
 *				a cycle with four colors and another with 3 colors,
 *				then max_cycle_colors == (4 + 3) == 7.
 *	
 * 		palette_update_hertz:	The rate at which you want the palette 
 *				handling interrupt routine to be invoked.  This should equal
 *				the maximum number of times per second that any 
 *				color cycle	will change. Or if no color cycling is used then 
 *				the rate at which you want palette change requests to be
 *				handled.
 *				Must be <= 60.
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		NON-ZERO on success.
 *		0 on Failure.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short OpenDBufGraphics (
	short mode, 
	short max_shapes, 
	size_t max_brush_size,
	short max_palette_colors, 
	short max_cycle_colors, 
	short palette_update_hertz
)
{
#if __MSDOS__
	GfxRoutinePtr	*aptr;
	GfxRoutinePtr	**rptr;
#endif

	if (OpenMemory ())
	{
		if (!OpenGraphics (mode, max_brush_size, max_palette_colors,
			max_cycle_colors, palette_update_hertz)) 
			{
				goto odgcleanup3;
			}

		SAList1 = MEM_calloc ((size_t)max_shapes, sizeof (AreaRecType));
		SAList2 = MEM_calloc ((size_t)max_shapes, sizeof (AreaRecType));
		if (!SAList1 || !SAList2) {
			GlobalErr = ERR_OUT_OF_MEMORY;
			GEprintf (GlobalErrStr[ERR_OUT_OF_MEMORY]);
			goto odgcleanup2;
		}

#if __MSDOS__
			switch (mode) {
			case MODE_MCGA:
#if USE_MCGA
				aptr = GetMCGADBufRoutineTable ();
				break;
#else
				SetGlobalErr (ERR_GENERIC);
				GEprintf ("MCGA DBUF Routines not linked in\n");
				goto odgcleanup2;
#endif
			case MODE_EGA:
#if USE_EGA
				aptr = GetEGADBufRoutineTable ();
				break;
#else
				SetGlobalErr (ERR_GENERIC);
				GEprintf ("MCGA DBUF Routines not linked in\n");
				goto odgcleanup2;
#endif
			case MODE_TANDY:
#if USE_TANDY
				aptr = GetTGADBufRoutineTable ();
				break;
#else
				SetGlobalErr (ERR_GENERIC);
				GEprintf ("MCGA DBUF Routines not linked in\n");
				goto odgcleanup2;
#endif
			case MODE_CGA:
#if USE_CGA
				aptr = GetCGADBufRoutineTable ();
				break;
#else
				SetGlobalErr (ERR_GENERIC);
				GEprintf ("MCGA DBUF Routines not linked in\n");
				goto odgcleanup2;
#endif
			default:
				GlobalErr = ERR_GENERIC;
				GEprintf ("Invalid graphics mode specified");
				goto odgcleanup2;
			}

			rptr = Routines;

			while (*rptr) {
				**rptr = *aptr;
				rptr++;
				aptr++;
			}
#endif

		if (!C_OpenDBufGraphics (mode, max_brush_size)) {
			goto odgcleanup2;
		}

		SetWorkPage (WorkBuffer);
		ResetSALists ();
		return (short)TRUE;

//odgcleanup:
//		CloseDBufGraphics ();
odgcleanup2:
		CloseGraphics ();
odgcleanup3:
		CloseMemory ();
	}
	return FALSE;

} /* OpenDBuffGraphics */

/*********************************************************************
 *
 * CloseDBufGraphics
 *
 * SYNOPSIS
 *		void CloseDBufGraphics (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CloseDBufGraphics (void)
{

	/* Free SALists */
	if (SAList2)	MEM_free (SAList2);
	if (SAList1)	MEM_free (SAList1);
	C_CloseDBufGraphics ();
	CloseGraphics ();
	CloseMemory ();

} /* CloseDBufGraphics */

