/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * GRAPHICS.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 12/18/89
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		09/27/93 Monday (dcc) - updated for MS-DOS Watcom C.
 *		09/29/93 Wednesday (dcc) - enhanced to restore old screen mode better
 *		12/07/93 Tuesday (dcc) - add VESA support for Save/Restore video mode
 *
*/

#include <echidna/platform.h>
#include "switches.h"
#include "eswitchs.h"

#include <stdio.h>
#include <string.h>
#if USE_GRAFX
#include <echidna/grafx.h>
#else
#include <echidna/graphics.h>
#endif
#include <echidna/memory.h>
#include <echidna/eerrors.h>
#include <echidna/timer.h>

#if __MSDOS__
#if __MSDOS32X__
#include <echidna/rmdos.h>
#define DPMI_INT 0x31
#else
#include <dos.h>
#endif
#endif

#ifndef USE_CGA
#define USE_CGA 0
#endif

#ifndef USE_EGA
#define USE_EGA 0
#endif

#ifndef USE_TANDY
#define USE_TANDY 0
#endif

#ifndef USE_MCGA
#define USE_MCGA 0
#endif

/******************************** T Y P E S *******************************/


/**************************** EXTERNAL ROUTINES ***************************/
#if __MSDOS__
extern void GetMCGARoutineTable(GfxRoutinePtr **grptr, GfxDataTbl **gdptr);
extern void GetEGARoutineTable(GfxRoutinePtr **grptr, GfxDataTbl **gdptr);
extern void GetTGARoutineTable(GfxRoutinePtr **grptr, GfxDataTbl **gdptr);
extern void GetCGARoutineTable(GfxRoutinePtr **grptr, GfxDataTbl **gdptr);
#endif

/**************************** GLOBAL VARIABLES ****************************/
/* Not used but left for compatability of graphics.h with GRAFXLIB */
PaletteType		*CurrentPalette;

short			GraphicsMode = 0;
ULONG 			VBTimer;		/* 60Hz count up timer */

#if __MSDOS__
int   far32			*ClipLeft;
int   far32			*ClipTop;
int   far32			*ClipWidth;
int   far32			*ClipHeight;
int   far32			*ClipRight;
int   far32			*ClipBottom;
UBYTE far32			*PenColor;
UBYTE far32			*PaperColor;
#if __MSDOS32X__
GPage far32			*GrDraw;
#else
GPage far32			*GrSeg;
#endif
GPage far32			*WorkBuffer;
GPage far32			*BackBuffer;
GPage far32			*DisplayBuffer;
UBYTE far32			*EvenPatternTbl;
UBYTE far32			*OddPatternTbl;
#endif

short				 GRoutine(C_OpenGraphics)(size_t size);
void				 GRoutine(C_CloseGraphics)(void);

void				 GRoutine(DrawPixel)(int x, int y);
void				 GRoutine(DrawRect)(int x, int y, int width, int height);
void				 GRoutine(ClearWorkPage)(void);
void				 GRoutine(DrawGShape)(GShape far32 *shape, int x, int y);
void				 GRoutine(DrawGShapeNoTrans)(GShape far32 *shape, int x, int y);
void				 GRoutine(Restore)(int x, int y, int width, int height);
void				 GRoutine(UnRestore)(int x, int y, int width, int height);
void				 GRoutine(ShowRect)(int x, int y, int width, int height);
GShape				*GRoutine(ConvertGShape)(
								ConvertSType	*cs
						);
void				 GRoutine(FreeGShape)(GShape *shapeptr);

#if !USE_GRAFX
short			 	 GRoutine(OpenPainter) (
								short max_colors,
								short max_cycle_colors,
								short hertz
							);
void				 GRoutine(ClosePainter) (void);

void				 GRoutine(UsePaletteN)(
								PaletteType	*palette,
								short		 src_offset,
								short		 dst_offset,
								short		 numcolors
							);
void				 GRoutine(FadeToPaletteN) (
								PaletteType	*palette,
								short		 src_offset,
								short		 dst_offset,
								short		 numcolors,
								short		 hz60frames
							);
void				 GRoutine(UseColorCycleSet)(
								ColorCycleSetType	*color_cycle_set,
								short				 activate
							);
void				 GRoutine(FadeToColorCycleSet) (
								ColorCycleSetType	*color_cycle_set,
								short		 		 hz60frames
							);
void				 GRoutine(UsePaintBoxN) (
								PaintBoxType	*paintbox,
								short			 src_offset,
								short			 dst_offset,
								short			 numcolors, 
								short			 activate_color_cycling
							);
void				 GRoutine(FadeToPaintBoxN) (
								PaintBoxType	*paintbox,
								short			 src_offset,
								short			 dst_offset,
								short			 numcolors,
								short			 hz60frames
							);
#endif

#if USE_GRAFX
void				 GRoutine(ClippedCopyRect)(void *src, int sx, int sy,
										void *dest, int dx, int dy,
										int width, int height);
void				 GRoutine(CopyRect)(void *src, int sx, int sy,
										void *dest, int dx, int dy,
										int width, int height);
void				 GRoutine(XORRect)(int x, int y, int w, int h);
SaveAreaType		*GRoutine(SaveArea) (int x, int y, int width, int height);
void				 GRoutine(RestoreArea) (SaveAreaType *sa);

#endif

#if __MSDOS__
void far32	 		*GRoutine(GetGShapeLoadBuf)(void);
#endif

#if __MSDOS__
static GfxRoutinePtr *Routines[] = {
	(GfxRoutinePtr *)&C_OpenGraphics,
	(GfxRoutinePtr *)&C_CloseGraphics,
	(GfxRoutinePtr *)&DrawPixel,
	(GfxRoutinePtr *)&DrawRect,
	(GfxRoutinePtr *)&ClearWorkPage,
	(GfxRoutinePtr *)&DrawGShape,
	(GfxRoutinePtr *)&Restore,
	(GfxRoutinePtr *)&UnRestore,
	(GfxRoutinePtr *)&ShowRect,
	(GfxRoutinePtr *)&ConvertGShape,
	(GfxRoutinePtr *)&FreeGShape,
	(GfxRoutinePtr *)&DrawGShapeNoTrans,
	(GfxRoutinePtr *)&GetGShapeLoadBuf,
	#if !USE_GRAFX
	(GfxRoutinePtr *)&UsePaletteN,
	(GfxRoutinePtr *)&FadeToPaletteN,
	(GfxRoutinePtr *)&UseColorCycleSet,
	(GfxRoutinePtr *)&FadeToColorCycleSet,
	(GfxRoutinePtr *)&UsePaintBoxN,
	(GfxRoutinePtr *)&FadeToPaintBoxN,
	(GfxRoutinePtr *)&OpenPainter,
	(GfxRoutinePtr *)&ClosePainter,
	#endif
	#if USE_GRAFX
	(GfxRoutinePtr *)&XORRect,
	(GfxRoutinePtr *)&CopyRect,
	(GfxRoutinePtr *)&ClippedCopyRect,
	(GfxRoutinePtr *)&SaveArea,
	(GfxRoutinePtr *)&RestoreArea,
	#endif
	NULL,
};
#endif


/***************************** R O U T I N E S ****************************/

#if __MSDOS__

static UBYTE OldMode;
static UBYTE OldHeight;
static UBYTE OldRows;

/*********************************************************************
 *
 * SaveVideoMode
 *
 * PURPOSE
 *		Determine current video mode, save it so we can restore it
 *		later using RestoreVideoMode().
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		12/07/93 Tuesday (dcc) - created.
 *		05/19/94 Thursday (dcc) - removed stuff to save VESA graphics state
 *
*/
static void SaveVideoMode(void)
{
	union REGS		 regs;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SaveVideoMode";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	// Use stinky old video BIOS to save state as well

	regs.h.ah = 0x0F;

#if __MSDOS32X__
	int386 (0x10, &regs, &regs);
#else
	int86 (0x10, &regs, &regs);
#endif

	OldMode = regs.h.al;
	
	regs.h.ah = 0x11;
	regs.h.al = 0x30;
	regs.h.bh = 0x03;
#if __MSDOS32X__
	int386 (0x10, &regs, &regs);
#else
	int86 (0x10, &regs, &regs);
#endif

	OldHeight = regs.h.cl;
	OldRows = regs.h.dl;

} /* SaveVideoMode */


/*********************************************************************
 *
 * RestoreVideoMode
 *
 * PURPOSE
 *		Restore vido mode that was previously saved by SaveVideoMode().
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		12/07/93 Tuesday (dcc) - created.
 *		05/19/94 Thursday (dcc) - removed stuff to restore VESA graphics state
 *
*/
static void RestoreVideoMode(void)
{
	union REGS		 regs;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "RestoreVideoMode";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	// First, use video BIOS to restore video mode

	regs.h.ah = 0x00;
	regs.h.al = OldMode;

#if __MSDOS32X__
	int386 (0x10, &regs, &regs);
#else
	int86 (0x10, &regs, &regs);
#endif

	if (OldRows > 25 && (OldMode <= 0x03 || OldMode == 0x07))
	{
	/* BUGBUG: This code is a TOTAL KLUDGE! It's only purpose in
		life is to try to deal with Norton Control Center setting
		the number of lines to something other than 25. NCC does it
		through some non-standard way which we can't figure out how
		to read, and therefore we can't reproduce. So, if you set the
		video mode to 50 display lines, this code works, but if you
		set it 40 lines, this code does NOT work. */

#if __MSDOS32X__
		UBYTE far32		*ROWS = rmMK_FP32 (0x0040,0x0084);
		UWORD far32		*POINTS=rmMK_FP32 (0x0040,0x0085);
		UBYTE far32		*INFO = rmMK_FP32 (0x0040,0x0087);
#else
		UBYTE far32		*ROWS = MK_FP (0x0040,0x0084);
		UWORD far32		*POINTS=MK_FP (0x0040,0x0085);
		UBYTE far32		*INFO = MK_FP (0x0040,0x0087);
#endif
		UBYTE 			 temp;

		temp        = *INFO;
		*INFO       = temp | 0x01;

		regs.h.ah = 0x11;
		regs.h.bl = 0x00;
		if (OldHeight >= 16)
		{
			regs.h.al = 0x14;
#if __MSDOS32X__
			int386 (0x10, &regs, &regs);
#else
			int86 (0x10, &regs, &regs);
#endif
		}
		else if (OldHeight >= 14)
		{
			regs.h.al = 0x11;
#if __MSDOS32X__
			int386 (0x10, &regs, &regs);
#else
			int86 (0x10, &regs, &regs);
#endif
		}
		else if (OldHeight >= 8)
		{
			regs.h.al = 0x12;
#if __MSDOS32X__
			int386 (0x10, &regs, &regs);
#else
			int86 (0x10, &regs, &regs);
#endif
		}

		*INFO       = temp;
		*POINTS		= OldHeight;
		*ROWS		= OldRows;
	}
} /* RestoreVideoMode */
#endif


/*********************************************************************
 *
 * OpenGraphics
 *
 * PURPOSE
 *		Init graphics routines for specific mode.
 *
 * INPUT
 *		mode:	One of MODE_MCGA, MODE_EGA, MODE_CGA, MODE_TANDY.
 *	
 *		max_brush_size:	How much space to allocate for brush loading buffer.
 *	
 *		max_colors: Number of colors in largest palette or cycle range used.
 *	
 *		max_cycle_colors: Total number of colors in largest cycle set used.
 *				For example if you use only one cycle set, and it has
 *				a cycle with four colors and another with 3 colors,
 *				then max_cycle_colors == (4 + 3) == 7.
 *	
 * 		palette_update_hertz:	The rate at which you want the palette 
 *				handling interrupt routine to be invoked.  This should equal
 *				the maximum number of times per second that any 
 *				color cycle	will change. Or if no color cycling is used then 
 *				the rate at which you want palette change requests to be
 *				handled.
 *				Must be <= 60.
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		NON-ZERO on success.
 *		0 on Failure.
 *
 * HISTORY
 *		09/29/93 Wednesday (dcc) - enhanced to save OldHeight
 *		12/07/93 Tuesday (dcc) - SaveVideoMode() now separate routine
 *
 * SEE ALSO
 *
*/
short OpenGraphics (
	short mode, 
	size_t max_brush_size,
	short max_palette_colors, 
	short max_cycle_colors, 
	short palette_update_hertz
)
{
#if __MSDOS__
	GfxRoutinePtr	*aptr;
	GfxRoutinePtr	**rptr;
	GfxDataTbl		*dptr;

	switch (mode) {
	case MODE_MCGA:
#if USE_MCGA
		GetMCGARoutineTable (&aptr, &dptr);
		break;
#else
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("MCGA Gfx Routines not linked in\n");
		return FALSE;
#endif
	case MODE_EGA:
#if USE_EGA
		GetEGARoutineTable (&aptr, &dptr);
		break;
#else
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("EGA Gfx Routines not linked in\n");
		return FALSE;
#endif
	case MODE_TANDY:
#if USE_TANDY
		GetTGARoutineTable (&aptr, &dptr);
		break;
#else
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("TANDY Gfx Routines not linked in\n");
		return FALSE;
#endif
	case MODE_CGA:
#if USE_CGA
		GetCGARoutineTable (&aptr, &dptr);
		break;
#else
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("CGA Gfx Routines not linked in\n");
		return FALSE;
#endif
	}

	rptr = Routines;

	while (*rptr) {
		**rptr = *aptr;
		rptr++;
		aptr++;
	}

	ClipLeft       = dptr->ClipLeft;
	ClipTop        = dptr->ClipTop;
	ClipWidth      = dptr->ClipWidth;
	ClipHeight     = dptr->ClipHeight;
	ClipRight      = dptr->ClipRight;
	ClipBottom     = dptr->ClipBottom;
	PenColor       = dptr->PenColor;
	PaperColor     = dptr->PaperColor;
#if __MSDOS32X__
	GrDraw         = dptr->GrDraw;
#else
	GrSeg          = dptr->GrSeg;
#endif
	WorkBuffer     = dptr->WorkBuffer;
	BackBuffer     = dptr->BackBuffer;
	DisplayBuffer  = dptr->DisplayBuffer;
	EvenPatternTbl = dptr->EvenPatternTable;
	OddPatternTbl  = dptr->OddPatternTable;

#endif

	if (OpenMemory ())
	{
#if __MSDOS__
		SaveVideoMode();
#endif

		GraphicsMode = mode;

		if (!C_OpenGraphics (max_brush_size)) {
			return FALSE;
		}

		SetWorkPage (DisplayBuffer);

#if !USE_GRAFX
		/** Open Interrupt driven timer and interrupt routine handler */
#define ooga_jma 1
#if ooga_jma
	 		OpenTimers();
#endif

		/******************************************/
		/* for now, we'll let the Amiga vblank in */
		/* EASYDBUF.C take care of VBTimer        */

#if USE_AMIGA_VBLANK
#else
		InstallTimer (&VBTimer, 60, sizeof (VBTimer), 1);
#endif

		return OpenPainter (max_palette_colors, max_cycle_colors, 
			palette_update_hertz);
#else
		max_palette_colors   = max_palette_colors;
		max_cycle_colors     = max_cycle_colors;
		palette_update_hertz = palette_update_hertz;

		return TRUE;
#endif
	}
	return FALSE;
} /* OpenGraphics */


/*********************************************************************
 *
 * CloseGraphics
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		09/29/93 Wednesday (dcc) - enhanced to restore OldRows,
 *						choose between int 10H, AH=11H, AL = 08H, 14H, or 16H.
 *		12/07/93 Tuesday (dcc) - RestoreVideoMode() now separate routine
 *
 * SEE ALSO
 *
*/
void CloseGraphics (void)
{
#if !USE_GRAFX
	ClosePainter ();

#if ooga_jma
	CloseTimers();
#endif
#endif

	C_CloseGraphics ();

#if __MSDOS__
	RestoreVideoMode();
#endif

	CloseMemory ();

} /* CloseGraphics */


/*********************************************************************
 *
 * SetGlobalPattern
 *
 * SYNOPSIS
 *		void	SetGlobalPattern (
 *			PatmapTableType	*pmt
 *		)
 *
 * PURPOSE
 *		To set up the global pattern table used for drawing non-brush
 *		graphics (rectangles, fill screen, ...) with a pattern table of
 *		your choice.
 *
 * INPUT
 *		pmt:	A pattern table structure usually loaded by LoadBPI().
 *
 * AFFECTS
 *		Changes global pattern table.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		LoadBPI()
*/
#if __MSDOS__
void	SetGlobalPattern (
	PatmapTableType	*pmt
)
{
	short	numpats = pmt->NumPatterns;
	
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetGlobalPattern";
#endif
#if REQUIRE
	
#endif
/*-------------------------------------------------------------------------*/
	if (GraphicsMode != MODE_MCGA) {
		memcpy (EvenPatternTbl, pmt->Even, numpats);
		memcpy (OddPatternTbl, pmt->Odd, numpats);
	}

} /* SetGlobalPattern */
#endif

#if USE_GRAFX
/*********************************************************************
 *
 * DrawBox
 *
 * PURPOSE
 *		Draw an unfilled rectangle.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

void DrawBox (int x, int y, int w, int h)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawBox";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	BeforeGraphics ();
	DrawRect (x, y, w, 1);
	DrawRect (x, y + h - 1, w, 1);
	DrawRect (x, y, 1, h);
	DrawRect (x + w - 1, y, 1, h);
	AfterGraphics ();

} /* DrawBox */


/*********************************************************************
 *
 * DrawThickBox
 *
 * PURPOSE
 *		Draw an unfilled rectangle with the given line thickness.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DrawThickBox (int x, int y, int w, int h, int thickness)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawThickBox";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	BeforeGraphics ();
	DrawRect (x, y, w, thickness);								/* top */
	DrawRect (x, y + h - thickness, w, thickness);				/* bottom */
	DrawRect (x, y+thickness, thickness, h - (thickness<<1));	/* left */
	DrawRect (x + w - thickness, y+thickness, thickness, h - (thickness<<1));	/* right */
	AfterGraphics ();

} /* DrawThickBox */

/*********************************************************************
 *
 * DrawXorBox
 *
 * PURPOSE
 *		Draw an unfilled rectangle.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DrawXorBox (int x, int y, int w, int h)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawXorBox";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	BeforeGraphics ();
	DrawXorHLine (x, y, w);			/* top */
	DrawXorHLine (x, y + h - 1, w);	/* bottom */
	DrawXorVLine (x, y+1, h-2);			/* left */
	DrawXorVLine (x + w - 1, y+1, h-2);	/* right */
	AfterGraphics ();

} /* DrawXorBox */
 
/*********************************************************************
 *
 * DrawPatBox
 *
 * PURPOSE
 *		Draw an unfilled rectangle.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DrawPatBox (int x, int y, int w, int h)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawPatBox";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	BeforeGraphics ();
	DrawPatHLine (x, y, w);			/* top */
	DrawPatHLine (x, y + h - 1, w);	/* bottom */
	DrawPatVLine (x, y+1, h-2);			/* left */
	DrawPatVLine (x + w - 1, y+1, h-2);	/* right */
	AfterGraphics ();

} /* DrawPatBox */

/*********************************************************************
 *
 * DrawPatXorBox
 *
 * PURPOSE
 *		Draw an unfilled rectangle.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DrawPatXorBox (int x, int y, int w, int h)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawPatXorBox";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	BeforeGraphics ();
	DrawPatXorHLine (x, y, w);			/* top */
	DrawPatXorHLine (x, y + h - 1, w);	/* bottom */
	DrawPatXorVLine (x, y+1, h-2);			/* left */
	DrawPatXorVLine (x + w - 1, y+1, h-2);	/* right */
	AfterGraphics ();

} /* DrawPatXorBox */
#endif

/**************************************************************************
 *
 * ResetVTimer
 *
 * SYNOPSIS
 *		ResetVTimer (short ticks)
 *
 * PURPOSE
 *		Setup timer? to end in 'ticks' ticks.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static ULONG DelayTicks;
static ULONG StartTicks;
void ResetVTimer (short ticks)
{
	StartTicks = VBTimer;
	DelayTicks = ticks;

} /* ResetVTimer */

#if __AMIGAOS__
#if MULTITASK
void WaitToBeActive (void)
{
	struct Message	*msg;

	StopSound (0);
	AClearPointer ();
	if (BeforePause) {
		BeforePause ();
	}
	if (!(msg = GetMsg (MainWindow->UserPort))) {
		WaitPort (MainWindow->UserPort);
	}
	while (msg = GetMsg (MainWindow->UserPort)) {
		ReplyMsg (msg);
	}
	ScreenToFront (AmigaScreen);
	if (AfterPause) {
		AfterPause ();
	}
	ARestorePointer ();
}
#endif
#endif


short CheckVTimer (void)
{
#if __AMIGAOS__
#if MULTITASK
	/* LATTICE wants this here 'cause it's only used here */
	static struct Screen	*oldfrontscreen;

	if (oldfrontscreen != IntuitionBase->FirstScreen) {
		oldfrontscreen = IntuitionBase->FirstScreen;
		if (IntuitionBase->FirstScreen == AmigaScreen) {
			RestoreView ();
		} else {
			WaitToBeActive ();
			RestoreView ();
		}
		ResetVTimer (DelayTicks);
	}
#endif
#endif
	if (DelayTicks) {
		return (short)(VBTimer - StartTicks < DelayTicks);
	}
	return FALSE;
}

void WaitVTimer (void)
{
#if 0
	return;
#else
	while (CheckVTimer());
	ResetVTimer ((short)DelayTicks);
#endif
}

