;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** MRECTP.ASM
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 12/13/89
;**   MODIFIED : 10/10/93
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	Protected-mode routine to draw a rectangle in current color on MCGA screen.
;**
;** HISTORY
;**
;**

	IDEAL

;=====================================================================
;**
;** MCGA_DrawRect
;**
;** PURPOSE
;**	Draws a rectangle on the current <MGrDraw> rastport in the current color
;**
;** INPUT
;**	EAX	: WORD x
;**	EDX	: WORD y
;**	EBX	: WORD width
;**	ECX	: WORD height
;**
;** REGISTER USED
;**	ESI, EDI
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC MCGA_DrawRect

	BeforeGraphics

	cld

	push	es

	push	ds
	pop	es	;ES = DS (BUGBUG: is this valid?)

	shl	edx, 1
	sub	edi, edi
	mov	di, [LineSt+edx]	; Get Starting address of y row
	add	edi, eax	; add x column.

	add	edi, [MGrDraw]	; Add base address of current page.

	if USE_GRAFX
	sub	edx, edx
	mov	dl, [MPenColor]	; Get Pen Color.
	mov	al, [_RemapTable+edx]	; Look up thru RemapTable
	else
	mov	al, [MPenColor]	; Get Pen Color.
	inc	al	; Inc color up one.
	endif
	mov	ah, al	; put it in both halfs of AX
	mov	edx, eax
	shl	eax, 16
	mov	ax, dx	; Pen color in all four bytes of EAX

	mov	edx, ecx	; EDX = height

	mov	ecx, ebx	; ECX = width
	mov	esi, ebx
	shr	esi, 2	; ESI = width / 4

	neg	ebx	; compute modulo of rect.
	add	ebx, SCREEN_WIDTH	; SCREEN_WIDTH - width.

	and	ecx, 3	; ECX = width mod 4
	je	@@mod4is0
	dec	ecx
	je	@@mod4is1
	dec	ecx
	je	@@mod4is2

	ALIGN	4
@@mod4is3:
	mov	ecx, esi	; ECX = width / 4

	rep	stosd	; Draw a row of pixels
	stosw
	stosb

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is3
	jmp	SHORT @@done

	ALIGN	4
@@mod4is0:
	mov	ecx, esi	; ECX = width / 4

	rep	stosd	; Draw a row of pixels

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is0
	jmp	SHORT @@done

	ALIGN	4
@@mod4is1:
	mov	ecx, esi	; ECX = width / 4

	rep	stosd	; Draw a row of pixels
	stosb

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is1
	jmp	SHORT @@done

	ALIGN	4
@@mod4is2:
	mov	ecx, esi	; ECX = width / 4

	rep	stosd	; Draw a row of pixels
	stosw

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is2

@@done:
	pop	es

	AfterGraphics
	ret

	ENDP MCGA_DrawRect


;=====================================================================
;**
;** MCGA_RectClip
;**
;** PURPOSE
;**	Clipping for rect routines.
;**
;** INPUT
;**	EAX	: DWORD x
;**	EDX	: DWORD y
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**
;** REGISTER USED
;**	ESI, EDI
;**
;** RETURN VALUE
;**	Carry clear if no part of rect to draw/xor/whatever.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC	MCGA_RectClip

	;** Clip the top left width and height.
	cmp	eax, [MClipRight]
	jg	@@no_rect	; starts past right edge.
	cmp	edx, [MClipBottom]
	jg	@@no_rect	; starts past bottom edge
	mov	esi, eax	; get 1 past right side
	add	esi, ebx
	cmp	esi, [MClipLeft]
	jle	@@no_rect	; ends before left edge.
	mov	edi, edx	; get 1 below bottom side 
	add	edi, ecx
	cmp	edi, [MClipTop]
	jle	@@no_rect	; ends before top edge.

	push	esi

	;* Clip sides that overlap clip window 
	mov	esi, eax
	sub	esi, [MClipLeft]	; check left edge.
	jge	@@left_ok
	add	ebx, esi	; clip width.
	sub	eax, esi	; clip left.

@@left_ok:
	mov	esi, edx
	sub	esi, [MClipTop]	; check top edge.
	jge	@@top_ok
	add	ecx, esi	; clip height.
	sub	edx, esi	; clip top.

@@top_ok:
	pop	esi
	dec	esi
	sub	esi, [MClipRight]	; check right edge.
	jle	@@right_ok
	sub	ebx, esi	; clip width.	

@@right_ok:
	dec	edi
	sub	edi, [MClipBottom]	; check bottom edge.
	jle	@@bottom_ok
	sub	ecx, edi	; clip height.	
@@bottom_ok:
	stc
	ret

@@no_rect:
	clc
	ret

	ENDP


;=====================================================================
;**
;** void MCGA_DrawRect(int x, int y, int width, int height);
;**
;** PURPOSE
;**	Draws a rectangle on the current workpage in the current color.
;**
;** INPUT
;**	EAX	: DWORD x
;**	EDX	: DWORD y
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	PUBLIC	MCGA_DrawRectAsm_
	MPROC	MCGA_DrawRectAsm_

	push	ebp
	mov	ebp, esp

	push	edi	; Save ESI and EDI
	push	esi

	call	MCGA_RectClip
	jnc	@@no_rect

	call	MCGA_DrawRect	; Draw the Rect
@@no_rect:
	pop	esi	; Restore ESI and EDI
	pop	edi

	pop	ebp
	ret

	ENDP	MCGA_DrawRectAsm_


	if	USE_GRAFX
;=====================================================================
;**
;** MCGA_XORRect
;**
;** PURPOSE
;**	Draws a rectangle on the current <MGRDraw> rastport XORed in the
;**	current color
;**
;** INPUT
;**	EAX	: DWORD x
;**	EDX	: DWORD y
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**
;** REGISTER USED
;**	ESI, EDI
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC MCGA_XORRect

	BeforeGraphics

	if 0
	cmp	ecx,0	; Skip if height = 0
	je	@@no_rect
	cmp	ebx,0	; Skip if width = 0
	je	@@no_rect
	endif

	shl	edx, 1
	sub	edi, edi
	mov	di, [LineSt+edx]	; Get Starting address of y row
	add	edi, eax	; add x column.

	add	edi, [MGrDraw]	; Add base address of current page.

	mov	al, [MPenColor]	; Get Pen Color.
	mov	ah, al	; put it in both halfs of AX
	mov	edx, eax
	shl	eax, 16
	mov	ax, dx	; Pen color in all four bytes of EAX

	mov	edx, ecx	; EDX = height

	mov	ecx, ebx	; ECX = width
	mov	esi, ebx
	shr	esi, 2	; ESI = width / 4

	neg	ebx	; compute modulo of rect.
	add	ebx, SCREEN_WIDTH	; SCREEN_WIDTH - width.

	and	ecx, 3	; ECX = width mod 4
	je	@@mod4is0
	dec	ecx
	je	@@mod4is1
	dec	ecx
	je	@@mod4is2

	ALIGN	4
@@mod4is3:
	mov	ecx, esi	; ECX = width / 4
	or	ecx, ecx
	je	@@skip3
@@xor3:
	xor	[edi], eax	; XOR row of pixels
	add	edi, 4
	loop	@@xor3
@@skip3:
	xor	[edi], ax	; XOR extra word
	add	edi, 2
	xor	[edi], al	; XOR extra byte
	inc	edi

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is3
	jmp	SHORT @@done

	ALIGN	4
@@mod4is0:
	mov	ecx, esi	; ECX = width / 4
	or	ecx, ecx
	je	@@skip0
@@xor0:
	xor	[edi], eax	; XOR row of pixels
	add	edi, 4
	loop	@@xor0
@@skip0:
	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is0
	jmp	SHORT @@done

	ALIGN	4
@@mod4is1:
	mov	ecx, esi	; ECX = width / 4
	or	ecx, ecx
	je	@@skip1
@@xor1:
	xor	[edi], eax	; XOR row of pixels
	add	edi, 4
	loop	@@xor1
@@skip1:
	xor	[edi], al	; XOR extra byte
	inc	edi

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is1
	jmp	SHORT @@done

	ALIGN	4
@@mod4is2:
	mov	ecx, esi	; ECX = width / 4
	or	ecx, ecx
	je	@@skip2
@@xor2:
	xor	[edi], eax	; XOR row of pixels
	add	edi, 4
	loop	@@xor2
@@skip2:
	xor	[edi], ax	; XOR extra word
	add	edi, 2

	add	edi, ebx	; move to next row
	dec	edx
	jg	@@mod4is2
@@no_rect:
@@done:
	AfterGraphics
	ret

	ENDP MCGA_XORRect


;=====================================================================
;**
;** void MCGA_XORRect(int x, int y, int width, int height);
;**
;** PURPOSE
;**	Draws a rectangle on the current workpage in the current color.
;**
;** INPUT
;**	EAX	: DWORD x
;**	EDX	: DWORD y
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	PUBLIC  MCGA_XORRect_
	MPROC	MCGA_XORRect_

	push	ebp
	mov	ebp, esp

	push	edi	; Save ESI and EDI
	push	esi

	call	MCGA_RectClip
	jnc	@@no_rect

	call	MCGA_XORRect	; Draw the Rect
@@no_rect:
	pop	esi	; Restore ESI and EDI
	pop	edi

	pop	ebp
	ret

	ENDP	MCGA_XORRect_
 	endif

