;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** MRESTORP.ASM
;**
;** PROGRAMMER : Daniel C. Chang
;**    VERSION : 00.000
;**    CREATED : 07/23/89
;**   MODIFIED : 10/25/93
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	Protected-mode ShowRect/Restore for MCGA.
;**
;** HISTORY
;**	01/16/92 Thursday (DCC) - add support for very large shapes
;**	05/14/91 Tuesday (DCC) - fixed so source & dest offset may be != 0
;**	09/27/93 Monday (dcc) - created based on real mode MRESTORE.ASM.
;**
	IDEAL

; MCGA_CopyRect - copy rectangle from source x, y of byte area pointed
; to by ESI to destination byte area pointed to by EDI to destination
; x, y of xsize, ysize.
; Known bug: doesn't clip to source rectangle.
;	if USE_GRAFX
; Known bug: doesn't clip to destination rectangle.
;	endif
; by Daniel Chang, 07/23/89.
;
; INPUT:
;
; ESI - points to source rectangular byte area
; EDI - points to destination rectangular byte area
; EAX - source x position of left edge of rectangle to copy from
; EBX - source y position of top edge of rectangle to copy from
; ECX - destination x position of left edge of rectangle to copy to
; EDX - destination y position of top edge of rectangle to copy to
; [sw] - width of source rectangle to copy
; [sh] - height of source rectangle to copy
; [sRowByte] - width of source screen in bytes
; [dRowByte] - width of destination screen in bytes
;
;	if !USE_GRAFX
; [cs:MClipLeft] - left edge of destination clip window
; [cs:MClipRight] - right edge of destination clip window
; [cs:MClipTop] - top edge of destination clip window
; [cs:MClipBottom] - bottom edge of destination clip window
;	endif
;
; REGISTERS CHANGED: EAX, EBX, ECX, EDX, ESI, EDI, EBP

	STRUC	ByteMap
Width	DW	?
Height	DW	?
gfxData	DD	?
TColor	DW	?
	ENDS

	ENDS	_TEXT

	SEGMENT	_DATA

	ALIGN	4

sRowByte dd	?	;source bytemap bytes per row
;sHeight	dd	?	;source bytemap number of rows
dRowByte dd	?	;destination bytemap bytes per row
sw	dd	?
sh	dd	?

	; local variable

dskip	dd	?	;bytes to skip to start of next row

	ENDS	_DATA

	SEGMENT	_TEXT

	MPROC	MCGA_CopyRect

	ife	USE_GRAFX

	; Check for clipping on destination left edge.
	; If we clip, adjust width, srcX, and dstY.

	mov	ebp, ecx
	sub	ebp, [MClipLeft]	;EBP = dstX-MClipLeft
	jge	@@dxOK	;=> no clipping

	add	[sw], ebp	;width = width + (dstX-MClipLeft)
	sub	eax, ebp	;srcX = srcX - (dstX-MClipLeft)
	mov	ecx, [MClipLeft]	;dstX = MClipLeft
@@dxOK:
	; Check for clipping on destination top edge.
	; If we clip, adjust height, srcY, and dstY.

	mov	ebp, edx
	sub	ebp, [MClipTop]	;EBP = dstY-MClipTop
	jge	@@dyOK	;=> no clipping

	add	[sh], ebp	;height = height + (dstY-MClipTop)
	sub	ebx, ebp	;srcY = srcY - (dstY-MClipTop)
	mov	edx, [MClipTop]	;dstY = MClipTop
@@dyOK:
	; Check for clipping on destination right edge.
	; If we clip, adjust width.

	mov	ebp, [MClipRight]
	inc	ebp
	sub	ebp, ecx
	cmp	[sw], ebp	;width <= MClipRight+1 - dstX
	jle	@@wOK	;=> yes, no clip
	mov	[sw], ebp	;width = MClipRight+1 - dstX
@@wOK:
	; Check for clipping on destination bottom edge.
	; If we clip, adjust height.

	mov	ebp, [MClipBottom]
	inc	ebp
	sub	ebp, edx
	cmp	[sh], ebp	;height <= MClipBottom+1 - dstY
	jle	@@hOK	;=> yes, no clip
	mov	[sh], ebp	;height = MClipBottom+1 - dstY
@@hOK:
	endif

	cld

	push	es

	push	ds
	pop	es	;ES = DS (BUGBUG: is this valid?)

	mov	ebp, [sw]	;save # of bytes to copy per row
	or	ebp, ebp
	jle	@@done	;=> width <= 0, no work
	cmp	[sh], 0
	jle	@@done	;=> height <= 0, no work

	push	edx	;save dstY
	push	eax	;save srcX

	mov	eax, ebx
	mul	[sRowByte]	;EAX = srcY * [sRowByte]
	pop	ebx
	add	eax, ebx	;EAX = srcY * [sRowByte] + srcX
	add	esi, eax	;ESI = offset within source bytemap

	pop	eax	;restore dstY
	mul	[dRowByte]	;EAX = dstY * [dRowByte]
	add	eax, ecx	;EAX = dstY * [dRowByte] + dstX
	add	edi, eax	;EDI = offset within dest bytemap

	mov	edx, [sRowByte]
	sub	edx, ebp	;EDX = bytes to skip to next copy area

	mov	eax, [dRowByte]
	sub	eax, ebp
	mov	[dskip], eax	;dskip = bytes to skip to next dest row

	mov	ebx, [sh]	;# of rows to copy
	mov	ecx, ebp
	shr	ebp, 2	;# bytes per row / 4

	and	ecx, 3	; ECX = # bytes per row mod 4
	je	@@mod4is0
	dec	ecx
	je	@@mod4is1
	dec	ecx
	je	@@mod4is2

	ALIGN	4
@@mod4is3:
	mov	ecx, ebp	;get number of bytes to copy
	rep	movsd	;copy row
	movsw
	movsb

	add	esi, edx	;move to next row in source

	add	edi, [dskip]	;move to next row in destination

	dec	ebx
	jg	@@mod4is3	;=> more rows to copy
@@done:
	pop	es
	ret

	ALIGN	4
@@mod4is0:
	mov	ecx, ebp	;get number of bytes to copy
	rep	movsd	;copy row

	add	esi, edx	;move to next row in source

	add	edi, [dskip]	;move to next row in destination

	dec	ebx
	jg	@@mod4is0	;=> more rows to copy
	pop	es
	ret

	ALIGN	4
@@mod4is1:
	mov	ecx, ebp	;get number of bytes to copy
	rep	movsd	;copy row
	movsb

	add	esi, edx	;move to next row in source

	add	edi, [dskip]	;move to next row in destination

	dec	ebx
	jg	@@mod4is1	;=> more rows to copy
	pop	es
	ret

	ALIGN	4
@@mod4is2:
	mov	ecx, ebp	;get number of bytes to copy
	rep	movsd	;copy row
	movsw

	add	esi, edx	;move to next row in source

	add	edi, [dskip]	;move to next row in destination

	dec	ebx
	jg	@@mod4is2	;=> more rows to copy
	pop	es
	ret
	ENDP	MCGA_CopyRect


	if USE_GRAFX

;=====================================================================
;**
;** MCGA_CopyRect_
;**
;** PURPOSE
;**	C glue for copy rect as above.  C Prototype is
;**
;**	MCGA_CopyRect (
;**		ByteMap far32	*sBM,
;**		int	 FromX,
;**		int	 FromY,
;**		ByteMap far32	*dBM,
;**		int	 ToX,
;**		int	 ToY,
;**		int	 GWidth,
;**		int	 Height
;**		)
;**
;** INPUT
;**	EAX	: DWORD sBM
;**	EDX	: DWORD  FromX
;**	EBX	: DWORD  FromY
;**	ECX	: DWORD dBM
;**	[ebp+8]	: DWORD  ToX
;**	[ebp+12]: DWORD  ToY
;**	[ebp+16]: DWORD  GWidth
;**	[ebp+20]: DWORD  Height
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	ALIGN	4

	PUBLIC	MCGA_CopyRect_
	MPROC	MCGA_CopyRect_

	push	ebp
	mov	ebp, esp

	push	edi
	push	esi

	BeforeGraphics

	mov	esi, eax	; Get Pointer to Source ByteMap struct
	mov	ax, [(ByteMap ptr esi).Width]
	cwde
	mov	[sRowByte], eax	; Set width of Source ByteMap
	mov	esi, [(ByteMap ptr esi).gfxData]	; Get Pointer to ByteMap image

	mov	edi, ecx	; Get Pointer to Dest ByteMap
	mov	ax, [(ByteMap ptr edi).Width]
	cwde
	mov	[dRowByte], eax	; Set width of Dest ByteMap
	mov	edi, [(ByteMap ptr edi).gfxData]	; Get Pointer to ByteMap image

	mov	eax, [ebp+16]	;GWidth
	mov	[sw], eax
	mov	eax, [ebp+20]	;Height
	mov	[sh], eax

;;;	mov	ebx, [FromY]
	mov	ecx, [ebp+8]	;ToX
	mov	eax, edx
	mov	edx, [ebp+12]	;ToY

	call	MCGA_CopyRect

	AfterGraphics

	pop	esi
	pop	edi

	pop	ebp
	ret	16
	ENDP	MCGA_CopyRect_

	endif

;
; C PROTOTYPE:
;
; void far MCGA_Restore(int xmin, int ymin, int width, int height);
;
; MCGA_Restore_ - restores from background buffer to the current work page.
;
;** INPUT
;**	EAX	: DWORD xmin
;**	EDX	: DWORD ymin
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**	MBackBuffer : -> to source buffer
;**	MWorkBuffer : -> to dest buffer
; by Daniel Chang, 07/23/89.
;**	09/30/93 Thursday (dcc) - protected mode version created.

	ALIGN	4

	MPROC	MCGA_Restore_

	push	ebp
	mov	ebp, esp

	push	edi
	push	esi

	BeforeGraphics

;;;	mov	edx, edx	;EDX = ymin
	mov	ebx, edx	;EBX = ymin
	mov	[sh], ecx	;[sh] = height
;;;	mov	eax, eax	;EAX = xmin
	mov	ecx, eax	;ECX = xmin
	mov	[sw], ebx	;[sw] = width

	mov	esi, SCREEN_WIDTH
	mov	[sRowByte],esi
	mov	[dRowByte],esi

	mov	esi, [MBackBuffer]	;source buffer
	mov	edi, [MWorkBuffer]	;destination buffer

	call	MCGA_CopyRect

	AfterGraphics

	pop	esi
	pop	edi

	pop	ebp
	ret
	ENDP	MCGA_Restore_

;
; C PROTOTYPE:
;
; void far MCGA_UnRestore(int xmin, int ymin, int width, int height);
;
; MCGA_UnRestore_ - 'unrestores' from the current work page to background buffer.
;
;** INPUT
;**	EAX	: DWORD xmin
;**	EDX	: DWORD ymin
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**	MWorkBuffer : -> to source buffer
;**	MBackBuffer : -> to dest buffer
; by Daniel Chang, 07/23/89.
;**	09/30/93 Thursday (dcc) - protected mode version created.

	ALIGN	4

	MPROC	MCGA_UnRestore_

	push	ebp
	mov	ebp, esp

	push	edi
	push	esi

	BeforeGraphics

;;;	mov	edx, edx	;EDX = ymin
	mov	ebx, edx	;EBX = ymin
	mov	[sh], ecx	;[sh] = height
;;;	mov	eax, eax	;EAX = xmin
	mov	ecx, eax	;ECX = xmin
	mov	[sw], ebx	;[sw] = width

	mov	esi, SCREEN_WIDTH
	mov	[sRowByte],esi
	mov	[dRowByte],esi

	mov	esi, [MWorkBuffer]	;source buffer
	mov	edi, [MBackBuffer]	;destination buffer

	call	MCGA_CopyRect

	AfterGraphics

	pop	esi
	pop	edi

	pop	ebp
	ret
	ENDP	MCGA_UnRestore_

;
; C PROTOTYPE:
;
; void far MCGA_ShowRect(int xmin, int ymin, int width, int height);
;
; MCGA_ShowRect_ - copies from work page to the display page.
;
;** INPUT
;**	EAX	: DWORD xmin
;**	EDX	: DWORD ymin
;**	EBX	: DWORD width
;**	ECX	: DWORD height
;**	MWorkBuffer : -> to source buffer
;**	MDisplayBuffer : -> to dest buffer
; by Daniel Chang, 07/23/89.
;**	09/30/93 Thursday (dcc) - protected mode version created.

	ALIGN	4

	MPROC	MCGA_ShowRect_

	push	ebp
	mov	ebp, esp

	push	edi
	push	esi

	BeforeGraphics

;;;	mov	edx, edx	;EDX = ymin
	mov	ebx, edx	;EBX = ymin
	mov	[sh], ecx	;[sh] = height
;;;	mov	eax, eax	;EAX = xmin
	mov	ecx, eax	;ECX = xmin
	mov	[sw], ebx	;[sw] = width

	mov	esi, SCREEN_WIDTH
	mov	[sRowByte],esi
	mov	[dRowByte],esi

	mov	esi, [MWorkBuffer]	;source buffer
	mov	edi, [MDisplayBuffer]	;destination buffer

	call	MCGA_CopyRect

	AfterGraphics

	pop	esi
	pop	edi

	pop	ebp
	ret
	ENDP	MCGA_ShowRect_

