/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * SHELLSIM.c
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 12/22/89
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		main() routine for simulating gaming shell.
 *
 * HISTORY
 *
*/

#include <echidna\platform.h>
#include "switches.h"

#define	TEST_PIXELS		1
#define TEST_RESTORE	1
#define TEST_SHAPES		1
#define TEST_ISHAPES	1
#define TEST_RECT		1
#define TEST_CLEAR		1
#define TEST_DBUF		1

#include <conio.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <echidna/etypes.h>
#include <echidna/eerrors.h>
#include <echidna/argparse.h>
#include <echidna/brushsup.h>
#include <echidna/dbuf.h>
#include "test.h"
#include "testdata.h"

/**************************** C O N S T A N T S ***************************/
#define MAX_SHAPES		200
#define TEST_FPF		"testdata.fpf"

/********************* G L O B A L   V A R I A B L E S ********************/

/********************************* EXTERNS ********************************/
#define B8(a,b,c,d,e,f,g,h)	((a << 7) + (b << 6) + (c << 5) + (d << 4) + (e << 3) + (f << 2) + (g << 1) + h)
#define WD(v)	((v) & 0xFF),((v) >> 8)
UBYTE test_sprite1[] = {
	WD(0), WD(2), WD(0), WD(3),     /* ; x_adj, x_len, y_adj, y_len */
	2,0x0ff,0,B8(0,0,1,1,1,1,1,1),B8(0,1,0,0,0,0,0,0), 0,
	2,B8(1,1,1,1,1,1,0,0),B8(0,0,0,0,0,0,1,1),B8(0,0,0,0,1,1,1,1),B8(0,1,0,1,0,0,0,0), 0,
	2,0x0ff,0,B8(0,0,1,1,1,1,1,1),B8(0,1,0,0,0,0,0,0),
};

UBYTE test_sprite2[] = {
	WD(0), WD(6), WD(0), WD(15),       /*      ; x_adj, x_len, y_adj, y_len */
	2,0x0ff,0,B8(1,1,0,0,0,0,0,0),B8(0,0,1,0,1,0,1,0), 0, 6,B8(0,0,0,0,1,1,1,1),B8(1,0,1,0,0,0,0,0),0,B8(1,0,1,0,1,0,1,0),
	1,B8(1,0,1,0,1,0,1,0), 4,0x0ff,0,B8(1,1,1,1,1,1,0,0),B8(0,0,0,0,0,0,1,0),
	2,B8(1,0,1,0,1,0,1,0),B8(1,0,1,0,1,0,1,0), 4,B8(0,0,1,1,1,1,1,1),B8(1,0,0,0,0,0,0,0),B8(1,1,0,0,0,0,0,0),B8(0,0,0,1,0,1,1,0),
	2,B8(1,0,1,0,1,0,1,0),B8(1,0,1,0,1,0,1,0), 2,B8(0,0,0,0,0,0,1,1),B8(1,0,0,1,0,1,0,0),0x0f0,B8(0,0,0,0,1,0,1,0), 0,
	2,B8(0,0,1,1,1,1,1,1),B8(1,0,0,0,0,0,0,0),0,B8(0,1,0,1,0,1,1,0),
	3,B8(1,0,1,0,1,0,1,0),B8(1,0,1,0,1,0,1,0),B8(1,0,0,1,0,1,0,1), 2,0x0f0,B8(0,0,0,0,1,0,1,0),0x0f,B8(1,0,1,0,0,0,0,0),
	4,B8(0,1,0,1,0,1,0,1),B8(1,0,1,0,1,0,1,0),B8(1,0,1,0,1,0,1,0),B8(0,1,0,1,0,1,0,1),2,B8(1,1,0,0,0,0,0,0),
	B8(0,0,1,0,1,0,1,0),B8(0,0,0,0,1,1,1,1),B8(1,0,1,0,0,0,0,0),
	0, 2,B8(1,1,0,0,0,0,0,0),B8(0,0,0,1,0,1,0,1),0,B8(0,1,0,1,0,1,0,1), 1,B8(0,1,0,1,0,1,0,1), 2,
	B8(0,0,0,0,0,0,1,1),B8(0,1,0,1,0,1,0,0),0,B8(1,0,1,0,1,0,1,0),
	0, 2,B8(0,0,1,1,1,1,1,1),B8(1,0,0,0,0,0,0,0),0x0f0,B8(0,0,0,0,0,1,0,1),
	2,B8(0,1,0,1,0,1,0,1),B8(0,1,0,1,0,1,0,1), 2,B8(0,0,0,0,0,0,1,1),B8(0,1,0,1,0,1,0,0),0,
	B8(1,0,1,0,1,0,1,0), 0, 2,0x0ff,0,0x0f0,B8(0,0,0,0,0,1,0,1),
	0, 2,B8(0,0,1,1,0,0,0,0),B8(0,1,0,0,0,1,0,1),B8(0,0,0,0,0,0,1,1),B8(0,1,0,1,0,1,0,0), 0, 2,
	B8(0,0,0,0,0,0,1,1),B8(0,1,0,1,0,1,0,0),B8(0,0,0,0,0,0,1,1),B8(1,0,1,0,1,0,0,0),
	0, 2,0x0ff,0,0x0f0,B8(0,0,0,0,0,1,0,1),
	0, 2,B8(0,0,1,1,1,1,0,0),B8(0,1,0,0,0,0,0,0),B8(1,1,0,0,1,1,1,1),B8(0,0,0,0,0,0,0,0), 0, 2,
	B8(0,0,0,0,0,0,1,1),B8(0,1,0,1,0,1,0,0),B8(0,0,1,1,1,1,1,1),B8(1,0,0,0,0,0,0,0),
	0, 2,0x0ff,0,0x0f0,B8(0,0,0,0,0,1,1,0),
	0, 2,B8(0,0,0,0,0,0,1,1),B8(1,0,1,0,1,0,0,0),0x0ff,0, 1,B8(0,1,0,1,0,1,1,0), 3,0x0ff,0,
	B8(1,1,0,0,0,0,0,0),B8(0,0,1,0,1,0,1,0),
	0, 2,B8(0,0,0,0,0,0,1,1),B8(1,0,1,0,1,1,0,0),0x0ff,0, 0, 4,B8(1,1,0,0,0,0,1,1),
	B8(0,0,0,0,1,0,0,0),B8(1,1,0,0,0,0,0,0),B8(0,0,1,1,1,0,1,0),
	0, 3,0x0f,B8(1,0,1,1,0,0,0,0),B8(0,0,0,0,0,0,1,1),B8(0,0,1,0,0,0,0,0), 0, 3,0x0ff,0,0x0f0,B8(0,0,0,0,1,1,1,0),
	0, 3,B8(0,0,1,1,1,1,1,1),B8(1,1,0,0,0,0,0,0),B8(0,0,1,1,1,1,1,1),B8(1,0,0,0,0,0,0,0), 0, 3,0x0ff,0,
	B8(1,1,1,1,1,1,0,0),B8(0,0,0,0,0,0,1,1),
	0, 5,0x0ff,0,0x0ff,0,
};

UBYTE test_sprite3[] = {
	WD(0),WD(5),WD(0),WD(3),
	3,0x0ff,0,0x0f,0x10,0,4,0x0ff,0,0x0f0,0x01,
	1,0x11,5,0x0ff,0,0x0f,0x10,0,2,0x0ff,0,0x0ff,0,
};

UBYTE test_sprite4[] = {
	WD(0),WD(12),WD(0),WD(15),
	3,0x0ff,0,0x0f0,0x06,2,0x66,0x66,9,0x0ff,0,0x0ff,0,
	4,0x66,0x66,0x66,0x66,8,0x0ff,0,0x0f0,0x06,4,0x66,0x66,0x66,0x66,
	7,0x0f,0x60,0x0f0,0x02,6,0x26,0x66,0x66,0x66,0x66,0x62,
	2,0x0f,0x20,0x0ff,0,1,0x44,2,0x0f,0x40,0x0ff,0,
	8,0x22,0x26,0x66,0x66,0x66,0x66,0x62,0x22,2,0x0ff,0,0,0x44,1,0x44,2,0x0ff,0,0,0x22,
	7,0x22,0x66,0x66,0x66,0x66,0x22,0x22,2,0x0f0,0x04,0,0x44,1,0x44,2,0x0ff,0,0x0f0,0x02,
	6,0x22,0x22,0x66,0x66,0x22,0x22,2,0x0f,0x20,0x0f0,0x04,1,0x44,
	4,0x0f,0x40,0,0x22,5,0x22,0x22,0x22,0x22,0x22,2,0x0f,0x20,0,0x44,1,0x44,4,0x0ff,0,0,0x22,0,
	2,0x0f,0x20,0,0x22,1,0x22,2,0x0f,0x20,0,0x22,0,1,0x0f,0x20,0x0ff,0,1,0x44,
	5,0x0f,0x40,0,0x22,
	0,2,0x0f,0x20,0x0f0,0x08,0,2,0x0f0,0x08,0x0ff,0,1,0x22,2,0x0f,0x20,0x0f,0x40,
	0,5,0x0ff,0,0,0x2c,
	1,0x0cc,4,0x0f,0x0c0,0,0x22,1,0x24,5,0x0ff,0,0x0f0,0x0c,
	2,0x0cc,0x0cc,4,0x0f,0x70,0x0f0,0x06,0,6,0x00f,0x40,0x0f0,0x07,
	2,0x0cc,0x0c7,4,0x0ff,0,0,0x64,0,7,0x0f,0x60,0,0x7c,
	0,5,0x0f,0x70,0x0f,0x40,0,7,0x0ff,0,0x0f0,0x07,0,10,0x0ff,0,0x0ff,0,
};

UBYTE test_sprite5[] = {
	WD(0),		/* Size */
	WD(3 * 8),	/* Width */
	WD(3),		/* Height */
	WD(0),		/* TColor */
	WD(0), WD(3), WD(0), WD(3),	/* x_adj, x_len, y_adj, y_len */
	0,0,0,0, 1,1,1,1, 0,0,0,0,
	0,0,0,0, 3,3,3,3, 0x80, 0x80, 0x80, 0x80,
	0,0,0,0, 1,1,1,1, 0,0,0,0,
	0,0,0,0,
};

UBYTE test_sprite6[] = {
	WD(0),		/* Size */
	WD(4 * 8),	/* Width */
	WD(15),		/* Height */
	WD(0),		/* TColor */
	WD(0), WD(4), WD(0), WD(15),	/* x_adj, x_len, y_adj, y_len */
	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,1,1,1), B8(0,0,0,0,0,1,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,0,0,0,0,0), B8(1,1,1,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,1,1,1,1), B8(0,0,0,0,1,1,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,0,0,0,0), B8(1,1,1,1,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,1,1,1,1,1), B8(0,0,0,1,1,1,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,0,0,0), B8(1,1,1,1,1,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,1,1,1,1,1,1,1), B8(0,0,0,1,1,1,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,0), B8(1,1,1,1,1,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,1,1,1,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,1), B8(0,0,0,1,1,1,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,1), B8(1,1,1,1,1,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,1,1,1,1,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,1), B8(0,0,0,0,1,1,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,1), B8(1,1,1,1,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,1,1,1,1,1,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,1,1,1,1,1,1,1), B8(0,0,0,0,0,0,1,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,1), B8(1,1,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,1,1,1,1,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,1,1,1,1,1,1), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,1,1,1,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(1,1,1,1,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,1,1,1,0,1,1), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(1,1,1,0,1,1,1,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(1,1,1,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,1,1,1,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,1),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,1,1,1,0), B8(0,0,0,0,0,0,0,0), B8(0,1,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(1,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,0,0), B8(0,0,1,0,0,0,0,0), B8(0,0,0,0,1,1,1,1), B8(0,0,0,0,1,1,1,1),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,1,1,1,0), B8(0,0,0,0,0,0,0,1), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,0,0,0,1,0), B8(0,0,0,0,0,0,1,0), B8(0,1,1,1,1,1,1,0), B8(0,1,1,1,1,1,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,1,0,0), B8(0,0,0,0,0,1,1,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,1,0,0,0,1,0,0), B8(0,1,0,0,0,1,0,0), B8(0,1,1,1,1,1,0,0), B8(0,0,1,1,1,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,1,0,1,0), B8(0,0,0,0,1,1,1,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,1,0,1,0,0,0), B8(0,0,1,0,1,0,0,0), B8(0,0,1,1,1,0,0,0), B8(0,0,0,1,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,1,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
	B8(0,0,0,1,0,0,0,0), B8(0,0,0,1,0,0,0,0), B8(0,0,0,1,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),
	B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0), B8(0,0,0,0,0,0,0,0),

	0,0,0,0,
};

void Nothing (void)
{
}

#define TRANS	1
#define DATA	0

static UBYTE	*data;
static UBYTE	 bcount;
static UBYTE	 mode;
static UBYTE	 tfirst;

static UBYTE nextbyte (void)
{
	for (;;) {
		if (mode == TRANS) {
			if (bcount) {
				bcount--;
				if (tfirst) {
					tfirst = FALSE;
					data++;
					return (*data++);
				}
				if (!bcount) {
					data++;
					return (*data++);
				}
				return 0;
			}
			bcount  = *data++;
			mode    = DATA;
		}
		if (mode == DATA) {
			if (bcount) {
				bcount--;
				return (*data++);
			}
			bcount = (*data++);
			mode   = TRANS;
			tfirst = TRUE;
		}
	}
}

void TestDrawShape (GShape *gs)
{
	UBYTE far	*buffer;
	UBYTE		*out;
	CGAGShape	*cs;
	UWORD		 width;
	UWORD		 height;
	UWORD		 odd = FALSE;

	buffer = MK_FP(*DisplayBuffer, 0);

	cs     = gs->Data.CGA.cs[0];
	data   = (UBYTE *)cs + CGAGSHAPESIZE;
	mode   = TRANS;
	bcount = (*data++);
	tfirst = TRUE;

	height = cs->DHeight;
	while (height) {
		out     = buffer;
		width   = cs->DWidth;
		while (width) {
			*out++ = nextbyte ();
			width--;
		}
		buffer += (odd ? (-0x2000 + (320 / 4)) : (0x2000));
		odd = !odd;
		height--;
	}

}

/********************************* Test() *********************************/
void Test (void)
{
	short	 i;
	short	 j;
	short	 x;
	short	 y;
	short	 c;
	short	 h;
	short	 w;
	short	 sh;
	GShape	 gsx;
	GShape	*gs;
	BPIType	*bpi;
	GShape	**shapes;

#if TEST_ISHAPES
	gs = &gsx;
	if (GraphicsMode == MODE_CGA) {
		gsx.Width  = 0;
		gsx.Height = 0;
		gsx.TColor = 0;
		gsx.Data.CGA.cs[0] = (CGAGShape *)test_sprite2;
		gsx.Data.CGA.cs[1] = (CGAGShape *)test_sprite2;
		gsx.Data.CGA.cs[2] = (CGAGShape *)test_sprite2;
		gsx.Data.CGA.cs[3] = (CGAGShape *)test_sprite2;
		gsx.Data.CGA.pcc   = 0;
	} else if (GraphicsMode == MODE_TANDY) {
		gsx.Width  = 0;
		gsx.Height = 0;
		gsx.TColor = 0;
		gsx.Data.TGA.ts[0] = (TGAGShape *)test_sprite4;
		gsx.Data.TGA.ts[1] = (TGAGShape *)test_sprite4;
		gsx.Data.TGA.pcc   = 0;
	} else if (GraphicsMode == MODE_EGA){
		gs = (GShape *)test_sprite6;
	}
#endif

#if TEST_SHAPES
	bpi = LoadBPI (TEST_FPF, TEST_BPI_FPI);
	if (!bpi) {
		return;
	}

	shapes = bpi->Shapes;
#endif

	SetPalette (&bpi->Palette);

	SetWorkPage (BackBuffer);
	SetPaperColor (1);
	ClearWorkPage ();
	Restore (-50, 60, 50, 10);
	Restore (-50, 60, 40, 10);
	RestoreShowScreen ();

#if TEST_DBUF
	SetWorkPage (WorkBuffer);
	x  = 0;
	y  = 0;
	sh = 0;
	do {
		DBufStampGShape (shapes[sh], 100, 100);
		DBufPeepGShape (shapes[sh], x, y);
		DBufStampGShape (shapes[sh], 200, 100);
		DBufPeepGShape (shapes[sh], 100, 150);
		DBufFlipScreen ();
		printf ("%03d\n\033[1A", sh);
		c = getch ();
		if (c == 27) {
			return;
		}
		switch (c) {
		case 'w':
			y--;
			break;
		case 'x':
			y++;
			break;
		case 'a':
			x--;
			break;
		case 'd':
			x++;
			break;
		case '+':
			sh++;
			break;
		case '-':
			sh--;
			break;
		case 'p':
			FreeBPI (bpi);
			return;
		}
		if (sh < 0) sh = 0;
		if (sh >= bpi->NumShapes) sh = bpi->NumShapes - 1;
	} while (c != 'q' && c != '\n');
#endif

#if TEST_ISHAPES
	SetWorkPage (DisplayBuffer);
	if (GraphicsMode != MODE_MCGA) {
		if (GraphicsMode == MODE_CGA) {
			TestDrawShape (gs);
			c = getch ();
			if (c == 27) {
				return;
			}
		}
		SetWorkPage (DisplayBuffer);
		sh = 3;
		SetPenColor (sh);
		x = 0;
		y = 0;
		while (c != 'q' && c != '\n') {
			ClearWorkPage ();
#if TEST_SHAPES
			DrawGShape (shapes[1], x, y);
#else
			DrawGShape (gs, x, y);
#endif

/*			DrawRect (x, y, 20, 20);
*/
			c = getch ();
			if (c == 27) {
				return;
			}
			switch (c) {
			case 'w':
				y--;
				break;
			case 'x':
				y++;
				break;
			case 'a':
				x -= 1;
				break;
			case 'd':
				x += 1;
				break;
			case '[':
				sh--;
				break;
			case ']':
				sh++;
				break;
			}
			SetPenColor (sh);
		}
	}
#endif

#if TEST_SHAPES
	SetWorkPage (DisplayBuffer);
	for (x = 0; x < 40; x++) {
		DrawGShape (shapes[0], x,   0 + x);
		DrawGShape (shapes[1], x,  20 + x);
		DrawGShape (shapes[2], x,  40 + x);
/*		DrawGShape (shapes[3], x,  60 + x);
		DrawGShape (shapes[4], x,  80 + x);
		DrawGShape (shapes[5], x, 100 + x);
		DrawGShape (shapes[6], x, 120 + x);
		DrawGShape (shapes[7], x, 140 + x);
		DrawGShape (shapes[8], x, 160 + x);
*/		c = getch ();
		if (c == 27) {
			return;
		}
	}
#endif

#if TEST_RESTORE
	SetWorkPage (BackBuffer);

	for (x = 0; x < 320; x++) {
		for (y = 0; y < 200; y++) {
			SetPenColor ((x + y) % 9 + 1);
			DrawPixel (x, y);
		}
	}
	Restore (-10, -20, 30, 50);
	Restore (300, 190, 50, 60);

	SetWorkPage (DisplayBuffer);
	SetPaperColor (0);
	SetPenColor (3);
	for (j = 0; j < 100; j++) {
		ClearWorkPage ();
		x = random (310);
		y = random (190);
		w = random (320 - x);
		h = random (200 - y);
		for (i = 0; i < w; i++) {
			DrawPixel (x + i, y);
			DrawPixel (x + i, y + h - 1);
		}
		for (i = 0; i < h; i++) {
			DrawPixel (x, y + i);
			DrawPixel (x + w - 1, y + i);
		}
		c = getch ();
		if (c == 27) {
			return;
		}
		Restore (x + 1, y + 1, w - 2, h - 2);
		ShowRect (x + 1, y + 1, w - 2, h - 2);
		c = getch ();
		if (c == 27) {
			return;
		}
		if (c == 'M' || c == 'm') {
			j = 0;
		}
	}
#endif

#if TEST_PIXELS
	SetWorkPage (DisplayBuffer);

	SetPaperColor (0);
	ClearWorkPage ();
	SetPenColor (1);
	for (x = 0; x < 10; x++) {
		DrawPixel (x, 0);
		DrawPixel (0, x);
	}
	if (GraphicsMode == MODE_CGA) {
		SetPaperColor (0);
		ClearWorkPage ();
		DrawGShape (gs, 0, 0);
	}
	c = getch ();
	if (c == 27) {
		return;
	}


	for (i = 1; i < 10; i++) {
		SetPenColor (i);
		for (x = 0; x < 50; x++) {
			DrawPixel (x + i * 3, x);
		}
		c = getch ();
		if (c == 27) {
			return;
		}
	}
	for (i = 1; i < 10; i++) {
		SetPenColor (i);
		for (x = 0; x < 50; x++) {
			for (y = 0; y < 50; y++) {
				DrawPixel (i * 10 + x, y);
			}
		}
		c = getch ();
		if (c == 27) {
			return;
		}
	}
	for (i = 1; i < 10; i++) {
		SetPenColor (i);
		for (j = 0; j < 1000; j++) {
			DrawPixel (random (320), random (200));
		}
		c = getch ();
		if (c == 27) {
			return;
		}
	}
#endif

#if TEST_CLEAR
	for (i = 0; i < 10; i++) {
		SetPaperColor (i);
		ClearWorkPage ();
		c = getch ();
		if (c == 27) {
			return;
		}
	}
#endif

#if TEST_RECT
	for (j = 0; j < 1000; j++) {
		SetPenColor (random (10));
		x = random (300);
		y = random (200);
		DrawRect (x, y, random (300 - x) + 20, random (200 - y));
	}
	c = getch ();
	if (c == 27) {
		return;
	}

	SetPaperColor (0);
	ClearWorkPage ();

	j = 1;
	for (x = 1; x < 70; x++) {
		SetPenColor (j);
		j++;
		if (j == 4) j = 1;
		DrawRect (100 - x, x, x, x);
		DrawRect (102, x, x, x);
		c = getch ();
		if (c == 27) {
			return;
		}
	}

	SetPaperColor (0);
	ClearWorkPage ();

	SetPenColor (1);
	for (x = 0; x < 4; x++) {
		DrawRect (16 + x, x * 3, 1, 2);
		DrawRect (32 + x, x * 3, 2, 2);
		DrawRect (48 + x, x * 3, 3, 2);
		DrawRect (64 + x, x * 3, 4, 2);
		DrawRect (80 + x, x * 3, 5, 2);
		DrawRect (96 + x, x * 3, 6, 2);
		DrawRect (128 + x, x * 3, 7, 2);
		DrawRect (144 + x, x * 3, 8, 2);
		DrawRect (160 + x, x * 3, 9, 2);
		DrawRect (176 + x, x * 3, 10, 2);
		DrawRect (192 + x, x * 3, 11, 2);
		DrawRect (208 + x, x * 3, 12, 2);

	}
	c = getch ();
	if (c == 27) {
		return;
	}
#endif

/*
	for (x = 0; x < 50; x++) {
		CGA_DrawShape (&test_sprite1, x, x);
		c = getch ();
		if (c == 27) {
			return;
		}
	}
	for (x = 0; x < 50; x++) {
		CGA_DrawShape (&test_sprite2, x + 100, x + 50);
		c = getch ();
		if (c == 27) {
			return;
		}
	}
*/

}

/******************************** TEMPLATE ********************************/
#define ARG_G	0

char Usage[] = "Usage: tank {-<switch>}\n";

ArgSpec Template[] = {
	CHRKEYWORD_ARG, "G", "\t-g<graphics mode> 0 = MCGA, 1 = EGA, 2 = TANDY, 3 = CGA\n",
	0, NULL, NULL,
};

/********************************* M A I N ********************************/
int main (
	int argc,
	char **argv
)
{
	short		 grmode = MODE_MCGA;
	char		**newargs;
	
	newargs = argparse (argc, argv, Template);
	if (!newargs) {
		printf ("%s\n", GlobalErrMsg);
		printarghelp (Usage, Template);
	} else {
		if (newargs[ARG_G]) {
			grmode = atoi (newargs[ARG_G]);
		}
	 	if (OpenDBufGraphics (grmode, MAX_SHAPES, 32000U)) {
			Test ();
			CloseDBufGraphics ();
		}

		if (GlobalErr) {
			printf ("%s", GlobalErrMsg);
			return FALSE;
		}
	}
	return TRUE;
} /* main */

