/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * TPalette.C
 *
 * PROGRAMMER : Gregg A. Tavares and Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 03/05/90
 *   MODIFIED : 10/08/94
 *       TABS : 05 09
 *
 * DESCRIPTION
 *		Routines for setting the TGA palette.
 *
 * HISTORY
 *		10/08/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/
#include <echidna/platform.h>
#include "switches.h"

#include <dos.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <echidna/etypes.h>
#include <echidna/alloc.h>
#include <echidna/graphics.h>
#include <echidna/eerrors.h>
#include <echidna/timer.h>
#include <echidna/memory.h>

#if REQUIRE
	#include <stdio.h>
#endif

/******************** E X T E R N A L   R O U T I N E S *******************/
extern void far TGA_ASM_WriteColors (void);

/******************************** CONSTANTS *******************************/
#define MAX_TGA_COLORS					16
#define NUM_TGA_FADE_FRAMES	4

static struct Cyc {
	short 		 MaxColors;			/* Max cycle colors routines handle */
	short		 FadeFrames;		/* Fade duration count-down timer.
							 		 * Non-zero if fade in progress */
	short		 TGAFadeFrames;		/* Number of actual color changes left */
	short		 FadeSkips;			/* # of Fade Frames to do nothing on. */
	short		 ToBlack;			/* True if fade is to black */
	ColorCycleSetType	*UserSet;	/* User Color Cycling Set */
	UBYTE		*RGB;				/* Private copy of cycle color data */
	UBYTE		*TFadeDstRGB;		/* Temp copy of fade-to colors */
	struct {
		BYTE Change;
		BYTE Fade;
		BYTE Activate;
		short Hz60FadeFrames;		
		ColorCycleSetType *CycleSet;
	} Request;
};

static struct Pal {
	short		 MaxColors;		/* Max palette colors routines handle */
	short		 FadeFrames;	/* Fade duration count-down timer.
								 * Non-zero if fade in progress */
	short		 TGAFadeFrames;	/* Number of actual color changes left */
	short		 FadeSkips;		/* # of Fade Frames to do nothing on. */
	short		 SrcOffset;		/* Offset in src pal of start of fade range*/
	short		 DstOffset;		/* Offset in Dst pal of start of fade range*/
	short		 NumColors;		/* NumColors to fade */
	short		 ToBlack;		/* True if fade is to black */
	PaletteType	*UserPalette;	/* User palette structure */
	UBYTE		 RGB[MAX_TGA_COLORS];/* Private copy of palette color data */
	UBYTE		 FadeDstRGB[MAX_TGA_COLORS]; /* Temp copy of fade-to colors*/
	struct {
		BYTE Change;
		BYTE Fade;
		short SrcOffset;
		short DstOffset;
		short NumColors;
		short Hz60FadeFrames;	
		PaletteType *Palette;
	} Request;
};

static struct PaintCtrlStruct {
	int				 InterruptHandle;
	int				 VBlankInterruptHandle;
	short			 PHertz;			/* Highest palette change rate */
	short			 Hz60ToPHertz;	
	volatile struct Cyc		 Cyc;			/* Info for color cycling */
	volatile struct Pal		 Pal;			/* Info for palette changes */
};

/****************************** E X T E R N S *****************************/
extern volatile short TGA_ActivationFlag;
extern UBYTE *TGAColors;

/****************************** G L O B A L S *****************************/

static struct PaintCtrlStruct D;

/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * TGA_Paint
 *
 * SYNOPSIS
 *		void TGA_Paint (void) {
 *
 * PURPOSE
 *		To Make changes requested by user.  This routine is called by
 *		an interrupt that is installed by OpenPainter().
 *
 * INPUT
 *		
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
InterruptRoutineType TGA_Paint (void) 
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_Paint";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short changed = FALSE;

	/********** P R O C E S S   F A D I N G   P A L E T T E  **********/
	if (D.Pal.FadeFrames) {
		if (!D.Pal.TGAFadeFrames) {
			int register i;
			UBYTE	*crn_color;
			UBYTE	*dst_color;

			dst_color = D.Pal.FadeDstRGB;
			crn_color = D.Pal.RGB +  D.Pal.SrcOffset;
			for (i = D.Pal.NumColors; i; i--) {
				UBYTE crn, dst;

				if ((crn = *crn_color) != (dst = *dst_color)) {
					UBYTE c_rgb, d_rgb;
					/* Different RGB values? */
					if ((c_rgb = crn & 0x7) != (d_rgb = dst & 0x7)) {
						/* Are both gray values (black .. white)? */
						if ((c_rgb == 7 || !c_rgb) && 
							(d_rgb == 7 || !d_rgb))
						{/* Special case */
							*crn_color = (crn & 0x8) ? 7 : 8;
						} else { 
							/* Match RGB value, leave intensity same */
							*crn_color = (crn & 0x8) | d_rgb;
						}
					} else { /* only intensity bit differs */
						/* Make them completely the same */
						*crn_color = *dst_color;
					}
				}
				crn_color++;
				dst_color++;
			}
				
			D.Pal.TGAFadeFrames = D.Pal.FadeSkips;
			--D.Pal.FadeFrames;
			changed = TRUE;
		} else {
			--D.Pal.TGAFadeFrames;
		}
	}
	
	/********** P R O C E S S   F A D I N G   C Y C L E  **********/
	if (D.Cyc.FadeFrames) {
		if (!D.Cyc.TGAFadeFrames) {
			int register i;
			UBYTE	*crn_color;
			UBYTE	*dst_color;

			dst_color = D.Cyc.TFadeDstRGB;
			crn_color = D.Cyc.RGB;
			for (i = D.Cyc.MaxColors; i; i--) {
				UBYTE crn, dst;

				if ((crn = *crn_color) != (dst = *dst_color)) {
					UBYTE c_rgb, d_rgb;
					/* Different RGB values? */
					if ((c_rgb = crn & 0x7) != (d_rgb = dst & 0x7)) {
						/* Are both gray values (black .. white)? */
						if ((c_rgb == 7 || !c_rgb) && 
							(d_rgb == 7 || !d_rgb))
						{/* Special case */
							*crn_color = (crn & 0x8) ? 7 : 8;
						} else { 
							/* Match RGB value, leave intensity same */
							*crn_color = (crn & 0x8) | d_rgb;
						}
					} else { /* only intensity bit differs */
						/* Make them completely the same */
						*crn_color = *dst_color;
					}
				}
				crn_color++;
				dst_color++;
			}
				
			D.Cyc.TGAFadeFrames = D.Cyc.FadeSkips;
			--D.Cyc.FadeFrames;
			changed = TRUE;
		} else {
			--D.Cyc.TGAFadeFrames;
		}
	} 

	/**********  P R O C E S S   C O L O R   C Y C L I N G  *********/
	if (ColorCyclingActive && D.Cyc.UserSet->Active) {
		ColorCycleType	*cycle;
		UBYTE		*rgbptr;
		short i;

		cycle = D.Cyc.UserSet->Cycles;
		rgbptr = D.Cyc.RGB;
		for (i = 0; i < D.Cyc.UserSet->NumCycles; i++) {
			UBYTE *reg;
			WORD icolor;
			UBYTE *newcolor;
			short i;
			WORD hertz = cycle->Hertz;

			if (cycle->Active) {
				/* Update Cycle Timer etc. if it has a speed */
				if (hertz) {
					/* Reset Timer and Cycle Colors if necessary */
					if (cycle->Timer <= 0) {
						cycle->Timer = D.PHertz / hertz;

						/* Cycle Colors */
						if (cycle->Direction) {
							if (++cycle->CurrentColor >= cycle->NumColors) {
								cycle->CurrentColor = 0; 
							}
						} else {
							if (--cycle->CurrentColor < 0) {
								cycle->CurrentColor = cycle->NumColors - 1;
							}
						}
					} 
					--cycle->Timer;
				}

				/* Copy Cycle Colors into Shadow Palette if they Differ */
				reg = cycle->Registers;
				icolor = cycle->CurrentColor;
				newcolor = &rgbptr[icolor];
				for (i = cycle->NumRegisters; i; i--) {
					UBYTE *crntcolor = &D.Pal.RGB[*reg];
					if (*crntcolor != *newcolor) {
						*crntcolor = *newcolor;
						changed = TRUE;
					}
					++newcolor; ++icolor;
					if (icolor >= cycle->NumColors) {
						icolor = 0;
						newcolor = rgbptr;
					}
					++reg;
				}

			}
			rgbptr += cycle->NumColors;
			++cycle;
		}
	}

	/****** P R O C E S S   P A L E T T E   R E Q U E S T S ********/
	if (D.Pal.Request.Change) {
		/* Is palette currently fading? */
		if (D.Pal.FadeFrames) { /* Terminate the fade */
			D.Pal.FadeFrames = 0;
		}
		/* Initiate palette change */
		{
			register int 		 i; 
			UBYTE	*src_color;
			UBYTE	*dst_color;
			UBYTE	toblack;

			toblack = (D.Pal.UserPalette = D.Pal.Request.Palette) == NULL;
			src_color = D.Pal.UserPalette->Data.EGA + D.Pal.Request.SrcOffset;
			dst_color = D.Pal.RGB + D.Pal.Request.DstOffset;

			for (i = D.Pal.Request.NumColors; i; i--) {
				*dst_color++ = (toblack) ? 0 : *src_color++;	
			}
		}
		/* Turn off change request */
		D.Pal.Request.Change = FALSE;

		changed = TRUE;
		
	} else if (D.Pal.Request.Fade) { /****** PALETTE FADE REQUEST ****/
		int register i;
		UBYTE	*src_color;
		UBYTE	*dst_color;

		if (D.Pal.Request.Palette) {
			D.Pal.UserPalette = D.Pal.Request.Palette;
		} 
		D.Pal.ToBlack = (D.Pal.Request.Palette == NULL);
		D.Pal.SrcOffset		= D.Pal.Request.SrcOffset;
		D.Pal.DstOffset		= D.Pal.Request.DstOffset;
		D.Pal.NumColors		= D.Pal.Request.NumColors;
		D.Pal.FadeFrames	= (D.Pal.Request.Hz60FadeFrames + 
			D.Hz60ToPHertz - 1) / D.Hz60ToPHertz;
		D.Pal.TGAFadeFrames = D.Pal.FadeSkips = 
			D.Pal.FadeFrames / (NUM_TGA_FADE_FRAMES - 1);
		D.Pal.FadeFrames = NUM_TGA_FADE_FRAMES - 1;

		/* Copy fade-to colors to fade buffer */
		src_color = D.Pal.UserPalette->Data.EGA + D.Pal.DstOffset;
		dst_color = D.Pal.FadeDstRGB;
		for (i = D.Pal.NumColors; i; i--) {
			*dst_color++ =(D.Pal.ToBlack) ? 0 :  *src_color++;
		}

		D.Pal.Request.Fade = FALSE;
	}

	/****** P R O C E S S   C Y C L E   R E Q U E S T S ********/
	/* Process Cycle Requests (change or fade) */
	if (D.Cyc.Request.Change) {
		/* Is cycle currently fading? */
		if (D.Cyc.FadeFrames) { /* Terminate the fade */
			D.Cyc.FadeFrames = 0;
		}
		/* Install new user cycle set */
		D.Cyc.UserSet = D.Cyc.Request.CycleSet;

		/* Make private copy of RGB data */
		memcpy (D.Cyc.RGB, D.Cyc.UserSet->Colors, 
			D.Cyc.UserSet->TotalColors * sizeof (UBYTE));

		if (D.Cyc.Request.Activate) ColorCyclingActive = TRUE;

		/* Turn off change request */
		D.Cyc.Request.Change = FALSE;

	} else if (D.Cyc.Request.Fade) { /***** CYCLE FADE REQUEST *****/
		int register i;
		UBYTE *src_color;
		UBYTE *dst_color;
		
		/* Turn off request for fade */
		D.Cyc.Request.Fade = FALSE;

		if (D.Cyc.Request.CycleSet) {
			D.Cyc.UserSet = D.Cyc.Request.CycleSet;
		} 
		ColorCyclingActive = TRUE;
		D.Cyc.ToBlack = (D.Cyc.Request.CycleSet == NULL);
		D.Cyc.FadeFrames	= (D.Cyc.Request.Hz60FadeFrames + 
			D.Hz60ToPHertz - 1) / D.Hz60ToPHertz;
		D.Cyc.TGAFadeFrames = D.Cyc.FadeSkips = 
			D.Cyc.FadeFrames / (NUM_TGA_FADE_FRAMES - 1);
		D.Cyc.FadeFrames = NUM_TGA_FADE_FRAMES - 1;

		/* Copy fade-to colors to fade buffer */
		src_color = D.Cyc.UserSet->Colors;
		dst_color = D.Cyc.TFadeDstRGB;
		for (i = D.Cyc.MaxColors; i; i--) {
			*dst_color++ =(D.Cyc.ToBlack) ? 0 :  *src_color++;
		}
	}

	/* Maintain public flags */
	PaletteIsFading = D.Pal.FadeFrames;
	CycleSetIsFading = D.Cyc.FadeFrames;

	/* Activate color stuffer if something changed */
	TGA_ActivationFlag = changed;

}
} /* TGA_Paint */

/*********************************************************************
 *
 * TGA_ClosePainter
 *
 * SYNOPSIS
 *		void TGA_ClosePainter (void) {
 *
 * PURPOSE
 *		To clean up stuff done in TGA_OpenPainter()
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_ClosePainter (void) 
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_ClosePainter";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	if (D.VBlankInterruptHandle > 0) RemoveVBlank (D.VBlankInterruptHandle);
	if (D.InterruptHandle > 0) RemoveInterrupt (D.InterruptHandle);
	if (D.Cyc.TFadeDstRGB) MEM_free (D.Cyc.TFadeDstRGB);
	if (D.Cyc.RGB) MEM_free (D.Cyc.RGB);

}
} /* TGA_ClosePainter */

/*********************************************************************
 *
 * TGA_OpenPainter
 *
 * SYNOPSIS
 *		short TGA_OpenPainter (	
 *			short max_colors, 
 *			short max_cycle_colors, 
 *			short hertz
 *		)
 *
 * PURPOSE
 *		To initialize and allocate structures for controlling palette.
 *
 * INPUT
 *		max_colors: # of colors in largest palette or cycle range used.
 *		max_cycle_colors: Total # of colors in largest cycle set used.
 *				For example if you use only one cycle set, and it has
 *				a cycle range with four colors and another with 3 colors,
 *				then mac_cycle_colors == (4 + 3) == 7.
 * 		hertz:	Maximum number of times per second that any color cycle
 *				will change. Or if no cycle colors are used then the rate
 *				at which you would like the the palette handling interrupt
 *				routine to check for palette change requests.
 *				Must be <= 60.
 *
 *		IMPORTANT:	OpenTimers() must be called before this routine is 
 *				called, becuase it tries to install interrupts.
 *	
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		TRUE on success.
 *		FALSE on failure.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short TGA_OpenPainter (	
	short max_colors, 
	short max_cycle_colors, 
	short hertz
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_OpenPainter";
#endif
#if REQUIRE
	if (max_colors < 2) {
		SetGlobalErr ( ERR_REQUIREMENTS);
		GEprintf ("Error: max_colors < 2 in TGA_OpenPainter()\n");
		return FALSE;
	}
#endif
/*-------------------------------------------------------------------------*/
{
	/* Clear Palette Control Structure */
	memset (&D, 0, sizeof (D));

	max_colors = min (max_colors, MAX_TGA_COLORS);
	D.Pal.MaxColors = PaletteMaxColors = max_colors;
	D.Cyc.MaxColors = max_cycle_colors;
	D.PHertz = hertz;
	D.Hz60ToPHertz = 60 / D.PHertz;
	D.InterruptHandle = -1;
	D.VBlankInterruptHandle = -1;
	PaletteIsFading = CycleSetIsFading = FALSE;

	if (D.Cyc.MaxColors) {
		/* Allocate Space for Private color cycle colors */
		if ((D.Cyc.TFadeDstRGB = (UBYTE *)MEM_calloc 
			(D.Cyc.MaxColors, sizeof (UBYTE))) == NULL)
		{
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("TGA_OpenPainter:OOM D.Cyc.TFadeDstRGB\n");
			goto CLOSE_FAIL;
		}

		/* Allocate Space for Private color Fading cycle colors */
		if ((D.Cyc.RGB = (UBYTE *)MEM_calloc 
			(D.Cyc.MaxColors, sizeof (UBYTE))) == NULL)
		{
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("TGA_OpenPainter:OOM D.Cyc.RGB\n");
			goto CLOSE_FAIL;
		}
	}

	/* Install Palette handling interrupt routine */
	if (!(D.InterruptHandle = InstallInterrupt (TGA_Paint, D.PHertz))) {
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("TGA_OpenPainter: couldn't install TGA_Paint()\n");
		goto CLOSE_FAIL;
	}

	/* Install vertical blank color stuffer */
	if (!(D.VBlankInterruptHandle = InstallVBlank (TGA_ASM_WriteColors))) {
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("TGA_OpenPainter: couldn't install VBlank\n");
		goto CLOSE_FAIL;
	}
	

	/* Set Palette to Black (black because D.Pal.RGB allocated with calloc()) */
	TGAColors = D.Pal.RGB;
	TGA_ActivationFlag = TRUE;

	return TRUE;
CLOSE_FAIL:

	TGA_ClosePainter();
	return FALSE;
}
} /* TGA_OpenPainter */

/*********************************************************************
 *
 * TGA_UsePaletteN
 *
 * SYNOPSIS
 *		void	TGA_UsePaletteN  (
 *			PaletteType	*palette,
 *			short		 src_offset,
 *			short		 dst_offset,
 *			short		 numcolors
 *		) 
 *
 * PURPOSE
 *		To set a range of colors in the hardware palette from the
 *		corresponding range in a user palette, or to black if 'palette'
 *		is NULL.
 *
 * INPUT
 *		palette:	User palette to get colors from, or NULL if you want
 *					to set colors to black.
 *		src_offset:	First color of range in palette to copy from.
 *		dst_offset:	First color of range in palette to write to.
 *		numcolors:	Number of colors to set.
 *
 * AFFECTS
 *		Sets up flags to tell interrupt driven Palette updating routine to
 *		copy the color range to the Private (shadow) palette then to hardware
 *		palette. Waits until palette is set before returning.
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_UsePaletteN  (
	PaletteType	*palette,
	short		 src_offset,
	short		 dst_offset,
	short		 numcolors
) 
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_UsePaletteN";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short tmp;

	/* Do nothing if offset is out of palette range */
	if (dst_offset >= D.Pal.MaxColors) return;

	/* Reduce numcolors if it is out of palette range */
	if ((tmp = (numcolors + dst_offset)) > D.Pal.MaxColors) {
		numcolors -= (tmp - D.Pal.MaxColors);
	}

	D.Pal.Request.SrcOffset = src_offset;
	D.Pal.Request.DstOffset = dst_offset;
	D.Pal.Request.NumColors = numcolors;
	D.Pal.Request.Palette = palette;
	D.Pal.Request.Change = TRUE;

	/* Wait til request recieved by interrupt driven Paint() routine */
	while (D.Pal.Request.Change) {}

	/* Wait for palette to be all set */
	while (TGA_ActivationFlag) {}
}
} /* TGA_UsePaletteN */

/*********************************************************************
 *
 * TGA_UseColorCycleSet
 *
 * SYNOPSIS
 *		void TGA_UseColorCycleSet (
 *			ColorCycleSetType	*color_cycle_set,
 *			short				 activate
 *		)
 *
 * PURPOSE
 *		To install a new cycle set and optionally activate it.
 *
 * INPUT
 *		color_cycle_set:
 *					Installed
 *		cycle set	CycleSet	Action
 *		---------	---------	----------
 *		valid		valid		Installs your cycle set.
 *		valid		NULL		Installs your cycle set.
 *		NULL		valid		Installs Null cycle set.
 *		NULL		NULL		No action.
 *	
 *		activate:			TRUE if cycling should be activated.
 *
 * AFFECTS
 *		Puts in request to change the cycle-set and waits til request is
 *		handled before returning.
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_UseColorCycleSet (
	ColorCycleSetType	*color_cycle_set,
	short				 activate
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_UseColorCycleSet";
#endif
#if REQUIRE
	if (color_cycle_set && color_cycle_set->TotalColors > D.Cyc.MaxColors) {
	 	SetGlobalErr (ERR_GENERIC);
		GEprintf1 ("Error: TGA_UseColorCycleSet(): TotalColors > (MaxColors=%d)\n",
			D.Cyc.MaxColors);
		return;
	}
#endif
/*-------------------------------------------------------------------------*/
{
	if (!color_cycle_set && !D.Cyc.UserSet) return;

	D.Cyc.Request.CycleSet = color_cycle_set;
	D.Cyc.Request.Activate = activate;
	D.Cyc.Request.Change = TRUE;

	/* Wait til request recieved by interrupt driven Paint() routine */
	while (D.Cyc.Request.Change) {}

}
} /* TGA_UseColorCycleSet */

/*********************************************************************
 *
 * TGA_UsePaintBoxN
 *
 * SYNOPSIS
 *		void TGA_UsePaintBoxN (
 *			PaintBoxType	*paintbox,
 *			short			 src_offset,
 *			short			 dst_offset,
 *			short		 	 numcolors,
 *			short			 activate_color_cycling
 *		)
 *
 * PURPOSE
 *		To install new palette, and install new color cycling set and
 *		optionally activate it.
 *
 * INPUT
 *					paintbox	Installed 
 *		paintbox	->Palette	Palette		Action
 *		--------	---------	---------	------------
 *		valid		valid		valid		Installs your palette.
 *		valid		valid		NULL		Installs your palette.
 *		valid		NULL		valid		Sets palette to black and 
 *												installs NULL palette.
 *		valid		NULL		NULL		No action.
 *		NULL					valid		Sets palette to black and
 *												installs NULL palette.
 *		NULL					NULL		No action.
 *
 * 
 *					paintbox	Installed 
 *		paintbox	->CycleSet	CycleSet	Action
 *		--------	----------	--------	----------
 *		valid		valid		valid		Installs your cycle set.
 *		valid		valid		NULL		Installs your cycle set.
 *		valid		NULL		valid		Installs Null cycle set.
 *		valid		NULL		NULL		No action.
 *		NULL					valid		Installs Null cycle set.
 *		NULL					NULL		No action.
 *	
 *		src_offset:	First color of range in palette to copy from.
 *		dst_offset:	First color of range in palette to write to.
 *		numcolors:	Number of colors in palette range.
 *		activate_color_cyclinge:	non-zero if you want color cycling
 *					to be activated after cycle set is installed.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_UsePaintBoxN (
	PaintBoxType	*paintbox,
	short			 src_offset,
	short			 dst_offset,
	short			 numcolors,
	short			 activate_color_cycling
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UsePaintBox";
#endif
#if REQUIRE
#endif
/*-------------------------------------------------------------------------*/
{
	PaletteType			*palette = NULL;
	ColorCycleSetType	*cycleset = NULL;
	short tmp;
	short do_cyc;

	if (paintbox) {
		palette = paintbox->Palette;
		cycleset = paintbox->CycleSet;
	}

	/* Do nothing if offset is out of palette range */
	if (dst_offset >= D.Pal.MaxColors) return;

	/* Reduce numcolors if it is out of palette range */
	if ((tmp = (numcolors + dst_offset)) > D.Pal.MaxColors) {
		numcolors -= (tmp - D.Pal.MaxColors);
	}

	D.Pal.Request.SrcOffset = src_offset;
	D.Pal.Request.DstOffset = dst_offset;
	D.Pal.Request.NumColors = numcolors;
	D.Pal.Request.Palette = palette;

	D.Cyc.Request.CycleSet = cycleset;
	D.Cyc.Request.Activate = activate_color_cycling;

	do_cyc = (cycleset || D.Cyc.UserSet);
	D.Pal.Request.Change = (palette || D.Pal.UserPalette);
	D.Cyc.Request.Change = do_cyc;

	/* Wait til request recieved by interrupt driven Paint() routine */
	while (D.Pal.Request.Change || D.Cyc.Request.Change) {}

	/* Wait for palette to be all set */
	while (TGA_ActivationFlag) {}

}

} /* TGA_UsePaintBoxN */

/*********************************************************************
 *
 * TGA_FadeToPaletteN
 *
 * SYNOPSIS
 *		void TGA_FadeToPaletteN (
 *			PaletteType *palette, 
 *			short		 src_offset,
 *			short		 dst_offset,
 *			short		 numcolors,
 *			short		 hz60frames
 *		) 
 *
 * PURPOSE
 *		To fade from the current palette colors to those in 'palette'.
 *	
 * INPUT
 *		palette:	
 *					Installed
 *		palette		Palette		Action
 *		---------	---------	----------
 *		valid		valid		Fades from installed palette to 
 *									your palette.
 *		valid		NULL		Fades from black to your palette.
 *		NULL		valid		Fades from installed palette 
 *									to black then installs NULL palette.
 *		NULL		NULL		No action.
 *	
 *		src_offset:	First color of range in palette to fade from.
 *		dst_offset:	First color of range in palette to fade to.
 *		numcolors:	Number of colors to fade.
 *		hz60frames:	Duration of fade in 1/60 second frames.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_FadeToPaletteN (
	PaletteType *palette, 
	short		 src_offset,
	short		 dst_offset,
	short		 numcolors,
	short		 hz60frames
) 
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_FadeToPaletteN";
#endif
#if REQUIRE
	if (palette && palette->NumColors > D.Pal.MaxColors) {
		SetGlobalErr (ERR_GENERIC);
		GEprintf1 ("Error: TGA_FadeToPaletteN(): NumColors > (MaxColors=%d)\n",
			D.Pal.MaxColors);
		return;
	}
#endif
/*-----
#endif
/*-------------------------------------------------------------------------*/
{
	short tmp;

	if (!palette && !D.Pal.UserPalette) return;

	/* Do nothing if offset is out of palette range */
	if (dst_offset >= D.Pal.MaxColors) return;

	/* Reduce numcolors if it is out of palette range */
	if ((tmp = (numcolors + dst_offset)) > D.Pal.MaxColors) {
		numcolors -= (tmp - D.Pal.MaxColors);
	}

	D.Pal.Request.SrcOffset = src_offset;
	D.Pal.Request.DstOffset = dst_offset;
	D.Pal.Request.NumColors = numcolors;
	D.Pal.Request.Hz60FadeFrames = hz60frames;
	D.Pal.Request.Palette = palette;
	D.Pal.Request.Fade = TRUE;

	/* Wait til fade has started before exiting */
	while (D.Pal.Request.Fade) {}

}
} /* TGA_FadeToPaletteN */

/*********************************************************************
 *
 * TGA_FadeToColorCycleSet
 *
 * SYNOPSIS
 *		void TGA_FadeToColorCycleSet (
 *			ColorCycleSetType	*color_cycle_set,
 *			short				 hz60frames
 *		)
 *
 * PURPOSE
 *		To fade from the current colors in the current CycleSet to 
 *		those in 'color_cycle_set'.		
 *
 * INPUT
 *		color_cycle_set: 
 *						Installed 
 *		color_cycle_set	CycleSet	Action
 *		---------------	--------	---------
 *		valid			valid		Installs your cycle set but fades
 *										from old installed cycleset
 *										colors to your cycleset colors.
 *		valid			NULL		Installs your cycle set and fades 
 *										from black cycle set colors 
 *										to your cycleset colors.
 *		NULL			valid		Fades installed cycle set colors 
 *										to black then installs NULL
 *										cycle set.
 *		NULL			NULL		No action.
 *
 *		hz60frames:	Duration of fade in 1/60 second frames.
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_FadeToColorCycleSet (
	ColorCycleSetType	*color_cycle_set,
	short				 hz60frames
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_FadeToColorCycleSet";
#endif
#if REQUIRE
	if (color_cycle_set && color_cycle_set->TotalColors > D.Cyc.MaxColors) {
	 	SetGlobalErr (ERR_GENERIC);
		GEprintf1 ("Error: TGA_FadeToColorCycleSetSet(): TotalColors > (MaxColors=%d)\n",
			D.Cyc.MaxColors);
		return;
	}
#endif
/*-------------------------------------------------------------------------*/
{
	if (color_cycle_set || D.Cyc.UserSet) {
		D.Cyc.Request.Hz60FadeFrames = hz60frames;
		D.Cyc.Request.CycleSet = color_cycle_set;
		D.Cyc.Request.Fade = TRUE;

		/* Wait til fade has started before exiting */
		while (D.Cyc.Request.Fade) {}
	}

}
} /* TGA_FadeToColorCycleSet */

/*********************************************************************
 *
 * TGA_FadeToPaintBoxN
 *
 * SYNOPSIS
 *		void TGA_FadeToPaintBoxN (
 *			PaintBoxType *paintbox,
 *			short		 src_offset,
 *			short		 dst_offset,
 *			short		 numcolors,
 *			short		 hz60frames
 *		)
 *
 * PURPOSE
 *		To fade from the colors in the current paintbox to those in
 *		'paintbox'.
 *
 * INPUT
 *		paintbox:	
 * 
 *					paintbox	Installed 
 *		paintbox	->Palette	Palette		Action
 *		--------	---------	-------		----------
 *		valid		valid		valid		Fades from installed palette to 
 *												your palette.
 *		valid		valid		NULL		Fades from black to your palette.
 *		valid		NULL		valid		Fades from installed palette 
 *												to black then installs 
 *												NULL palette.
 *		valid		NULL		NULL		No action.
 *		NULL					valid		Fades from installed palette 
 *												to Black then installs 
 *												NULL palette.
 *		NULL					NULL		No Action.
 *
 * 
 *					paintbox	Installed 
 *		paintbox	->CycleSet	CycleSet	Action
 *		--------	----------	--------	----------
 *		valid		valid		valid		Installs your cycle set but fades
 *												from old installed cycleset
 *												colors to your cycleset colors.
 *		valid		valid		NULL		Installs your cycle set and fades 
 *												from black cycle set colors 
 *												to your cycleset colors.
 *		valid		NULL		valid		Fades installed cycle set colors
 *												to black then installs 
 *												NULL cycle set.
 *		valid		NULL		NULL		No action.
 *		NULL					valid		Fades installed cycle set colors 
 *												to black then installs
 *												NULL cycle set.
 *		NULL					NULL		No Action.                           
 *
 *		src_offset:	First color of range in palette to fade from.
 *		dst_offset:	First color of range in palette to fade to.
 *		numcolors:	Number of colors in palette range.
 *		hz60frames:	Duration of fade in 1/60 second frames.
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_FadeToPaintBoxN (
	PaintBoxType *paintbox,
	short		 src_offset,
	short		 dst_offset,
	short		 numcolors,
	short		 hz60frames
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "TGA_FadeToPaintBoxN";
#endif
#if REQUIRE
	if (paintbox) {
		if (paintbox->CycleSet && 
			paintbox->CycleSet->TotalColors > D.Cyc.MaxColors) 
		{
		 	SetGlobalErr (ERR_GENERIC);
			GEprintf1 ("Error: TGA_FadePaintBoxN(): TotalColors > (MaxColors=%d)\n",
				D.Cyc.MaxColors);
			return;
		}
		if (paintbox->Palette && paintbox->Palette->NumColors >
			D.Pal.MaxColors)
		{
		 	SetGlobalErr (ERR_GENERIC);
			GEprintf1 ("Error: TGA_FadePaintBoxN(): NumColors > (MaxColors=%d)\n",
				D.Pal.MaxColors);
			return;
		}
	}
#endif
/*-------------------------------------------------------------------------*/
{
	PaletteType			*palette = NULL;
	ColorCycleSetType	*cycleset = NULL;
	short tmp;
	short do_cyc;

	if (paintbox) {
		palette = paintbox->Palette;
		cycleset = paintbox->CycleSet;
	}

	/* Do nothing if offset is out of palette range */
	if (dst_offset >= D.Pal.MaxColors) return;

	/* Reduce numcolors if it is out of palette range */
	if ((tmp = (numcolors + dst_offset)) > D.Pal.MaxColors) {
		numcolors -= (tmp - D.Pal.MaxColors);
	}

	D.Pal.Request.Palette = palette;
	D.Pal.Request.SrcOffset = src_offset;
	D.Pal.Request.DstOffset = dst_offset;
	D.Pal.Request.NumColors = numcolors;
	D.Pal.Request.Hz60FadeFrames = D.Cyc.Request.Hz60FadeFrames = hz60frames;

	D.Cyc.Request.CycleSet = cycleset;
	
	do_cyc = (cycleset || D.Cyc.UserSet);
	D.Pal.Request.Fade = (palette || D.Pal.UserPalette);
	D.Cyc.Request.Fade = do_cyc;

	/* Wait til fade has started before exiting */
	while (D.Pal.Request.Fade || D.Cyc.Request.Fade) {}

}
} /* TGA_FadeToPaintBoxN */

