/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * TSUP.c
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 03/05/90
 *   MODIFIED : 10/08/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		C routine support for TGA mode
 *
 * HISTORY
 *		10/08/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <alloc.h>
#include <mem.h>
#include <dos.h>
#include <echidna/alloc.h>
#include <echidna/eerrors.h>
#include <echidna/graphics.h>
#include <echidna/pixlscan.h>
#include <echidna/memory.h>

/***************************** E X T E R N A L ****************************/
extern far TGA_ASMOpenGraphics (void);
extern far TGA_ASMCloseGraphics (void);
extern far TGA_DrawRectAsm (short x, short y, short width, short height);
extern GfxRoutinePtr TGARoutineTable[];
extern GfxDataTbl	 TGADataTable;
extern UWORD TDMemTbl[];

/**************************** C O N S T A N T S ***************************/
#define DISPB	0
#define WORKB	1
#define BACKB	2

/************************************  ************************************/

/*********************************************************************
 *
 * TGA_GetGShapeLoadBuf
 *
 * SYNOPSIS
 *		void far *TGA_GetGShapeLoadBuf (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void far *TGA_GetGShapeLoadBuf (void)
{
	if (TDMemTbl[BACKB]) {
		return MK_FP(TDMemTbl[BACKB], 0x0000);
	} else {
		return MK_FP(TDMemTbl[DISPB], 0x0000);
	}

} /* TGA_GetGShapeLoadBuf */

/*********************************************************************
 *
 * TGA_DrawRect
 *
 * SYNOPSIS
 *		TGA_DrawRect (short x, short y, short width, short height)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DrawRect (short x, short y, short inwidth, short inheight)
{
	short width  = inwidth;
	short height = inheight;

	if (x < 0) {
		width = inwidth - (0 - x);
		x     = 0;
	} else if (x > 0 + 320 - inwidth) {
		width = (0 + 320) - x;
	}

	if (y < 0) {
		height = inheight - (0 - y);
		y      = 0;
	} else if (y > 0 + 200 - inheight) {
		height = (0 + 200) - y;
	}

	if (width > 0 && height > 0) {
		TGA_DrawRectAsm (x, y, width, height);
	}
} /* TGA_DrawRect */

/*********************************************************************
 *
 * GetTGARoutineTable
 *
 * SYNOPSIS
 *		GfxRoutinePtr *GetTGARoutineTable (void)
 *
 * PURPOSE
 *		Return pointer to TGARoutineTable.  Calling this routine
 *		should force loading of the TGA routine overlay.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void GetTGARoutineTable (
	GfxRoutinePtr	**grptr,
	GfxDataTbl		**gdptr
) {
	*grptr = TGARoutineTable;
	*gdptr = &TGADataTable;
} /* GetTGARoutineTable */


/*********************************************************************
 *
 * TGA_FreeShape
 *
 * SYNOPSIS
 *		void	TGA_FreeShape (
 *			GShape	*shapeptr
 *		)
 *
 * PURPOSE
 *		To Free the memory allocated for shape by To_MCG_Shape.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_FreeShape(GShape *shape)
{
	UWORD	 s;
	UBYTE	 pcc = shape->Data.TGA.pcc;

	for (s = 0; s < 2; s++) {
		if (s && !(pcc & PCC_FOUR_COPIES)) {
			continue;
		}
		if (shape->Data.TGA.ts[s]) {
			MEM_free (shape->Data.TGA.ts[s]);
		}
	}
	MEM_free (shape);
} /* TGA_FreeShape */

static UBYTE cmasks[] = { 0xF0, 0x0F, };
static UBYTE	*ps_ColorMap;
static UWORD	 ps_Size;
static short	 ps_DWidth;
static UBYTE	*ps_EPatternMap;
static UBYTE	*ps_OPatternMap;
static UBYTE	*ps_PatMap;
static short	 ps_DeltaCol;
static short	 ps_DeltaRow;
static UBYTE	*ps_PixlPtr;
static short	 ps_ColCount;
static short	 ps_TrueWidth;
static short	 ps_TrueWidthHold;
static UBYTE	 ps_tskip;
static UBYTE	 ps_tskiphold;
static UBYTE	 ps_tpad;
static UBYTE	 ps_tpadhold;
static UBYTE	 ps_odd;
static UBYTE	 pixel;
static UBYTE	 cdata;
static UBYTE	 cmask;
static UBYTE 	 tcolor;
static UBYTE	 count;
static UBYTE	 color;
static UBYTE	*ps_EOPatMap = NULL;
static UBYTE	*ps_OOPatMap = NULL;
static UBYTE	*ps_EWorkPat;
static UBYTE	*ps_OWorkPat;

static void getpixl (void)
{
	/* Get two pixels and their mask */
	cdata = 0;

	if (ps_Size) {
		cmask = 0;
		for (pixel = 0; pixel < 2; pixel++) {
			if (!ps_TrueWidth) {
				ps_TrueWidth = ps_TrueWidthHold;
				ps_tskip     = ps_tskiphold;
				ps_tpad      = ps_tpadhold;
				ps_ColCount  = 0;
				ps_PixlPtr  += ps_DeltaRow;
				ps_odd = !ps_odd;
				if (ps_odd) {
					ps_PatMap = ps_OWorkPat;
				} else {
					ps_PatMap = ps_EWorkPat;
				}
			}
			ps_TrueWidth--;
			if (ps_tskip) {
				ps_tskip--;
				color = tcolor;
			} else if (ps_ColCount < ps_DWidth) {
				ps_Size--;
				ps_ColCount++;
				color = ps_ColorMap[*ps_PixlPtr];
				ps_PixlPtr += ps_DeltaCol;
			} else if (ps_tpad) {
				ps_tpad--;
				color = tcolor;
			}

			cmask |= ((color == tcolor) ? cmasks[pixel] : 0);
			cdata |= ps_PatMap[color] & ((color == tcolor) ? 0 : cmasks[pixel]);
		}
	} else {
		cmask = 0xFF;
	}
}

/*********************************************************************
 *
 * TGA_ConvertShape
 *
 * SYNOPSIS
 *		GShape	*TGA_ConvertShape (
 *			GShape			*gshape, 
 *			short				 Hflip,
 *			short				 Vflip,
 *			UBYTE			 pcc,
 *			GShape			*outshape
 *		)
 *
 * PURPOSE
 *		To Create a shape usable by the TGA shape routines.
 *
 * INPUT
 *		gshape:		Raw unpacked MCGA shape.
 *		attrib:		Value from BrushPak that tells me if I need to flip
 *					and/or remap the shape.
 *
 * EFFECTS
 *		Makes Converted copy of 'gshape' in 'outshape' and then allocates 
 *		space for and copies 'outshape'
 *
 * RETURN VALUE
 *		Returns pointer to allocated TGA shape if succesful.
 *		Returns NULL if unable to allocate space.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
GShape	*TGA_ConvertShape (
	ConvertSType	*cs
)
{
	GShape		*gshape;
	short		 Hflip;
	short		 Vflip;
	short		 newxoff;
	UBYTE		 pcc;
	GShape		*outshape;
	TGAGShape	*TGAshape		= NULL;
	UBYTE 		*outbuf;
	UBYTE 		*countbyteptr;
	UWORD 		 origsize;
	UWORD 		 packedsize;
	short		 ps_2xDWidth;
	UWORD		 s;

	if (!cs) {
		if (ps_EOPatMap)	DestroyTempBuffer (ps_EOPatMap, 512);
		ps_EOPatMap = NULL;
		return NULL;
	}

	if (!ps_EOPatMap) {
		ps_EOPatMap = CreateTempBuffer (512);
		if (!ps_EOPatMap) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("TGA_ConvertShape:OOM allocating odd pattern maps");
			return NULL;
		}
		ps_OOPatMap = ps_EOPatMap + 256;
	}

	gshape 			= cs->ShapeData; 
	ps_ColorMap		= cs->ColorMap;
	ps_EPatternMap	= cs->PatternMap->Even;
	ps_OPatternMap	= cs->PatternMap->Odd;
	Hflip			= cs->HFlip;
	Vflip			= cs->VFlip;
	pcc				= cs->pcc;

	/* if new pattern create odd pattern set */
	if (cs->ExpandPatterns) {
		UBYTE	*oop;
		UBYTE	*oep;
		UBYTE	*nop;
		UBYTE	*nep;

		oep = ps_EPatternMap;
		oop = ps_OPatternMap;
		nep = ps_EOPatMap;
		nop = ps_OOPatMap;
		
		for (s = 0; s < 256; s++) {
			*nep++ = (*oep << 4) | ((*oep >> 4) & 0x0F);
			*nop++ = (*oop << 4) | ((*oop >> 4) & 0x0F);
			oep++;
			oop++;
		}
	}

	/* Allocate Space for Worst Case Packing */
	if ( (outshape = (GShape *)MEM_calloc (1, GSHAPESIZE + TGASDATASIZE)) == NULL)
	{
		SetGlobalErr (ERR_OUT_OF_MEMORY);
		GEprintf ("OOM: in TGA_ConvertShape");
		return NULL;
	}

	/* Copy Shape Header info to outshape */
	memcpy (outshape, gshape, GSHAPESIZE);
	tcolor = gshape->TColor;
	outshape->Data.TGA.pcc = pcc;
#if 0
	outshape->TColor = ps_EPatternMap[tcolor] + (ps_OPatternMap[tcolor] << 8);
#else
	outshape->Data.TGA.TPatEven = ps_EPatternMap[tcolor];
	outshape->Data.TGA.TPatOdd = ps_OPatternMap[tcolor];
#endif

	/* make requested number of shapes */
	for (s = 0; s < 2; s++) {

		if (s && !(pcc & PCC_FOUR_COPIES)) {
			outshape->Data.TGA.ts[1] = TGAshape;
			continue;
		}

		if (s) {
			ps_EWorkPat = ps_EOPatMap;
			ps_OWorkPat = ps_OOPatMap;
		} else {
			ps_EWorkPat = ps_EPatternMap;
			ps_OWorkPat = ps_OPatternMap;
		}

		/* Pack Shape */
		ResetPixlScan (gshape, Hflip, Vflip);

		if (Hflip) {
			newxoff = gshape->Width - ps_DWidth - gshape->Data.MCGA.XOffset + s;
		} else {
			newxoff = gshape->Data.MCGA.XOffset + s;
		}

		origsize = (((gshape->Data.MCGA.DWidth + newxoff - 1) >> 1) -
					(newxoff >> 1) + 1) *
					gshape->Data.MCGA.DHeight * 3;
		origsize = max (origsize, 6) + TGAGSHAPESIZE + 5;
		TGAshape = MEM_malloc (origsize);
		if (!TGAshape) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("OOM: in TGA_ConvertShape (2)");
			goto TGAcleanup;
		}
		outbuf = (UBYTE *)TGAshape + TGAGSHAPESIZE;
		packedsize = 0;

		TGAshape->XOffset = newxoff & 0xFFFE;

		if (Vflip) {
			TGAshape->YOffset = gshape->Height - 
				gshape->Data.MCGA.DHeight - gshape->Data.MCGA.YOffset;
		} else {
			TGAshape->YOffset = gshape->Data.MCGA.YOffset;
		}
		TGAshape->DWidth  = ((ps_DWidth + (newxoff & 0x01)) + 1) >> 1;
		TGAshape->DHeight = gshape->Data.MCGA.DHeight;

		ps_tskiphold     = newxoff & 0x01;
		ps_tpadhold      = 1 - ((newxoff + ps_DWidth - 1) & 0x01);
		ps_TrueWidthHold = ps_tskiphold + ps_DWidth + ps_tpadhold;

		ps_TrueWidth	 = ps_TrueWidthHold;
		ps_tskip     	 = ps_tskiphold;
		ps_tpad      	 = ps_tpadhold;

		ps_odd = TGAshape->YOffset & 0x01;
		if (ps_odd) {
			ps_PatMap = ps_OWorkPat;
		} else {
			ps_PatMap = ps_EWorkPat;
		}

		if (ps_Size) {
			getpixl ();
			do {

				/* remember address for transparent count */
				countbyteptr = outbuf++;

				/* save starting trans mask/data */
				*outbuf++   = cmask;
				*outbuf++   = cdata;
				packedsize += 3;

				/* Count Run of Transparent Bytes including start/end */
				getpixl ();
				count = 2;
				while (++count != 0 && cmask == 0xFF && ps_Size) {
					getpixl();
				}
				*countbyteptr = (UWORD)(--count);

				/* write out next trailing trans mask/data */
				*outbuf++   = cmask;
				*outbuf++   = cdata;
				packedsize += 2;

				/* remember address for data count */
				countbyteptr = outbuf++;

				/* Count Run of Non-Transparent Bytes */
				getpixl ();
				count = 0;
				while (++count != 0 && !cmask && ps_Size) {
					*outbuf++ = cdata;
					getpixl ();
				}
				*countbyteptr = --count;
				packedsize   += (UWORD)count + 1;
			} while (ps_Size || ps_tpad);

			/* store last bits of data */
			*outbuf++   = 2;
			*outbuf++   = cmask;
			*outbuf++   = cdata;
			getpixl ();
			*outbuf++   = cmask;
			*outbuf++   = cdata;
			packedsize += 5;
		}

		/* ReAlloc result to proper size. */
		outshape->Data.TGA.ts[s] = TGAshape;
		freesome (origsize - (packedsize + TGAGSHAPESIZE));
	}

	return outshape;

TGAcleanup:
	TGA_FreeShape (outshape);
	return NULL;
} /* TGA_ConvertShape */

/*********************************************************************
 *
 * TGA_OpenGraphics
 *
 * SYNOPSIS
 *		short TGA_OpenGraphics(void)
 *
 * PURPOSE
 *		Init TGA graphics routines.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

short TGA_OpenGraphics(void)
{
	TGA_ASMOpenGraphics ();

	return 1;

} /* TGA_OpenGraphics */

/*********************************************************************
 *
 * TGA_CloseGraphics
 *
 * SYNOPSIS
 *		void TGA_CloseGraphics (void)
 *
 * PURPOSE
 *		Free resources used by TGA graphics routines.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_CloseGraphics (void)
{
	TGA_ASMCloseGraphics ();

} /* TGA_CloseGraphics */

/*********************************************************************
 *
 * TGA_DrawShapeNoTrans
 *
 * SYNOPSIS
 *		TGA_DrawShapeNoTrans (GShape *shape, short x, short y)
 *
 * PURPOSE
 *		DrawShape without transparency.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void TGA_DrawShapeNoTrans (GShape *shape, short x, short y)
{
	SetPenColor (255);
#if 0
	EvenPatternTbl[255] = shape->TColor;
	OddPatternTbl[255]  = shape->TColor >> 8;
#else
	EvenPatternTbl[255] = shape->Data.TGA.TPatEven;
	OddPatternTbl[255]  = shape->Data.TGA.TPatOdd;
#endif
	DrawRect (x, y, shape->Width, shape->Height);
	DrawGShape (shape, x, y);

} /* TGA_DrawShapeNoTrans */

