/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * XTRAMEM.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 11/04/91
 *   MODIFIED : 10/10/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		
 *
 * HISTORY
 *		08/29/92 (GAT) - Changed AllocXTRAofXMT() to look at bits
 *						 instead of types.  In otherwords you can
 *						 pass (xmtEMS|xmsXMS) to alloc from one or
 *						 the other but not xmsSTD.
 *		11/18/92 (GAT) - changed AvailXTRA() to return -1 if not used.
 *		12/06/92 (GAT) - Added Zortech 
 *		05/18/93 (dcc) - Add ReleaseXTRA().
 *		06/29/93 (dcc) - Added Watcom C/C++32 support
 *		09/30/93 (GAT) - Added Symantec C/C++32 for MPW support
 *		11/17/93 (GAT) - Added MinMemFree support
 *		10/06/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include <echidna/alloc.h>
#include <echidna/listfunc.h>
#include <echidna/memory.h>
#include <echidna/ems.h>
#include <echidna/xms.h>
#include <echidna/xtramem.h>

#if __TURBOC__
#include <mem.h>
#endif

#if __WATCOMC__
#include <malloc.h>
#endif

/**************************** C O N S T A N T S ***************************/

#define FREEBLOCKSIZE 2048

/******************************** T Y P E S *******************************/

/****************************** G L O B A L S *****************************/

#if (__MSDOS32X__ || __AMIGAOS__ || __MACOS__ || __EGGS_SGI__)

void	*__LastXTRA;

#else

/*	szLargestXTRA - the largest memory block that the calling program
	claims it will ever allocate. This value is initialized to 65496,
	which is currently the largest XMS block you may allocate. */

static UWORD	 szLargestXTRA = 65496U;

static XTRAPntr	 LastXTRA;
static short	 LastWindow;

/******************************* M A C R O S ******************************/

#define XPNT(xp)	((MEMPntr *)(xp))

/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * SetXTRAMemoryPoolSize
 *
 * PURPOSE
 *		Call this routine to set the maximum amount of XTRA memory
 *		(in Kbytes) that may be allocated via AllocXTRA().
 *
 *		It is NOT necessary to call this routine. It is probably
 *		most useful to assist in debugging low XTRA memory conditions.
 *
 *		If you call this routine with 0, it will use all available
 *		XTRA memory.
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		That EMS and XMS shares the same pool of memory.
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		03/07/94 Monday (dcc) - created.
 *
*/
void SetXTRAMemoryPoolSize(UWORD cXTRAKBytes)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetXTRAMemoryPoolSize";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	SetEMSMemoryPoolSize(cXTRAKBytes);
	SetXMSMemoryPoolSize(cXTRAKBytes);

} /* SetXTRAMemoryPoolSize */


/*********************************************************************
 *
 * LastActivateXTRA
 *
 * SYNOPSIS
 *		XTRAPntr LastActivateXTRA (void)
 *
 * PURPOSE
 *		Returns the last XTRA pointer passed to ActivateXTRA or
 *		ActivateXTRAatWin so that a routine may save the state
 *		of XTRA memory.  NOTE: UpdateXTRA is not called. See example
 *		below.
 *	
 *		NOTE: ActivateXTRA must have already been called once for this
 *		routine to return a valid value.
 *	
 *		NOTE: It is up to you to assure that calling ActivateXTRA to
 *		activate some other XTRA mem does not lose any changes to
 *		previous XTRA memory since UpdateXTRA is not called.
 *
 *		BUGBUG: This function does NOT work correctly with
 *		ActivateXTRAatWin! It is designed to work with ActivateXTRA only!
 *
 * INPUT
 *
 *
 * EFFECTS
 *	
 *	
 * EXAMPLE
 *		#include <echidna/xtramem.h>
 *
 *		void foo (void)
 *		{
 *			XTRAPntr	 xtra;
 *			XTRAPntr	 old;
 *			char		*data;
 *	
 *			old  = LastActivateXTRA;		// get and save current XTRA pntr
 *	
 *			xtra = AllocXTRA (100);			// allocate some XTRA mem
 *			data = ActivateXTRA (xtra);		// make it visible.
 *	
 *			strcpy (data, "Now is the...");	// use the memory
 *			UpdateXTRA (xtra);				// save it.
 *	
 *			ActivateXTRA (old);				// restore old visible XTRA mem
 *		}
 *	
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		ActivateXTRA, ActivateXTRAatWin
 *	
*/
XTRAPntr LastActivateXTRA (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "LastActivateXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	return LastXTRA;

} /* LastActivateXTRA */


/*********************************************************************
 *
 * SetLargestXTRA
 *
 * SYNOPSIS
 *		void SetLargestXTRA (size_t size)
 *
 * PURPOSE
 *		Defines the largest contiguous block the program will ever
 *		allocate. Typically, you will call this routine with a multiple
 *		of 16384. The intent is that if you specify the maximum to
 *		be less than 64K, the EMS routines may be able to save memory
 *		in allocating the pointers to keep track of an EMS memory block.
 *		If this routine is never called, the value defaults to the
 *		pre-initialized value szLargestXTRA.
 *
 * INPUT
 *		size - the largest block your program will ever allocate.
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetLargestXTRA (size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetLargestXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	szLargestXTRA = size;

	SetLargestEMS(size);

} /* SetLargestXTRA */


/*********************************************************************
 *
 * ActivateXTRAatWin
 *
 * SYNOPSIS
 *		void far32 *ActivateXTRAatWin (XTRAPntr *xp, short window)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void far32 *ActivateXTRAatWin (XTRAPntr xp, short window)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ActivateXTRAatWin";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		void far32	*mem;

		LastXTRA   = xp;
		LastWindow = window;

		switch (XPNT(xp)->xmt) {
		case xmtEMS:
			mem = ActivateEMSatWin ((EMSPntr *) xp, window);
			break;
		case xmtXMS:
			mem = ActivateXMSatWin ((XMSPntr *) xp, window);
			break;
		case xmtSTD:
			mem = XPNT(xp)->Memory;
			break;
		#if	REQUIRE
		default:
			puts ("ActivateXTRAatWin(): bad memory type\n");
			exit (1);
			break;
		#endif
		}
		return mem;
	}

} /* ActivateXTRAatWin */

/*********************************************************************
 *
 * UpdateXTRA
 *
 * SYNOPSIS
 *		void UpdateXTRA (XTRAPntr *xp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void UpdateXTRA (XTRAPntr xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UpdateXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		switch (XPNT(xp)->xmt) {
		case xmtXMS:
			UpdateXMS ((XMSPntr *) xp);
			break;
		case xmtEMS:
		case xmtSTD:
			break;
		#if	REQUIRE
		default:
			puts ("UpdateXTRA(): bad memory type\n");
			exit (1);
			break;
		#endif
		}
	}
} /* UpdateXTRA */


/*********************************************************************
 *
 * ReleaseXTRA
 *
 * SYNOPSIS
 *		void ReleaseXTRA (XTRAPntr *xp)
 *
 * PURPOSE
 *		See xms#ReleaseXMS() for explanation.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ReleaseXTRA (XTRAPntr xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReleaseXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		switch (XPNT(xp)->xmt) {
		case xmtXMS:
			ReleaseXMS ((XMSPntr *) xp);
			break;
		case xmtEMS:
		case xmtSTD:
			break;
		#if	REQUIRE
		default:
			puts ("ReleaseXTRA(): bad memory type\n");
			exit (1);
			break;
		#endif
		}
	}
} /* ReleaseXTRA */


/*********************************************************************
 *
 * FreeXTRA
 *
 * SYNOPSIS
 *		void FreeXTRA (XTRAPntr *xp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FreeXTRA (XTRAPntr xp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreeXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		switch (XPNT(xp)->xmt) {
		case xmtEMS:
			FreeEMS ((EMSPntr *) xp);
			break;
		case xmtXMS:
			FreeXMS ((XMSPntr *) xp);
			break;
		case xmtSTD:
			FreeMemOnPara (XPNT(xp)->Memory);
			MEM_free (xp);
			break;
		#if	REQUIRE
		default:
			puts ("FreeXTRA(): bad memory type\n");
			exit (1);
			break;
		#endif
		}
	}

} /* FreeXTRA */

/*********************************************************************
 *
 * AllocXTRA
 *
 * SYNOPSIS
 *		XTRAPntr AllocXTRA (size_t size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
XTRAPntr AllocXTRA (size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AllocXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		void far32	*mem;

		if (size > szLargestXTRA)		/* can't allocate something too big! */
			return NULL;

		mem = AllocEMS (size);
		if (mem) {
			XPNT(mem)->xmt = xmtEMS;
		} else {
			mem = AllocXMS (size);
			if (mem) {
				XPNT(mem)->xmt = xmtXMS;
			} else {
				mem = MEM_calloc (1, sizeof(MEMPntr));
				if (mem) {
					XPNT(mem)->Memory = AllocMemOnPara (size);
					if (XPNT(mem)->Memory) {
						XPNT(mem)->xmt = xmtSTD;
					} else {
						MEM_free (mem);
						return NULL;
					}
				} else {
					return NULL;
				}
			}
		}

		return (XTRAPntr)mem;
	}

} /* AllocXTRA */


/*********************************************************************
 *
 * AllocXTRAofXMT
 *
 * SYNOPSIS
 *		XTRAPntr AllocXTRAofXMT (size_t size, XMT xmt)
 *
 * PURPOSE
 *		Force allocate XTRA memory of type <xmt>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Pointer to allocated XTRA memory of type <xmt>,
 *		or NULL if unsuccessful.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
XTRAPntr AllocXTRAofXMT (size_t size, XMT xmt)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AllocXTRAofType";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		void far32	*mem = NULL;

		if (size > szLargestXTRA)		/* can't allocate something too big! */
			return NULL;

		if (xmt & xmtEMS)
		{
			mem = AllocEMS (size);
			if (mem)
				xmt = xmtEMS;
		}
		if (!mem && (xmt & xmtXMS))
		{
			mem = AllocXMS (size);
			if (mem)
				xmt = xmtXMS;
		}
		if (!mem && (xmt & xmtSTD))
		{
			mem = MEM_calloc (1, sizeof(MEMPntr));
			if (mem) {
				xmt = xmtSTD;
				XPNT(mem)->Memory = AllocMemOnPara (size);
				if (!XPNT(mem)->Memory) {
					MEM_free (mem);
					return NULL;
				}
			}
		}
		#if	REQUIRE
		if (xmt & ~(xmtSTD|xmtEMS|xmtXMS))
		{
			puts ("AllocXTRAofXMT(): bad memory type\n");
			exit (1);
		}
		#endif

		if (mem)
			XPNT(mem)->xmt = xmt;

		return (XTRAPntr)mem;
	}

} /* AllocXTRAofType */


/*********************************************************************
 *
 * ReadXTRA
 *
 * SYNOPSIS
 *		void ReadXTRA (XTRAPntr *xp, UWORD uwOffset, UWORD uwLength,
 *					void far32 *pMem)
 *
 * PURPOSE
 *		Copies <uwLength> bytes from <uwOffset> from the start of
 *		XTRA memory chunk <xp> to <pMem>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ReadXTRA (XTRAPntr xp, UWORD uwOffset, UWORD uwLength, void far32 *pMem)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		switch (XPNT(xp)->xmt) {
		case xmtEMS:
			ReadEMS((EMSPntr *) xp, uwOffset, uwLength, pMem);
			break;
		case xmtXMS:
			ReadXMS((XMSPntr *) xp, uwOffset, uwLength, pMem);
			break;
		case xmtSTD:
			memcpy(pMem, (UBYTE *) XPNT(xp)->Memory + uwOffset, uwLength);
			break;
		#if	REQUIRE
		default:
			puts ("ReadXTRA(): bad memory type\n");
			exit (1);
			break;
		#endif
		}
	}

} /* ReadXTRA */

/*********************************************************************
 *
 * WriteXTRA
 *
 * SYNOPSIS
 *		void WriteXTRA (void far32 *pMem, UWORD uwLength,
 *						XTRAPntr *xp, UWORD uwOffset)
 *
 * PURPOSE
 *		Copies <uwLength> bytes from <pMem> to
 *		<uwOffset> from the start of XTRA memory chunk <xp>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void WriteXTRA (void far32 *pMem, UWORD uwLength, XTRAPntr xp, UWORD uwOffset)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		switch (XPNT(xp)->xmt) {
		case xmtEMS:
			WriteEMS(pMem, uwLength, (EMSPntr *) xp, uwOffset);
			break;
		case xmtXMS:
			WriteXMS(pMem, uwLength, (XMSPntr *) xp, uwOffset);
			break;
		case xmtSTD:
			memcpy((UBYTE *) XPNT(xp)->Memory + uwOffset, pMem, uwLength);
			break;
		#if	REQUIRE
		default:
			puts ("WriteXTRA(): bad memory type\n");
			exit (1);
			break;
		#endif
		}
	}

} /* WriteXTRA */


/*********************************************************************
 *
 * AvailSTD
 *
 * SYNOPSIS
 *		long AvailSTD(void)
 *
 * PURPOSE
 *		Return number of FREEBLOCKSIZE blocks we are able to
 *		allocate * FREEBLOCKSIZE / 1024 (free memory in K's).
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Memory free in K bytes.
 *
 * HISTORY
 *		10/10/94 (dcc) - add calls to MEM_GetShowOOM() & MEM_SetShowOOM().
 *
 * SEE ALSO
 *
*/
static long AvailSTD(void)
{

	ListType	 FreeListX;
	ListType	*FreeList = &FreeListX;
	UWORD		 i, j = 0;
	UBYTE		*pMem;
	NodeType	*pNod;
#if DEBUG
	BOOL		 f;
#endif

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AvailSTD";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	InitList(FreeList);

#if DEBUG
	f = MEM_GetShowOOM();				/* Save show OOM state. */
	MEM_SetShowOOM(FALSE);
#endif

	for (i = 0; i < 65535U; i++)
	{

		if ((pMem = MEM_malloc(FREEBLOCKSIZE)) == NULL)
		{
			break;
		}
		((NodeType *)(pMem))->Type = 0;	/* Fix so AddTail does generate error */
		AddTail(FreeList, pMem);
	}
#if DEBUG
	MEM_SetShowOOM(f);					/* Restore show OOM state. */
#endif

	while ((pNod = (NodeType *) RemHead(FreeList)) != NULL)
	{
		MEM_free(pNod);
		j++;
	}

	return (long)(i * (FREEBLOCKSIZE / 1024));
} /* AvailSTD */


/*********************************************************************
 *
 * AvailXTRA
 *
 * SYNOPSIS
 *		long AvailXTRA(XMT xmt)
 *
 * PURPOSE
 *		Return available memory in K bytes of type <xmt>.
 *		<xmt> may be the sum of more than one type of memory,
 *		(e.g., xmtEMS+xmtSTD), in which case this routine
 *		will return the sum of all specified types of memory.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		-1 if type of memory(s) not used yet.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
long AvailXTRA(XMT xmt)
{

	long ulAvail = 0;
	long ulfree  = 0;
	BOOL flag    = FALSE;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AvailXTRA";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (xmt & xmtEMS)
	{
		ulfree = AvailEMS ();
		if (ulfree >= 0)
		{
			flag     = TRUE;
			ulAvail += ulfree;
		}
	}

	if (xmt & xmtXMS)
	{
		ulfree = AvailXMS ();
		if (ulfree >= 0)
		{
			flag     = TRUE;
			ulAvail += ulfree;
		}
	}

	if (xmt & xmtSTD)
	{
		ulfree = AvailSTD ();
		if (ulfree >= 0)
		{
			flag     = TRUE;
			ulAvail += ulfree;
		}
	}

	return (flag ? ulAvail : (-1L));

} /* AvailXTRA */
#endif	/*!__MSDOS32X__*/

#if __MSDOS__
	#if __MSDOS32X__
		static long	MinMem		= LONG_MAX;	/* Used to track min free mem */
	#else
		static long	MinMem		= LONG_MAX;	/* Used to track min free mem */
		static long	MinMemEMS	= (-1L);
		static long	MinMemXMS	= (-1L);
	#endif
#endif

/*********************************************************************
 *
 * MinMemFree
 *
 * SYNOPSIS
 *		void MinMemFree (void)
 *
 * PURPOSE
 *		Tracks the minimum amount of memory free for reporting to the
 *		user.
 *
 * INPUT
 *		NONE.
 *
 * EFFECTS
 *		MinMem is update if less memory is available then before.
 *
 * RETURN VALUE
 *		NONE.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void MinMemFree (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "MinMemFree";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	#if __MSDOS__
		#if __MSDOS32X__
			#if __TURBOC__
				MinMem    = min (coreleft(), MinMem);
			#else
				MinMem    = min (_memavl(), MinMem);
			#endif
		#else
			MinMem    = min (coreleft(), MinMem);
			if (MinMemEMS < 0)
			{
				MinMemEMS = AvailXTRA(xmtEMS);
			}
			else
			{
				MinMemEMS = min(AvailXTRA(xmtEMS), MinMemEMS);
			}
			if (MinMemXMS < 0)
			{
				MinMemXMS = AvailXTRA(xmtXMS);
			}
			else
			{
				MinMemXMS = min(AvailXTRA(xmtXMS), MinMemXMS);
			}
		#endif
	#else
	#endif
} /* MinMemFree */

/*********************************************************************
 *
 * PrintMinMemFree
 *
 * SYNOPSIS
 *		void  PrintMinMemFree (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintMinMemFree (void)
{
	#if __MSDOS__
		#if __MSDOS32X__
  			printf ("\nMem Free: %8ld bytes\n", MinMem);
		#else
			{
				char	buf[80];

				printf ("\nConventional Mem Free: %8ld bytes\n", MinMem);
				if (MinMemEMS >= 0) {
					sprintf (buf, "%8ld bytes", MinMemEMS * 1024L);
				} else {
					sprintf (buf, "not used");
				}
				printf ("         EMS Mem free: %s\n", buf);
				if (MinMemXMS >= 0) {
					sprintf (buf, "%8ld bytes", MinMemXMS * 1024L);
				} else {
					sprintf (buf, "not used");
				}
				printf ("         XMS Mem free: %s\n", buf);
			}
		#endif
	#endif
}

