/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * DATAFILE.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 07/25/90
 *   MODIFIED : 09/21/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Routines to create various format file.
 *		*	Intel-Hex format files.
 *		*	DB style statements
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <stdio.h>
#include <echidna/eerrors.h>
#include <echidna/eio.h>
#include <echidna/listfunc.h>
#include <echidna/emacros.h>
#include <echidna/datafile.h>

/**************************** C O N S T A N T S ***************************/

#define OFF_NUM_BYTES	0
#define OFF_ADDRESS		1
#define OFF_END_FLAG	3
#define OFF_DATA		4

#define HEADER_SIZE		(1 + 2 + 1)
#define MAX_BYTES		(0x18)

/******************************** T Y P E S *******************************/

typedef struct {
	NodeType	 Node;
	ULONG		 LenPos;
	ULONG		 Length;
} CPEPosType;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*------------------------------------------------------------------------*/
/**# MODULE:DATATEFILE_IntelHex                                           */
/*------------------------------------------------------------------------*/

static UBYTE  HexBuffer[HEADER_SIZE + MAX_BYTES + 2];
static UBYTE *HexOutPtr;
static UWORD  ByteCount;
static UWORD  DstAddress;
static UBYTE  StartFlag = FALSE;
static char   HexChrs[] = "0123456789ABCDEF";

/*********************************************************************
 *
 * WriteHexLine
 *
 * SYNOPSIS
 *		static int WriteHexLine (FILE *fp)
 *
 * PURPOSE
 *		Write Hex Line to File.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static int WriteHexLine (FILE *fp)
{
	int		 len;
	UBYTE	*data;
	UBYTE	 checksum;
	char	 line[80];
	char	*out;
	int		 i;
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteHexLine";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	HexBuffer[OFF_NUM_BYTES] = (UBYTE) ByteCount;
	HexBuffer[OFF_ADDRESS]   = (UBYTE) (DstAddress >> 8);
	HexBuffer[OFF_ADDRESS+1] = (UBYTE) (DstAddress & 0xFF);
	HexBuffer[OFF_END_FLAG]  = 0;

	len  = HEADER_SIZE + ByteCount;
	data = HexBuffer;
	checksum = 0;
	for (i = 0; i < len; i++) {
		checksum += *data++;
	}
	*data = (UBYTE) (256 - checksum);
	len++;

	out  = line;
	data = HexBuffer;
	*out++ = ':';
	for (i = 0; i < len; i++) {
		*out++ = HexChrs[*data >> 4];
		*out++ = HexChrs[*data++ & 0x0F];
	}
	*out++ = '\n';
	*out++ = '\0';
	HexOutPtr   = &HexBuffer[OFF_DATA];
	DstAddress += (UWORD) ByteCount;
	ByteCount   = 0;

	return (fputs (line, fp) != EOF);

} /* WriteHexLine */

/*********************************************************************
 *
 * StartIntelHexSection
 *
 * SYNOPSIS
 *		short StartIntelHexSection (FILE *fp, UWORD Addr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short StartIntelHexSection (FILE *fp, UWORD Addr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StartIntelHexSection";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	fp = fp;
	if (!StartFlag) {
		DstAddress = Addr;
		ByteCount  = 0;
		HexOutPtr  = &HexBuffer[OFF_DATA];
		StartFlag  = TRUE;
		return TRUE;
	} else {
		SetGlobalErr (ERR_GENERIC);
		GEprintf ("StartIntelHexSection: Started a new Section before Ending a previous section");
		return FALSE;
	}

} /* StartIntelHexSection */

/*********************************************************************
 *
 * WriteIntelHex
 *
 * SYNOPSIS
 *		short WriteIntelHex (FILE *fp, UBYTE *data, size_t size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short WriteIntelHex (FILE *fp, UBYTE *data, size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteIntelHex";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while (size) {
		*HexOutPtr++ = *data++;
		ByteCount++;
		if (ByteCount == MAX_BYTES) {
			if (!WriteHexLine (fp)) {
				return FALSE;
			}
		}
		size--;
	}

	return TRUE;

} /* WriteIntelHex */

/*********************************************************************
 *
 * EndIntelHexSection
 *
 * SYNOPSIS
 *		int EndIntelHexSection (FILE *fp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int EndIntelHexSection (FILE *fp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EndIntelHexSection";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	StartFlag = FALSE;
	return WriteHexLine (fp);

} /* EndIntelHexSection */

/*********************************************************************
 *
 * EndIntelHexFile
 *
 * SYNOPSIS
 *		int EndIntelHexFile (FILE *fp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int EndIntelHexFile (FILE *fp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EndIntelHexFile";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	return (fputs (":00000001FF\n", fp) != EOF);

} /* EndIntelHexFile */


/************************************  ************************************/
/************************************  ************************************/
/************************************  ************************************/

/*------------------------------------------------------------------------*/
/**# MODULE:DATATEFILE_DataStatements                                     */
/*------------------------------------------------------------------------*/

#define BUFFERSIZE	BufferSize
#define MAXBUFSIZE	80

static char		*Header;
static char		*Format;
static char		*Delim;
static UBYTE	 Buffer[MAXBUFSIZE];
static UBYTE	*b;
static int		 BufferSize;
static int		 Count;
static int		 Line;
static UWORD	 ByteCount;
static UWORD	 Flags;

/*********************************************************************
 *
 * WriteDataLine
 *
 * SYNOPSIS
 *		short WriteDataLine (FILE *fp)
 *
 * PURPOSE
 *		Write one line of data.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static short WriteDataLine (FILE *fp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteDataLine";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short	i;

		b = Buffer;
		if (!(Flags & DATAST_NOLINEINFO) && !(Line & 0x0F)) {
			fprintf (fp, ";L%-4d: ", Line);
			fprintf (fp, Format, ByteCount);
			fprintf (fp, "\n");
		}
		fprintf (fp, Header);
		for (i = 0; i < Count - 1 ; i++) {
			fprintf (fp, Format, *b++);
			fprintf (fp, Delim);
		}
		fprintf (fp, Format, *b++);
		if (Flags & DATAST_ASMLINENO) {
			fprintf (fp, "\t;%d\n", Line);
		} else {
			fprintf (fp, "\n");
		}
		b          = Buffer;
		ByteCount += (UWORD) Count;
		Count      = 0;
		Line++;
		return TRUE;
	}

} /* WriteDataLine */

/*********************************************************************
 *
 * StartDataStatements
 *
 * SYNOPSIS
 *		void StartDataStatements (
 *			char	*header,
 *			char	*format,
 *			char	*delimiter,
 *			short	 bytesperline,
 *			short	 flags)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void StartDataStatements (char *header, char *format, char *delimiter, short bytesperline, UWORD flags)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StartDataStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	Header     = header;
	Format     = format;
	Delim      = delimiter;
	Count      = 0;
	BufferSize = min(bytesperline,MAXBUFSIZE);
	b          = Buffer;
	Line       = 0;
	ByteCount  = 0;
	Flags      = flags;

} /* StartDataStatements */

/*********************************************************************
 *
 * WriteDataStatements
 *
 * SYNOPSIS
 *		short WriteDataStatements (FILE *fp, UBYTE *data, size_t size)
 *
 * PURPOSE
 *		Write Data Statements
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short WriteDataStatements (FILE *fp, UBYTE *data, size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteDataStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while (size) {
		*b++ = *data++;
		size--;
		Count++;
		if (Count == BUFFERSIZE) {
			if (!WriteDataLine (fp)) {
				return FALSE;
			}
		}
	}
	return TRUE;

} /* WriteDataStatements */

/*********************************************************************
 *
 * EndDataStatements
 *
 * SYNOPSIS
 *		short EndDataStatements (FILE *fp)
 *
 * PURPOSE
 *		Write Last Data Line if any.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short EndDataStatements (FILE *fp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EndDataStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (Count) {
		return WriteDataLine (fp);
	}
	return TRUE;

} /* EndDataStatements */

/*------------------------------------------------------------------------*/
/**# MODULE:DATATEFILE_StartDBHEXStatements                               */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * StartDBHEXStatements
 *
 * SYNOPSIS
 *		void StartDBHEXStatements (void)
 *
 * PURPOSE
 *		Setup WriteDataStatements for 'standard' DB statements with hex data
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void StartDBHEXStatements (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StartDBHEXStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	StartDataStatements ("\tDB ", "$%02x", ",", 16, NULL);

} /* StartDBHEXStatements */

/*------------------------------------------------------------------------*/
/**# MODULE:DATATEFILE_StartDBDECStatements                               */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * StartDBDECStatements
 *
 * SYNOPSIS
 *		void StartDBDECStatements (void)
 *
 * PURPOSE
 *		Setup WriteDataStatements for 'standard' DB statements with DEC data
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void StartDBDECStatements (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StartDBDECStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	StartDataStatements ("\tDB ", "$%3d", ",", 16, NULL);

} /* StartDBDECStatements */

/************************************  ************************************/
/************************************  ************************************/
/************************************  ************************************/

/*------------------------------------------------------------------------*/
/**# MODULE:DATATEFILE_WordStatements                                     */
/*------------------------------------------------------------------------*/

#define BUFFERSIZE	BufferSize
#define MAXBUFSIZE	80

static char		*Header;
static char		*Format;
static char		*Delim;
static UWORD	 wBuffer[MAXBUFSIZE];
static UWORD	*w;
static int		 BufferSize;
static int		 Count;
static int		 Line;
static UWORD	 WordCount;
static UWORD	 Flags;

/*********************************************************************
 *
 * WriteWordLine
 *
 * SYNOPSIS
 *		short WriteWordLine (FILE *fp)
 *
 * PURPOSE
 *		Write one line of data.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static short WriteWordLine (FILE *fp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteWordLine";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short	i;

		w = wBuffer;
		if (!(Flags & DATAST_NOLINEINFO) && !(Line & 0x0F)) {
			fprintf (fp, ";L%-4d: ", Line);
			fprintf (fp, Format, WordCount);
			fprintf (fp, "\n");
		}
		fprintf (fp, Header);
		for (i = 0; i < Count - 1 ; i++) {
			fprintf (fp, Format, *w++);
			fprintf (fp, Delim);
		}
		fprintf (fp, Format, *w++);
		if (Flags & DATAST_ASMLINENO) {
			fprintf (fp, "\t;%d\n", Line);
		} else {
			fprintf (fp, "\n");
		}
		w          = wBuffer;
		WordCount += (UWORD) Count;
		Count      = 0;
		Line++;
		return TRUE;
	}

} /* WriteWordLine */

/*********************************************************************
 *
 * StartWordStatements
 *
 * SYNOPSIS
 *		void StartWordStatements (
 *			char	*header,
 *			char	*format,
 *			char	*delimiter
 *			short	 wordsperline
 *			short	 flags)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void StartWordStatements (
	char	*header,
	char	*format,
	char	*delimiter,
	short	 bytesperline,
	UWORD	 flags
) {

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StartWordStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	Header     = header;
	Format     = format;
	Delim      = delimiter;
	Count      = 0;
	BufferSize = min (bytesperline, MAXBUFSIZE);
	w          = wBuffer;
	Line       = 0;
	WordCount  = 0;
	Flags      = flags;

} /* StartWordStatements */

/*********************************************************************
 *
 * WriteWordStatements
 *
 * SYNOPSIS
 *		short WriteWordStatements (FILE *fp, UWORD *data, size_t size)
 *
 * PURPOSE
 *		Write Data Statements
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short WriteWordStatements (FILE *fp, UWORD *data, size_t wordcount)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteWordStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while (wordcount) {
		*w++ = *data++;
		wordcount--;
		Count++;
		if (Count == BUFFERSIZE) {
			if (!WriteWordLine (fp)) {
				return FALSE;
			}
		}
	}
	return TRUE;

} /* WriteWordStatements */

/*********************************************************************
 *
 * EndWordStatements
 *
 * SYNOPSIS
 *		short EndWordStatements (FILE *fp)
 *
 * PURPOSE
 *		Write Last Data Line if any.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short EndWordStatements (FILE *fp)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EndWordStatements";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (Count) {
		return WriteWordLine (fp);
	}
	return TRUE;

} /* EndWordStatements */

/************************************  ************************************/
/************************************  ************************************/
/************************************  ************************************/

/*------------------------------------------------------------------------*/
/**# MODULE:DATATEFILE_CPEFiles                                           */
/*------------------------------------------------------------------------*/

static ULONG	 CPEBytesWritten;
static ULONG	 CPELenPos = 0;
static ListType	 CPEListX;
static ListType	*CPEList = &CPEListX;
static char		*CPEName;

/*********************************************************************
 *
 * OpenCPEFileWrite
 *
 * SYNOPSIS
 *		int OpenCPEFileWrite (char *filename)
 *
 * PURPOSE
 *		Open a file and write the CPE format header into it.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		file handle, (-1) if error
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int OpenCPEFileWrite (char *filename)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "OpenCPEFileWrite";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		static	UBYTE	Header[] = { 'C', 'P', 'E', 1 };

		int		fh;

		fh = EIO_WriteOpen (filename);
		if (fh == (-1)) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf1 ("\nOpenCPEFileWrite: Couldn't open '%s'", filename);
			return (-1);
		}

		EIO_Write (fh, Header, sizeof (Header));

		InitList (CPEList);
		CPELenPos = 0;
		CPEName   = filename;

		return fh;
	}

} /* OpenCPEFileWrite */

/*********************************************************************
 *
 * OpenCPEMemSection
 *
 * SYNOPSIS
 *		short OpenCPEMemSection (int fh, ULONG address)
 *
 * PURPOSE
 *		Write CPE memory section header to file and track bytes written.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short OpenCPEMemSection (int fh, ULONG address)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "OpenCPEMemSection";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		UBYTE	Data[9];

		if (CPELenPos != 0) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nOpenCPEMemSection:Section already open");
			return FALSE;
		}

		Data[0] = 1;
		Data[1] = (UBYTE)((address & 0x000000FFUL) >>  0);
		Data[2] = (UBYTE)((address & 0x0000FF00UL) >>  8);
		Data[3] = (UBYTE)((address & 0x00FF0000UL) >> 16);
		Data[4] = (UBYTE)((address & 0xFF000000UL) >> 24);
		
		CPEBytesWritten = 0;
		CPELenPos       = EIO_Seek (fh, 0, EIO_SEEK_CURRENT) + 5;

		if (EIO_Write (fh, Data, sizeof (Data)) != sizeof (Data)) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nOpenCPEMemSection:Error writing header");
			return FALSE;
		}
		
		return TRUE;
	}
} /* OpenCPEMemSection */

/*********************************************************************
 *
 * WriteCPEMemSection
 *
 * SYNOPSIS
 *		short WriteCPEMemSection (int fh, void *buffer, ULONG size)
 *
 * PURPOSE
 *		Write bytes to an CPE file and track bytes written.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short WriteCPEMemSection (int fh, void *buffer, ULONG size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteCPEMemSection";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		if (EIO_Write (fh, buffer, size) != size) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nWriteCPEMemSection:Error writing buffer");
			return FALSE;
		}

		CPEBytesWritten += size;
		return TRUE;
	}
} /* WriteCPEMemSection */

/*********************************************************************
 *
 * CloseCPEMemSection
 *
 * SYNOPSIS
 *		short CloseCPEMemSection (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short CloseCPEMemSection (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CloseCPEMemSection";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		CPEPosType	*cpt;

		cpt = CreateNode (sizeof (CPEPosType), NULL);
		if (!cpt) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEcatf ("CloseCPEMemSection:OOM CPEPosType");
			return FALSE;
		}

		cpt->LenPos = CPELenPos;
		cpt->Length = CPEBytesWritten;
		AddTail (CPEList, cpt);

		CPELenPos = 0;

		return TRUE;

	}
} /* CloseCPEMemSection */

/*********************************************************************
 *
 * WriteEntireCPEMemSection
 *
 * SYNOPSIS
 *		short WriteEntireCPEMemSection (
 *			int		 fh,
 *			ULONG	 address,
 *			void	*buffer,
 *			ULONG	 size
 *		)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short WriteEntireCPEMemSection (
	int		 fh,
	ULONG	 address,
	void	*buffer,
	ULONG	 size
) {

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteEntireCPEMemSection";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		if (OpenCPEMemSection (fh, address)) {
			if (WriteCPEMemSection (fh, buffer, size)) {
				return CloseCPEMemSection ();
			}
		}
		return FALSE;
	}
} /* WriteEntireCPEMemSection */

/*********************************************************************
 *
 * CloseCPEFileWrite
 *
 * SYNOPSIS
 *		short CloseCPEFileWrite (int fh)
 *
 * PURPOSE
 *		Close CPE file and update all length headers.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short CloseCPEFileWrite (int fh)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CloseCPEFileWrite";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		CPEPosType	*cpt;
		UBYTE		 Data[4];
		UBYTE		 End = 0;

		if (CPELenPos != 0) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nCloseCPEFileWrite:MemSection still open");
			return FALSE;
		}

		if (EIO_Write (fh, &End, sizeof (End)) != sizeof (End)) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf ("\nOpenCPEMemSection:Error writing CPE EOF");
			return FALSE;
		}

		EIO_Close (fh);

		fh = EIO_UpdateOpen (CPEName);
		if (fh == (-1)) {
			SetGlobalErr (ERR_GENERIC);
			GEcatf1 ("\nCloseCPEFileWrite:Couldn't reopen '%s'", CPEName);
			return FALSE;
		}

		while ((cpt = RemHead (CPEList)) != NULL) {
			Data[0] = (UBYTE)((cpt->Length & 0x000000FFUL) >>  0);
			Data[1] = (UBYTE)((cpt->Length & 0x0000FF00UL) >>  8);
			Data[2] = (UBYTE)((cpt->Length & 0x00FF0000UL) >> 16);
			Data[3] = (UBYTE)((cpt->Length & 0xFF000000UL) >> 24);

			EIO_Seek (fh, cpt->LenPos, EIO_SEEK_BEGINNING);

			EIO_Write (fh, Data, sizeof (Data));

			DeleteNode (cpt);
		}

		EIO_Close (fh);
		return TRUE;
	}
} /* CloseCPEFileWrite */

