/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * Eerrors.c
 *
 * PROGRAMMER : Gregg A. Tavares 
 *    VERSION : 00.000
 *    CREATED : 08/09/89 
 *   MODIFIED : 09/28/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *	General Error routines
 *
 * HISTORY
 *	05/26/91 Sunday (RGM) - Put #include <echidna/etypes.h> earlier in the
 *		food chain 'cause of the way how stupid <exec/memory.h> and
 *		<exec/nodes.h> need it but don't pull it in.  Sheesh.
 *
*/

#include <echidna\platform.h>
#include "switches.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <echidna/etypes.h>

#if __TURBOC__
#include <dos.h>
#include <alloc.h>
#endif
#if __AMIGAOS__
#include <exec\memory.h>
#if AZTEC_C
#include <functions.h>
#endif
#if LATTICE
#include <proto\exec.h>
#endif
#endif
#if __MACSC__
	#include <ErrMgr.h>
#endif
#include <echidna\eerrors.h>

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_Globals                                               */
/*------------------------------------------------------------------------*/

int		 GlobalErr;
char	*GlobalErrMsg;
char	*CurrentFuncName;

char	*GlobalErrStr[] = {
	"",
	"Out of memory ",
	"Could not open file ",
	"Reading file ",
	"Failed Requirement ",
};

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS___PrintError                                          */
/*------------------------------------------------------------------------*/

static char GlobalErrMsgBuffer[1024];

/*********************************************************************
 *
 * ClearGlobalError
 *
 * SYNOPSIS
 *		void ClearGlobalError (void)
 *
 * PURPOSE
 *		Reset/Clear the global error and error msg.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ClearGlobalError (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ClearGlobalError";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	GlobalErr             = 0;
	GlobalErrMsg          = GlobalErrMsgBuffer;
	GlobalErrMsgBuffer[0] = '\0';

} /* ClearGlobalError */

/**************************************************************************
 *
 * __PrintError
 *
 * SYNOPSIS
 *		void __PrintError (int append, char *fmt, ...)
 *
 * USAGE
 *		GEprintf1 ("Couldn't allocate memory");
 *
 * PURPOSE
 *		Create ErrorMessage in GlobalErrMsgBuffer.
 *
 * INPUTS
 *		fmt	: format string.
 *
 * RESULTS
 *		Prints error message and args to GlobalErrMsgBuffer and
 *  	sets GlobalErrMsg to point to GlobalErrMsgBuffer.
 *
 * BUGS
 *		none.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void __PrintError (int append, char *fmt, ...)
{
	va_list		 ap;	/* points to each unnamed arg in turn */
	char		*out;

	if (append) {
		out	 = &GlobalErrMsgBuffer[strlen(GlobalErrMsgBuffer)];
	} else {
		out  = GlobalErrMsgBuffer;
		out += sprintf (out, "ERROR: ");
	}
	GlobalErrMsg = GlobalErrMsgBuffer;

	va_start (ap,fmt); /* make ap point to 1st unnamed arg */
	vsprintf (out,fmt,ap);
	va_end (ap);	/* clean up when done */
	

} /* PrintError */

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS___PrintFreeMem                                        */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * __PrintFreeMem
 *
 * PURPOSE
 *		Print amount of free memory left.
 *
 * INPUT
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
*/
void __PrintFreeMem (void)
{
#if __TURBOC__

#if __MSDOS16__
	printf ("freemem = %lu, %lu\n", coreleft(), farcoreleft());
#else
	printf ("freemem not implemented\n");
#endif

#elif __WATCOMC__

	printf ("freemem not implemented\n");

#elif __ZTC__

	printf ("freemem not implemented\n");

#elif __AMIGAOS__

	printf ("freemem = %lu\n", AvailMem (NULL));

#elif __EGGS_SGI__

	printf ("freemem not implemented\n");
#else
#error Need __PrintFreeMem() for this system.
#endif

} /* __PrintFreeMem */


/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_GetSysErrorVector                                     */
/*------------------------------------------------------------------------*/

/* Watcom C32: GetSysErrorVector() & SetSysErrorVector() are in IBMERRP.ASM */
#if !__WATCOMC__
/*********************************************************************
 *
 * GetSysErrorVector
 *
 * PURPOSE
 *		Return current MS-DOS critical error handler vector.
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		10/12/93 Tuesday (dcc) - created.
 *
*/
INT far32 *GetSysErrorVector(void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GetSysErrorVector";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

#if __TURBOC__
#if __MSDOS16__
	return getvect(0x24);
#else
	printf ("GetSysErrorVector not implemented\n");
	return NULL;
#endif
#elif __AMIGAOS__

	printf ("GetSysErrorVector not implemented\n");

#elif
#error Need GetSysErrorVector() for this system.
#endif

} /* GetSysErrorVector */
#endif


/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_SetSysErrorVector                                     */
/*------------------------------------------------------------------------*/

#if !__WATCOMC__
/*********************************************************************
 *
 * SetSysErrorVector
 *
 * PURPOSE
 *		Set MS-DOS critical error handler vector.
 *
 * INPUT
 *		
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		
 *
 * HISTORY
 *		10/12/93 Tuesday (dcc) - created.
 *
*/
void SetSysErrorVector(INT far32 *pInt)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetSysErrorVector";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

#if __TURBOC__
#if __MSDOS16__
	setvect (0x24, pInt);
#else
	pInt = pInt;
	printf ("SetSysErrorVector not implemented\n");
#endif
#elif __AMIGAOS__

	printf ("SetSysErrorVector not implemented\n");

#elif
#error Need SetSysErrorVector() for this system.
#endif

} /* SetSysErrorVector */
#endif


/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_SetSysErrorFunc                                       */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * SetSysErrorFunc
 *
 * SYNOPSIS
 *		void SetSysErrorFunc (SysErrorFuncPtr sefptr)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
extern SysErrorFuncPtr far32 SysErrorFunc;
#if (__TURBOC__ || __WATCOMC__)
extern INT IBMErrIntHeader;
#endif
void SetSysErrorFunc (SysErrorFuncPtr sefptr)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetSysErrorFunc";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

#if (__TURBOC__ || __WATCOMC__)
	
	SysErrorFunc = sefptr;
	SetSysErrorVector (IBMErrIntHeader);

#elif __AMIGAOS__

	printf ("SetSysErrorFunc not implemented\n");

#elif __MACOS__

	printf ("SetSysErrorFunc not implemented\n");

#elif __EGGS_SGI__

	printf ("SetSysErrorFunc not implemented\n");

#else
#error Need SetSysErrorFunc() for this system.
#endif

} /* SetSysErrorFunc */

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_FailMess                                              */
/*------------------------------------------------------------------------*/

/**************************************************************************
 *
 * FailMess 
 *
 * SYNOPSIS
 *		void	FailMess (char *fmt, ...)
 *
 * USAGE
 *		FailMess ("Couldn't allocate memory");
 *
 * PURPOSE
 *		Print failure message and line number. Exit program.
 *
 * INPUTS
 *		fmt	: format string.
 *
 * RESULTS
 *		Prints failure message and line number to stdout. 
 *		Program terminates with exit code 20.
 *
 * BUGS
 *		none.
 *
 * HISTORY
 *	
 *
 * SEE ALSO
 *
*/
void	FailMess (char	*fmt, ...)
{
	va_list ap;	/* points to each unnamed arg in turn */

	fprintf (stdout, "FAILED: ");

	va_start (ap,fmt); /* make ap point to 1st unnamed arg */
	vfprintf (stdout, fmt,ap);
	va_end (ap);	/* clean up when done */

	exit(20);

} /* FailMess */

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_ErrMess                                               */
/*------------------------------------------------------------------------*/

/**************************************************************************
 *
 * ErrMess 
 *
 * SYNOPSIS
 *		void	ErrMess (char *fmt, ...)
 *
 * USAGE
 *		ErrMess ("Missing bracket");
 *
 * PURPOSE
 *		Print error message and line number.
 *
 * INPUTS
 *		fmt	: format string.
 *
 * RESULTS
 *		Prints message and line number to stdout.
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

int	ErrorCount	= 0;

void	ErrMess (char	*fmt, ...)
{
	va_list ap;	/* points to each unnamed arg in turn */

	++ErrorCount;
	fprintf (stdout, "ERROR: ");

	va_start (ap,fmt); /* make ap point to 1st unnamed arg */
	vfprintf (stdout, fmt,ap);
	va_end (ap);	/* clean up when done */

} /* ErrMess */

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_WarnMess                                              */
/*------------------------------------------------------------------------*/

/**************************************************************************
 *
 * WarnMess
 *
 * SYNOPSIS
 *		void	WarnMess (char *fmt, ...)
 *
 * USAGE
 *		WarnMess ("Suspicious ptr to ptr conversion.");
 *
 * PURPOSE
 *		Print warning message and line number. 
 *
 * INPUTS
 *		fmt	: format string.	
 *
 * RESULTS
 *		Prints warning message and line number to stdout.
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/

int		WarnCount	= 0;

void	WarnMess (char	*fmt, ...)
{
	va_list ap;	/* points to each unnamed arg in turn */

	++WarnCount;
	fprintf (stdout, "WARNING: ");

	va_start (ap,fmt); /* make ap point to 1st unnamed arg */
	vfprintf (stdout, fmt,ap);
	va_end (ap);	/* clean up when done */

} /* WarnMess */

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_OutOfMemErr                                           */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * OutOfMemErr
 *
 * SYNOPSIS
 *		void	OutOfMemErr (char *fmt, ...)
 *
 * USAGE
 *		OutOfMemErr ("Invalid tag: %s",ClassTagStr[classtag]);
 *
 * PURPOSE
 *		To print out of memory error messages. 
 *
 * INPUTS
 *
 *
 * RESULTS
 *
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void	OutOfMemErr (	char	*fmt, ...)
{
	va_list ap;	/* points to each unnamed arg in turn */

	fprintf (stdout, "OUT OF MEMORY: ");

	va_start (ap,fmt); /* make ap point to 1st unnamed arg */
	vfprintf (stdout, fmt,ap);
	va_end (ap);	/* clean up when done */

	exit(20);
} /* OutOfMemErr */

/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_MacSystemErrMsg                                       */
/*------------------------------------------------------------------------*/

#if __MACSC__
char *MacSystemErrMsg (int err)
{
	static	char	errmsg[256];
	
	return GetSysErrText (err, errmsg);
}
#endif

