/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * TESTEIO.C
 *
 * PROGRAMMER : Dan Chang
 *    VERSION : 00.000
 *    CREATED : 07/06/93
 *   MODIFIED : 07/27/94
 *       TABS : 04 07
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *		Test the EIO.C file.
 *
 * HISTORY
 *		07/06/93 Tuesday (dcc) - created.
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/eio.h>

#include <string.h>

#if __TURBOC__
#include <alloc.h>
#endif

#if ERRORS
#error This program must be compiled with #define ERRORS 0
#endif

/* These routines are still not tested by this program:

EIO_InsureEndSlash
EIO_GetTempFileName
EIO_fnmerge
EIO_Path
EIO_Name
EIO_Ext
EIO_Filename
EIO_PrintEscString
EIO_SetFileDate
EIO_GetFileAttrib
EIO_SetFileAttrib
EIO_GetFileComment
EIO_SetFileComment
EIO_CopyFile
EIO_MakeDir
EIO_Read
*/

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*------------------------------------------------------------------------*/
/**# MODULE:EERRORS_Globals                                               */
/*------------------------------------------------------------------------*/

int		 GlobalErr;


/*------------------------------------------------------------------------*/
/**# MODULE:STRINGS_lchinstr                                              */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * lchinstr
 *
 * SYNOPSIS
 *		int lchinstr (char ch, char *str)
 *
 * USAGE
 *		FirstDotPosition = lchinstr ('.', filenamestr);
 *
 * PURPOSE
 *		To find the position of the first occurence from the left of the
 *		character 'ch' in string 'str'.
 *
 * INPUTS
 *		ch		: character to find.
 *		str		: string in which to find character. (may be NULL)
 *
 * RESULTS
 *		If character is in string then returns positon (0..strlen (str)) of
 *		leftmost occurence of character in string. Else returns < 0.
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int lchinstr (
	char	 ch,
	char	*str
)
{
	int position = -1;
	int i;

	if (!str)
		return -1;
	for (i = 0; *str && (position < 0); i++, str++){
		if (ch == *str)
			position = i;
 	}
	return position;
} /* lchinstr */


/*------------------------------------------------------------------------*/
/**# MODULE:STRINGS_rstrchinstr                                           */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * rstrchinstr
 *
 * SYNOPSIS
 *		int rstrchinstr (char *chset, char *str)
 *
 * USAGE
 *		LastPathDelimPosition = rstrchinstr ("\/", path);
 *
 * PURPOSE
 *		To find the position of the last occurence in string 'str' of
 *		any character from string 'chset' except NULL character.
 *
 * INPUTS
 *		chset	: string of characters to find.
 *		str		: string in which to find characters. (may be NULL)
 *
 * RESULTS
 *		If character from 'chset' is in string then returns positon
 *		(0..strlen (str)) of rightmost occurence of characters in string.
 *		Else returns < 0.
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int rstrchinstr (
	char	*chset,
	char	*str
)
{
	int position = -1;
	int i;

	if (!str || !chset)
		return -1;
	for (i = 0; *str; i++, str++){
		if (lchinstr (*str,chset) >= 0)
			position = i;
 	}
	return position;
} /* rstrchinstr */

int main(int argc, char **argv)
{
	argc = argc;
#if __ZTC__
	printf("EIO_GetNextFile() is broken\n");
#else
	{
		DirTracker *pdt;
		short		fOk = TRUE;

		printf("Files in current directory:\n\n");

		pdt = EIO_GetFirstFile("*.*", FALSE);
		if (pdt)
		{
			while (fOk && pdt->Status)
			{
				printf("%s\n", pdt->Path);
				fOk = EIO_GetNextFile(pdt);
			}
			EIO_FreeDirTracker(pdt);
		}
		if (!fOk || !pdt)
		{
			printf ("Error reading directory\n");
		}
	}
	printf("\n");
#endif
	{
		char sz[EIO_MAXPATH];
		int fh;
		UBYTE huge32 *pub;
		long l;

		EIO_GetTempFileName(sz);
		printf("Got temporary file %s\n", sz);

#if __TURBOC__
		pub = farmalloc(66666UL);
#else
		pub = malloc(66666UL);
#endif
		if (pub)
		{
			{
				long i;

				for (i = 0; i < 66666UL; i++)
					*(pub+i) = 0;
				for (i = 0; i < 256; i++)
					*(pub+65536U+i) = i;
			}
			if ((fh = EIO_WriteOpen(sz)) != -1)
			{
				EIO_Write(fh, pub, 66666UL);
				l = EIO_FileLength(fh);
				EIO_Close(fh);
				printf("Wrote %ld bytes to %s\n", l, sz);
			}
			else
			{
				printf("Unable to create file %s\n", sz);
			}
#if __TURBOC__
			farfree(pub);
#else
			free(pub);
#endif
		}
		else
		{
			printf("Unable to allocate 66666 bytes.\n");
		}
		{
			FileDateType fdt, fdt2, fdt3;

			if (EIO_GetFileDate(sz, &fdt))
				printf("Got file date of %s\n", sz);
			else
				printf("Coulnd't get file date of %s\n", sz);

			if (EIO_GetFileDate(argv[0], &fdt2))
				printf("Got file date of %s\n", argv[0]);
			else
				printf("Coulnd't get file date of %s\n", argv[0]);

			if (EIO_GetFileDate(sz, &fdt3))
				printf("Got file date of %s\n", sz);
			else
				printf("Coulnd't get file date of %s\n", sz);

			printf("Compare %s=%s: %d\n", sz, argv[0], EIO_CmpDates(&fdt, &fdt2));
			printf("Compare %s=%s: %d\n", sz, sz, EIO_CmpDates(&fdt, &fdt3));
		}
	}
	printf("\n");
	{
		char sz[EIO_MAXDIR];

		if (EIO_CurrentDir(sz))
			printf("Current directory is %s\n", sz);
		else
			printf("Couldn't get current directory\n");
	}
	printf("\n");
	{
		DirTracker *pdt;
		BOOL fOk = TRUE;

		pdt = EIO_GetFirstEnv();
		if (pdt)
		{
			while (fOk && pdt->Status)
			{
				printf("%s=%s\n", pdt->Path, pdt->EnvString);
				fOk = EIO_GetNextEnv(pdt);
			}
			EIO_FreeDirTracker(pdt);
		}
		if (!fOk && !pdt)
			printf ("Error reading environment variables\n");
	}
	printf("\n");
	{
		short ft;

		ft = EIO_FileType(argv[0]);
		if (!ft)
			printf ("%s not found\n", argv[0]);
		else if (ft == EIO_TYPE_FILE)
			printf ("%s is a file\n", argv[0]);
		else if (ft == EIO_TYPE_DIRECTORY)
			printf ("%s is a directory\n", argv[0]);
	}
	printf("\n");
	{
		short ft;
		char	 dir[EIO_MAXDIR];
		char	 name[EIO_MAXFILE];
		char	 ext[EIO_MAXEXT];

		EIO_fnsplit (argv[0], dir, name, ext);
		printf ("dir  = '%s'\n", dir);
		printf ("name = '%s'\n", name);
		printf ("ext  = '%s'\n", ext);

		dir[strlen(dir)-1] = 0;
		ft = EIO_FileType(dir);
		if (!ft)
			printf ("%s not found\n", dir);
		else if (ft == EIO_TYPE_FILE)
			printf ("%s is a file\n", dir);
		else if (ft == EIO_TYPE_DIRECTORY)
			printf ("%s is a directory\n", dir);
	}
	printf("\n");
	{
// This also tests EIO_GetFirstPath(), EIO_GetNextPath(), EIO_FileExists()

		char sz[EIO_MAXPATH];

#define szFile "PKUNZIP.EXE"

		if (EIO_FindFile(szFile, NULL, sz))
			printf("Found %s at %s\n", szFile, sz);
		else
			printf("%s not found in path\n", szFile);

		{
			char *args[1];

			args[0] = NULL;

			EIO_SpawnProgram(sz, args);

			printf("\nReturned from %s\n", sz);
		}
	}
	return 0;
}

