/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * ATIMLIST.H
 *
 * PROGRAMMER : R.G. Marquez 
 *    VERSION : 00.000
 *    CREATED : 06/23/91 
 *   MODIFIED : 07/27/94
 *       TABS : 04 09
 *
 *	     \|///-_
 *	     \oO///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *		Header for Amiga timer list functions (see ATIMLIST.C).
 *
 * HISTORY
 *	06/23/91 Sunday (RGM) - Created.
 *
*/
#ifndef ATIMLIST_H
#define ATIMLIST_H

#include <echidna/platform.h>
#include "switches.h"

#include <exec/types.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/

typedef struct ATimer_MinNode {
	struct ATimer_MinNode	__far	*Next;
	struct ATimer_MinNode	__far	*Prev;
} ATimer_MinNode;


typedef struct {
	struct ATimer_MinNode	__far	*Head;
	struct ATimer_MinNode	__far	*Tail;
	struct ATimer_MinNode	__far	*TailPred;
} ATimer_MinList;


/****************************** E X T E R N S *****************************/


/****************************** G L O B A L S *****************************/


/******************************* L O C A L S ******************************/


/******************************* M A C R O S ******************************/

/* ate_Insert (after) */

#define ate_InitList(list_ptr)\
	((ATimer_MinList __far *) (list_ptr))->Head	=\
		(void __far *) &((ATimer_MinList __far *) (list_ptr))->Tail;\
	((ATimer_MinList __far *) (list_ptr))->Tail	= (void __far *) NULL;\
	((ATimer_MinList __far *) (list_ptr))->TailPred=\
		(void __far *) &((ATimer_MinList __far *) (list_ptr))->Head

#define ate_Remove(node_ptr)\
	((ATimer_MinNode __far *) (node_ptr))->Prev->Next	=\
		((ATimer_MinNode __far *) (node_ptr))->Next;\
	((ATimer_MinNode __far *) (node_ptr))->Next->Prev	=\
		((ATimer_MinNode __far *) (node_ptr))->Prev
	
#define ate_AddHead(list_ptr,node_ptr)\
	((ATimer_MinNode __far *) (node_ptr))->Prev	=\
		(void __far *) &((ATimer_MinList __far *) (list_ptr))->Head;\
	((ATimer_MinNode __far *) (node_ptr))->Next	=\
		(void __far *) ((ATimer_MinList __far *) (list_ptr))->Head;\
	((ATimer_MinList __far *) (list_ptr))->Head	= (void __far *) (node_ptr);\
	((ATimer_MinNode __far *) (node_ptr))->Next->Prev =\
		(void __far *) (node_ptr)

#define ate_AddTail(list_ptr,node_ptr)\
	((ATimer_MinNode __far *) (node_ptr))->Prev	=\
		(void __far *) ((ATimer_MinList __far *) (list_ptr))->TailPred;\
	((ATimer_MinNode __far *) (node_ptr))->Next	=\
		(void __far *) &((ATimer_MinList __far *) (list_ptr))->Tail;\
	((ATimer_MinList __far *) (list_ptr))->TailPred	=\
		(void __far *) (node_ptr);\
	((ATimer_MinNode __far *) (node_ptr))->Prev->Next =\
		(void __far *) (node_ptr)

#define ate_Head(list_ptr) \
	(((ATimer_MinList __far *) (list_ptr))->Head)

#define ate_Tail(list_ptr) \
	(((ATimer_MinList __far *) (list_ptr))->TailPred)

#define ate_Prev(node_ptr) \
	(((ATimer_MinNode __far *) (node_ptr))->Prev)

#define ate_Next(node_ptr) \
	(((ATimer_MinNode __far *) (node_ptr))->Next)

#define ate_IsHead(node_ptr) \
	(! ((ATimer_MinNode __far *) (node_ptr))->Prev)

#define ate_IsTail(node_ptr) \
	(! ((ATimer_MinNode __far *) (node_ptr))->Next)

#define ate_IsEmpty(list_ptr)\
	(ate_IsTail(ate_Head(list_ptr)))

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern void __far * __far __regargs ate_RemHead(void __far *list_ptr);

extern void __far * __far __regargs ate_RemTail(void __far *list_ptr);

#endif /* ATIMLIST_H */

