;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** VBLANK.ASM
;**
;** PROGRAMMER : Dan Chang
;**    VERSION : 00.000
;**    CREATED : 04/18/91
;**   MODIFIED : 03/25/94
;**       TABS : 09 17
;**
;**
;** DESCRIPTION
;**	8253 Programmable interval timer routines.
;**
;**	These routines used to sychronize to the vertical blank,
;**	but this synchronization code breaks under Windows, and is
;**	commented out. BUGBUG: fix this someday...
;**
;** HISTORY
;**	05/12/93 Wednesday (dcc) - default to 60Hz instead of 420Hz.
;**			Automatically increases speed if necessary.
;**	09/13/93 Monday (dcc) - BUGFIX: 2nd Interrupt routine never called
;**	09/13/93 Monday (dcc) - BUGFIX: revise already installed code check
;**	03/18/94 Friday (dcc) - revise Reset_Vint() to be more safe.
;**
	public	__OpenTimerInterrupt
	public	__CloseTimerInterrupt

	public	__TimerRecord
	public	__uwFreeHead

	public	__TimerCount
	public	__NewTimerCount

	public	__VBlankAddress
	public	__VBlankActive
	public	__LastVBlankActive

	IDEAL

	INCLUDE	"TIMERMDL.inc"		;JMA

	WSEG	TIMER_SEG

MAXVBLANKS	equ	8		;number of vblanks you may install
MAXTIMERRECORDS	equ	256

ACTIVE	equ	(10000000b)
MODIFY	equ	(01000000b)
COUNTUP	equ	(00001000b)
ROUTINE	equ	(00000100b)

;32bits	equ	(00000010b)
;16bits equ	(00000001b)
;08bits	equ	(00000000b)

vb60Hz	equ	19886	;(1193180/60)

	STRUC	TimerType
uwNext	dw	?
uwPrev	dw	?
ulAddress	dd	?
ulCounter	dd	?
ulTrigger	dw	?
		db	?
ubFlags		db	?
	ENDS

__TimerRecord	TimerType	MAXTIMERRECORDS dup (<>)
__uwFreeHead	dw	?

old_timer_disp	dw	0		; old timer interrupt vector
old_timer_seg	dw	0

__TimerCount	dw	?		; current timer count value loaded
__NewTimerCount	dw	?		; new timer count value to load

__VBlankAddress	dd	MAXVBLANKS dup (?)	; address of vblank routine to call

__VBlankActive	db	MAXVBLANKS dup (?)	; set TRUE if VBlankDisp is valid

__LastVBlankActive	dw	?	; last vblank active in list

VblankCount	dw	?		; keep track of counts between calls
					; to vertical blanking routine
VblankTrigger	dw	?		; when to do vblank (VblankCount >= )

OldTimerCount	dw	?		; keep track of counts between calls
OldTimerCountH	dw	?		; to old timer interrupt routine

; Vint - this is the actual interrupt routine. It gets called 420
; times per second. Call the routine that needs to happen in vertical blank.
; Call all other routines that depend on the interrupt.
;
; by Danno

	PROC	Vint

        push	ax
        push	bx
	push	cx
        push	dx
        push	bp
	push	si
	push	di
        push	ds
	push	es

	mov	ax, TIMER_SEG
	mov	ds, ax

	mov	ax, [__TimerCount]
	cmp	ax, [__NewTimerCount]
	jne	@@FastEnough		;=> still running fast enough

	; Need to speed up interrupt rate

	mov	bx, [__NewTimerCount]
        mov   al, 36h                   ; set new interrupt rate
        out   43h, al                   ;
        jmp   $+2                       ;
        mov   al, bl                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
        mov   al, bh                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
@@FastEnough:

	mov	ax, [__TimerCount]
	add	[VblankCount], ax
	mov	ax, [VblankCount]
	cmp	ax, [VblankTrigger]
	jb	@@notVblank

	; Handle vertical blank routines

	sub	ax, vb60Hz
	mov	[VblankCount], ax

;!!	; Vertical blank should occur within this interrupt period.
;!!	; Wait for it to start...

;!!	mov	ah, 1000b
;!!	mov	dx, 3dah

;!!	in	al, dx			;check vertical retrace
;!!	test	al, ah
;!!	jnz	@@retrace		;=> already in vertical retrace
;!!@@waitretrace:
;!!	in	al, dx
;!!	test	al, ah
;!!	jz	@@waitretrace		;=> wait for retrace to happen
;!!
;!!	mov	bx, [__TimerCount]	; DCC load current timer value
;!!
;!!        mov   al, 36h                   ; set new interrupt rate
;!!        out   43h, al                   ;
;!!        jmp   $+2                       ;
;!!        mov   al, bl                    ;
;!!        out   40h, al                   ;
;!!        jmp   $+2                       ;
;!!        mov   al, bh                    ;
;!!        out   40h, al                   ;
;!!        jmp   $+2                       ;
;!!
;!!@@retrace:
	sub	si, si
	sub	di, di
@@nextVBL:
	cmp	[__VBlankActive+si], 0
	jz	@@noVBL

	call	[dword ptr __VBlankAddress+di]
@@noVBL:
	add	di, 4
	inc	si
	cmp	si, [__LastVBlankActive]
	jb	@@nextVBL

;!!	sub	ax, ax			; reset VblankCount
@@notVblank:
;!!	mov	[VblankCount], ax

	sub	si, si
	jmp	SHORT @@updateLoop

	; Remove timer/interrupt from active list.

@@remove:
	; First, unlink entry from active list.

	mov	ax, [si+__TimerRecord.uwNext]
	mov	di, [si+__TimerRecord.uwPrev]
	mov	[di+__TimerRecord.uwNext], ax

	mov	ax, [si+__TimerRecord.uwPrev]
	mov	di, [si+__TimerRecord.uwNext]
	mov	[di+__TimerRecord.uwPrev], ax

	; Then add node to head of free list.

	mov	di, [__uwFreeHead]
	mov	[di+__TimerRecord.uwPrev], si
	mov	[si+__TimerRecord.uwNext], di
	mov	[si+__TimerRecord.uwPrev], 0ffffh
	mov	[__uwFreeHead], si

	; Set offset index to previous active record in list
	; (so when @@updateLoop gets next record it'll be correct).

	mov	si, ax

	; Top of loop: go through all active timer records.

@@updateLoop:
	mov	si, [si+__TimerRecord.uwNext]	; get next record offset

	; See if last record reached (_TimerRecourd.uwNext == 0ffffh)?

	mov	ax, [si+__TimerRecord.uwNext]
	inc	ax
	jnz	@@notdone
	jmp	@@done			;=> last record reached
@@notdone:
	; See if record is being modified? If so, skip it.

	mov	al, [si+__TimerRecord.ubFlags]
	test	al, MODIFY
	jnz	@@updateLoop		;=> this record is being updated

	; See if record is active? If not, delete it.

	test	al, ACTIVE
	jz	@@remove		;=> remove record

	; See if ulTrigger is zero (timer suspended)?

	sub	ah, ah
	mov	al, [byte ptr si+2+__TimerRecord.ulTrigger]
	or	ax, [word ptr si+__TimerRecord.ulTrigger]
	jz	@@updateLoop		;=> this timer is suspended

	; Add time elapsed since last interrupt to total time elapsed for this record.

	mov	ax, [__TimerCount]
	add	[word ptr si+__TimerRecord.ulCounter], ax
	adc	[word ptr si+2+__TimerRecord.ulCounter], 0

	; Compare total time elapsed with the trigger value (3 byte quantity).

	sub	bh, bh			;zero out 4th byte of trigger
	mov	bl, [byte ptr si+2+__TimerRecord.ulTrigger]	;get 3rd byte

	mov	ax, [word ptr si+__TimerRecord.ulCounter]
	sub	ax, [word ptr si+__TimerRecord.ulTrigger]
	mov	dx, [word ptr si+2+__TimerRecord.ulCounter]
	sbb	dx, bx
	jb	@@updateLoop		;=> not time to inc timer/call routine yet

	; Subtract out Trigger value and save new total time elapsed.

	mov	[word ptr si+__TimerRecord.ulCounter], ax
	mov	[word ptr si+2+__TimerRecord.ulCounter], dx

	; Increment/decrement timer or call routine?

	mov	al, [si+__TimerRecord.ubFlags]
	test	al, ROUTINE
	jz	@@IncOrDec		;=> ROUTINE bit not set, inc/dec timer

	; Call user install routine.

	call	[dword ptr si+__TimerRecord.ulAddress]
	jmp	@@updateLoop

	; Increment or decrement memory location

@@IncOrDec:
	les	di, [dword ptr si+__TimerRecord.ulAddress]

	test	al, COUNTUP
	jz	@@countDN		;=> decrement memory location

	; Decrement memory location.

	and	al, 11b
	jnz	@@notBup

	inc	[byte ptr es:di]
	jmp	@@updateLoop
@@notBup:
	dec	al
	jnz	@@notWup

	inc	[word ptr es:di]
	jmp	@@updateLoop
@@notWup:
	inc	[word ptr es:di]
	jnz	@@nextEntry
	inc	[word ptr es:di+2]
@@nextEntry:
	jmp	@@updateLoop

	; Increment memory location.
@@countDN:
	and	al, 11b
	jnz	@@notBdn

	mov	al, [byte ptr es:di]
	or	al, al
	jz	@@toLoop		;=> byte counter already at zero
	dec	al
	mov	[byte ptr es:di], al
	jmp	@@updateLoop
@@notBdn:
	dec	al
	jnz	@@notWdn

	mov	ax, [word ptr es:di]
	or	ax, ax
	jz	@@toLoop		;=> word counter already at zero
	dec	ax
	mov	[word ptr es:di], ax
	jmp	@@updateLoop
@@notWdn:
	mov	ax, [word ptr es:di]
	or	ax, [word ptr es:di+2]
	jz	@@toLoop		;=> long word counter already at zero

	cmp	[word ptr es:di], 0
	jnz	@@notHI
	dec	[word ptr es:di+2]
@@notHI:
	dec	[word ptr es:di]
@@toLoop:
	jmp	@@updateLoop
@@done:
	mov	ax, [__TimerCount]

	mov	bx, [__NewTimerCount]
	mov	[__TimerCount], bx	; __TimerCount = __NewTimerCount

	add	[OldTimerCount], ax
	adc	[OldTimerCountH], 0
	jnz	@@CallOld		; => time to call old timer interrupt

	mov	al, 20h			;eoi
	out	20h, al
	pop	es
	pop	ds
	pop	di
	pop	si
	pop	bp
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	iret

	; Call old timer interrupt routine.

@@CallOld:
	dec	[OldTimerCountH]	; update OldTimerCount
	pop	es
	pop	ds
	pop	di
	pop	si
	pop	bp
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	jmp	[Dword Ptr cs:old_timer_disp]

	ENDP	Vint

; SaveTimer - save old timer interrupt vector. We ASSUME old time rate
; was 65536 (about 18.2 times per second).
;
; by Danno

	PROC	SaveTimer

	push	ax
	push	bx
	push	es

        mov	ax, 3508h		; get old timer interrupt vector
        int	21h

        mov	ax, es
        mov	[cs:old_timer_seg], ax	; save old vector
        mov	[cs:old_timer_disp], bx

	pop	es
	pop	bx
	pop	ax
	ret

	ENDP	SaveTimer

;/******************************************************************/
;/*                                                                */
;/*       PROC-NAME   : Set_Vint                                   */
;/*       AUTHOR      : JOHN REEGO                                 */
;/*       PURPOSE     : vector the timer interrupt                 */
;/*                                                                */
;/*       INPUT       : none                                       */
;/*                                                                */
;/*       OUTPUT      : none                                       */
;/*                                                                */
;/*       USED REGS   : none                                       */
;/*                                                                */
;/*       REMARKS     :                                            */
;/*                                                                */
;/*       PROCEDURES  : none                                       */
;/*                                                                */
;/******************************************************************/

	PROC	Set_Vint

        push  ax                     ; save regs
        push  dx                     ;
        push  ds                     ;

        mov   ax, cs
        mov   ds, ax

	sub	ax, ax
	mov	[cs:OldTimerCount], ax	; DCC - reset old timer count
	mov	[cs:OldTimerCountH], ax

        mov   dx, offset Vint        ; ds:dx = new interrupt vector
        mov   ax, 2508h              ; set new hardware timer interrupt
        int   21h                    ;

        pop   ds                     ; restore regs
        pop   dx                     ;
        pop   ax                     ;
        ret                          ;

	ENDP    Set_Vint

;/******************************************************************/
;/*                                                                */
;/*       PROC-NAME   : Reset_Vint                                 */
;/*       AUTHOR      : JOHN REEGO                                 */
;/*       PURPOSE     : reset the timer interrupt vector           */
;/*                                                                */
;/*       INPUT       : none                                       */
;/*                                                                */
;/*       OUTPUT      : none                                       */
;/*                                                                */
;/*       USED REGS   : ax,bx,cx,dx                                */
;/*                                                                */
;/*       REMARKS     :                                            */
;/*                                                                */
;/*       PROCEDURES  : None                                       */
;**	03/18/94 Friday (dcc) - go paranoid - reorder, disable int */
;/*                                                                */
;/******************************************************************/

	PROC	Reset_Vint

        push  ax                        ; save regs
        push  dx                        ;
        push  ds                        ;
	pushf

	cli

	; Get old timer interrupt vector.

	lds	dx, [dword ptr cs:old_timer_disp]

        mov   ax, 2508h                 ; restore old vector
        int   21h                       ;

	cli

        mov   al,36H               ; Timer mode 3, load rate Low/High
        out   43H,al               ;
        jmp   $+2                  ;
        jmp   $+2                  ;
        sub   ax, ax               ;
        out   40H,al               ;
        jmp   $+2                  ;
        jmp   $+2                  ;
        out   40H,al               ; 65536 for 18.2 times per second.

	sub	ax, ax
	mov	[cs:old_timer_disp], ax	; DCC set flag: routine restored
	mov	[cs:old_timer_seg], ax

	popf
        pop   ds                   ; restore regs
        pop   dx                   ;
        pop   ax                   ;
        ret                        ;

	ENDP	Reset_Vint

; Read_VBlank - figure out how long vertical blanking is taking.
;
; Set the following variables:
; VblankCount
; VblankTrigger
;
; This routine is set up to deal with 60Hz cards (EGA) and 70Hz cards (VGA).
; It set the interrupt rate at 420Hz, which is 7xEGA rate & 6xVGA rate.
; VblankMultiple will be set to 7 or 6 as appropriate.
;
; Not even. It now just sets the interrupt rate == 60Hz.
; BUGBUG: The old VBLANK measuring code is UNRELIABLE under Windows.
; So we just set the rate to 60Hz.

	PROC    Read_VBlank

        push  ds
        mov   ax, cs
        mov   ds, ax

	mov	bx, vb60Hz
	mov	[__TimerCount], bx	; DCC set current timer value
	mov	[__NewTimerCount], bx	; DCC set new timer value
	mov	[VblankTrigger], bx	; DCC before we need to call vbl routine

	sub	ax, ax
	mov	[VblankCount], ax	; DCC - reset vblank timer count

        mov   ah, 1000b                 ;
        mov   dx, 3dah                  ;
@@lpxx:                                 ; LOOP while not in VB
        in    al, dx                    ;
        jmp   $+2                       ;
        test  al, ah                    ;
        jz    @@lpxx                    ;

	IF	0
@@lpyy:                                 ; LOOP while in VB
        in    al, dx                    ;
        jmp   $+2                       ;
        test  al, ah                    ;
        jnz   @@lpyy                    ;
	ENDIF

	CLI

        mov   al, 36h                   ; set new interrupt rate
        out   43h, al                   ;
        jmp   $+2                       ;
        mov   al, bl                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
        mov   al, bh                    ;
        out   40h, al                   ;
        jmp   $+2                       ;

	STI

        pop   ds
        ret

	ENDP    Read_VBlank

; __OpenTimerInterrupt - initialize "vertical blank interrupt". This routine
; actually installs an interrupt routine that runs 420 times per
; second, and calls your routine whenever appropriate.
;
; by Danno

	MPROC	__OpenTimerInterrupt

	push	si
	push	di
	push	bp

	; See if our interrupt handler is already installed.

	mov	ax, [cs:old_timer_disp]	;see if already restored or
	or	ax, [cs:old_timer_seg]	;    if never installed
	jnz	@@installed		;=> already installed

	call	SaveTimer		;save old timer interrupt vector
	call	Read_VBlank		;how fast are thing happening?
	call	Set_Vint		;start the interrupt
@@installed:
	pop	bp
	pop	di
	pop	si
	ret

	ENDP	__OpenTimerInterrupt

; __CloseTimerInterrupt - all good things must end eventually, and so remove
; our timer interrupt and restore the timer to normal speed.
;
; by Danno

	MPROC	__CloseTimerInterrupt

	push	si
	push	di
	push	bp

	; See if we've already restored.

	mov	ax, [cs:old_timer_disp]	;see if already restored or
	or	ax, [cs:old_timer_seg]	;    if never installed
	jz	@@notinstalled		;=> don't restore

	call	Reset_Vint
@@notinstalled:
	pop	bp
	pop	di
	pop	si
	ret

	ENDP	__CloseTimerInterrupt
	ENDS
	END

