/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * TPMasks.H
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/18/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Masks for getting TumePack tile set attribute values.
 *
 * HISTORY
 *
*/

#ifndef TPMASKS_H
#define TPMASKS_H

#include <echidna/platform.h>
#include "switches.h"

/**************************** C O N S T A N T S ***************************/
/* Tile set attribute flags bit positions */
#define TSAT_EGA_PAT_LO_BIT		24
#define TSAT_CGA_PAT_LO_BIT		16
#define TSAT_SET_SIZE_LO_BIT	8
#define TSAT_PCC_LO_BIT			4

#define TINF_USER_TYPE_LO_BIT	24

/* Tile set attribute flag masks */
#define TSAT_EGA_PAT_MASK	(0xFFL << TSAT_EGA_PAT_LO_BIT)
#define TSAT_CGA_PAT_MASK	(0xFFL << TSAT_CGA_PAT_LO_BIT)
#define TSAT_SET_SIZE_MASK	(0xFFL << TSAT_SET_SIZE_LO_BIT)
#define TSAT_PCC_MASK		(0x0FL << TSAT_PCC_LO_BIT)

#define TINF_USER_TYPE_MASK	(0xFFL << TINF_USER_TYPE_LO_BIT)
#define TINF_SIZE_MASK		(0x00FFFFFFL)


#define TSAT_FREE_CODE_MASK		0x0FL

#define TSAT_DIRECT_FREE		0
#define TSAT_SHAPE_FREE			1

/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define TileSize(info)		((info) & TINF_SIZE_MASK)
#define TileUserType(info)	(((info) & TINF_USER_TYPE_MASK) >>	\
								TINF_USER_TYPE_LO_BIT)

#define TilePCC(att)		(((att) & TSAT_PCC_MASK) >> TSAT_PCC_LO_BIT)
#define TileSetSize(att)	(((att) & TSAT_SET_SIZE_MASK) >>	\
										TSAT_SET_SIZE_LO_BIT)
#define EGAPatIndex(att)	(((att) & TSAT_EGA_PAT_MASK) >>	\
										TSAT_EGA_PAT_LO_BIT)
#define CGAPatIndex(att)	(((att) & TSAT_CGA_PAT_MASK) >>	\
										TSAT_CGA_PAT_LO_BIT)

#define TileFreeCode(att)		((att) & TSAT_FREE_CODE_MASK)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/


#endif /* TPMASKS_H */

