/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * XPakSup.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/18/90
 *   MODIFIED : 02/05/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Routines used by both TumeSup.C and BrushSup.C.
 *
 * HISTORY
 *		10/07/94 (dcc) - Updated to use <echidna/alloc.h>.
 *		02/05/95 (dcc) - #include "xpaksup.h" to verify function prototypes.
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdlib.h>
#include <string.h>
#include <string.h>

#include <echidna/etypes.h>
#include <echidna/alloc.h>
#include <echidna/graphics.h>
#include <echidna/eerrors.h>
#include <echidna/fpfload.h>

#include "xpaksup.h"					// verify function prototypes

/**************************** C O N S T A N T S ***************************/
#define NUM_CGA_COLORS	4


/******************************** T Y P E S *******************************/


typedef struct {
	WORD CGA_Code;
	WORD EGA_NumColors;
	WORD EGA_NumCycles;
	WORD MGA_NumColors;
	WORD MGA_NumCycles; 
	WORD MGA_TotalCycleColors;	/* Total of colors in all cycles */
	WORD MGA_TotalCycleRegs;	/* Total of registers in all cycles */
	WORD EGA_TotalCycleColors;	/* Total of colors in all cycles */
	WORD EGA_TotalCycleRegs;	/* Total of registers in all cycles */
} PBX_HeaderType;


/****************************** G L O B A L S *****************************/

UBYTE	*RawBuf;

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * GetCopy
 *
 * SYNOPSIS
 *		void	*GetCopy (
 *			void	*data,
 *			size_t	size
 *		)
 *
 * PURPOSE
 *		Allocate 'size' mem and copy data into it and return ptr to it.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Returns pointer to copy if succesful, else returns NULL.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void	*GetCopy (
	void	*data,
	size_t	size
)
{
	void *ptr;

	if ((ptr = MEM_malloc(size)) != NULL) {
		memcpy (ptr, data, size);
	} else {
		SetGlobalErr ( ERR_OUT_OF_MEMORY);
		GEprintf (GlobalErrStr[ERR_OUT_OF_MEMORY]);
		GEcatf ("GetCopy ()\n");
	}
	return ptr;
} /* GetCopy */


/*********************************************************************
 *
 * ReadPBX
 *
 * SYNOPSIS
 *		PaintBoxType *ReadPBX (size_t min_read_size)
 *	
 *	
 * PURPOSE
 *		To make a PaintBoxType struct from the raw data in PBX format.
 *		CSSI format:
 *		struct {
 *    		WORD CGACode;
 *    		WORD EGA_NumColors;
 *    		WORD EGA_NumCycles;
 *    		WORD MGA_NumColors;   \* MCGA or Amiga *\
 *    		WORD MGA_NumCycles;     \* MCGA or Amiga *\
 *
 *    		\* MCGA or Amiga Cycle Set Header Info *\
 *    		struct {
 *        		WORD TotalColors;    \* Total of colors in all cycles *\
 *        		WORD TotalRegisters; \* Total of registers in all cycles *\
 *    		} CycleSetHeader[((MGA_NumCycles) ? 1 : 0)];
 *
 *    		\* MCGA or Amiga Cycles Data *\
 *    		struct {
 *        		WORD NumColors;     \* Number of colors in Color Cycle.    *\
 *        		WORD NumRegisters;  \* Number of registers in Color Cycle. *\
 *        		WORD Hertz;         \* Changes per second *\
 *        		WORD Direction;     \* 0 = low..high reg.; 1 = high..low reg.*\
 *        		WORD Active;        \* Always non-zero.
 *        		WORD Timer;         \* 0; Count down timer for timing cycle *\
 *        		WORD CurrentColor;  \* Index of color to put in Registers[0] *\
 *        		ULONG Pad0;          \* Unused (space for Colors ptr)*\
 *        		ULONG Pad1;          \* Unused (space for Registers ptr)*\
 *    		} ColorCycles[MGA_NumCycles];
 *    		ColorType Colors[CycleSetHeader[0].TotalColors];
 *    		UBYTE Registers[CycleSetHeader[0].TotalRegisters];
 *
 *    		\* MCGA or Amiga Palette *\
 *    		ColorType PaletteColors[MGA_NumColors];
 *
 *    		\* EGA Cycle Set Header Info *\
 *    		struct {
 *        		WORD TotalColors;    \* Total of colors in all EGA cycles *\
 *        		WORD TotalRegisters; \* Total of registers in all EGA cycles *\
 *    		} EGACycleSetHeader[((EGA_NumCycles) ? 1 : 0)];
 *
 *    		\* EGA Cycles Data *\
 *    		struct {
 *        		WORD NumColors;     \* Number of colors in EGA Color Cycle.    *\
 *        		WORD NumRegisters;  \* Number of registers in EGA Color Cycle. *\
 *        		WORD Hertz;         \* Changes per second *\
 *        		WORD Direction;     \* 0 = low..high reg.; 1 = high..low reg.*\
 *        		WORD Active;        \* Always non-zero.
 *        		WORD Timer;         \* 0; Count down timer for timing cycle *\
 *        		WORD CurrentColor;  \* Index of color to put in Registers[0] *\
 *        		ULONG Pad0;          \* Unused (space for Colors ptr)*\
 *        		ULONG Pad1;          \* Unused (space for Registers ptr)*\
 *    		} ColorCycles[EGA_NumCycles];
 *    		UBYTE Colors[EGACycleSetHeader[0].TotalColors];
 *    		UBYTE Registers[EGACycleSetHeader[0].TotalRegisters];
 *
 *    		\* EGA Palette *\
 *    		UBYTE EGAPaletteColors[EGA_NumColors];
 *		} PBX;
 *
 * INPUT
 *		
 *		min_read_size:	Minimum amout to read from file.
 *
 *		IMPOTANT: This routine reads data through the FPF routines and
 *			it expects that you have already opened the file and the
 *			FPF reader is at the start of the data.  This routine will
 *			read at least 'size' bytes, unless it gets a read error
 *			or can't allocate memory.
 *
 * AFFECTS
 *		Allocates enough memory to fit entire PaintBoxType struct and
 *		dependent fields etc.
 *
 *
 * RETURN VALUE
 *		On success, returns pointer to allocated PaintBoxType struct.
 *		Check fields of PaintBox to find out what was created from 
 *		PBX data (Palette, CycleSet).  
 *		A NULL field in the paintbox indicates there was no data for that
 *		structure in the PBX to create that structure.
 *
 *		On failure (Unable to allocate memory) returns NULL.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PaintBoxType *ReadPBX (size_t min_read_size)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadPBX";
#endif
#if REQUIRE
#endif
/*-------------------------------------------------------------------------*/
{
	int count;
	PBX_HeaderType		 pbx_head;
	PaintBoxType		*paintbox = NULL;
	PaletteType			*palette = NULL;
	ColorCycleSetType	*cycleset = NULL;
	ColorCycleType		*cycles = NULL;
	void				*pal_colors = NULL;
	void				*cycle_colors = NULL;
	UBYTE				*cycle_regs = NULL;				
	WORD num_pal_colors;
	size_t pal_colors_size;
	WORD num_cycles;
	size_t cycles_size;
	WORD total_cycle_colors;
	size_t cycle_colors_size;
	WORD total_cycle_regs;
	size_t cycle_regs_size;
	size_t total_read_size = 0;
	size_t a_color_size = sizeof (ColorType);

	/* Clear header struct */
	memset (&pbx_head, 0, sizeof (pbx_head));

	/** Allocate paintbox structure **/
	if ((paintbox = (PaintBoxType *)MEM_calloc (1, sizeof (PaintBoxType))) == NULL)
	{
		SetGlobalErr ( ERR_OUT_OF_MEMORY);
		GEprintf ("ReadPBX:OOM allocating paintbox: ");
		goto FAIL_EXIT;
	}

	/* Read PBX Header 
	 * CGA_Code, EGA_NumColors, EGA_NumCycles, 
	 * MGA_NumColors and MGA_NumCycles 
	 */
	if ((FPF_read (&pbx_head, 5 * sizeof (WORD))) !=
		5 * sizeof (WORD)) 
	{
		SetGlobalErr ( ERR_READING_FILE);
		GEcatf ("\nReadPBX:Error reading PBX header:");
		goto FAIL_EXIT;
	}
	total_read_size += 5 * sizeof (WORD);

	if (pbx_head.MGA_NumColors
#if __MSDOS__
		|| (pbx_head.EGA_NumColors && (MODE_EGA == GraphicsMode ||
			MODE_TANDY == GraphicsMode))
		|| (MODE_CGA == GraphicsMode)
#endif	
	)
	{
		/** Allocate Palette **/
		if ((palette = (PaletteType *)MEM_calloc (1, sizeof (PaletteType))) == NULL)
		{
			SetGlobalErr ( ERR_OUT_OF_MEMORY);
			GEprintf ("ReadPBX:OOM allocating palette struct: ");
			goto FAIL_EXIT;
		}
		paintbox->Palette = palette;

#if __MSDOS__
		switch (GraphicsMode) {
		case MODE_CGA: 
			palette->NumColors = NUM_CGA_COLORS;
			palette->Data.CGA = pbx_head.CGA_Code;
			goto DONE;
		case MODE_EGA: case MODE_TANDY:
			num_pal_colors = pbx_head.EGA_NumColors;
			pal_colors_size = pbx_head.EGA_NumColors * sizeof (UBYTE);
			break;
		default:
#endif
			num_pal_colors = pbx_head.MGA_NumColors;
			pal_colors_size = pbx_head.MGA_NumColors * sizeof (ColorType);
#if __MSDOS__
			break;
		}
#endif
		/* Allocate space for palette colors data */
		if ((pal_colors = MEM_calloc (1, pal_colors_size)) == NULL)
		{
			SetGlobalErr ( ERR_OUT_OF_MEMORY);
			GEprintf ("ReadPBX:OOM pal colors: ");
			goto FAIL_EXIT;
		}
	}

	/* Does it have CycleSet header? */
	if (pbx_head.MGA_NumCycles) 
	{
		/* Read MGA_TotalCycleColors and MGA_TotalCycleRegs of Cycle set */
		if ( (FPF_read (&pbx_head.MGA_TotalCycleColors, 2 * sizeof (WORD))) !=
			2 * sizeof (WORD)) 
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEcatf ("\nReadPBX:Error reading PBX CycleSet header:");
			goto FAIL_EXIT;
		}
		total_read_size += 2 * sizeof (WORD);

	}

	num_cycles = pbx_head.MGA_NumCycles;
	cycles_size = pbx_head.MGA_NumCycles 
			* sizeof (ColorCycleType); 
	total_cycle_colors = pbx_head.MGA_TotalCycleColors;
	cycle_colors_size = pbx_head.MGA_TotalCycleColors 
			* sizeof (ColorType);
	total_cycle_regs = pbx_head.MGA_TotalCycleRegs;
	cycle_regs_size = pbx_head.MGA_TotalCycleRegs * sizeof (UBYTE);

#if __MSDOS__
	if (MODE_EGA == GraphicsMode || MODE_TANDY == GraphicsMode) 
	{
		size_t skip_size = cycles_size + cycle_colors_size + 
			cycle_regs_size + (pbx_head.MGA_NumColors * sizeof (ColorType));

		/* Skip over MCGA data */
		if (FPF_skip (skip_size) == -1) 
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEprintf ("ReadPBX: skipping MCGA part: ");
			goto FAIL_EXIT;
		}
		total_read_size += skip_size;

		if (pbx_head.EGA_NumCycles) {
			/* Read EGA_TotalCycleColors and EGA_TotalCycleRegs of Cycle set */
			if ( (FPF_read (&pbx_head.EGA_TotalCycleColors, 2 * sizeof (WORD))) !=
				2 * sizeof (WORD)) 
			{
				SetGlobalErr ( ERR_READING_FILE);
				GEcatf ("\nReadPBX:Error reading EGA CycleSet header:");
				goto FAIL_EXIT;
			}
			total_read_size += 2 * sizeof (WORD);
		}
		num_cycles = pbx_head.EGA_NumCycles;
		cycles_size = pbx_head.EGA_NumCycles 
				* sizeof (ColorCycleType); 
		total_cycle_colors = pbx_head.EGA_TotalCycleColors;
		cycle_colors_size = pbx_head.EGA_TotalCycleColors 
				* sizeof (UBYTE);
		total_cycle_regs = pbx_head.EGA_TotalCycleRegs;
		cycle_regs_size = pbx_head.EGA_TotalCycleRegs * sizeof (UBYTE);
		num_pal_colors = pbx_head.EGA_NumColors;
		pal_colors_size = pbx_head.EGA_NumColors * sizeof (UBYTE);
		
		a_color_size = sizeof (UBYTE);
	}
#endif

	if (cycles_size) 
	{

		/* Allocate space for color cycles data */
		if ((cycles = (ColorCycleType *)MEM_calloc (1, cycles_size)) == NULL)
		{
			SetGlobalErr ( ERR_OUT_OF_MEMORY);
			GEprintf ("ReadPBX:OOM cycles: ");
			goto FAIL_EXIT;
		}
		/* Read cycles into buffer */
		if ( (count = (int) FPF_read (cycles, cycles_size)) == -1 || 
			(unsigned)count != cycles_size)
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEcatf ("\nReadPBX:Error reading cycles data:");
			goto FAIL_EXIT;
		}
		total_read_size += cycles_size;
	}

	if (cycle_colors_size) 
	{
		/* Allocate space for cycle color data */
		if ((cycle_colors = (UBYTE *)MEM_calloc (1, cycle_colors_size)) == NULL)
		{
			SetGlobalErr ( ERR_OUT_OF_MEMORY);
			GEprintf ("ReadPBX:OOM cycle colors: ");
			goto FAIL_EXIT;
		}
		/* Read cycles into buffer */
		if ( (count = (int) FPF_read (cycle_colors, cycle_colors_size)) == -1 || 
			(unsigned)count != cycle_colors_size)
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEcatf ("\nReadPBX:Error reading cycle colors data:");
			goto FAIL_EXIT;
		}
		total_read_size += cycle_colors_size;
	}

	if (cycle_regs_size) {
		/* Allocate space for register data */
		if ((cycle_regs = (UBYTE *)MEM_calloc (1, cycle_regs_size)) == NULL)
		{
			SetGlobalErr ( ERR_OUT_OF_MEMORY);
			GEprintf ("ReadPBX:OOM cycle regs: ");
			goto FAIL_EXIT;
		}

		/* Read data into buffer */
		if ( (count = (int) FPF_read (cycle_regs, cycle_regs_size)) == -1 || 
			(unsigned)count != cycle_regs_size)
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEcatf ("\nReadPBX:Error reading cycle regs data:");
			goto FAIL_EXIT;
		}
		total_read_size += cycle_regs_size;
	}

	/* Set up pointers to data */

	/* Set up color cycles if any */
	if (cycles_size) 
	{
		int i;
		UBYTE *color;
		UBYTE *reg;
		ColorCycleType *cycle;

		/* Allocate Cycle Set struct */
		if ((cycleset = (ColorCycleSetType *)MEM_calloc 
			(1, sizeof (ColorCycleSetType))) == NULL)
		{
			SetGlobalErr ( ERR_OUT_OF_MEMORY);
			GEprintf ("ReadPBX:OOM allocating cycle set struct: ");
			goto FAIL_EXIT;
		}
		paintbox->CycleSet = cycleset;
		cycleset->Active = TRUE;
		cycleset->NumCycles = num_cycles;
		cycleset->TotalColors = total_cycle_colors;
		cycleset->TotalRegisters = total_cycle_regs;
		cycleset->Cycles = cycles;
		cycleset->Colors = cycle_colors;
		cycleset->Registers = cycle_regs;

		/* connect cycles to their color and register data */
		cycle = cycles;
		color = cycle_colors;
		reg = cycle_regs;
		for (i = num_cycles; i; i--) {
			cycle->Colors = color;
			cycle->Registers = reg;
			color += cycle->NumColors * a_color_size;
			reg += cycle->NumRegisters;
			++cycle;
		}
	}

	/* Set up palette if any */
	if (pal_colors_size) 
	{
		/* Read data into buffer */
		if ( (count = (int) FPF_read (pal_colors, pal_colors_size)) == -1 || 
			(unsigned)count != pal_colors_size)
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEcatf ("\nReadPBX:Error reading cycle regs data:");
			goto FAIL_EXIT;
		}
		total_read_size += pal_colors_size;

		palette->NumColors = num_pal_colors;
		/* This should work for EGA to, 'cause its a union of pointers */
		palette->Data.RGB = pal_colors;
	}

DONE: 
	/* Read remaining mimimum read size, if any */
	if (total_read_size < min_read_size) 
	{
		if (FPF_skip (min_read_size - total_read_size) == -1) 
		{
			SetGlobalErr ( ERR_READING_FILE);
			GEprintf ("ReadPBX:reading min_read_size: ");
			goto FAIL_EXIT;
		}
	}

	/* Success */
	return paintbox;

FAIL_EXIT:

	/* Free any memory allocated */
	if (cycleset) MEM_free (cycleset);
	if (cycle_regs) MEM_free (cycle_regs);
	if (cycle_colors) MEM_free (cycle_colors);
	if (cycles) MEM_free (cycles);
	if (pal_colors) MEM_free (pal_colors);
	if (palette) MEM_free (palette);
	if (paintbox) MEM_free (paintbox);

	return NULL;
}
} /* ReadPBX */

