/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * tP16.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 02.060
 *    CREATED : 01/29/92
 *   MODIFIED : 11/08/95
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 *	THIS IS AN UNPUBLISHED WORK CONTAINING ECHIDNA'S CONFIDENTIAL AND
 *	PROPRIETARY INFORMATION. DISCLOSURE, USE OR REPRODUCTION WITHOUT
 *	WRITTEN AUTHORIZATION OF ECHIDNA IS PROHIBITED.
 *
 * DESCRIPTION
 *
 *  Room Types
 *		0 Normal Level
 *		1 Table Room
 *		2 Picture Room
 *		3 Parallax Room
 *		4 Mode7 Room
 *		5 Flat Conversion Room
 *		6 Layered Conversion Room
 *
 *  Tileset Types
 *		0 Image Tiles (16 color SNES or SEGA Genesis)
 *		1 Contour Tiles
 *		2 Special Tiles
 *		3 Object Tiles
 *		4 2 BitPlane Image Tiles (SNES)
 *		5 8 BitPlane Image Tiles (SNES)
 *		6 Mode 7 Image Tiles (SNES)
 *		
 *
 *	SEGA Screen Mem
 *  +-----+-----+-----+-----+-----+-----+-----+-----+
 *  | pri | cp1 | cp0 | vf  | hf  | pt10| pt9 | pt8 |
 *  +-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+
 *    | pt7 | pt6 | pt5 | pt4 | pt3 | pt2 | pt1 | pt0 |
 *    +-----+-----+-----+-----+-----+-----+-----+-----+
 *
 *	SNES Screen Mem
 *  +-----+-----+-----+-----+-----+-----+-----+-----+
 *  | vf  | hf  | pri | cl2 | cl1 | cl0 | nm9 | nm8 |
 *  +-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+
 *    | nm7 | nm6 | nm5 | nm4 | nm3 | nm2 | nm1 | nm0 |
 *    +-----+-----+-----+-----+-----+-----+-----+-----+
 *
 *	SNES Mode 7 Screen/Character Mem
 *  +-----+-----+-----+-----+-----+-----+-----+-----+
 *  | ft7 | ft6 | ft5 | ft4 | ft3 | ft2 | ft1 | ft0 |  Font Data
 *  +-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+
 *    | nm7 | nm6 | nm5 | nm4 | nm3 | nm2 | nm1 | nm0 | Screen Memory
 *    +-----+-----+-----+-----+-----+-----+-----+-----+
 *	
 * HISTORY
 *		01/05/94 (dcc) - fix some small typos
 *2.051	02/12/94 (dcc) - fix animation block not flippable in map
 *2.052	04/10/94 (dcc) - output color cycling information to *.cyc
 *		04/10/94 (dcc) - print error if not 4 or 8 layers in level room
 *		08/03/94 (dcc) - update copyright notice (unpublished)
 *2.053	08/03/94 (dcc) - add support for +T switch (AddFlipToTBL)
 *2.054	08/24/94 (dcc) - add support to read new 'ROOM''LAYR' chunks
 *2.055 09/29/94 (GAT) - merge support for -v1 contour bands 06/08/94
 *		09/29/94 (GAT) - merge support for +u layer 5 object lists 06/08/94
 *		09/29/94 (dcc) - compiles cleanly with 32-bit Borland C++ 4.02
 *		10/01/94 (dcc) - don't merge tilesets with the same names if they
 *						 have different UserTypes or UserNumbers (BEIFFLIB).
 *2.056	11/30/94 (dcc) - Fix error OOM: CharGroup (2) always generated if
 *						 there are more than 937 256-color characters.
 *		11/30/94 (dcc) - Add calls to ReleaseXTRA().
 *2.058	01/22/95 (dcc) - Comment out some unused variables.
 *		01/23/95 (dcc) - Call UpdateXTRA().
 *2.059	06/26/95 (dcc) - Allocate flage for composite rooms so that
 *						 +k switch doesn't crash program.
 *2.060	06/28/95 (dcc) - When I "fixed" animation block not flippable for
 *						 v2.051, I actually broke it. Restored to previous.
 *2.061	07/02/95 (dcc) - Reset animation room tiles' tsi->Flags[] so
 *						 TFLG_CONVERT are no longer set. This way animated
 *						 tiles don't get added to the character set twice.
 *2.062	11/08/95 (dcc) - Abort if tiles other than 8x8 or 16x16 found.
 *
 * SEE ALSO
 *		tP16.DOC -- Word for Windows file.
 *	
 * TODO	
 *		* If char column is not animated then exclude it from frame (don't
 *		  generate a column for it) and then generate it as normal later.
 *
 *		  this would require one bit/flag for each corner of tile so each
 *		  corner could be marked as converted
 *	
 *		* Handle CompTiles in non-0 layers???
 *		* Add Error reports for outside colorsets + FirstColor
 *	
 *		* Check Contours for repeats and deal with XY Symmtry?
 *		* Add support for table rooms like M.C. Kids 1 with
 *		  the number of rows per table specified by GUIDE or by
 *		  room number.  If room number >=2 then room number else
 *		  guide.
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <ctype.h>
#include <limits.h>
#include <echidna/etypes.h>
#include <echidna/version.h>
#include <echidna/listfunc.h>
#include <echidna/datafile.h>
#include <echidna/argparse.h>
#include <echidna/eio.h>
#include <echidna/eerrors.h>
#include <echidna/xtramem.h>
#include <echidna/btume.h>
#include <echidna/strings.h>

#if !defined(FUNC_NAMES)
#define FUNC_NAMES 0
#endif

/**************************** C O N S T A N T S ***************************/

#define GRID_TILES			1

/*
** Tileset Types
*/
#define TILESET_IMAGE		0
#define TILESET_CONTOUR		1
#define TILESET_SPECIAL		2
#define TILESET_OBJECT		3
#define TILESET_2BITS		4
#define	TILESET_8BITS		5
#define TILESET_MODE7		6
#define TILESTE_GLOBAL16	7	// not used here
#define TILESET_LOGIC		8
#define	TILESET_ANGLE		9
#define	TILESET_ELEVATION	10
#define	TILESET_GROUP		11
#define	TILESET_PATH		12
#define	TILESET_THING		13

#define NUM_TILE_TYPES		14

/*
** Room Types
*/
#define	ROOM_LEVEL				0
#define ROOM_TABLE				1
#define ROOM_PICTURE			2
#define ROOM_PARALLAX			3
#define ROOM_MODE7				4
#define ROOM_FLAT_CONVERSION	5
#define ROOM_LAYERED_CONVERSION	6
#define ROOM_BLOCK_TABLE		7
#define ROOM_ANIMATION			8
#define ROOM_REPLACEMENT		9

/*
** Dest Hardware Modes
*/
#define	DEST_GENESIS	0
#define DEST_SNES		1
#define	DEST_GEAR		2

/*
** SEGA Genesis Hardware bits
*/
#define SEGA_XFLIP			0x0800U
#define	SEGA_YFLIP			0x1000U
#define	SEGA_PRIORITY		0x8000U
#define SEGA_COLORSET_MASK	0x6000U
#define SEGA_CHAR_MASK		0x07FFU
#define SEGA_COLORSHIFT		13
#define SEGA_MAX_COLORSET	3

/*
** Super NES Hardware bits
*/
#define SNES_XFLIP			0x4000U
#define	SNES_YFLIP			0x8000U
#define	SNES_PRIORITY		0x2000U
#define SNES_COLORSET_MASK	0x1C00U
#define SNES_CHAR_MASK		0x03FFU
#define SNES_COLORSHIFT		10
#define SNES_MAX_COLORSET	7

/*
** GameGear Hardware bits
*/
#define	GEAR_XFLIP			0x0200U
#define GEAR_YFLIP			0x0400U
#define GEAR_PRIORITY		0x1000U
#define	GEAR_COLORSET_MASK	0x0800U
#define GEAR_CHAR_MASK		0x01FFU
#define GEAR_COLORSHIFT		9
#define GEAR_MAX_COLORSET	1

/*
** tUME Map TileFlag Bits
*/
#define	TUME_XFLIP			0x40
#define	TUME_YFLIP			0x20
#define	TUME_PRIORITY		0x10
#define TUME_COLORSET_FLAG	0x80
#define TUME_COLORSET_MASK	0x07

/*
** Contour Flip Bits
*/
#define CONTOUR_XFLIP		0x8000U
#define CONTOUR_YFLIP		0x4000U
#define CONTOUR_XSYM		0x2000U
#define CONTOUR_YSYM		0x1000U

/*
** 16x16 Tile Flags
*/
#define TFLG_BLANK			0x01	/* Tile Is Blank					*/
#define TFLG_XSYM			0x02	/* X flipping will have no effect	*/
#define TFLG_YSYM			0x04	/* Y flipping will have no effect	*/
#define TFLG_DONTPACK		0x08	/* Tile should not be packed		*/
#define TFLG_CONVERT		0x10	/* Convert this tile to chars		*/
#define TFLG_ANIM			0x20	/* Tile is in anim					*/
#define	TFLG_DONE			0x40	/* Tile has been converted			*/
#define TFLG_TOP			0x80	/* Tile is on top of anim column	*/

/*
** 8x8 Charceter flags
*/
#define CFLG_NOTBLANK		TFLG_BLANK	/* Char is NOT blank			*/
#define CFLG_NOTXSYM		TFLG_XSYM	/* Char is not horiz symetrical	*/
#define CFLG_NOTYSYM		TFLG_YSYM	/* Char is not vert symetrical	*/
#define CFLG_DONTPACK		TFLG_DONTPACK	/* Don't pack this char		*/

#define GROUP_STILE_SIZE	LARGEST_XTRAMEM
#define MAX_GROUP_STILES	(GROUP_STILE_SIZE / sizeof (STile))
#define GROUP_CHAR_SIZE		LARGEST_XTRAMEM

/*
** Contour Types
*/
#define CONTOUR_NOFLIP		0
#define CONTOUR_GENFLIP		1
#define CONTOUR_USEBITS		2
#define CONTOUR_XFLIPBIT	0x0001
#define CONTOUR_YFLIPBIT	0x0002

/*
** File Consts
*/
#define FILE_PALETTE	0
#define FILE_CONTOUR	1
#define FILE_FONT		2
#define FILE_MODE7		3
#define	FILE_TABLE		4
#define	FILE_BLOCK		5
#define FILE_RM_MODE7	6
#define	FILE_RM_LEVEL	7
#define FILE_RM_PARA	8
#define	FILE_RM_PICT	9
#define FILE_FLOOR		10
#define FILE_ANIMATION	11
#define	FILE_OBJECTTBL	12

#define FILE_TYPES		13

/******************************** T Y P E S *******************************/

typedef struct {
	NodeType	 Node;
	UWORD		 X;
	UWORD		 Y;
	UWORD		 Object;
	UWORD		 ObjectNum;
} MapObjType;

/*
** ByteTile
**
** Used to navigate an 8x8 TMGX tile since 8x8 TMGX tiles are stored in words
*/
typedef struct {
	UBYTE	Data;
	UBYTE	pad;
} ByteTile;

/*
** ImageTile
**
** Used to hold 4 characters that make a 16x16 tile.
*/
typedef	struct {
	UWORD	TL;
	UWORD	TR;
	UWORD	BL;
	UWORD	BR;
} ImageTile;

/*
** ContourFlags
**
** Used to track used contour tiles and their symmetry info.
*/
typedef struct {
	UWORD	ContourID;
	UWORD	Flags;
} ContourFlags;

/*
** ContourType
**
** Used to hold the Contour/Special/Object information for a particular tile.
*/
typedef struct {
	UWORD	Contour;
	UWORD	Special;
	UWORD	Object;
} ContourType;

/*
** STile
**
** Used to hold all output information for a particular tile
*/
typedef struct {
	ImageTile	 IM;
	ContourType	 CT;
} STile;

/*
** STileGroup
**
** Used to track all STiles because if we tried to allocate enough
** memory for all possible STiles we would have to allocate
** room->Width * room->Height * 8(colorsets) * 2(priority) * 2(xflip) *
** 2(yflip) * 256(objects) * 256(special) * 65536(contour) * roomcount.
*/
typedef struct {
	NodeType	 Node;
	int			 TileCount;
	XTRAPntr	 STiles;
} STileGroup;

/*
** CharGroup
**
** Used the same as STileGroup but for characters
*/
typedef struct {
	NodeType	 Node;
	int			 CharCount;
	XTRAPntr	 Chars;
} CharGroup;

/*
** TilesetInfo
**
** Used to hold runtime information about each tileset.
*/
typedef struct {
	UWORD		 FirstChar;		/* First Character used in Tileset: NOT USED	*/
	ImageTile	*ImageTiles;	/* if 16x16 tileset, 4 chars that make this tile*/
	UWORD		*ImageChars;	/* if 8x8 tileset, chars that make each tile	*/
	UBYTE		*Flags;			/* Flags for each tile							*/
} TilesetInfo;

/*
** RoomInfo
**
** Used to hold runtime information about each room
*/
typedef struct {
	ListType	 AnimGroupListX;
	ListType	*AnimGroupList;
} RoomInfo;

/*
** AnimGroup
**
** Used to hold linked list of anims that are a certain number of frames
** and from the same room
*/
typedef struct {
	NodeType	 Node;
	ListType	 ColumnListX;
	ListType	*ColumnList;
	int			 Frames;
	int			 TileType;
	int			 XPos;		// first column
	int			 XPos2;		// last column (if not merged)
} AnimGroup;

/*
** MixedAnimGroup
**
** Used to hold framesize and offset of mixed group
**
** Note, this is somewhat bogus.  I should have organized the animations
** being and AnimGroup which contained list of AnimTypes which contained
** a list of AnimColumns but since I didn't I have do use this structure
** at the end to create the info I would have had if I had used the
** better organization.
*/
typedef struct {
	NodeType	 Node;
	int			 Frames;
	int			 FrameCounter;
	unsigned	 CharOffset;
	UWORD		*FrameIDs;
	UWORD		*FrameOffsets;
	UWORD		*FrameSizes;
} MixedAnimGroup;

/*
** AnimColumn
**
** Used to hold one 'column' of cycle data
**
*/
typedef struct {
	NodeType	 Node;
	XTRAPntr	 FrameData;
	UWORD		 CharID;
	BOOL		 UsedFlag;
} AnimColumn;

/*
** SegaChar
**
** Used to hold the graphic data for a SEGA or SNES char.
*/
typedef struct {
	UBYTE		 Data[32];
} SegaChar;

typedef struct {
	UBYTE		 Data[16];
} ContourChar;

typedef struct {
	UBYTE		 Data[16];
} Color4Char;

typedef struct {
	UBYTE		 Data[64];
} Color256Char;

typedef struct {
	UBYTE		 Data[64];
} Mode7Char;

/****************************** G L O B A L S *****************************/

TilesetType		*CurrentTileset;	/* Used for ERROR reporting		*/
unsigned		 CurrentTileNum;	/* Used for ERROR reporting		*/

char			*ContourName;		/* Filename for .CON file		*/
TilesetType		**CTilesTable;		/* QuickRef Contour TSet Table	*/
ContourFlags	*ContourTable;		/* Used Contours				*/
//ContourFlags	*LastContour;		/* Last Contour					*/
unsigned		 NumContours;		/* Number of Contour used		*/

ListType		 STileListX;		/* List of STileGroups			*/
ListType		*STileList = &STileListX;
unsigned		 NumTiles		= 0;

UWORD			 NumChars[NUM_TILE_TYPES];	/* Number of character		*/
UWORD			 MaxChars[NUM_TILE_TYPES];	/* Max number of character	*/
UWORD			 FirstChar[NUM_TILE_TYPES];	/* First Char# of this type	*/
//UWORD			 NumBlocks[NUM_TILE_TYPES];	/* Number of Blocks of this type */
ListType		 CharListX[NUM_TILE_TYPES];	/* Character Lists */
ListType		*CharList[NUM_TILE_TYPES];

int				 charfh;					/* char(font) file handle	*/

int				 LargestBlank;
int				 ContourSize	= 16;
BOOL			 OnlyUsedChars  = FALSE;	/* Font has only used chars	*/
//int				 NumConvTables  = 0;		/* Number of Conversion Tbls*/
int				 ParallaxWidth  = 0;		/* Width of parallax screen */
int				 ParallaxHeight = 0;		/* Height of parallax screen*/
BOOL			 DebugSwitch	= FALSE;	/* Print EXTRA info			*/
BOOL			 WriteTiles		= TRUE;		/* Write Tiles				*/
BOOL			 WriteRooms		= TRUE;		/* Write Rooms				*/
BOOL			 WriteFiles		= TRUE;		/* Write Files				*/
BOOL			 PackFont		= TRUE;		/* Pack the Font			*/
BOOL			 ZeroAnimChars	= FALSE;	/* Zero Anim Chars in Font	*/
BOOL			 InterleaveAni	= FALSE;	/* Interleave Animation		*/
BOOL			 WriteContour	= TRUE;		/* Use & Write Contour info	*/
int				 FirstLetter	= 0;		/* First Character to use	*/
int				 FirstColor		= 0;		/* First Colorset to use	*/
int				 DestHardware	= DEST_GENESIS;	/* SNES or SEGA data	*/
BOOL			 LittleEndian	= FALSE;	/* Word Sex					*/
BOOL			 BinaryOut		= FALSE;	/* All files written as bin	*/
BOOL			 AddFlipToTBL	= FALSE;	/* Add flip/colorset to TBL	*/
BOOL			 TinyMode		= FALSE;	/* Pack Flip/Color/Pri bits	*/
int				 ContourMode	= 0;		/* Allow Flipped Contours	*/
int				 ContourStyle	= 0;		/* Contour Style			*/
BOOL			 DumpMode7		= FALSE;	/* Dump 32K for Mode7		*/
BOOL			 BigSNESTiles	= FALSE;	/* Make 16x16 SNES tiles	*/
BOOL			 JustTop		= 0;		/* Animation Mode			*/
BOOL			 SeparateAnim	= FALSE;	/* Separate Animations		*/
BOOL			 CompressAnim	= TRUE;		/* Compress Animations		*/
BOOL			 MakeObjectList = FALSE;	/* Make Object List			*/
BOOL			 FileSizes		= FALSE;	/* Add File Sizes to files	*/
											/* Room Width/Height or		*/
											/* Tile Counts				*/
BOOL			 MergeLayers	= FALSE;	/* Allow Contour/Special/	*/
											/*   Object tiles to be mixd*/
BOOL			 Other2BitMode  = FALSE;	/* 2Bit tile use colors 0-3,*/
											/* 16-19, 32-35... instead	*/
											/* of 0-3, 4-7, 8-11...		*/

BOOL			 BinFileTable[FILE_TYPES] =	/* Flags for Binary Files	*/
					{
						FALSE,	/* FILE_PALETTE		*/
						FALSE,	/* FILE_CONTOUR		*/
						TRUE,	/* FILE_FONT		*/
						TRUE,	/* FILE_MODE7		*/
						FALSE,	/* FILE_TABLE		*/
						FALSE,	/* FILE_BLOCK		*/
						TRUE,	/* FILE_RM_MODE7	*/
						TRUE,	/* FILE_RM_LEVEL	*/
						TRUE,	/* FILE_RM_PARA		*/
						TRUE,	/* FILE_RM_PICT		*/
						FALSE,	/* FILE_FLOOR		*/
						FALSE,	/* FILE_ANIMATION	*/
						FALSE,	/* FILE_OBJECTTBL	*/
					};

char			*MapName;					/* Filename of Map			*/
char			 TSName[EIO_MAXPATH];		/* Filename of char(font)	*/
char			*OutPath;					/* Output Directory			*/

/*
** Hardware bits, SNES or SEGA, depending on DestHardware flag
*/
UWORD			 HDWR_XFLIP;
UWORD			 HDWR_YFLIP;
UWORD			 HDWR_PRIORITY;
UWORD			 HDWR_COLORSET_MASK;
UWORD			 HDWR_CHAR_MASK;
UWORD			 HDWR_COLORSHIFT;
UWORD			 HDWR_MAX_COLORSET;

/******************************* M A C R O S ******************************/

#define GET_TOGGLE_SWITCH(var,arg)	\
	{ if ((arg)) (var) = (SWITCH_VALUE((arg)) == '+'); }

/******************************* T A B L E S ******************************/

static int MAX_GROUP_CHARS[NUM_TILE_TYPES] =
{
	GROUP_CHAR_SIZE	/ sizeof (SegaChar),		/* TILESET_IMAGE	*/
	GROUP_CHAR_SIZE / sizeof (ContourChar),		/* TILESET_CONTOUR	*/
	0,											/* TILESET_SPECIAL	*/
	0,											/* TILESET_OBJECTS	*/
	GROUP_CHAR_SIZE / sizeof (Color4Char),		/* TILESET_2BITS	*/
	GROUP_CHAR_SIZE / sizeof (Color256Char),	/* TILESET_8BITS	*/
	GROUP_CHAR_SIZE / sizeof (Mode7Char),		/* TILESET_MODE7	*/

	0,0,0,0,0,0,0
};

UWORD BlankChar[NUM_TILE_TYPES];

static	UBYTE ImageTypes[] = {
	1,	/* TILESET_IMAGE	*/
	0,	/* TILESET_CONTOUR	*/
	0,	/* TILESET_SPECIAL	*/
	0,	/* TILESET_OBJECTS	*/
	1,	/* TILESET_2BITS	*/
	1,	/* TILESET_8BITS	*/
	0,	/* TILESET_MODE7	*/
};

/*
** Flags for Colors to Ignore
*/
UBYTE IgnoreColors[256] = {  1, 1, 0, };

/*
** Table of character sizes for different types of characters.
*/
int CharSize[] = {
	sizeof (SegaChar),		/* TILESET_IMAGE	*/
	sizeof (ContourChar),	/* TILESET_CONTOUR	*/
	0,						/* TILESET_SPECIAL	*/
	0,						/* TILESET_OBJECTS	*/
	sizeof (Color4Char),	/* TILESET_2BITS	*/
	sizeof (Color256Char),	/* TILESET_8BITS	*/
	sizeof (Mode7Char),		/* TILESET_MODE7	*/
};

static	UBYTE AnimOrder[] = {
	2,	/* TILESET_IMAGE	*/
	0,	/* TILESET_CONTOUR	*/
	0,	/* TILESET_SPECIAL	*/
	0,	/* TILESET_OBJECTS	*/
	1,	/* TILESET_2BITS	*/
	3,	/* TILESET_8BITS	*/
	4,	/* TILESET_MODE7	*/
};

/*
** Just some zero data for writing to a file
*/
UBYTE ZeroData[256];

/*
** FlipTable for SEGA graphic data.  Given a number from 0 <-> 255
** representing SEGA graphic data, what number would you need to repsent
** the same graphics data horizontally flipped.
*/
UBYTE FlipTable[] = {
	0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x80,0x90,0xa0,0xb0,0xc0,0xd0,0xe0,0xf0,
	0x01,0x11,0x21,0x31,0x41,0x51,0x61,0x71,0x81,0x91,0xa1,0xb1,0xc1,0xd1,0xe1,0xf1,
	0x02,0x12,0x22,0x32,0x42,0x52,0x62,0x72,0x82,0x92,0xa2,0xb2,0xc2,0xd2,0xe2,0xf2,
	0x03,0x13,0x23,0x33,0x43,0x53,0x63,0x73,0x83,0x93,0xa3,0xb3,0xc3,0xd3,0xe3,0xf3,
	0x04,0x14,0x24,0x34,0x44,0x54,0x64,0x74,0x84,0x94,0xa4,0xb4,0xc4,0xd4,0xe4,0xf4,
	0x05,0x15,0x25,0x35,0x45,0x55,0x65,0x75,0x85,0x95,0xa5,0xb5,0xc5,0xd5,0xe5,0xf5,
	0x06,0x16,0x26,0x36,0x46,0x56,0x66,0x76,0x86,0x96,0xa6,0xb6,0xc6,0xd6,0xe6,0xf6,
	0x07,0x17,0x27,0x37,0x47,0x57,0x67,0x77,0x87,0x97,0xa7,0xb7,0xc7,0xd7,0xe7,0xf7,
	0x08,0x18,0x28,0x38,0x48,0x58,0x68,0x78,0x88,0x98,0xa8,0xb8,0xc8,0xd8,0xe8,0xf8,
	0x09,0x19,0x29,0x39,0x49,0x59,0x69,0x79,0x89,0x99,0xa9,0xb9,0xc9,0xd9,0xe9,0xf9,
	0x0a,0x1a,0x2a,0x3a,0x4a,0x5a,0x6a,0x7a,0x8a,0x9a,0xaa,0xba,0xca,0xda,0xea,0xfa,
	0x0b,0x1b,0x2b,0x3b,0x4b,0x5b,0x6b,0x7b,0x8b,0x9b,0xab,0xbb,0xcb,0xdb,0xeb,0xfb,
	0x0c,0x1c,0x2c,0x3c,0x4c,0x5c,0x6c,0x7c,0x8c,0x9c,0xac,0xbc,0xcc,0xdc,0xec,0xfc,
	0x0d,0x1d,0x2d,0x3d,0x4d,0x5d,0x6d,0x7d,0x8d,0x9d,0xad,0xbd,0xcd,0xdd,0xed,0xfd,
	0x0e,0x1e,0x2e,0x3e,0x4e,0x5e,0x6e,0x7e,0x8e,0x9e,0xae,0xbe,0xce,0xde,0xee,0xfe,
	0x0f,0x1f,0x2f,0x3f,0x4f,0x5f,0x6f,0x7f,0x8f,0x9f,0xaf,0xbf,0xcf,0xdf,0xef,0xff,
};

UBYTE SNESFlipTable[] = {
	0x00,0x80,0x40,0xc0,0x20,0xa0,0x60,0xe0,0x10,0x90,0x50,0xd0,0x30,0xb0,0x70,0xf0,
	0x08,0x88,0x48,0xc8,0x28,0xa8,0x68,0xe8,0x18,0x98,0x58,0xd8,0x38,0xb8,0x78,0xf8,
	0x04,0x84,0x44,0xc4,0x24,0xa4,0x64,0xe4,0x14,0x94,0x54,0xd4,0x34,0xb4,0x74,0xf4,
	0x0c,0x8c,0x4c,0xcc,0x2c,0xac,0x6c,0xec,0x1c,0x9c,0x5c,0xdc,0x3c,0xbc,0x7c,0xfc,
	0x02,0x82,0x42,0xc2,0x22,0xa2,0x62,0xe2,0x12,0x92,0x52,0xd2,0x32,0xb2,0x72,0xf2,
	0x0a,0x8a,0x4a,0xca,0x2a,0xaa,0x6a,0xea,0x1a,0x9a,0x5a,0xda,0x3a,0xba,0x7a,0xfa,
	0x06,0x86,0x46,0xc6,0x26,0xa6,0x66,0xe6,0x16,0x96,0x56,0xd6,0x36,0xb6,0x76,0xf6,
	0x0e,0x8e,0x4e,0xce,0x2e,0xae,0x6e,0xee,0x1e,0x9e,0x5e,0xde,0x3e,0xbe,0x7e,0xfe,
	0x01,0x81,0x41,0xc1,0x21,0xa1,0x61,0xe1,0x11,0x91,0x51,0xd1,0x31,0xb1,0x71,0xf1,
	0x09,0x89,0x49,0xc9,0x29,0xa9,0x69,0xe9,0x19,0x99,0x59,0xd9,0x39,0xb9,0x79,0xf9,
	0x05,0x85,0x45,0xc5,0x25,0xa5,0x65,0xe5,0x15,0x95,0x55,0xd5,0x35,0xb5,0x75,0xf5,
	0x0d,0x8d,0x4d,0xcd,0x2d,0xad,0x6d,0xed,0x1d,0x9d,0x5d,0xdd,0x3d,0xbd,0x7d,0xfd,
	0x03,0x83,0x43,0xc3,0x23,0xa3,0x63,0xe3,0x13,0x93,0x53,0xd3,0x33,0xb3,0x73,0xf3,
	0x0b,0x8b,0x4b,0xcb,0x2b,0xab,0x6b,0xeb,0x1b,0x9b,0x5b,0xdb,0x3b,0xbb,0x7b,0xfb,
	0x07,0x87,0x47,0xc7,0x27,0xa7,0x67,0xe7,0x17,0x97,0x57,0xd7,0x37,0xb7,0x77,0xf7,
	0x0f,0x8f,0x4f,0xcf,0x2f,0xaf,0x6f,0xef,0x1f,0x9f,0x5f,0xdf,0x3f,0xbf,0x7f,0xff,
};

/***************************** R O U T I N E S ****************************/


/*********************************************************************
 *
 * ParseAnimArgs
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int ParseAnimArgs (char *s)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ParseAnimArgs";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		BOOL	 done	= FALSE;
		BOOL	 value	= FALSE;
		ULONG	 id		= 0;

#define Arg3ID(a,b,c)	((((ULONG)(a))<<16)|(((ULONG)(b))<<8)|((ULONG)(c)))

		while (!done)
		{
			if (!*s || *s == '+' || *s == '-')
			{
				if (id)
				{

					switch (id)
					{
					case Arg3ID('M','R','G'):	SeparateAnim  = !value;	break;
					case Arg3ID('C','M','P'):	CompressAnim  =  value;	break;
					case Arg3ID('I','N','T'):	InterleaveAni =  value;	break;
					case Arg3ID('T','O','P'):	JustTop       =  value;	break;
					case Arg3ID('Z','R','O'):	ZeroAnimChars =  value;	break;
					default:
						ErrMess ("Bad -l parameter\n");
						break;
					}
					
					id = 0;
				}
				if (*s == '+')
				{
					value = TRUE;
				}
				if (*s == '-')
				{
					value = FALSE;
				}
				if (!*s)
				{
					done = TRUE;
				}
			}
			if (isalpha (*s))
			{
				id = (id << 8) | toupper (*s);
			}
			s++;
		}
	}
	return (!ErrorCount);
} /* ParseAnimArgs */


/*********************************************************************
 *
 * WritePalette
 *
 * PURPOSE
 *		Writes palette data to a file.  Write 64 color if SEGA
 *		and 128 if SNES.
 *
 * INPUT
 *		name	: Name of map.
 *		cr		: Pointer to color palette of room.
 *
 * RETURN VALUE
 *		NONE.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void WritePalette (char *name, ColorRegister *cr, int roomtiletype)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WritePalette";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (WriteFiles) {
		FILE			*fp;
		int				 i;
		int				 j;
		char			 rmfile[EIO_MAXPATH];

		EIO_fnmerge (rmfile, OutPath, name, ".pal");

		printf ("Writing Palette '%s'\n", rmfile);

		BinaryOut = BinFileTable[FILE_PALETTE];


		
		if (BinaryOut) {
			fp = fopen (rmfile, "wb");
		} else {
			fp = fopen (rmfile, "w");
		}
		if (!fp) {
			FailMess ("Couldn't open file '%s'\n", rmfile);
		}

		if (!BinaryOut) {
			fprintf (fp,
				";*\n"
				";* %s\n"
				";*\n"
				";* Palette generated by tP16\n"
				";*\n",
				rmfile
				);
		}

		switch (DestHardware)
		{
		case DEST_SNES:
			{
				UWORD	work;
				int		sets;

				switch (roomtiletype) {
				case TILESET_IMAGE:
					sets = 16;
					break;
				case TILESET_2BITS:
					sets = 4;
					break;
				case TILESET_8BITS:
					sets = 32;
					break;
				case TILESET_MODE7:
					sets = 32;
					break;
				default:
					sets = 16;
					break;
				}

				for (i = 0; i < sets / 2; i++) {
					if (!BinaryOut) {
						fprintf (fp, "\tHEX\t");
					}
					for (j = 0; j < 15; j++) {
						work = (UWORD)
							   (
								((cr->red   >> 3) <<  0) |
								((cr->green >> 3) <<  5) |
								((cr->blue  >> 3) << 10)
							   );
						if (!BinaryOut) {
							fprintf (fp,
								"%02x%02x",
								work & 0xFF,
								work >> 8
								);
						} else {
							fputc (work & 0xFF, fp);
							fputc (work >> 8, fp);
						}
						cr++;
					}
					work = (UWORD)
						   (
							((cr->red   >> 3) <<  0) |
							((cr->green >> 3) <<  5) |
							((cr->blue  >> 3) << 10)
						   );
				
					if (!BinaryOut) {
						fprintf (fp,
							"%02x%02x\t;%d-%d\n",
							work & 0xFF,
							work >> 8,
							i * 8,
							i * 8 + 7
							);
					} else {
						fputc (work & 0xFF, fp);
						fputc (work >> 8, fp);
					}

					cr++;
				}
			}
			break;
		case DEST_GENESIS:
			{
				for (i = 0; i < 8; i++) {
					if (!BinaryOut) {
						fprintf (fp, "\tPALETTE ");
					}
					for (j = 0; j < 7; j++) {
						if (!BinaryOut) {
							fprintf (fp,
								"$%x%x%x,",
								cr->red   >> 5,
								cr->green >> 5,
								cr->blue  >> 5
								);
						} else {
							UWORD	work;

							work = (UWORD)
								   (
									(((cr->red   >> 4) & 0x0E) << 0) |
									(((cr->green >> 4) & 0x0E) << 4) |
									(((cr->blue  >> 4) & 0x0E) << 8)
								   );

							fputc (work >> 8, fp);
							fputc (work & 0xFF, fp);
						}
						cr++;
					}
					if (!BinaryOut) {
						fprintf (fp,
							"$%x%x%x\t;Pens %d-%d\n",
							cr->red   >> 5,
							cr->green >> 5,
							cr->blue  >> 5,
							i * 8,
							i * 8 + 7
							);
					} else {
						UWORD	work;

						work = (UWORD)
							   (
								(((cr->red   >> 4) & 0x0E) << 0) |
								(((cr->green >> 4) & 0x0E) << 4) |
								(((cr->blue  >> 4) & 0x0E) << 8)
							   );

						fputc (work >> 8, fp);
						fputc (work & 0xFF, fp);
					}
					cr++;
				}
			}
			break;
		case DEST_GEAR:
			{
				UWORD	work;
				int		sets;

				sets = 4;
				for (i = 0; i < sets / 2; i++) {
					if (!BinaryOut) {
						fprintf (fp, "\tPALETTE\t");
					}
					for (j = 0; j < 15; j++) {
						work = (UWORD)
							   (
								((cr->red   >> 4) <<  0) |
								((cr->green >> 4) <<  4) |
								((cr->blue  >> 4) <<  8)
							   );
						if (!BinaryOut) {
							fprintf (fp,
								"$%d%d%d,",
								cr->red   >> 4,
								cr->green >> 4,
								cr->blue  >> 4
								);
						} else {
							fputc (work & 0xFF, fp);
							fputc (work >> 8, fp);
						}
						cr++;
					}
					work = (UWORD)
						   (
							((cr->red   >> 4) <<  0) |
							((cr->green >> 4) <<  4) |
							((cr->blue  >> 4) <<  8)
						   );
				
					if (!BinaryOut) {
							fprintf (fp,
								"$%d%d%d\t;Pens %d-%d\n",
								cr->red   >> 4,
								cr->green >> 4,
								cr->blue  >> 4,
								i * 8,
								i * 8 + 7
								);
					} else {
						fputc (work & 0xFF, fp);
						fputc (work >> 8, fp);
					}

					cr++;
				}
			}
			break;
		}

		fclose (fp);

	}
} /* WritePalette */


/*********************************************************************
 *
 * WriteColorCycle
 *
 * PURPOSE
 *		Write the color cycle to a text file <room_name>.cyc.
 *
 *		NOTE: this routine currently ignores the BINARY switch.
 *
 * INPUT
 *		char *szFile		: file name to write
 *		CycleInfo **rgpcyc	: array of pointers to CycleInfo
 *		UWORD ccyc			: number of cycle info for current room
 *
 * ASSUMES
 *		
 *
 * RETURN VALUE
 *		NONE.
 *
 * HISTORY
 *		04/10/94 Sunday (dcc) - created.
 *
*/
void WriteColorCycle(char *szFile, CycleInfo **rgpcyc, UWORD ccyc)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteColorCycle";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (ccyc && WriteFiles)
	{
		int				 i;
		FILE			*hfilOut;
		char			 szCyc[EIO_MAXPATH];

		EIO_fnmerge (szCyc, OutPath, szFile, ".cyc");

		printf("Writing Color cycle '%s'\n", szCyc);

		hfilOut = fopen(szCyc, "w");

		fprintf(hfilOut, 
				";*\n"
				";* %s\n"
				";*\n"
				";* Color cycling information generated by tP16\n"
				";*\n"
				, szCyc);

		fprintf(hfilOut,
				"\n"
				"\t; array of offsets to color cycling data structures\n"
				"\n"
				);

		for (i = 0; i < ccyc; i++)
		{
			fprintf(hfilOut, "\tOFFSET\tCYC%02x\n", i);
		}

		for (i = 0; i < ccyc; i++)
		{
			int j;

			fprintf(hfilOut,
					"\n"
					"\t; color cycling speed, direction, # of colors, # of registers\n"
					"\n"
					);

			fprintf(hfilOut,
					"CYC%02x:\n"
					"\tCYCLE\t$%04x, $%04x, $%04x, $%04x\n",
					i,
					rgpcyc[i]->Speed,
					rgpcyc[i]->Direction,
					rgpcyc[i]->NumColors,
					rgpcyc[i]->NumRegs
					);

			fprintf(hfilOut,
					"\n"
					"\t; array of offsets to each color cycling frame\n"
					"\n"
					);

			for (j = 0; j < rgpcyc[i]->NumColors; j++)
			{
				fprintf(hfilOut, "\tOFFSET\tFRM%02x_%02x\n", i, j);
			}

			for (j = 0; j < rgpcyc[i]->NumColors; j++)
			{
				int k, l;

				fprintf(hfilOut, "\nFRM%02x_%02x:\n", i, j);

				for (k = 0; k < rgpcyc[i]->NumRegs; k++)
				{
					UWORD work;

					l = (k + j) % rgpcyc[i]->NumColors;
					switch (DestHardware)
					{
					case DEST_SNES:
						work = (UWORD)
							   (
								((rgpcyc[i]->Colors[l].red   >> 3) <<  0) |
								((rgpcyc[i]->Colors[l].green >> 3) <<  5) |
								((rgpcyc[i]->Colors[l].blue  >> 3) << 10)
							   );
						break;
					case DEST_GENESIS:
						work = (UWORD)
							   (
								(((rgpcyc[i]->Colors[l].red   >> 4) & 0x0E) << 0) |
								(((rgpcyc[i]->Colors[l].green >> 4) & 0x0E) << 4) |
								(((rgpcyc[i]->Colors[l].blue  >> 4) & 0x0E) << 8)
							   );
						break;
					case DEST_GEAR:
						work = (UWORD)
							   (
								((rgpcyc[i]->Colors[l].red   >> 4) << 0) |
								((rgpcyc[i]->Colors[l].green >> 4) << 4) |
								((rgpcyc[i]->Colors[l].blue  >> 4) << 8)
							   );
						break;
					}
					fprintf(hfilOut,
							"\tSET\t$%04x, $%04x\n",
							rgpcyc[i]->Registers[k],
							work
							);
				}
			}
		}
	}
} /* WriteColorCycle */


/*********************************************************************
 *
 * ConvertSEGAtoSNES
 *
 * PURPOSE
 *		Converts one SEGA 8x8 chuck graphic character to one
 *		SNES 8x8 planar graphic character.
 *
 * INPUT
 *		sega	: pointer to a SegaChar.
 *
 *
 * EFFECTS
 *		SEGA characters are stored 2 pixels per bytes.  Bits 7-4 are
 *		left pixel, Bits 3-0 are right pixel.
 *	
 *		SNES characters are stored planar, first byte is 0 bits of
 *		top 8 pixels. Second byte is 1 bits of top 8 pixels.
 *		16th byte is 2 bits of top 8 pixels. 17th byte is 3 bits of
 *		top 8 pixels.
 *	
 * RETURN VALUE
 *		NONE
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ConvertSEGAtoSNES (SegaChar *sega)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ConvertSEGAtoSNES";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		SegaChar	 SNES;
		UBYTE		*p01 = &SNES.Data[ 0];
		UBYTE		*p23 = &SNES.Data[16];
		UBYTE		 d0;
		UBYTE		 d1;
		UBYTE		 d2;
		UBYTE		 d3;
		UBYTE		*src = sega->Data;
		int			 y;

		for (y = 0; y < 8; y++) {
			d0 = *src++;
			d1 = *src++;
			d2 = *src++;
			d3 = *src++;
			
			*p01++ = (UBYTE)
					 (
						((d0 & 0x10) << 3) |
						((d0 & 0x01) << 6) |
						((d1 & 0x10) << 1) |
						((d1 & 0x01) << 4) |
						((d2 & 0x10) >> 1) |
						((d2 & 0x01) << 2) |
						((d3 & 0x10) >> 3) |
						((d3 & 0x01)     )
					 );

			*p01++ = (UBYTE)
					 (
						((d0 & 0x20) << 2) |
						((d0 & 0x02) << 5) |
						((d1 & 0x20)     ) |
						((d1 & 0x02) << 3) |
						((d2 & 0x20) >> 2) |
						((d2 & 0x02) << 1) |
						((d3 & 0x20) >> 4) |
						((d3 & 0x02) >> 1)
					 );

			*p23++ = (UBYTE)
					 (
						((d0 & 0x40) << 1) |
						((d0 & 0x04) << 4) |
						((d1 & 0x40) >> 1) |
						((d1 & 0x04) << 2) |
						((d2 & 0x40) >> 3) |
						((d2 & 0x04)     ) |
						((d3 & 0x40) >> 5) |
						((d3 & 0x04) >> 2)
					 );

			*p23++ = (UBYTE)
					 (
						((d0 & 0x80)     ) |
						((d0 & 0x08) << 3) |
						((d1 & 0x80) >> 2) |
						((d1 & 0x08) << 1) |
						((d2 & 0x80) >> 4) |
						((d2 & 0x08) >> 1) |
						((d3 & 0x80) >> 6) |
						((d3 & 0x08) >> 3)
					 );
		}

		*sega = SNES;
	}
} /* ConvertSEGAtoSNES */


/*********************************************************************
 *
 * ConvertSEGAtoGEAR
 *
 * PURPOSE
 *		Converts one SEGA 8x8 chuck graphic character to one
 *		GEAR 8x8 planar graphic character.
 *
 * INPUT
 *		sega	: pointer to a SegaChar.
 *
 *
 * EFFECTS
 *		SEGA characters are stored 2 pixels per bytes.  Bits 7-4 are
 *		left pixel, Bits 3-0 are right pixel.
 *	
 *		GEAR characters are stored planar, first byte is 0 bits of
 *		top 8 pixels. Second byte is 1 bits of top 8 pixels.
 *		3th byte is 2 bits of top 8 pixels. 4th byte is 3 bits of
 *		top 8 pixels.
 *	
 * RETURN VALUE
 *		NONE
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ConvertSEGAtoGEAR (SegaChar *sega)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ConvertSEGAtoGEAR";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		SegaChar	 GEAR;
		UBYTE		*p = &GEAR.Data[0];
		UBYTE		 d0;
		UBYTE		 d1;
		UBYTE		 d2;
		UBYTE		 d3;
		UBYTE		*src = sega->Data;
		int			 y;

		for (y = 0; y < 8; y++) {
			d0 = *src++;
			d1 = *src++;
			d2 = *src++;
			d3 = *src++;
			
			*p++ = (UBYTE)
				   (
					((d0 & 0x10) << 3) |
					((d0 & 0x01) << 6) |
					((d1 & 0x10) << 1) |
					((d1 & 0x01) << 4) |
					((d2 & 0x10) >> 1) |
					((d2 & 0x01) << 2) |
					((d3 & 0x10) >> 3) |
					((d3 & 0x01)     )
				   );

			*p++ = (UBYTE)
				   (
					((d0 & 0x20) << 2) |
					((d0 & 0x02) << 5) |
					((d1 & 0x20)     ) |
					((d1 & 0x02) << 3) |
					((d2 & 0x20) >> 2) |
					((d2 & 0x02) << 1) |
					((d3 & 0x20) >> 4) |
					((d3 & 0x02) >> 1)
				   );

			*p++ = (UBYTE)
				   (
					((d0 & 0x40) << 1) |
					((d0 & 0x04) << 4) |
					((d1 & 0x40) >> 1) |
					((d1 & 0x04) << 2) |
					((d2 & 0x40) >> 3) |
					((d2 & 0x04)     ) |
					((d3 & 0x40) >> 5) |
					((d3 & 0x04) >> 2)
				   );

			*p++ = (UBYTE)
				   (
					((d0 & 0x80)     ) |
					((d0 & 0x08) << 3) |
					((d1 & 0x80) >> 2) |
					((d1 & 0x08) << 1) |
					((d2 & 0x80) >> 4) |
					((d2 & 0x08) >> 1) |
					((d3 & 0x80) >> 6) |
					((d3 & 0x08) >> 3)
				   );
		}

		//
		// **COPY** local data to destination character
		//
		*sega = GEAR;
	}
} /* ConvertSEGAtoGEAR */


/*********************************************************************
 *
 * CopyChar
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
BOOL CopyChar (int ndx, void *dest, int tiletype)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CopyChar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	CharGroup	*chgrp;

	chgrp = Head (CharList[tiletype]);
	while (!EndOfList(chgrp))
	{
		if (ndx < chgrp->CharCount)
		{
			UBYTE	*data;

			data = ActivateXTRA (chgrp->Chars);
			memcpy (dest, data + CharSize[tiletype] * ndx, CharSize[tiletype]);
			ReleaseXTRA (chgrp->Chars);
			return TRUE;
		}
		ndx  -= chgrp->CharCount;
		chgrp = Next (chgrp);
	}
	return FALSE;
}
} /* CopyChar */


/*********************************************************************
 *
 * FindChar
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *2.056	11/30/94 (dcc) - Compare with MAX_GROUP_CHARS[]-1 instead of
 *						 MAX_GROUP_CHARS[].
 *
 * SEE ALSO
 *
*/
CharGroup *FindChar (
		void	*normchar,
		void	*xflipchar,
		void	*yflipchar,
		void	*xyflipchar,
		int		 tiletype,
		UWORD	*cID,
		UWORD	*flipBits,
		BOOL	*sameOut)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FindChar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	CharGroup		*chgrp;
	UBYTE huge32	*chkptr;
	UWORD			 c		  = 0;
	BOOL			 same	  = FALSE;
	UWORD			 FlipBits = 0x0000;
	int				 size     = CharSize[tiletype];
	int				 ch;

	chgrp = Head (CharList[tiletype]);
	while (!EndOfList (chgrp))
	{
		chkptr = ActivateXTRA (chgrp->Chars);

		for (ch = 0; ch < chgrp->CharCount; ch++) {
			if (!memcmp ((void *)chkptr, normchar, size)) {
				same = TRUE;
				break;
			}
			if (xflipchar && !memcmp ((void *)chkptr, xflipchar, size)) {
				FlipBits = HDWR_XFLIP;
				same     = TRUE;
				break;
			}
			if (yflipchar && !memcmp ((void *)chkptr, yflipchar, size)) {
				FlipBits = HDWR_YFLIP;
				same     = TRUE;
				break;
			}
			if (xyflipchar && !memcmp ((void *)chkptr, xyflipchar, size)) {
				FlipBits = HDWR_XFLIP | HDWR_YFLIP;
				same     = TRUE;
				break;
			}
			chkptr += size;
			c++;
		}
		ReleaseXTRA(chgrp->Chars);

		if (same || (ch == chgrp->CharCount && chgrp->CharCount != MAX_GROUP_CHARS[tiletype]-1))
		{
			break;
		}
		else
		{
			chgrp = Next (chgrp);
		}
	}

	*sameOut  = same;
	*cID      = c;
	*flipBits = FlipBits;

	return chgrp;
}
} /* FindChar */


/*********************************************************************
 *
 * AddGroupChar
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void AddGroupChar (CharGroup *chgrp, void *normchar, int tiletype, int c)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddGroupChar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	UBYTE			*chr;
	int				 size = CharSize[tiletype];

	if (!chgrp)
	{
		chgrp = Head (CharList[tiletype]);
	}

	if (EndOfList (chgrp) || c == MAX_GROUP_CHARS[tiletype]) {
		chgrp = CreateNode (sizeof (CharGroup), NULL);
		if (!chgrp) {
			OutOfMemErr ("OOM: CharGroup\n");
		}
		chgrp->Chars = AllocXTRA (GROUP_CHAR_SIZE);
		if (!chgrp->Chars) {
			OutOfMemErr ("OOM: CharGroup (2)\n");
		}
		AddTail (CharList[tiletype], chgrp);
	}
	chr  = ActivateXTRA (chgrp->Chars);
	chr += chgrp->CharCount * size;
	memcpy (chr, normchar, size);
	UpdateXTRA (chgrp->Chars);
	chgrp->CharCount++;
	NumChars[tiletype]++;
	if (DebugSwitch) {
		printf ("type %d chars = %d\r", tiletype, NumChars[tiletype]);
	}
}
} /* AddGroupChar */


/*********************************************************************
 *
 * AddCharImage
 *
 * PURPOSE
 *		Converts 256 color 8x8 piece of tUME graphic data to
 *		an 16 color 8x8 SEGA character with a colorset from 0 <-> 7.
 *
 *		Checks for some color errors like using colors from two different
 *		colorsets.
 *
 *		Generates x flipped, y flipped and xy flipped versions of the
 *		character.
 *
 *		Checks for and Flags Blank characters and symetry of characters.
 *
 *		Checks Character against all previously generated characters
 *		to see if an indentical character has already been created
 *		including flipped characeters.
 *
 *		If character is unique it is added to list/pile of characters.
 *
 *		Returns SEGA character number with flipbits and colorset bits
 *		set.
 *
 * INPUT
 *		data	: pointer to tUME graphic data.
 *		modulo	: bytes to skip to get from end of row to start of next.
 *		schar	: pointer to UWORD to fill in with SEGA character info
 *		packit	: FALSE = don't look of duplicate character just add
 *				  this character unconditionally.	
 *
 *
 * RETURN VALUE
 *		returns flags for character.  CFLG_NOTXSYM, CFLG_NOTYSYM, CFLG_NOTBLANK
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UBYTE AddCharImage (
		UBYTE	*data,
		int		 modulo,
		UWORD	*schar,
		BOOL	 packit,
		int		 tiletype,
		void	**norm,
		void	**xflp,
		void	**yflp,
		void	**xyflp
		)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddCharImage";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		static SegaChar	 NewChar;
		static SegaChar	 XFlipped;
		static SegaChar	 XYFlipped;
		static SegaChar	 YFlipped;
		int			 y1;
		int			 x1;
		int			 color;
		int			 bcolor;
		UBYTE		*newptr;
		UBYTE		 zero = 0x00;
		UBYTE		 flags;
		UBYTE		 cerr = FALSE;

		/*
		** Store new data at end of character table/pile/list
		*/
		newptr = (UBYTE far32 *)&NewChar;
		color  = -1;
		bcolor = -1;


		/*
		** Convert tUME graphic data to SEGA graphic data
		*/
		for (y1 = 0; y1 < 8; y1++) {
			for (x1 = 0; x1 < 4; x1++) {
				UBYTE	p0;
				UBYTE	p1;
				UBYTE	c0;
				UBYTE	c1;

				p0 = (UBYTE) (*data   & 0x0F);
				c0 = (UBYTE) ((*data++ & 0xF0) >> 4);

				p1 = (UBYTE) (*data   & 0x0F);
				c1 = (UBYTE) ((*data++ & 0xF0) >> 4);

				*newptr++  = (UBYTE) ((p0 << 4) | p1);
				zero      |= p0 | p1;

				/*
				** Ignore colors 0 and 1 of each colorset
				** when determining color of character.
				*/

				if (IgnoreColors[p0]) {
					if (p0 && c0 && bcolor == -1) {
						bcolor = c0;
					}
					c0 = 0;
				}
				if (IgnoreColors[p1]) {
					if (p1 && c1 && bcolor == -1) {
						bcolor = c1;
					}
					c1 = 0;
				}

				/*
				** Track colorset and check for errors.
				*/
				if (c0 || c1) {
					if (c0 && c1 && (c0 != c1)) {
						if (!cerr) {
							cerr = TRUE;
							WarnMess ("Bad colors in tileset '%s' tile #%d\n",
								EIO_Filename(CurrentTileset->FileSpec),
								CurrentTileNum);
						}
					}
					if (c0) {
						if (c0 > HDWR_MAX_COLORSET) {
							if (!cerr) {
								cerr = TRUE;
								WarnMess ("Bad colors in tileset '%s' tile #%d\n",
									EIO_Filename(CurrentTileset->FileSpec),
									CurrentTileNum);
							}
						} else {
							if (color == -1) {
								color = c0;
							} else if (color != c0) {
								if (!cerr) {
									cerr = TRUE;
									WarnMess ("Bad colors in tileset '%s' tile #%d\n",
										EIO_Filename(CurrentTileset->FileSpec),
										CurrentTileNum);
								}
							}
						}
					} else {
						if (c1 > HDWR_MAX_COLORSET) {
							if (!cerr) {
								cerr = TRUE;
								WarnMess ("Bad colors in tileset '%s' tile #%d\n",
									EIO_Filename(CurrentTileset->FileSpec),
									CurrentTileNum);
							}
						} else {
							if (color == -1) {
								color = c1;
							} else if (color != c1) {
								if (!cerr) {
									cerr = TRUE;
									WarnMess ("Bad colors in tileset '%s' tile #%d\n",
										EIO_Filename(CurrentTileset->FileSpec),
										CurrentTileNum);
								}
							}
						}
					}									
				}
			}
			data += modulo;
		}

		/*
		** If no non-zero colorset was found then it must be colorset 0
		*/
		if (color == (-1)) {
			if (bcolor == (-1)) {
				color = 0;
			} else {
				color = bcolor;
			}
		}

		if (norm || (PackFont && packit)) {
			/*
			** Flip character to check for flipped chars.
			*/
			{
				int		 i;
				int		 j;
				UBYTE	*orig;
				UBYTE	*x;
				UBYTE	*y;
				UBYTE	*xy;
				UBYTE	 flip;

				orig = (UBYTE far32 *)&NewChar;
				x    = XFlipped.Data  + 3;
				y    = YFlipped.Data  + sizeof (SegaChar) - 4;
				xy   = XYFlipped.Data + sizeof (SegaChar) - 1;

				for (i = 0; i < 8; i++) {
					for (j =0; j < 4; j++) {
						flip = FlipTable[*orig];
						*x  = flip;
						*y  = *orig;
						*xy = flip;

						orig++;
						x--;
						y++;
						xy--;
					}
					x  += 8;
					y  -= 8;
				}			
			}
			{
				UBYTE	xsym;
				UBYTE	ysym;

				xsym = (UBYTE) (!memcmp ((void *)&NewChar, (void *)&XFlipped, sizeof (SegaChar)));
				ysym = (UBYTE) (!memcmp ((void *)&NewChar, (void *)&YFlipped, sizeof (SegaChar)));

				flags = (UBYTE)
						(
						 ((zero) ? 0 : CFLG_NOTBLANK) |
						 ((xsym) ? 0 : CFLG_NOTXSYM ) |
						 ((ysym) ? 0 : CFLG_NOTYSYM )
						);

			}
		} else {
			flags = CFLG_NOTBLANK | CFLG_NOTXSYM | CFLG_NOTYSYM;
		}



		if (!norm) {
			UWORD			 c;
			BOOL			 same;
			UWORD			 FlipBits = 0x0000;
			CharGroup		*chgrp    = NULL;

			same  = FALSE;
			c     = 0;

			/*
			** See if new character is already in character table/list/pile.
			*/
			if (PackFont && packit) {
				chgrp = FindChar (
							&NewChar,
							&XFlipped,
							&YFlipped,
							&XYFlipped,
							tiletype,
							&c,
							&FlipBits,
							&same);
			} else {
				c = NumChars[tiletype];
			}

			/*
			** Return character number + colorset and flipbits.
			*/

			c += FirstChar[tiletype];
			*schar = (UWORD) (c | ((color + FirstColor) << HDWR_COLORSHIFT) | FlipBits);

			/*
			** If character was unique then add it to table.
			**
			** Really it is already in the table so just move the
			** end of the table to cover the new character.
			*/
			if (PackFont && packit && !zero)
			{
				*schar = BlankChar[TILESET_IMAGE];
			}
			else
			{
				if (!same) {
					AddGroupChar (chgrp, &NewChar, tiletype, c);
				}
			}
		}
		else
		{
			*norm  = &NewChar;
			*xflp  = &XFlipped;
			*yflp  = &YFlipped;
			*xyflp = &XYFlipped;
			*schar = (UWORD) ((color + FirstColor) << HDWR_COLORSHIFT);
		}

		return flags;
	}
} /* AddCharImage */


/*********************************************************************
 *
 * AddChar2Bits
 *
 * PURPOSE
 *		Converts 256 color 8x8 piece of tUME graphic data to
 *		an 4 color 8x8 SNES character with a colorset from 0 <-> 7.
 *
 *		Checks for some color errors like using colors from two different
 *		colorsets.
 *
 *		Generates x flipped, y flipped and xy flipped versions of the
 *		character.
 *
 *		Checks for and Flags Blank characters and symetry of characters.
 *
 *		Checks Character against all previously generated characters
 *		to see if an indentical character has already been created
 *		including flipped characeters.
 *
 *		If character is unique it is added to list/pile of characters.
 *
 *		Returns SNES character number with flipbits and colorset bits
 *		set.
 *
 * INPUT
 *		data	: pointer to tUME graphic data.
 *		modulo	: bytes to skip to get from end of row to start of next.
 *		schar	: pointer to UWORD to fill in with SNES character info
 *		packit	: FALSE = don't look of duplicate character just add
 *				  this character unconditionally.	
 *
 *
 * RETURN VALUE
 *		returns flags for character.  CFLG_NOTXSYM, CFLG_NOTYSYM, CFLG_NOTBLANK
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UBYTE AddChar2Bits (
		UBYTE	 *data,
		int		 modulo,
		UWORD	*schar,
		BOOL	 packit,
		int		 tiletype,
		void	**norm,
		void	**xflp,
		void	**yflp,
		void	**xyflp
		)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddChar2Bits";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		UBYTE		 PlanedTile[16*8];
		UBYTE		 TMGCData[8*8];
		static Color4Char	 NewChar;
		static Color4Char	 XFlipped;
		static Color4Char	 XYFlipped;
		static Color4Char	 YFlipped;
		int			 y1;
		int			 color;
		int			 bcolor;
		UBYTE		*newptr;
		UBYTE		 zero = 0x00;
		UBYTE		 flags;
		UBYTE		 cerr = FALSE;

		/*
		** Convert TMGC data to TMGX data
		*/
		newptr = TMGCData;
		for (y1 = 0; y1 < 8; y1++) {
			memcpy (newptr, data, 8);
			data   += 8 + modulo;
			newptr += 8;
		}

		ConvertTMGCtoTMGX (TMGCData, 8, 8, PlanedTile);

		/*
		** Store new data at end of character table/pile/list
		*/
		newptr = (UBYTE far32 *)&NewChar;
		color  = -1;
		bcolor = -1;

		/*
		** Convert tUME graphic data to 2Bit graphic data
		*/
		data = PlanedTile;
		for (y1 = 0; y1 < 8; y1++) {
			UBYTE	p;
			UBYTE	c;

			*newptr++ = data[0 * 16];	/* Plane 0 */
			*newptr++ = data[1 * 16];	/* Plane 1 */

			p = (UBYTE)
				(
				 ((data[0 * 16] != 0) ? 0x01 : 0) |
				 ((data[1 * 16] != 0) ? 0x02 : 0)
				);

			/*
			** TODO: Check for Color errors if p > 3???
			*/

			if (Other2BitMode) {
				c = (UBYTE)
					(
					 ((data[4 * 16] != 0) ? 0x01 : 0) |
					 ((data[5 * 16] != 0) ? 0x02 : 0) |
					 ((data[6 * 16] != 0) ? 0x04 : 0) |
					 ((data[7 * 16] != 0) ? 0x08 : 0)
					);
			} else {
				c = (UBYTE)
					(
					 ((data[2 * 16] != 0) ? 0x01 : 0) |
					 ((data[3 * 16] != 0) ? 0x02 : 0) |
					 ((data[4 * 16] != 0) ? 0x04 : 0) |
					 ((data[5 * 16] != 0) ? 0x08 : 0)
					);
			}

			data += 2;

			zero |= p;

			/*
			** Ignore colors 0 and 1 of each colorset
			** when determining color of character.
			*/

			if (IgnoreColors[p]) {
				if (p && c && bcolor == -1) {
					bcolor = c;
				}
				c = 0;
			}

			/*
			** Track colorset and check for errors.
			*/
			if (c) {
				if (c > HDWR_MAX_COLORSET) {
					if (!cerr) {
						cerr = TRUE;
						WarnMess ("Bad colors in tileset '%s' tile #%d\n",
							EIO_Filename(CurrentTileset->FileSpec),
							CurrentTileNum);
					}
				} else {
					if (color == -1) {
						color = c;
					} else if (color != c) {
						if (!cerr) {
							cerr = TRUE;
							WarnMess ("Bad colors in tileset '%s' tile #%d\n",
								EIO_Filename(CurrentTileset->FileSpec),
								CurrentTileNum);
						}
					}
				}
			}
		}

		/*
		** If no non-zero colorset was found then it must be colorset 0
		*/
		if (color == (-1)) {
			if (bcolor == (-1)) {
				color = 0;
			} else {
				color = bcolor;
			}
		}

		if (norm || (PackFont && packit)) {
			/*
			** Flip character to check for flipped chars.
			*/
			{
				int		 i;
				int		 j;
				UBYTE	*orig;
				UBYTE	*x;
				UBYTE	*y;
				UBYTE	*xy;
				UBYTE	 flip;

				orig = (UBYTE far32 *)&NewChar;
				x    = XFlipped.Data;
				y    = YFlipped.Data  + sizeof (Color4Char) - 2;
				xy   = XYFlipped.Data + sizeof (Color4Char) - 2;

				for (i = 0; i < 8; i++) {
					for (j = 0; j < 2; j++) {
						flip  = SNESFlipTable[orig[j]];
						x[j]  = flip;
						y[j]  = orig[j];
						xy[j] = flip;
					}
					orig += 2;
					x    += 2;
					y    -= 2;
					xy   -= 2;
				}			
			}

			{
				UBYTE	xsym;
				UBYTE	ysym;

				xsym = (UBYTE) (!memcmp ((void *)&NewChar, (void *)&XFlipped, sizeof (Color4Char)));
				ysym = (UBYTE) (!memcmp ((void *)&NewChar, (void *)&YFlipped, sizeof (Color4Char)));

				flags =	(UBYTE)
						(
						 ((zero) ? 0 : CFLG_NOTBLANK) |
						 ((xsym) ? 0 : CFLG_NOTXSYM ) |
						 ((ysym) ? 0 : CFLG_NOTYSYM )
						);

			}
		} else {
			flags = CFLG_NOTBLANK | CFLG_NOTXSYM | CFLG_NOTYSYM;
		}


		if (!norm) {
			UWORD			 c;
			BOOL			 same;
			UWORD			 FlipBits = 0x0000;
			CharGroup		*chgrp    = NULL;

			same = FALSE;
			c    = 0;

			/*
			** See if new character is already in character table/list/pile.
			*/
			if (PackFont && packit) {
				chgrp = FindChar (
							&NewChar,
							&XFlipped,
							&YFlipped,
							&XYFlipped,
							tiletype,
							&c,
							&FlipBits,
							&same);
			} else {
				c = NumChars[tiletype];
			}

			/*
			** Return character number + colorset and flipbits.
			*/

			c += FirstChar[tiletype];
			*schar = (UWORD) (c | ((color + FirstColor) << HDWR_COLORSHIFT) | FlipBits);

			/*
			** If character was unique then add it to table.
			**
			** Really it is already in the table so just move the
			** end of the table to cover the new character.
			*/
			if (PackFont && packit && !zero)
			{
				*schar = BlankChar[TILESET_2BITS];
			}
			else
			{
				if (!same) {
					AddGroupChar (chgrp, &NewChar, tiletype, c);
				}
			}
		}
		else
		{
			*norm  = &NewChar;
			*xflp  = &XFlipped;
			*yflp  = &YFlipped;
			*xyflp = &XYFlipped;
			*schar = (UWORD) ((color + FirstColor) << HDWR_COLORSHIFT);
		}

		return flags;
	}
} /* AddChar2Bits */


/*********************************************************************
 *
 * AddChar8Bits
 *
 * PURPOSE
 *		Converts 256 color 8x8 piece of tUME graphic data to
 *		an 256 color 8x8 SNES character.
 *
 *		Generates x flipped, y flipped and xy flipped versions of the
 *		character.
 *
 *		Checks for and Flags Blank characters and symetry of characters.
 *
 *		Checks Character against all previously generated characters
 *		to see if an indentical character has already been created
 *		including flipped characeters.
 *
 *		If character is unique it is added to list/pile of characters.
 *
 *		Returns SNES character number with flipbits set.
 *
 * INPUT
 *		data	: pointer to tUME graphic data.
 *		modulo	: bytes to skip to get from end of row to start of next.
 *		schar	: pointer to UWORD to fill in with SNES character info
 *		packit	: FALSE = don't look of duplicate character just add
 *				  this character unconditionally.	
 *
 *
 * RETURN VALUE
 *		returns flags for character.  CFLG_NOTXSYM, CFLG_NOTYSYM, CFLG_NOTBLANK
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UBYTE AddChar8Bits (
		UBYTE	*data,
		int		 modulo,
		UWORD	*schar,
		BOOL	 packit,
		int		 tiletype,
		void	**norm,
		void	**xflp,
		void	**yflp,
		void	**xyflp
	)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddChar8Bits";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		UBYTE		 PlanedTile[16*8];
		UBYTE		 TMGCData[8*8];
		static Color256Char NewChar;
		static Color256Char XFlipped;
		static Color256Char XYFlipped;
		static Color256Char YFlipped;
		int			 y1;
		UBYTE		*newptr;
		UBYTE		 zero = 0x00;
		UBYTE		 flags;

		/*
		** Convert TMGC data to TMGX data
		*/
		newptr = TMGCData;
		for (y1 = 0; y1 < 8; y1++) {
			memcpy (newptr, data, 8);
			data   += 8 + modulo;
			newptr += 8;
		}

		#if 0
		if (DebugSwitch)
		{
			int		x;
			int		y;
			UBYTE	*d = TMGCData;

			printf ("-Orig-\n");
			for (y = 0; y < 8; y++)
			{
				for (x = 0; x < 8; x++)
				{
					printf ("%c", (*d & 0x3F) + 0x20);
					d++;
				}
				printf ("\n");
			}
		}
		#endif

		ConvertTMGCtoTMGX (TMGCData, 8, 8, PlanedTile);

		#if 0
		if (DebugSwitch)
		{
			int		x;
			int		y;
			int		p;
			UBYTE	b;

			printf ("-Planed-\n");
			for (y = 0; y < 8; y++)
			{
				for (p = 0; p < 8; p++)
				{
					b = 0x80;
					for (x = 0; x < 8; x++)
					{
				 		printf ("%c", ((PlanedTile[16 * p + y * 2]) & b) ? '*' : '.');
						b = b >> 1;
					}
					printf (" ");
				}
				printf ("\n");
			}
		}
		#endif

		/*
		** Store new data at end of character table/pile/list
		*/
		newptr = (UBYTE far32 *)&NewChar;

		/*
		** Convert tUME graphic data to 8Bit graphic data
		*/
		data = PlanedTile;
		for (y1 = 0; y1 < 8; y1++) {

			zero |= newptr[0 * 16    ] = data[0 * 16];	/* Plane 0 */
			zero |= newptr[0 * 16 + 1] = data[1 * 16];	/* Plane 1 */
			zero |= newptr[1 * 16    ] = data[2 * 16];	/* Plane 2 */
			zero |= newptr[1 * 16 + 1] = data[3 * 16];	/* Plane 3 */
			zero |= newptr[2 * 16    ] = data[4 * 16];	/* Plane 4 */
			zero |= newptr[2 * 16 + 1] = data[5 * 16];	/* Plane 5 */
			zero |= newptr[3 * 16    ] = data[6 * 16];	/* Plane 6 */
			zero |= newptr[3 * 16 + 1] = data[7 * 16];	/* Plane 7 */

			newptr += 2;
			data   += 2;
		}

		if (norm || (PackFont && packit)) {
			/*
			** Flip character to check for flipped chars.
			*/
			{
				int		 i;
				int		 j;
				UBYTE	*orig;
				UBYTE	*x;
				UBYTE	*y;
				UBYTE	*xy;
				UBYTE	 flip;

				orig = (UBYTE far32 *)&NewChar;
				x    = XFlipped.Data;
				y    = YFlipped.Data  + 16 - 2;
				xy   = XYFlipped.Data + 16 - 2;

				for (i = 0; i < 8; i++) {
					for (j = 0; j < 2; j++) {
						flip  = SNESFlipTable[orig[j + 0 * 16]];
						x[j + 0 * 16]  = flip;
						y[j + 0 * 16]  = orig[j];
						xy[j + 0 * 16] = flip;

						flip  = SNESFlipTable[orig[j + 1 * 16]];
						x[j + 1 * 16]  = flip;
						y[j + 1 * 16]  = orig[j];
						xy[j + 1 * 16] = flip;

						flip  = SNESFlipTable[orig[j + 2 * 16]];
						x[j + 2 * 16]  = flip;
						y[j + 2 * 16]  = orig[j];
						xy[j + 2 * 16] = flip;

						flip  = SNESFlipTable[orig[j + 3 * 16]];
						x[j + 3 * 16]  = flip;
						y[j + 3 * 16]  = orig[j];
						xy[j + 3 * 16] = flip;

					}
					orig += 2;
					x    += 2;
					y    -= 2;
					xy   -= 2;
				}			

			}

			#if 0
			if (DebugSwitch)
			{
				int		x;
				int		y;
				int		p;
				UBYTE	b;

				printf ("-NewChar-\n");
				for (y = 0; y < 8; y++)
				{
					for (p = 0; p < 8; p++)
					{
						b = 0x80;
						for (x = 0; x < 8; x++)
						{
				 			printf ("%c", ((NewChar.Data[16 * (p / 2) + (p & 0x01) + y * 2]) & b) ? '*' : '.');
							b = b >> 1;
						}
						printf (" ");
					}
					printf ("\n");
				}
			}
			#endif

			{
				UBYTE	xsym;
				UBYTE	ysym;

				xsym = (UBYTE) (!memcmp ((void *)&NewChar, (void *)&XFlipped, sizeof (Color256Char)));
				ysym = (UBYTE) (!memcmp ((void *)&NewChar, (void *)&YFlipped, sizeof (Color256Char)));

				flags = (UBYTE)
						(
						 ((zero) ? 0 : CFLG_NOTBLANK) |
						 ((xsym) ? 0 : CFLG_NOTXSYM ) |
						 ((ysym) ? 0 : CFLG_NOTYSYM )
						);

			}
		} else {
			flags = CFLG_NOTBLANK | CFLG_NOTXSYM | CFLG_NOTYSYM;
		}


		if (!norm)
		{
			UWORD			 c;
			BOOL			 same;
			UWORD			 FlipBits = 0x0000;
			CharGroup		*chgrp    = NULL;

			c    = 0;
			same = FALSE;

			/*
			** See if new character is already in character table/list/pile.
			*/
			if (PackFont && packit) {
				chgrp = FindChar (
							&NewChar,
							&XFlipped,
							&YFlipped,
							&XYFlipped,
							tiletype,
							&c,
							&FlipBits,
							&same);
			} else {
				c = NumChars[tiletype];
			}

			/*
			** Return character number + colorset and flipbits.
			*/

			c += FirstChar[tiletype];
			*schar = c | FlipBits;

			/*
			** If character was unique then add it to table.
			**
			** Really it is already in the table so just move the
			** end of the table to cover the new character.
			*/
			if (PackFont && packit && !zero)
			{
				*schar = BlankChar[TILESET_8BITS];
			}
			else
			{
				if (!same) {
					AddGroupChar (chgrp, &NewChar, tiletype, c);
				}
			}
		}
		else
		{
			*norm  = &NewChar;
			*xflp  = &XFlipped;
			*yflp  = &YFlipped;
			*xyflp = &XYFlipped;
			*schar = 0;
		}

		return flags;
	}
} /* AddChar8Bits */


/*********************************************************************
 *
 * AddCharMode7
 *
 * PURPOSE
 *		Converts 256 color 8x8 piece of tUME graphic data to
 *		an 16 color 8x8 SEGA character with a colorset from 0 <-> 7.
 *
 *		Checks for some color errors like using colors from two different
 *		colorsets.
 *
 *		Generates x flipped, y flipped and xy flipped versions of the
 *		character.
 *
 *		Checks for and Flags Blank characters and symetry of characters.
 *
 *		Checks Character against all previously generated characters
 *		to see if an indentical character has already been created
 *		including flipped characeters.
 *
 *		If character is unique it is added to list/pile of characters.
 *
 *		Returns SEGA character number with flipbits and colorset bits
 *		set.
 *
 * INPUT
 *		data	: pointer to tUME graphic data.
 *		modulo	: bytes to skip to get from end of row to start of next.
 *		schar	: pointer to UWORD to fill in with SEGA character info
 *		packit	: FALSE = don't look of duplicate character just add
 *				  this character unconditionally.	
 *
 *
 * RETURN VALUE
 *		returns flags for character.  CFLG_NOTXSYM, CFLG_NOTYSYM, CFLG_NOTBLANK
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UBYTE AddCharMode7 (
		UBYTE	*data,
		int		 modulo,
		UWORD	*schar,
		BOOL	 packit,
		int		 tiletype,
		void	**norm,
		void	**xflp,
		void	**yflp,
		void	**xyflp
)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddCharMode7";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		int			 y1;
		int			 zero;
		static Mode7Char	 NewChar;
		static Mode7Char	 BlankButt;
		UBYTE		*newptr;
		UBYTE		 flags;

		/*
		** Store new data at end of character table/pile/list
		*/
		newptr = (UBYTE far32 *)&NewChar;

		/*
		** Convert tUME graphic data to SEGA graphic data
		*/
		for (y1 = 0; y1 < 8; y1++) {
			memcpy (newptr, data, 8);
			data   += 8 + modulo;
			newptr += 8;
		}

		zero = memcmp (&NewChar, &BlankButt, sizeof (NewChar));

		/*
		** Since no flipping just set flags.
		*/
		flags = CFLG_NOTBLANK | CFLG_NOTXSYM | CFLG_NOTYSYM;

		if (!norm)
		{
			UWORD			 c;
			BOOL			 same;
			UWORD			 junk;
			CharGroup		*chgrp = NULL;

			same = FALSE;
			c    = 0;

			/*
			** See if new character is already in character table/list/pile.
			*/
			if (PackFont && packit) {
				chgrp = FindChar (
							&NewChar,
							NULL,
							NULL,
							NULL,
							tiletype,
							&c,
							&junk,
							&same);
			} else {
				c = NumChars[tiletype];
			}

			/*
			** Return character number + colorset and flipbits.
			*/

			*schar = c;

			/*
			** If character was unique then add it to table.
			**
			** Really it is already in the table so just move the
			** end of the table to cover the new character.
			*/
			if (PackFont && packit && !zero)
			{
				*schar = BlankChar[TILESET_MODE7];
			}
			else
			{
				if (!same) {
					AddGroupChar (chgrp, &NewChar, tiletype, c);
				}
			}
		}
		else
		{
			*norm  = &NewChar;
			*xflp  = &NewChar;
			*yflp  = &NewChar;
			*xyflp = &NewChar;
			*schar = 0;
		}

		return flags;
	}
} /* AddCharMode7 */


/*********************************************************************
 *
 * AddChar
 *
 * PURPOSE
 *		Converts 256 color 8x8 piece of tUME graphic data to
 *		an 16 color 8x8 SEGA character with a colorset from 0 <-> 7.
 *
 *		Checks for some color errors like using colors from two different
 *		colorsets.
 *
 *		Generates x flipped, y flipped and xy flipped versions of the
 *		character.
 *
 *		Checks for and Flags Blank characters and symetry of characters.
 *
 *		Checks Character against all previously generated characters
 *		to see if an indentical character has already been created
 *		including flipped characeters.
 *
 *		If character is unique it is added to list/pile of characters.
 *
 *		Returns SEGA character number with flipbits and colorset bits
 *		set.
 *
 * INPUT
 *		data	: pointer to tUME graphic data.
 *		modulo	: bytes to skip to get from end of row to start of next.
 *		schar	: pointer to UWORD to fill in with SEGA character info
 *		packit	: FALSE = don't look of duplicate character just add
 *				  this character unconditionally.	
 *
 *
 * RETURN VALUE
 *		returns flags for character.  CFLG_NOTXSYM, CFLG_NOTYSYM, CFLG_NOTBLANK
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UBYTE AddChar (UBYTE *data, int modulo, UWORD *schar, BOOL packit, int tiletype)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddChar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		switch (tiletype) {
		case TILESET_IMAGE:
			return AddCharImage (data, modulo, schar, packit, tiletype, NULL, NULL, NULL, NULL);
		case TILESET_2BITS:
			return AddChar2Bits (data, modulo, schar, packit, tiletype, NULL, NULL, NULL, NULL);
		case TILESET_8BITS:
			return AddChar8Bits (data, modulo, schar, packit, tiletype, NULL, NULL, NULL, NULL);
		case TILESET_MODE7:
			return AddCharMode7 (data, modulo, schar, packit, tiletype, NULL, NULL, NULL, NULL);
		default:
			FailMess ("AddChar:Bad Tile Type %d", tiletype);
		}
	}
	return 0;

} /* AddChar */


/*********************************************************************
 *
 * ReadTileData
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ReadTileData (TilesetType *ts, unsigned tileID, void *dest)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ReadTileData";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		TileDataGroup	*tdg;
		BOOL			 found;
		unsigned		 tcount;
		unsigned		 tilesize;

		tilesize = ts->TMG.Width * ts->TMG.Height;
		tcount   = tileID;
		found    = FALSE;

		tdg = Head (ts->TMG.TileGroupList);
		while (!found) {
			if (tcount < tdg->NumTiles) {
				found     = TRUE;
				ReadXTRA (tdg->TileData, tilesize * tcount, tilesize, dest);
			} else {
				tcount -= tdg->NumTiles;
				tdg = Next (tdg);
			}
		}
	}

} /* ReadTileData */


/*********************************************************************
 *
 * AddContour
 *
 * PURPOSE
 *		create a value that represents this contour tile with its
 *		flip bits and then check to see if the same contour in the
 *		same orientation has already been used.  If it hasn't then
 *		add this new contour/orientation to the used table.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Position in the used contour table (or contour index)
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
unsigned AddContour (UWORD tileID, UBYTE tileflags)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddContour";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		unsigned	ndx;
		BOOL	same;
		BOOL	checked;
		UWORD	cflags;


		cflags = (UWORD)
				 (
					((tileflags & TUME_XFLIP) ? CONTOUR_XFLIP : 0) |
					((tileflags & TUME_YFLIP) ? CONTOUR_YFLIP : 0)
				 );

		same    = FALSE;
		checked = FALSE;
		for (ndx = 0; ndx < NumContours; ndx++) {
			if (tileID == ContourTable[ndx].ContourID) {
				UWORD	oflags;

				checked = TRUE;
				oflags  = ContourTable[ndx].Flags;
				if (oflags & CONTOUR_XSYM) {
					cflags &= (UWORD) (~CONTOUR_XFLIP);
					cflags |=  CONTOUR_XSYM;
				}
				if (oflags & CONTOUR_YSYM) {
					cflags &= ~CONTOUR_YFLIP;
					cflags |=  CONTOUR_YSYM;
				}

				if (cflags == oflags) {
					same = TRUE;
					break;
				}
			}
		}

		if (!same) {
			if (!checked) {
				UBYTE		 Original[16*16];
				UBYTE		 XFlipped[16*16];
				UBYTE		 YFlipped[16*16];
				UWORD		 contID = tileID;
				UWORD		 tset;
				BOOL		 found;
				unsigned	 tcount;
				unsigned	 tilesize;
				TilesetType		*ts;
				TileDataGroup	*tdg;

				tset = 0;
				while (contID > CTilesTable[tset]->TileCount) {
					tset++;
					contID -= CTilesTable[tset]->TileCount;
				}

				ts = CTilesTable[tset];
				if (!ts->TMG.TileCount) {
					FailMess ("No Contour Data save with map! (AC)\n");
				}

				if (ts->Width == 16 && ts->Height == 16)
				{
					/*
					** Lookup 16x16 tile's data
					*/
					tilesize = ts->TMG.Width * ts->TMG.Height;
					tcount   = contID;
					found    = FALSE;

					tdg = Head (ts->TMG.TileGroupList);
					while (!found) {
						if (tcount < tdg->NumTiles) {
							found     = TRUE;
							ReadXTRA (tdg->TileData, tilesize * tcount, sizeof (Original), Original);
						} else {
							tcount -= tdg->NumTiles;
							tdg = Next (tdg);
						}
					}

					{
						int		x;
						int		y;
						UBYTE	byte;

						for (y = 0; y < 16; y++) {
							for (x = 0; x < 16; x++) {
								byte = Original[y * 16 + x];
								XFlipped[15 - x + y * 16] = byte;
								YFlipped[x + (15 - y) * 16] = byte;
							}
						}
					}

					if (!memcmp (Original, XFlipped, sizeof (Original))) {
						cflags &= (UWORD) (~CONTOUR_XFLIP);
						cflags |=  CONTOUR_XSYM;
					}
					if (!memcmp (Original, YFlipped, sizeof (Original))) {
						cflags &= ~CONTOUR_YFLIP;
						cflags |=  CONTOUR_YSYM;
					}
				}
				else if (ts->Width == 8 && ts->Height == 8)
				{
					/*
					** Lookup 8x8 tile's data
					*/
					UBYTE	tile8x8[8*8];

					if (ts->TileCount != 1024)
					{
						FailMess ("There must be 256 8x8 contour tiles\n");
					}

					{
						int		x;
						int		y;
						int		i;

						for (y = 0; y < 2; y++)
						{
							for (x = 0; x < 2; x++)
							{
								UBYTE	*dest;
								UBYTE	*src;

								tcount = tileID + 32 * y + x;
								ReadTileData (ts, tcount, tile8x8);

								dest = Original + x * 8 + y * (16 * 8);
								src  = tile8x8;

								for (i = 0; i < 8; i++)
								{
									memcpy (dest, src, 8);
									src  += 8;
									dest += 16;
								}
							}
						}
					}

					ReadTileData (ts, tcount, tile8x8);

					{
						int		x;
						int		y;
						UBYTE	byte;

						for (y = 0; y < 16; y++) {
							for (x = 0; x < 16; x++) {
								byte = Original[y * 16 + x];
								XFlipped[15 - x + y * 16] = byte;
								YFlipped[x + (15 - y) * 16] = byte;
							}
						}
					}

					if (!memcmp (Original, XFlipped, sizeof (Original))) {
						cflags &= (UWORD) (~CONTOUR_XFLIP);
						cflags |=  CONTOUR_XSYM;
					}
					if (!memcmp (Original, YFlipped, sizeof (Original))) {
						cflags &= ~CONTOUR_YFLIP;
						cflags |=  CONTOUR_YSYM;
					}
				}
				else
				{
					FailMess ("Writing Contour Tileset (AC): Tileset not 16x16 or 8x8 pixels\n");
				}

			}

			ContourTable[NumContours].ContourID = tileID;
			ContourTable[NumContours].Flags     = cflags;
			NumContours++;
		}

		return ndx;
	}
} /* AddContour */

static int OrderTable[] = {
	TILESET_2BITS,
	TILESET_IMAGE,
	TILESET_8BITS,
	TILESET_MODE7,
	TILESET_CONTOUR,
	-1,
};

/*********************************************************************
 *
 * InitTilesetCollections
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *2.059	06/26/95 (dcc) - Allocate flage for composite rooms so that
 *						 +k switch doesn't crash program.
 *
 * SEE ALSO
 *
*/
int InitTilesetCollections (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitTilesetCollections";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		TilesetType	**tsptr;
		TilesetType	*ts;
		TilesetInfo	*tsi;
		ListType	 clistx;
		ListType	*clist = &clistx;
		int			 count = 0;
		int			 tiletype;

		InitList (clist);

		/*
		** Count the Image tiles and
		** allocate a TilesetInfo for each tileset
		** and if the tileset is an image tileset
		** allocate tables to remember which SEGA characters
		** represent the tiles.
		*/
		tsptr = map->Tilesets + 1;
		while (*tsptr) {
			ts  = *tsptr;

			tsi = calloc (1, sizeof (TilesetInfo));
			if (!tsi) {
				OutOfMemErr ("WriteAllTiles:TilesetInfo\n");
			}

			ts->UserData = tsi;
			tiletype     = ts->UserType;
			switch (tiletype) {
			case TILESET_IMAGE:
			case TILESET_2BITS:
			case TILESET_8BITS:
			case TILESET_MODE7:

				/*
				** Skip Composite Tilesets
				*/
				if (ts->GridedTilesRoom) {
					tsi->Flags = calloc (ts->TileCount + 1, sizeof (UBYTE));/*2.059*/
					if (!tsi->Flags)
					{
						OutOfMemErr ("InitTilesetCollections:composite tiles\n");
					}
				} else {

					if (LargestBlank < CharSize[tiletype])
					{
						LargestBlank = CharSize[tiletype];
					}

					if (ts->Width == 8 && ts->Height == 8)
					{
						tsi->FirstChar = MaxChars[tiletype];
						MaxChars[tiletype] += ts->TileCount;
						if (!ts->TMG.TileCount) {
							ErrMess ("No tile images for tileset '%s'\n", EIO_Filename (ts->FileSpec));
						}
						tsi->ImageChars = calloc (ts->TileCount + 1, sizeof (UWORD));
						tsi->Flags      = calloc (ts->TileCount + 1, sizeof (UBYTE));
						if (!tsi->ImageChars || !tsi->Flags) {
							OutOfMemErr ("WriteAllTile:ImageChars\n");
						}
					}
					else if (ts->Width == 16 && ts->Height == 16)
					{
						tsi->FirstChar = MaxChars[tiletype];
						MaxChars[tiletype] += (UWORD) (ts->TileCount * 4);
						if (!ts->TMG.TileCount) {
							ErrMess ("No tile images for tileset '%s'\n", EIO_Filename (ts->FileSpec));
						}
						tsi->ImageTiles = calloc (ts->TileCount + 1, sizeof (ImageTile));
						tsi->Flags      = calloc (ts->TileCount + 1, sizeof (UBYTE));
						if (!tsi->ImageTiles || !tsi->Flags) {
							OutOfMemErr ("WriteAllTiles:ImageTiles\n");
						}
					}
					else
					{
						ErrMess ("WriteAllTiles:Tileset '%s' is not 8x8 or 16x16\n", EIO_Filename (ts->FileSpec));
                  exit(1);
					}

					//
					// if were converting all character then mark all
					// image tiles to be converted.
					//
					if (!OnlyUsedChars)
					{
						unsigned	t;
						UBYTE	*tflg;

						tflg = tsi->Flags + 1;
						for (t = 0; t < ts->TileCount; t++)
						{
							*tflg = TFLG_CONVERT;
							tflg++;
						}
					}
					else
					{
						tsi->Flags[1] = TFLG_CONVERT;
					}
				}
				break;
			case TILESET_CONTOUR:
				/*
				** Skip Composite Tilesets
				*/
				if (ts->GridedTilesRoom) {
					ErrMess ("WriteAllTiles:No support for Composite Contour tiles yet\n");
				} else {
					/*
					** Remember Contour Tileset Name for Filename later.
					*/
					if (!ContourName) {
						ContourName = ts->FileSpec;
					}

					/*
					** Gather Contour Tilesets into a table for quicker
					** referencing.
					*/
					if (ContourMode == CONTOUR_GENFLIP) {
						AddTail (clist, ts);
						count++;
					}

					if (ts->Width == 16 && ts->Height == 16)
					{
						tsi->FirstChar = MaxChars[tiletype];
						MaxChars[tiletype] += ts->TileCount;
						if (!ts->TMG.TileCount)
						{
							ErrMess ("No tile images for tileset '%s'\n", EIO_Filename (ts->FileSpec));
						}
						tsi->Flags = calloc (ts->TileCount + 1, sizeof (UBYTE));
						if (!tsi->Flags)
						{
							OutOfMemErr ("WriteAllTiles:ImageTiles\n");
						}
					}
					else if (ts->Width == 8 && ts->Height == 8)
					{
						tsi->FirstChar = MaxChars[tiletype];
						MaxChars[tiletype] += (UWORD) (ts->TileCount / 4);
						if (!ts->TMG.TileCount)
						{
							ErrMess ("No tile images for tileset '%s'\n", EIO_Filename (ts->FileSpec));
						}
						tsi->Flags = calloc (ts->TileCount + 1, sizeof (UBYTE));
						if (!tsi->Flags)
						{
							OutOfMemErr ("WriteAllTiles:ImageTiles\n");
						}
					}
					else
					{
						ErrMess ("WriteAllTiles:Contour Tileset '%s' is not 16x16 or 8x8\n", EIO_Filename (ts->FileSpec));
					}

				}
				break;
			}
			tsptr++;
		}

		/*
		** Allocate and Initialize Contour Tileset QuickRef Table.
		*/
		if (ContourMode == CONTOUR_GENFLIP) {
			if (count)
			{
				CTilesTable = calloc (count, sizeof (TilesetType *));
				if (!CTilesTable) {
					OutOfMemErr ("WriteAllTiles: CTilesTable\n");
				}
				count = 0;
				tsptr = map->Tilesets + 1;
				while (*tsptr) {
					ts  = *tsptr;

					if (ts->UserType == TILESET_CONTOUR) {
						CTilesTable[count] = ts;
						count++;
					}
					tsptr++;
				}
			}
			else
			{
				ContourMode = CONTOUR_NOFLIP;
			}
		}

		/*
		** Allocate contour table for collecting unique used contours
		*/
		if (ContourMode == CONTOUR_GENFLIP) {
			ContourTable = calloc (MaxChars[TILESET_CONTOUR] * 4, sizeof (ContourFlags));
			if (!ContourTable) {
				OutOfMemErr ("WriteAllTiles: ContourTable\n");
			}
//			LastContour = ContourTable;
			/*
			** Add First Contour -- Assume blank
			*/
			AddContour (0, 0);
		}

		/*
		** Find all Table Rooms and mark all IMAGE tiles used in 
		** the table room as DONTPACK so those tiles will
		** use their own unique SEGA/SNES characters.
		**
		** Also, add all contours in table room to contour table
		** so that they are first in the table.
		*/
		{
			RoomType	**roomptr;
			RoomType	*room;

			printf ("Finding used tiles\n");

			roomptr = map->Rooms + 1;
			while (*roomptr) {
				room = *roomptr;

				if (room->UserType == ROOM_TABLE ||
					room->UserType == ROOM_ANIMATION ||
					OnlyUsedChars) {
					int			 y;
					int			 x;
					int			 l;
					UBYTE		 imageflags;
					UBYTE		 animflags;
					PlotType	*floorplot;

					imageflags = (UBYTE)
								 (
								  TFLG_CONVERT | 
								  ((room->UserType == ROOM_TABLE) ? TFLG_DONTPACK : 0)
								 );

					for (l = 0; l < room->LayerCount; l++) {
						for (y = 0; y < room->Height; y++) {

							animflags = (UBYTE)
										(
										 (room->UserType == ROOM_ANIMATION) ?
											(
												(JustTop) ?
													(y ? 0 : TFLG_ANIM)
													:
													TFLG_ANIM
											)
											:
											0
										);



							floorplot = ((PlotType *)ActivateXTRA (room->Layers[l]->Plot[y]));
							for (x = 0; x < room->Width; x++) {
								UBYTE		 tilesetID;
								UWORD		 tileID;
								TilesetType	*ts;
								TilesetInfo	*tsi;

								tilesetID = floorplot->TilesetID;
								tileID    = floorplot->TileID;

								if (tilesetID) {
									ts  = map->Tilesets[tilesetID];
									tsi = ts->UserData;
									switch (ts->UserType) {
									case TILESET_IMAGE:
									case TILESET_2BITS:
									case TILESET_8BITS:
									case TILESET_MODE7:
										tsi->Flags[tileID] |= (imageflags | animflags);
										break;
									case TILESET_CONTOUR:
										if (room->UserType == ROOM_TABLE)
										{
											tileID = (UWORD) ((tileID - 1) + tsi->FirstChar);
											if (ContourMode == CONTOUR_GENFLIP)
											{
												AddContour (tileID,floorplot->PlotFlags);
											}
										}
										break;
									}
								}
								floorplot++;
							}
							ReleaseXTRA(room->Layers[l]->Plot[y]);
						}
					}
				}

				roomptr++;
			}

		}

		/*
		** Exit if there are any errors so far.
		*/
		return (!ErrorCount);
	}
} /* InitTilesetCollections */


/*********************************************************************
 *
 * GetFrameInfo
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int GetFrameInfo (
		MapType		*map,
		RoomType	*room,
		int			 x,
		int			*framesvar,
		int			*fheightvar,
		int			*tiletypevar
	)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GetFrameInfo";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		//
		// Find number of frames in this column
		// and height of 'row' (tile between nulls)
		//
		// Also, make sure that all tiles in column
		// are the same type (i.e. all 16 color...)
		//
		// And, make sure all sub columns are the same
		// height, (i.e. if there are three tiles before
		// the first null then there should be three tiles
		// before the next null and so on.
		//
		// complain about extra tiles too?  (tiles that are
		// past 2 nulls in a row or in a column where the
		// first tile is null.
		//
		int		frames   = 0;
		int		cheight  = 0;
		int		cheight2 = 0;
		BOOL	chflag   = FALSE;
		int		tiletype = -1;
		int		y;

		for (y = 0; y < room->Height; y++)
		{
			PlotType	*plot;
			UBYTE		 tsetID;
			UBYTE		 plotflags;

			plot      = ((PlotType *)ActivateXTRA (room->Layers[0]->Plot[y])) + x;
			tsetID    = plot->TilesetID;
			plotflags = plot->PlotFlags;
			ReleaseXTRA(room->Layers[0]->Plot[y]);

			if (!tsetID)
			{
				if (cheight2) 
				{
					if (!chflag)
					{
						cheight = cheight2;
						chflag  = TRUE;
					}
					if (cheight != cheight2)
					{
						ErrMess ("Frames in Animation room '%s' in column %d are not the same height\n", room->Name, x);
					}
					cheight2 = 0;
					frames++;
				}
			}
			else
			{
				int		thistype;

				thistype = map->Tilesets[tsetID]->UserType;
				if (ImageTypes[thistype])
				{
					if (tiletype < 0)
					{
						tiletype = thistype;
					}
					else
					{
						if (tiletype != thistype)
						{
							ErrMess ("Tiles of mixed types in Animation room '%s' at (%d, %d)\n", room->Name, x, y);
						}
					}

					if (plotflags)
					{
						WarnMess ("Tile in anim room '%s' at (%d,%d) is flipped or has colorset or priority\n", room->Name, x, y);
					}
				}
				else
				{
					ErrMess ("Non Image tile in anim room '%s' at (%d,%d)\n");
				}

				cheight2++;
			}
		}
		*framesvar   = frames;
		*fheightvar  = cheight;
		*tiletypevar = tiletype;
	}
	return (!ErrorCount);

} /* GetFrameInfo */


/*********************************************************************
 *
 * ConvertColumn
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *2.051	02/12/94 (dcc) - Fix animation block not flippable in map
 *						 (This routine used to discard the flags
 *						 returned by AddCharImage(). Now it actually
 *						 saves the value away in tsi->Flags[]...)
 *2.060	06/28/95 (dcc) - When I "fixed" animation block not flippable for
 *						 v2.051, I actually broke it. Restored to previous.
 *						 (I should have studied the code more carefully.)
 *2.061	07/02/95 (dcc) - Reset animation room tiles' tsi->Flags[] so
 *						 TFLG_CONVERT are no longer set. This way animated
 *						 tiles don't get added to the character set twice.
 *
 * SEE ALSO
 *
*/
int ConvertColumn (
			MapType		*map,
			RoomType	*room,
			int			 x,
			int			 f,
			AnimGroup	*agrp,
			int			 place,
			int			 offset,
			int			 modulo,
			int			 pass
		)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ConvertColumn";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		AnimColumn	*acol;
		XTRAPntr	 aptr;
		XTRAPntr	 xptr;
		XTRAPntr	 yptr;
		XTRAPntr	 xyptr;
		short		 cheight = 0;
		short		 y;
		short		 frame;
		unsigned	 fsize;
		short		 same = FALSE;
		UWORD		 firstcolor;

		acol = CreateNode (sizeof (AnimColumn), NULL);
		if (!acol)
		{
			OutOfMemErr ("AnimColumn\n");
		}

		fsize = CharSize[agrp->TileType] * agrp->Frames;
		aptr  = AllocXTRA (fsize);
		xptr  = AllocXTRA (fsize);
		yptr  = AllocXTRA (fsize);
		xyptr = AllocXTRA (fsize);
		if (!aptr || !xptr || !yptr || !xyptr)
		{
			OutOfMemErr ("AnimColumn data\n");
		}

		acol->FrameData = aptr;

		frame = 0;
		for (y = 0; y < room->Height; y++)
		{
			PlotType	*plot;
			UBYTE		 tsetID;
			UWORD		 tileID;

			plot   = ((PlotType *)ActivateXTRA (room->Layers[0]->Plot[y])) + x;
			tsetID = plot->TilesetID;
			tileID = plot->TileID;
			ReleaseXTRA(room->Layers[0]->Plot[y]);

			if (!tsetID)
			{
				cheight = 0;
			}
			else
			{
				if (cheight == f)
				{
					TilesetType	*ts;
					TilesetInfo	*tsi;

					ts = map->Tilesets[tsetID];
					tsi = ts->UserData;

					if (ts->GridedTilesRoom)
					{
						FailMess ("Animated Composite Tiles not yet implemented\n");
					}
					else
					{
						TileDataGroup	*tdg;
						unsigned		 tcount;
						unsigned		 tilesize;
						UBYTE			*data;
						UBYTE			*normal;
						UBYTE			*xflip;
						UBYTE			*yflip;
						UBYTE			*xyflip;
						UWORD			 color;
						short			 found;
						short			 tiletype;

						tiletype = ts->UserType;
						tcount   = tileID - 1;
						tilesize = ts->TMG.Width * ts->TMG.Height;
						found    = FALSE;
						
						tdg = Head (ts->TMG.TileGroupList);
						while (!found) {
							if (tcount < tdg->NumTiles) {
								found = TRUE;
								data  = ActivateXTRA (tdg->TileData);
								data += tilesize * tcount;
							} else {
								tcount -= tdg->NumTiles;
								tdg = Next (tdg);
							}
						}

						data += offset;

						//
						// ... for error reporting
						//
						CurrentTileset = ts;
						CurrentTileNum = tileID;

						switch (ts->UserType)
						{
						case TILESET_IMAGE:
							AddCharImage (data, modulo, &color, FALSE, tiletype,
										  (void **)&normal, (void **)&xflip, (void **)&yflip, (void **)&xyflip);
							tsi->Flags[tileID] &= ~TFLG_CONVERT;
							break;
						case TILESET_2BITS:
							AddChar2Bits (data, modulo, &color, FALSE, tiletype,
										  (void **)&normal, (void **)&xflip, (void **)&yflip, (void **)&xyflip);
							tsi->Flags[tileID] &= ~TFLG_CONVERT;
							break;
						case TILESET_8BITS:
							AddChar8Bits (data, modulo, &color, FALSE, tiletype,
										  (void **)&normal, (void **)&xflip, (void **)&yflip, (void **)&xyflip);
							tsi->Flags[tileID] &= ~TFLG_CONVERT;
							break;
						case TILESET_MODE7:
							AddCharMode7 (data, modulo, &color, FALSE, tiletype,
										  (void **)&normal, (void **)&xflip, (void **)&yflip, (void **)&xyflip);
							tsi->Flags[tileID] &= ~TFLG_CONVERT;
							break;
						default:
							FailMess ("ConvertColumn:Bad Tile Type %d", tiletype);
							break;
						}
						ReleaseXTRA(tdg->TileData);

						if (frame == 0)
						{
							firstcolor = color;
						}
						{
							UBYTE	*dest;

							dest  = ActivateXTRA (aptr);
							dest += (frame * CharSize[tiletype]);
							memcpy (dest, normal, CharSize[tiletype]);
							UpdateXTRA (aptr);

							dest  = ActivateXTRA (xptr);
							dest += (frame * CharSize[tiletype]);
							memcpy (dest, xflip, CharSize[tiletype]);
							UpdateXTRA (xptr);

							dest  = ActivateXTRA (yptr);
							dest += (frame * CharSize[tiletype]);
							memcpy (dest, yflip, CharSize[tiletype]);
							UpdateXTRA (yptr);

							dest  = ActivateXTRA (xyptr);
							dest += (frame * CharSize[tiletype]);
							memcpy (dest, xyflip, CharSize[tiletype]);
							UpdateXTRA (xyptr);
						}
					}
					frame++;
				}
				cheight++;
			}
		}

		//
		// got an entire 'char' column so...
		//
		// compare it against other columns in this group
		//
		{
			AnimColumn	*oldcol;
			UWORD		 count;
			UWORD		 flip = 0x0000;

			if (pass == 2 || CompressAnim)
			{
				count  = 0;
				oldcol = Head (agrp->ColumnList);
				while (!EndOfList (oldcol))
				{
					UBYTE	*old;
					UBYTE	*new;

					if (!(pass == 2 && !CompressAnim && oldcol->UsedFlag))
					{
						old = ActivateXTRAatWin (oldcol->FrameData, 2);

						new = ActivateXTRAatWin (aptr, 0);
						if (!memcmp (old, new, fsize))
						{
							same = TRUE;
							ReleaseXTRA(aptr);
							ReleaseXTRA(oldcol->FrameData);
							break;
						}
						ReleaseXTRA(aptr);

						new = ActivateXTRAatWin (xptr, 0);
						if (!memcmp (old, new, fsize))
						{
							same = TRUE;
							flip = HDWR_XFLIP;
							ReleaseXTRA(xptr);
							ReleaseXTRA(oldcol->FrameData);
							break;
						}
						ReleaseXTRA(xptr);

						new = ActivateXTRAatWin (yptr, 0);
						if (!memcmp (old, new, fsize))
						{
							same = TRUE;
							flip = HDWR_YFLIP;
							ReleaseXTRA(yptr);
							ReleaseXTRA(oldcol->FrameData);
							break;
						}
						ReleaseXTRA(yptr);

						new = ActivateXTRAatWin (xyptr, 0);
						if (!memcmp (old, new, fsize))
						{
							same = TRUE;
							flip = HDWR_XFLIP | HDWR_YFLIP;
							ReleaseXTRA(xyptr);
							ReleaseXTRA(oldcol->FrameData);
							break;
						}
						ReleaseXTRA(xyptr);

						ReleaseXTRA(oldcol->FrameData);
					}
					count++;
					oldcol = Next (oldcol);
				}
			}

			//
			// if this is the second pass then now that we have
			// the character ID put it in the tiles (ImageTile
			// stuff)
			//
			if (same && pass == 1 && DebugSwitch)
			{
				printf ("Column %d(%d) of room '%s' is the same as another\n",
						x,place,room->Name);
			}

			if (pass == 2)
			{
				oldcol->UsedFlag = TRUE;

				frame = 0;
				for (y = 0; y < room->Height; y++)
				{
					PlotType	*plot;
					UBYTE		 tsetID;
					UWORD		 tileID;

					plot   = ((PlotType *)ActivateXTRA (room->Layers[0]->Plot[y])) + x;
					tsetID = plot->TilesetID;
					tileID = plot->TileID;
					ReleaseXTRA(room->Layers[0]->Plot[y]);

					if (!tsetID)
					{
						cheight = 0;
					}
					else
					{
						if (cheight == f)
						{
							TilesetType	*ts;
							TilesetInfo	*tsi;

							ts  = map->Tilesets[tsetID];
							tsi = ts->UserData;

							if (tsi->Flags[tileID] & TFLG_TOP)
							{
								if (frame == 0)
								{
									ErrMess ("You can't have the same tile at the top of more than one column in anim room '%s'\n", room->Name);
								}
							}
							else
							{
								UWORD	c;

								c = oldcol->CharID | firstcolor | flip;

								switch (place)
								{
								case 0:
									tsi->Flags[tileID] |= TFLG_DONE;
									tsi->ImageChars[tileID] = c;
									if (!frame)
									{
										tsi->Flags[tileID] |= TFLG_TOP;
									}
									break;
								case 1:
									tsi->ImageTiles[tileID].TL = c;
									break;
								case 2:
									tsi->ImageTiles[tileID].TR = c;
									break;
								case 3:
									tsi->ImageTiles[tileID].BL = c;
									break;
								case 4:
									tsi->ImageTiles[tileID].BR = c;
									tsi->Flags[tileID] |= TFLG_DONE;
									if (!frame)
									{
										tsi->Flags[tileID] |= TFLG_TOP;
									}
									break;
								}
							}

							frame++;
						}
						cheight++;
					}
				}
			}
		}

		//
		// notes:
		//   Okay, my current thinking is that I will collect
		// all the character columns first, then I will sort
		// and renumber them.  Then I will collect them again
		// but this time since they've already been collected
		// once they will match the previous collection and
		// also, this second time the, each column will have
		// been given a char ID.
		//
		// Todo:
		//	 Check if a column is entirely the same frame and if
		// it is then don't add it to the frame but instead add
		// it to the normal characters.
		//
		// Question:
		//   Why did I have to check for character symmetry?  I forgot?
		//
		//	 Oh Yeah, it was because of both left and right characters
		// are symetrical and they are the same character then the tile
		// is symetrical and shouldn't generate a separate block entry
		// when it is flipped.  This is also true for top and botton
		// characters.
		//

		FreeXTRA (xyptr);
		FreeXTRA (yptr);
		FreeXTRA (xptr);

		if (same)
		{
			FreeXTRA (aptr);
			DeleteNode (acol);
		}
		else
		{
			AddTail (agrp->ColumnList, acol);
		}

		return (!ErrorCount);
	}
} /* ConvertColumn */


/*********************************************************************
 *
 * GatherColumns
 *
 * PURPOSE
 *		Find and covert all anim columns in specified room that are the
 *		the as the specified column
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int GatherColumns (
			MapType		*map,
			RoomType	*room,
			int			 x,
			AnimGroup	*agrp,
			int			 pass
)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GatherColumns";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		for (; x < room->Width; x++)
		{
			int		frames;
			int		fheight;
			int		tiletype;

			if (GetFrameInfo (map, room, x, &frames, &fheight, &tiletype))
			{
				if (frames == agrp->Frames && tiletype == agrp->TileType)
				{
					int		f;

					for (f = 0; f < fheight; f++)
					{
						if (room->TileWidth == 8)
						{
							ConvertColumn (map, room, x, f, agrp, 0, 0, 0, pass);
						}
						else if (room->TileHeight == 16)
						{
							ConvertColumn (map, room, x, f, agrp, 1,   0, 8, pass);
							ConvertColumn (map, room, x, f, agrp, 2,   8, 8, pass);
							ConvertColumn (map, room, x, f, agrp, 3, 128, 8, pass);
							ConvertColumn (map, room, x, f, agrp, 4, 136, 8, pass);
						}
						else
						{
							FailMess ("Tiles not 8x8 or 16x16 in room '%s'\n", room->Name);
						}
					}
				}
				else
				{
					//
					// if separate then stop as soon as something doesn't match
					//
					if (SeparateAnim)
					{
						agrp->XPos2 = x - 1;
						break;
					}
				}
			}
		}
	}
	return (!ErrorCount);

} /* GatherColumns */


/*********************************************************************
 *
 * CompareFrame2Frame
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		FALSE = frames are different
 *		TRUE  = frames are the same
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
BOOL CompareFrame2Frame (AnimGroup *grp, int f1, int f2)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CompareFrame2Frame";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		int		numframes = grp->Frames;

		while (!EndOfList (grp) && grp->Frames == numframes)
		{
			AnimColumn	*col;
			unsigned	 csize;

			csize = CharSize[grp->TileType];

			col = Head (grp->ColumnList);
			while (!EndOfList (col))
			{
				UBYTE	*data;
				UBYTE	*data1;
				UBYTE	*data2;

				data  = ActivateXTRA (col->FrameData);
				data1 = data + f1 * csize;
				data2 = data + f2 * csize;

				if (memcmp (data1, data2, csize))
				{
					ReleaseXTRA(col->FrameData);
					return FALSE;
				}
				ReleaseXTRA(col->FrameData);

				col = Next (col);
			}
			if (SeparateAnim)
			{
				break;
			}
			grp = Next (grp);
		}

		return TRUE;
	}
} /* CompareFrame2Frame */


/*********************************************************************
 *
 * WriteFrame
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void WriteFrame (int fh, AnimGroup *grp, int frame, unsigned *animsize, int zero)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteFrame";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		int		numframes = grp->Frames;

		while (!EndOfList (grp) && grp->Frames == numframes)
		{
			AnimColumn	*col;
			unsigned	 csize;

			csize = CharSize[grp->TileType];

			col = Head (grp->ColumnList);
			while (!EndOfList (col))
			{
				UBYTE	*data;

				data  = ActivateXTRA (col->FrameData);
				data += frame * csize;

				if (zero)
				{
					EIO_Write (fh, ZeroData, csize);
				}
				else
				{
					if (grp->TileType == TILESET_IMAGE)
					{
						switch (DestHardware)
						{
						case DEST_SNES:
							{
								SegaChar	 work;

								memcpy (&work, data, csize);
								ConvertSEGAtoSNES (&work);
								EIO_Write (fh, &work, csize);
							}
							break;
						case DEST_GENESIS:
							EIO_Write (fh, data, csize);
							break;
						case DEST_GEAR:
							{
								SegaChar	 work;

								memcpy (&work, data, csize);
								ConvertSEGAtoGEAR (&work);
								EIO_Write (fh, &work, csize);
							}
							break;
						}
					}
					else
					{
						EIO_Write (fh, data, csize);
					}
				}
				ReleaseXTRA(col->FrameData);
				*animsize += csize;

				col = Next (col);
			}
			if (SeparateAnim)
			{
				break;
			}
			grp = Next (grp);
		}
	}
} /* WriteFrame */


/*********************************************************************
 *
 * ConvertAnimTiles
 *
 * PURPOSE
 *		Search all animation rooms and create animation tables
 *		Assign characters for animations and set ImageTile info
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int ConvertAnimTiles (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ConvertAnimTiles";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		RoomType	**roomptr;
		RoomType	*room;
		unsigned	 charMemUsed = FirstChar[TILESET_2BITS] * sizeof (Color4Char);
		unsigned	 animsize    = 0;
		int			 adtfh;
		char		 adtname[EIO_MAXPATH];

		EIO_fnmerge (adtname, OutPath, EIO_Name(MapName), ".adt");
		strupr (adtname);

		printf ("Writing animation data '%s'\n", adtname);

		if (WriteFiles && WriteTiles)
		{
			adtfh = EIO_WriteOpen (adtname);
			if (adtfh == (-1))
			{
				FailMess ("Couldn't open '%s'\n", adtname);
			}
		}

		roomptr = map->Rooms + 1;
		while (*roomptr) {
			room = *roomptr;

			if (room->CompositeTSet)
			{
			}
			else
			{
				RoomInfo	*rinf;

				rinf = calloc (1, sizeof (RoomInfo));
				if (!rinf)
				{
					OutOfMemErr ("Couldn't allocate RoomInfo\n");
				}
				rinf->AnimGroupList = &rinf->AnimGroupListX;
				InitList (rinf->AnimGroupList);
				room->UserData = rinf;

				if (!room->Name || !strlen(room->Name))
				{
					FailMess ("Room size (%d, %d, %d) has no name!\n", room->Width, room->Height, room->LayerCount);
				}

				if (room->UserType == ROOM_ANIMATION)
				{
					int		x;

					if (room->Comments)
					{
						char	**s;

						s = room->Comments;
						while (*s)
						{
							ParseAnimArgs (*s);
							s++;
						}
					}

					printf ("Scanning Animation Room '%s'\n", room->Name);

					//
					// do each column
					//
					for (x = 0; x < room->Width; x++)
					{
						int		frames;
						int		fheight;
						int		tiletype;

						if (GetFrameInfo (map, room, x, &frames, &fheight, &tiletype))
						{
							//
							// See if this type of column has already been
							// scanned in this room
							//
							if (frames)
							{
								AnimGroup	*agrp;

								if (!SeparateAnim)
								{
									agrp = Head (rinf->AnimGroupList);
									while (!EndOfList (agrp))
									{
										if (agrp->Frames == frames && agrp->TileType == tiletype)
										{
											break;
										}
										agrp = Next (agrp);
									}
								}

								//
								// If we are at the end of the list of anim groups
								// then we didn't find a match so add this new type
								//
								if (SeparateAnim || EndOfList (agrp) )
								{
									AnimGroup	*newgrp;

									newgrp = CreateNode (sizeof (AnimGroup), NULL);
									if (!newgrp)
									{
										OutOfMemErr ("AnimGroup\n");
									}
									newgrp->ColumnList = &newgrp->ColumnListX;
									InitList (newgrp->ColumnList);
									newgrp->Frames   = frames;
									newgrp->TileType = tiletype;
									newgrp->XPos     = x;
									AddTail (rinf->AnimGroupList, newgrp);

									if (!GatherColumns (map, room, x, newgrp, 1))
									{
										return FALSE;
									}

									if (SeparateAnim)
									{
										x = newgrp->XPos2;
									}
								}
							}

							if (frames == 1)
							{
								WarnMess ("Animation with only one frame in room '%s' column %d\n", room->Name, x);
							}
						}
					}

					if (ErrorCount)
					{
						return FALSE;
					}

					//
					// sort the groups, first by number of frames
					// them by type of tile (4 color, 16 color, ...)
					//
					if (!SeparateAnim) 
					{
						short	done;

						do
						{
							AnimGroup	*grp;

							done = TRUE;

							grp = Head (rinf->AnimGroupList);
							while (!EndOfList (grp))
							{
								AnimGroup	*nextgrp;

								nextgrp = Next (grp);
								if (EndOfList (nextgrp))
								{
									break;
								}

								if ((nextgrp->Frames <  grp->Frames) ||
									(nextgrp->Frames == grp->Frames && AnimOrder[nextgrp->TileType] < AnimOrder[grp->TileType]))
								{
									Remove (grp);
									InsertAfter (nextgrp, grp);
									nextgrp = grp;

									done = FALSE;
								}
								grp = nextgrp;
							}
						} while (!done);
					}

					//
					// Okay, they are sorted so number them
					//
					{
						short	done;

						do
						{
							AnimGroup	*grp;

							done = TRUE;

							grp = Head (rinf->AnimGroupList);
							while (!EndOfList (grp))
							{
								AnimColumn	*col;

								col = Head (grp->ColumnList);
								while (!EndOfList (col))
								{
									col->CharID = (UWORD) ((charMemUsed + CharSize[grp->TileType] - 1) / CharSize[grp->TileType]);
									charMemUsed = (col->CharID + 1) * CharSize[grp->TileType];

									col = Next (col);
								}
								grp = Next (grp);
							}
						} while (!done);
					}
					
					//
					// Now, do the room again
					//
					// This time the columns will match and you can assign
					// CharID into the ImageTile (Block) information
					//
					//
					// do each type of group
					//

					{
						AnimGroup	*grp;

						grp = Head (rinf->AnimGroupList);
						while (!EndOfList (grp))
						{
							if (!GatherColumns (map, room, grp->XPos, grp, 2))
							{
								return FALSE;
							}
							grp = Next (grp);
						}
					}

					if (ErrorCount)
					{
						return FALSE;
					}

					//
					// Write out the stuff for this room
					//
					if (WriteFiles && WriteTiles)
					{
						ListType	 AnimListX;
						ListType	*AnimList = &AnimListX;
						AnimGroup	*grp;
						char		 anifile[EIO_MAXPATH];
						char		 rmname[9];
						FILE		*fp;

						InitList (AnimList);

						strncpy (rmname, room->Name, 8);
						rmname[8] = '\0';
						strupr (rmname);

						EIO_fnmerge (anifile, OutPath, rmname, ".ani");

						printf ("Writing Anim Macro file '%s'\n", anifile);

						BinaryOut = BinFileTable[FILE_ANIMATION];

						if (BinaryOut)
						{
							fp = fopen (anifile, "wb");
						}
						else
						{
							fp = fopen (anifile, "w");
						}
						if (!fp)
						{
							FailMess ("Couldn't open '%s'\n", anifile);
						}

						if (!BinaryOut)
						{
							fprintf (fp,
								";*\n"
								";* Animation from room '%s'\n"
								";*\n"
								,
								room->Name
							);
						}

						grp = Head (rinf->AnimGroupList);
						while (!EndOfList (grp))
						{
							int				 f;
							UWORD			*frameoffsets;
							UWORD			*framesizes;
							UWORD			*frameIDs;
							unsigned		 charoffset;
							MixedAnimGroup	*mag;

							frameoffsets = calloc (grp->Frames, sizeof (UWORD));
							framesizes   = calloc (grp->Frames, sizeof (UWORD));
							frameIDs     = calloc (grp->Frames, sizeof (UWORD));
							mag          = CreateNode (sizeof (MixedAnimGroup), NULL);

							if (!frameoffsets || !framesizes || !frameIDs || !mag)
							{
								OutOfMemErr ("frameoffsets\n");
							}


							{
								AnimColumn	*col;

								col = Head (grp->ColumnList);
								charoffset = col->CharID * CharSize[grp->TileType];
							}

							AddTail (AnimList, mag);
							mag->Frames       = grp->Frames;
							mag->CharOffset   = charoffset;
							mag->FrameIDs     = frameIDs;
							mag->FrameOffsets = frameoffsets;
							mag->FrameSizes   = framesizes;

							for (f = 0; f < grp->Frames; f++)
							{
								short	oldf;

								for (oldf = 0; oldf < f; oldf++)
								{
									if (CompareFrame2Frame (grp, oldf, f))
									{
										break;
									}
								}

								//
								// if this frame is unique then
								// write it's data
								//
								// Also, record it's data for the macros
								//
								if (oldf == f)
								{
									frameoffsets[f] = (UWORD) animsize;
									WriteFrame (adtfh, grp, f, &animsize, FALSE);
									framesizes[f] = (UWORD) (animsize - frameoffsets[f]);

									//
									// Write first frame to font
									//
									if (f == 0)
									{
										unsigned	junk;

										WriteFrame (charfh, grp, f, &junk, ZeroAnimChars);
									}
								}
								else
								{
									frameoffsets[f] = frameoffsets[oldf];
									framesizes[f]   = framesizes[oldf];
									if (DebugSwitch)
									{
										printf ("Anim Frame %2d of %2d is the same as frame %2d\n", f + 1, grp->Frames, oldf + 1);
									}
								}
								frameIDs[f] = oldf;

							}

							if (SeparateAnim)
							{
								grp = Next (grp);
							}
							else
							{
								f = grp->Frames;
								while (!EndOfList (grp))
								{
									if (grp->Frames != f)
									{
										break;
									}
									grp = Next (grp);
								}
							}
						}

						//
						// If ZeroAnimChars then create a macro table
						// that tells how to copy the first anim frames
						// to fill out the font
						//
						if (ZeroAnimChars)
						{
							MixedAnimGroup	*mag;

							mag = Head (AnimList);
							if (!BinaryOut)
							{
								fprintf (fp,
									"AINIT_%s_START:\n"
									,
									rmname
								);
							}

							{
								if (!BinaryOut)
								{
									fprintf (fp,
										"\tani_dma %s_DATA+$%04x,$%04x,$%04x	; Frame %2d (%2d) of %2d\n"
										,
										EIO_Name(adtname),
										mag->FrameOffsets[0],
										mag->CharOffset,
										mag->FrameSizes[0],
										1,
										mag->FrameIDs[0] + 1,
										mag->Frames
									);
								}

								mag = Next (mag);
							}

							if (!BinaryOut)
							{
								fprintf (fp,
									"AINIT_%s_LEN equ *-AINIT_%s_START\n\n\n"
									,
									rmname,
									rmname
								);
							}
						}

						//
						// write anim macros for this frame
						//
						if (!InterleaveAni)
						{
							MixedAnimGroup	*mag;
							short			 count = 0;

							mag = Head (AnimList);
							while (!EndOfList (mag))
							{
								short	f;

								if (!BinaryOut)
								{
									fprintf (fp,
										"AMAP_%s%d_START:\n"
										,
										rmname,
										count
									);
								}

								for (f = 0; f < mag->Frames; f++)
								{
									if (!BinaryOut)
									{
										fprintf (fp,
											"\tani_dma %s_DATA+$%04x,$%04x,$%04x	; Frame %2d (%2d) of %2d\n"
											,
											EIO_Name(adtname),
											mag->FrameOffsets[f],
											mag->CharOffset,
											mag->FrameSizes[f],
											f + 1,
											mag->FrameIDs[f] + 1,
											mag->Frames
										);
									}
									else
									{
										UWORD	data[3];

										data[0] = mag->FrameOffsets[f];
										data[1] = (UWORD) mag->CharOffset;
										data[2] = mag->FrameSizes[f];

										if (!LittleEndian)
										{
											WordSex(data[0]);
											WordSex(data[1]);
											WordSex(data[2]);
										}

										fwrite (&data, sizeof (data), 1, fp);
									}
								}

								if (!BinaryOut)
								{
									fprintf (fp,
										"AMAP_%s%d_LEN equ *-AMAP_%s%d_START\n\n\n"
										,
										rmname,
										count,
										rmname,
										count
									);
								}
								else
								{
									UWORD	data[3];

									data[0] = 0;
									data[1] = 0;
									data[2] = 0;

									fwrite (&data, 2, 1, fp);										
								}


								count++;
								mag = Next (mag);
							}
						}
						else
						{
							MixedAnimGroup	*mag;
							short			 done;

							if (!BinaryOut)
							{
								fprintf (fp,
									"AMAP_%s_START:\n"
									,
									rmname
								);
							}

							do {
								done = TRUE;


								mag = Head (AnimList);
								while (!EndOfList (mag))
								{
									int		f;

									f = mag->FrameCounter++;

									if (!BinaryOut)
									{
										fprintf (fp,
											"\tani_dma %s_DATA+$%04x,$%04x,$%04x	; Frame %2d (%2d) of %2d\n"
											,
											EIO_Name(adtname),
											mag->FrameOffsets[f],
											mag->CharOffset,
											mag->FrameSizes[f],
											f + 1,
											mag->FrameIDs[f] + 1,
											mag->Frames
										);
									}
									else
									{
										UWORD	data[3];

										data[0] = mag->FrameOffsets[f];
										data[1] = (UWORD) mag->CharOffset;
										data[2] = mag->FrameSizes[f];

										if (!LittleEndian)
										{
											WordSex(data[0]);
											WordSex(data[1]);
											WordSex(data[2]);
										}

										fwrite (&data, sizeof (data), 1, fp);
									}

									if (mag->FrameCounter != mag->Frames)
									{
										done = FALSE;
									}
									else
									{
										mag->FrameCounter = 0;
									}
									mag = Next (mag);
								}
							} while (!done);

							if (!BinaryOut)
							{
								fprintf (fp,
									"AMAP_%s_LEN equ *-AMAP_%s_START\n\n"
									,
									rmname,
									rmname
								);
							}
							else
							{
								UWORD	data[3];

								data[0] = 0;
								data[1] = 0;
								data[2] = 0;

								fwrite (&data, 2, 1, fp);										
							}
						}

						{
							MixedAnimGroup	*mag;

							while ((mag = RemTail (AnimList)) != NULL)
							{
								free (mag->FrameIDs);
								free (mag->FrameOffsets);
								free (mag->FrameSizes);
								DeleteNode (mag);
							}
						}

						fclose (fp);
					}
				}
			}
			roomptr++;
		}

		if (WriteFiles && WriteTiles)
		{
			EIO_Close (adtfh);
		}

		//
		// Write out padding to first character
		//
		{
			unsigned oldCharMemUsed;

			oldCharMemUsed = charMemUsed;
			charMemUsed = ((charMemUsed + LargestBlank - 1) / LargestBlank) * LargestBlank;

			if (oldCharMemUsed != charMemUsed)
			{
				EIO_Write (charfh, ZeroData, charMemUsed - oldCharMemUsed);
			}
		}

		{
			short	t;

			for (t = 0; t < NUM_TILE_TYPES; t++)
			{
				if (CharSize[t])
				{
					BlankChar[t] = (UWORD) (charMemUsed / CharSize[t]);
				}
			}
		}

		charMemUsed += LargestBlank;
		FirstChar[TILESET_2BITS] = (UWORD) ((charMemUsed + sizeof (Color4Char) - 1) / sizeof (Color4Char));

		return (!ErrorCount);
	}
} /* ConvertAnimTiles */


/*********************************************************************
 *
 * ConvertAllTiles
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int ConvertAllTiles (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ConvertAllTiles";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		TilesetType	**tsptr;
		TilesetType	*ts;
		TilesetInfo	*tsi;
		int			 tiletype;

		/*
		** Convert all Image Tiles to characters
		**   First Do 4 Color Chars
		**	 Then  Do 16 Color Chars
		**   Then  Do 256 Color Chars
		**   Then  Do Mode7 Chars
		*/
		{
			int			 ttypes;
		
			for (ttypes = 0; OrderTable[ttypes] >= 0; ttypes++) {
				tiletype = OrderTable[ttypes];

				tsptr = map->Tilesets + 1;
				while (*tsptr) {
					ts  = *tsptr;
					tsi = ts->UserData;
					CurrentTileset = ts;

					if (ts->UserType == tiletype) {
						if (tiletype != TILESET_CONTOUR) {
							printf ("Converting Tileset '%s'\n", EIO_Filename (ts->FileSpec));
							if (ts->GridedTilesRoom) {
							} else {

								/*
								** Convert all packable characters
								*/

								if (ts->Width == 8) {
									TileDataGroup	*tdg;
									short			 t;
									short			 tile;

									/*
									** Convert 8x8 Tiles to Sega Chars
									*/
						
									tile = 1;
									tdg  = Head (ts->TMG.TileGroupList);
									while (!EndOfList (tdg)) {
										UBYTE	*data;

										for (t = 0; t < tdg->NumTiles; t++) {
											UBYTE	tflags;

											data   = ActivateXTRA (tdg->TileData);
											data  += t * ts->TMG.Width * ts->TMG.Height;
											tflags = tsi->Flags[tile];
											if ((!(tflags & TFLG_DONTPACK)) &&
												(!(tflags & TFLG_ANIM)) &&
												(tflags & TFLG_CONVERT))
											{
												CurrentTileNum = tile - 1;
												tsi->Flags[tile] |= AddChar (data, 0, &tsi->ImageChars[tile], TRUE, tiletype);
											}
											ReleaseXTRA(tdg->TileData);
											tile++;
										}
										tdg = Next (tdg);
									}
								} else {
									TileDataGroup	*tdg;
									short			 t;
									short			 tile;
									/*
									** Convert 16x16 Tiles to Sega Chars
									*/
									tile = 1;
									tdg  = Head (ts->TMG.TileGroupList);
									while (!EndOfList (tdg)) {
										UBYTE		*data;
										UBYTE		 tflags;

										for (t = 0; t < tdg->NumTiles; t++) {
											UBYTE		 flags = 0x00;
											ImageTile	*im;

											tflags = tsi->Flags[tile];
											if (!(tflags & TFLG_DONTPACK) &&
												!(tflags & TFLG_ANIM) &&
												(tflags & TFLG_CONVERT))
											{
												CurrentTileNum = tile;
												im = &tsi->ImageTiles[tile];
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data          , 8, &im->TL, TRUE, tiletype);
												ReleaseXTRA(tdg->TileData);
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data       + 8, 8, &im->TR, TRUE, tiletype);
												ReleaseXTRA(tdg->TileData);
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data + 128    , 8, &im->BL, TRUE, tiletype);
												ReleaseXTRA(tdg->TileData);
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data + 128 + 8, 8, &im->BR, TRUE, tiletype);
												ReleaseXTRA(tdg->TileData);
												{
													UBYTE	tflags;

													tflags = (UBYTE) ((flags & CFLG_NOTBLANK) ^ CFLG_NOTBLANK);
													if (!(flags & CFLG_NOTXSYM) &&
														im->TL == im->TR &&
														im->BL == im->BR) {
														tflags |= TFLG_XSYM;
													}
													if (!(flags & CFLG_NOTYSYM) &&
														im->TL == im->BL &&
														im->TR == im->BR) {
														tflags |= TFLG_YSYM;
													}
													tsi->Flags[tile] |= tflags;
												}
												if (TinyMode)
												{
													UWORD	c0;
													UWORD	c1;
													UWORD	c2;
													UWORD	c3;
													UWORD	colorset;

													c0 = (im->TL & HDWR_COLORSET_MASK) ;
													c1 = (im->TR & HDWR_COLORSET_MASK) ;
													c2 = (im->BL & HDWR_COLORSET_MASK) ;
													c3 = (im->BR & HDWR_COLORSET_MASK) ;

													colorset = c0 | c1 | c2 | c3;

													if (!c0) c0 = colorset;
													if (!c1) c1 = colorset;
													if (!c2) c2 = colorset;
													if (!c3) c3 = colorset;

													if ((c0 != c1) ||
														(c0 != c2) ||
														(c0 != c3) ||
														(c1 != c2) ||
														(c1 != c3) ||
														(c2 != c3))
													{
														WarnMess ("Tile #%d uses more than 1 colorset in tileset '%s'\n", tile, EIO_Filename (ts->FileSpec));
													}
												}
											}
											tile++;
										}
										tdg = Next (tdg);
									}
					
								}

								/*
								** Convert all NONE packable characters
								*/

								if (ts->Width == 8) {
									TileDataGroup	*tdg;
									short			 t;
									short			 tile;
									/*
									** Convert 8x8 Tiles to Sega Chars
									*/

									tile = 1;
									tdg  = Head (ts->TMG.TileGroupList);
									while (!EndOfList (tdg)) {
										UBYTE	*data;
							
										for (t = 0; t < tdg->NumTiles; t++) {
											UBYTE	tflags;

											tflags = tsi->Flags[tile];
											if ((tflags & TFLG_DONTPACK) &&
												!(tflags & TFLG_ANIM) &&
												(tflags & TFLG_CONVERT))
											{
												CurrentTileNum = tile - 1;
												data  = ActivateXTRA (tdg->TileData);
												data += t * ts->TMG.Width * ts->TMG.Height;
												tsi->Flags[tile] |= AddChar (data, 0, &tsi->ImageChars[tile], FALSE, tiletype);
												ReleaseXTRA(tdg->TileData);
											}
											tile++;
										}
										tdg = Next (tdg);
									}
								} else {
									TileDataGroup	*tdg;
									short			 t;
									short			 tile;
									/*
									** Convert 16x16 Tiles to Sega Chars
									*/

									tile = 1;
									tdg  = Head (ts->TMG.TileGroupList);
									while (!EndOfList (tdg)) {
										UBYTE	*data;

										for (t = 0; t < tdg->NumTiles; t++) {
											UBYTE		 flags = 0x00;
											ImageTile	*im;
											UBYTE		 tflags;

											tflags = tsi->Flags[tile];
											if ((tflags & TFLG_DONTPACK) &&
												!(tflags & TFLG_ANIM) &&
												(tflags & TFLG_CONVERT))
											{
												CurrentTileNum = tile - 1;
												im = &tsi->ImageTiles[tile];
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data          , 8, &im->TL, FALSE, tiletype);
												ReleaseXTRA(tdg->TileData);
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data       + 8, 8, &im->TR, FALSE, tiletype);
												ReleaseXTRA(tdg->TileData);
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data + 128    , 8, &im->BL, FALSE, tiletype);
												ReleaseXTRA(tdg->TileData);
												data   = ActivateXTRA (tdg->TileData);
												data  += t * ts->TMG.Width * ts->TMG.Height;
												flags |= AddChar (data + 128 + 8, 8, &im->BR, FALSE, tiletype);
												ReleaseXTRA(tdg->TileData);
												{
													UBYTE	tflags;

													tflags = (UBYTE) ((flags & CFLG_NOTBLANK) ^ CFLG_NOTBLANK);
													if (!(flags & CFLG_NOTXSYM) &&
														im->TL == im->TR &&
														im->BL == im->BR) {
														tflags |= TFLG_XSYM;
													}
													if (!(flags & CFLG_NOTYSYM) &&
														im->TL == im->BL &&
														im->TR == im->BR) {
														tflags |= TFLG_YSYM;
													}
													tsi->Flags[tile] |= tflags;
												}
											}
											tile++;
										}
										tdg = Next (tdg);
									}
					
								}

							}
						} else if (ContourMode != CONTOUR_GENFLIP) {
							FILE	*fp;
							char	 tsname[EIO_MAXPATH];
							int		 t;
							int		 x;
							int		 y;
							TileDataGroup	*tdg;

							if (WriteFiles) {
								EIO_fnmerge (tsname, OutPath, EIO_Name(ts->FileSpec), ".con");
								printf ("Writing Contour Tileset '%s'\n", tsname);
							}

							if (ts->Width == 16 && ts->Height == 16)
							{
								BinaryOut = BinFileTable[FILE_CONTOUR];

								if (WriteFiles) {
									if (BinaryOut) {
										fp = fopen (tsname, "wb");
									} else {
										fp = fopen (tsname, "w");
									}

									if (!fp) {
										FailMess ("Writing Contour Tile: Couldn't open '%s'\n", tsname);
									}

									if (!BinaryOut) {
										switch (DestHardware)
										{
										case DEST_SNES:
											StartDataStatements ("\tHEX\t","%02x","",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
											break;
										case DEST_GENESIS:
											StartDataStatements ("\tdc.b\t","%d",",",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
											break;
										case DEST_GEAR:
											StartDataStatements ("\tdb\t","$%02x",",",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
											break;
										}
									}
								}

								tdg = Head (ts->TMG.TileGroupList);
								while (!EndOfList (tdg)) {
									UBYTE	*data;

									data = ActivateXTRA (tdg->TileData);

									for (t = 0; t < tdg->NumTiles; t++) {
										UBYTE	 oldcon[16];
										UBYTE	 contour[16][2];
										UWORD	 color = 0;

										for (x = 0; x < 16; x++) {
											BOOL	found = FALSE;
											int		first = 0;

											for (y = 0; y < 16; y++) {
												if (data[x + y * ts->TMG.Width]) {
													if (!found)
													{
														first = y;
														found = TRUE;
														color = data[x + y * ts->TMG.Width];
													}
												}
												else
												{
													if (found)
													{
														break;
													}
												}
											}
											if (!found)
											{
												first = -1;
												y     = -1;
											}
											oldcon[x]     = (UBYTE) (first + 1);
											contour[x][0] = (UBYTE) (first + 1);
											contour[x][1] = (UBYTE) (y - first);
										}

										if (WriteFiles) {
											if (ContourStyle == 1)
											{
												if (!LittleEndian)
												{
													WordSex (color);
												}
												if (BinaryOut) {
													fwrite (&color, 2, 1, fp);
													fwrite (contour, 32, 1, fp);
												} else {
													WriteDataStatements (fp, (UBYTE *)&color, 2);
													WriteDataStatements (fp, (UBYTE *)contour, 32);
												}
											}
											else
											{
												if (BinaryOut) {
													fwrite (oldcon, 16, 1, fp);
												} else {
													WriteDataStatements (fp, oldcon, 16);
												}
											}
										}
										data += ts->TMG.Width * ts->TMG.Height;
									}
									ReleaseXTRA(tdg->TileData);
									tdg = Next (tdg);
								}

								if (WriteFiles) {
									if (!BinaryOut) {
										EndDataStatements (fp);
									}
									fclose (fp);
								}
							}
							else if (ts->Width == 8 && ts->Height == 8)
							{
								short	i;
								UBYTE	cdata[16*16];
								UBYTE	 contour[16];

								if (ts->TileCount != 1024)
								{
									FailMess ("There must be 1024 8x8 contour tiles\n");
								}
								BinaryOut = BinFileTable[FILE_CONTOUR];

								if (WriteFiles) {
									if (BinaryOut) {
										fp = fopen (tsname, "wb");
									} else {
										fp = fopen (tsname, "w");
									}

									if (!fp) {
										FailMess ("Writing Contour Tile: Couldn't open '%s'\n", tsname);
									}

									if (!BinaryOut) {
										switch (DestHardware)
										{
										case DEST_SNES:
											StartDataStatements ("\tHEX\t","%02x","",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
											break;
										case DEST_GENESIS:
											StartDataStatements ("\tdc.b\t","%d",",",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
											break;
										case DEST_GEAR:
											StartDataStatements ("\tdb\t","$%02x",",",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
											break;
										}
									}
								}

								for (i = 0; i < 256; i++)
								{
									int		x1;
									int		y1;

									for (y1 = 0; y1 < 2; y1++)
									{
										for (x1 = 0; x1 < 2; x1++)
										{
											UBYTE	tile8x8[8*8];

											ReadTileData (ts,
														((i / 16) * 64) +
														((i % 16) *  2) +
														y1 * 32 + x1,
														tile8x8);

											{
												UBYTE	*dest;
												UBYTE	*src;
												short	 j;

												dest = cdata + x1 * 8 + y1 * (16 * 8);
												src  = tile8x8;

												for (j = 0; j < 8; j++)
												{
													memcpy (dest, src, 8);
													src  += 8;
													dest += 16;
												}
											}
										}
									}

									for (x = 0; x < 16; x++) {
										for (y = 0; y < 16; y++) {
											if (cdata[x + y * 16]) {
												break;
											}
										}
										if (y < 16) {
											contour[x] = (UBYTE) (y + 1);
										} else {
											contour[x] = 0;
										}
									}

									if (WriteFiles) {
										if (BinaryOut) {
											fwrite (contour, 16, 1, fp);
										} else {
											WriteDataStatements (fp, contour, 16);
										}
									}
								}

								if (WriteFiles) {
									if (!BinaryOut) {
										EndDataStatements (fp);
									}
									fclose (fp);
								}
							}
							else
							{
								FailMess ("Writing Contour Tileset: Tileset not 16x16 or 8x8 pixels\n");
							}

				
						} else { // ContourMode == CONTOUR_GENFLIP
						}
					}

					tsptr++;
				}

				/*
				** Compute Position of First Character in Character Font
				** for NEXT type of characters to be converted.
				**
				** Example:  If 37 4Color characters were found
				**           then they require 16 bytes each
				**			 taking a total of 592 bytes which
				**			 occupy the space of 18.5 16color
				**			 characters so, rounding up the first
				**			 available 16color character slot is
				**			 character 19.
				**
				** Also round up Current NumChars[tiletype] so that it is flush
				** with the next set of characters.
				**
				** Example:  In the above example there were 37 4color
				**			 characters taking 592 bytes.  The next available
				**			 16color character will start at byte 608 so if
				**			 we increase the number of 4color characters to
				**			 38 then things will work out when we write the
				**			 character to a file later.
				*/
				{
					int		nexttype;
					long	currsize;
					long	newstart;

					nexttype = OrderTable[ttypes + 1];

					switch (nexttype) {
					case TILESET_IMAGE:
					case TILESET_2BITS:
					case TILESET_8BITS:
						if (nexttype >= 0 && nexttype != TILESET_CONTOUR) {
							currsize  = ((long)FirstChar[tiletype] + (long)NumChars[tiletype]) * CharSize[tiletype];
							FirstChar[nexttype] = (UWORD)((currsize + CharSize[nexttype] - 1) / CharSize[nexttype]);
							newstart  = FirstChar[nexttype] * CharSize[nexttype];
							NumChars[tiletype] += (UWORD)((newstart - currsize) / CharSize[tiletype]);
						}
						break;
					}
				}
			}
		}
		return (!ErrorCount);
	}
} /* ConvertAllTiles */


/*********************************************************************
 *
 * WriteTileData
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		01/23/95 (dcc) - Should call UpdateXTRA() instead of ReleaseXTRA().
 *
 * SEE ALSO
 *
*/
BOOL WriteTileData (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteTileData";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		int			 tiletype;

		map = map;	// stop warning

		printf ("Total   4 Color 8x8 Characters = %d\n", NumChars[TILESET_2BITS]);
		printf ("Total  16 Color 8x8 Characters = %d\n", NumChars[TILESET_IMAGE]);
		printf ("Total 256 Color 8x8 Characters = %d\n", NumChars[TILESET_8BITS]);
		printf ("Total     Mode7 8x8 Characters = %d\n", NumChars[TILESET_MODE7]);

		/*
		** Write out SEGA character but convert them to SNES
		** character if SNES data is desired.
		*/
		if (WriteFiles && WriteTiles) {
			int		 ttypes;

			printf ("Writing Character Font '%s'\n", TSName);

			BinaryOut = BinFileTable[FILE_FONT];

			EIO_Write (charfh, ZeroData, LargestBlank);

			for (ttypes = 0; OrderTable[ttypes] >= 0; ttypes++) {
				tiletype = OrderTable[ttypes];

				switch (tiletype) {
				case TILESET_IMAGE:
					if (DestHardware != DEST_GENESIS) {
						CharGroup	*chgrp;
						short		 i;

						chgrp = Head (CharList[tiletype]);
						while (!EndOfList(chgrp))
						{
							SegaChar	*sc;

							sc = ActivateXTRA (chgrp->Chars);
							for (i = 0; i < chgrp->CharCount; i++) {
								switch (DestHardware)
								{
								case DEST_SNES:
									ConvertSEGAtoSNES (sc);
									break;
								case DEST_GEAR:
									ConvertSEGAtoGEAR (sc);
									break;
								}
								sc++;
							}
							UpdateXTRA(chgrp->Chars);
							chgrp = Next (chgrp);
						}
					}
				case TILESET_2BITS:
				case TILESET_8BITS:
					if (BigSNESTiles && tiletype == TILESET_IMAGE) {
						int		 numchars;
						int			 i;
						SegaChar	 snes;
						int			 ndx;

						for (numchars = 0; numchars < NumChars[tiletype]; numchars += 8) {
							ndx = (numchars);
							for (i = 0; i < 8; i++) {
								CopyChar (ndx, &snes, tiletype);
								EIO_Write (charfh, &snes, sizeof (SegaChar));
								CopyChar (ndx + 1, &snes, tiletype);
								EIO_Write (charfh, &snes, sizeof (SegaChar));
								ndx += 4;
							}
							ndx = (numchars + 2);
							for (i = 0; i < 8; i++) {
								CopyChar (ndx, &snes, tiletype);
								EIO_Write (charfh, &snes, sizeof (SegaChar));
								CopyChar (ndx + 1, &snes, tiletype);
								EIO_Write (charfh, &snes, sizeof (SegaChar));
							}
						}
					} else {
						CharGroup	*chgrp;
						int			 count = NumChars[tiletype];

						chgrp = Head (CharList[tiletype]);
						while (!EndOfList(chgrp))
						{
							void	*data;

							data = ActivateXTRA (chgrp->Chars);
							EIO_Write (charfh, data, (long)chgrp->CharCount * (long)(CharSize[tiletype]));
							ReleaseXTRA (chgrp->Chars);
							if (DebugSwitch)
							{
								printf ("Writing %d type %d chars\n", chgrp->CharCount, tiletype);
							}
							count-= chgrp->CharCount;
							chgrp = Next (chgrp);
						}

						if (count > 0 && DebugSwitch)
						{
							printf ("Writing %d type %d padding chars\n", count, tiletype);
						}
						while (count > 0)
						{
							EIO_Write (charfh, FlipTable, CharSize[tiletype]);
							count--;
						}

					}
					break;
				case TILESET_MODE7:
					if (!DumpMode7)
					{
						CharGroup	*chgrp;

						chgrp = Head (CharList[tiletype]);
						while (!EndOfList(chgrp))
						{
							void	*data;

							data = ActivateXTRA (chgrp->Chars);
							EIO_Write (charfh, data, (long)chgrp->CharCount * (long)(CharSize[tiletype]));
							ReleaseXTRA(chgrp->Chars);
							chgrp = Next (chgrp);
						}
					}
				}
			}
		}
		MinMemFree ();

		return (!ErrorCount);
	}
} /* WriteTileData */


/*********************************************************************
 *
 * ScanTiles
 *
 * PURPOSE
 *		Goes through tilesets, converts all Image tiles to
 *		SEGA or SNES characters.  Converts all Contour tiles to
 *		a contour table.  Remember which SEGA/SNES characters
 *		Image tiles were converted into.
 *
 * INPUT
 *		map		: pointer to a MapType.
 *
 * RETURN VALUE
 *		FALSE = Failure.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
BOOL ScanTiles (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ScanTiles";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		if (!WriteTiles)
		{
			return TRUE;
		}

		if (WriteTiles && WriteFiles)
		{
			EIO_fnmerge (TSName, OutPath, EIO_Name(MapName), ".chr");
			
			charfh = EIO_WriteOpen (TSName);
			if (charfh == (-1)) {
				FailMess ("Writing character set:Couldn't open '%s'\n", TSName);
			}
		}

		if (InitTilesetCollections (map))
		{
			if (ConvertAnimTiles (map))
			{
				return ConvertAllTiles (map);
			}
		}
		return FALSE;
	}
} /* ScanTiles */


/*********************************************************************
 *
 * CompositeTileChar
 *
 * PURPOSE
 *		Look up one SNES/SEGA character in a 2x2 8x8 composite tile.
 *
 * INPUT
 *		map		: pointer to a MapType.
 *		ts		: pointer to Composite Tileset.
 *		tileID	: number of composite tile (1 <-> ts->TileCount??)
 *		xoff	: x offset of character we want (0 <-> 1)
 *		yoff	: y offset of character we want (0 <-> 1)
 *
 * RETURN VALUE
 *		SEGA character representing requested 8x8 character.
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UWORD CompositeTileChar (
			MapType		*map,
			TilesetType	*ts,
			UWORD		 tileID,
			int			 xoff,
			int			 yoff)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CompositeTileChar";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		XTRAPntr	 old;
		RoomType	*room;
		PlotType	*plot;
		TilesetInfo	*tsi;
		int			 rowsize;
		int			 x;
		int			 y;
		UWORD		 newtileID;
		UBYTE		 newtilesetID;
		UBYTE		 newtileflags;
		UWORD		 segachar;

		/*
		** Lookup Composite Tile Room and compute
		** offset to 2x2 tiles that represent the
		** composite tile in question.
		*/
		room = ts->GridedTilesRoom;
		rowsize = room->Width / ts->TilesAcross;
		x  = (tileID - 1) % rowsize;
		y  = (tileID - 1) / rowsize;
		x *= ts->TilesAcross;
		y *= ts->TilesDown;
		x += xoff;
		y += yoff;

		/*
		** Lookup 1 tile from 2x2 tile.
		*/
		old   = LastActivateXTRA ();
		plot  = ActivateXTRA (room->Layers[0]->Plot[y]);
		plot += x;

		newtilesetID = plot->TilesetID;
		newtileID    = plot->TileID;
		newtileflags = plot->PlotFlags;

		ReleaseXTRA(room->Layers[0]->Plot[y]);

		/*
		** Look up SEGA character that represents this tile.
		*/
		if (!newtilesetID) {
			segachar = 0;
		} else {
			ts  = map->Tilesets[newtilesetID];
			tsi = ts->UserData;
			segachar = tsi->ImageChars[newtileID];

			/*
			** Handle Flip Bits.
			*/
			if ((newtileflags & TUME_XFLIP) && (newtileflags & TUME_YFLIP)) {
				segachar ^= (HDWR_XFLIP | HDWR_YFLIP);
			} else if (newtileflags & TUME_XFLIP) {
				segachar ^= HDWR_XFLIP;
			} else if (newtileflags & TUME_YFLIP) {
				segachar ^= HDWR_YFLIP;
			}

			/*
			** Handle Priority.
			*/
			segachar |= (UWORD) ((newtileflags & TUME_PRIORITY) ? HDWR_PRIORITY : 0);

			/*
			** Handle Colorsets.
			*/
			if (newtileflags & TUME_COLORSET_FLAG) {
				UWORD	colorset;

				colorset = (UWORD) ((newtileflags & TUME_COLORSET_MASK) << HDWR_COLORSHIFT);
				segachar = (UWORD) ((segachar & (~HDWR_COLORSET_MASK)) | colorset);
			}
		}

		ActivateXTRA (old);

		return segachar;
	}
} /* CompositeTileChar */

/************************************  ************************************/
/************************************  ************************************/
/************************************  ************************************/
/************************************  ************************************/
/************************************  ************************************/

/*********************************************************************
 *
 * AddObject
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *2.055	06/08/94 (GAT) - created.
 *
 * SEE ALSO
 *
*/
void AddObject (ListType *objlist, int x, int y, UWORD type, int sort, int *objectCount)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "AddObject";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		MapObjType	*mot;
		MapObjType	*tom;

		mot = CreateNode (sizeof (MapObjType), NULL);
		if (!mot) {
			OutOfMemErr ("AddObject: MapObjType\n");
		}

		mot->X         = (UWORD) x;
		mot->Y         = (UWORD) y;
		mot->Object    = type;
		(*objectCount)++;

		if (DebugSwitch)
		{
			printf ("Found Obj %d at (%d, %d)\n", mot->Object, mot->X, mot->Y);
		}

		if (!sort)
		{
			tom = Head (objlist);
			for (;;) {
				if (EndOfList (tom)) {
					InsertBefore (tom, mot);
					break;
				}
				if (tom->X == mot->X) {
					if (mot->Y < tom->Y) {
						InsertBefore (tom, mot);
						break;
					}
				}
				if (mot->X < tom->X) {
					InsertBefore (tom, mot);
					break;
				}
				tom = Next (tom);
			}
		}
		else
		{
			tom = Head (objlist);
			for (;;) {
				if (EndOfList (tom)) {
					InsertBefore (tom, mot);
					break;
				}
				if (tom->Y == mot->Y) {
					if (mot->X < tom->X) {
						InsertBefore (tom, mot);
						break;
					}
				}
				if (mot->Y < tom->Y) {
					InsertBefore (tom, mot);
					break;
				}
				tom = Next (tom);
			}
		}
	}
} /* AddObject */


/*********************************************************************
 *
 * GetRoomTileType
 *
 * PURPOSE
 *		Find first tile in room layer and return it's UserType
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int GetRoomTileType (MapType *map, RoomType *room, int layer)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "GetRoomTileType";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		short	y;

		for (y = 0; y < room->Height; y++)
		{
			PlotType	*plot;
			short		 x;

			plot = ((PlotType *)ActivateXTRA (room->Layers[layer]->Plot[y]));
			for (x = 0; x < room->Width; x++)
			{
				UBYTE	tsetID;

				tsetID = plot->TilesetID;
				if (tsetID)
				{
					TilesetType	*ts;

					ts = map->Tilesets[tsetID];
					ReleaseXTRA(room->Layers[layer]->Plot[y]);
					return (ts->UserType);
				}
				plot++;
			}
			ReleaseXTRA(room->Layers[layer]->Plot[y]);
		}
		return TILESET_IMAGE;
	}

} /* GetRoomTileType */


/*********************************************************************
 *
 * WriteMode7Room
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int WriteMode7Room (MapType *map, RoomType *room)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteMode7Room";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		PlotType		*floorplot;
		FILE			*fp;
		UBYTE			*CharRow[2];
		short			 x;
		short			 y;
		short			 l;
		BOOL			 smalltiles;
		char			 rmfile[EIO_MAXPATH];
		char			 rmname[9];
		unsigned		 widthrmchars;
		UBYTE			*fontdata;
		int				 fontcount = 0;
		short			 fontchardata = 0;
		short			 fontndx  = 0;
		short			 rowcount = 128;
		Mode7Char		 mode7char;

		fontcount = NumChars[TILESET_MODE7];

		/*
		** This is a MODE7 room
		*/

		strncpy (rmname, room->Name, 8);
		rmname[8] = '\0';

		if (WriteRooms)
		{

		BinaryOut = BinFileTable[FILE_RM_MODE7];

		EIO_fnmerge (rmfile, OutPath, rmname, ".m7r");
		if (WriteFiles) {
			printf ("Writing MODE7 Room '%s'\n", rmfile);
		}

		if (room->TileWidth == 8) {
			smalltiles    = TRUE;
			widthrmchars  = (room->Width);
		} else {
			smalltiles    = FALSE;
			widthrmchars  = (room->Width * 2);
		}
	
		CharRow[0] = malloc (widthrmchars * sizeof (UBYTE));
		CharRow[1] = malloc (widthrmchars * sizeof (UBYTE));
		if (!CharRow[0] || !CharRow[1]) {
			OutOfMemErr ("WriteAllRooms:CharRow\n");
		}

		for (l = 0; l < room->LayerCount; l++) {

			if (room->LayerCount > 1) {
				rmfile[strlen(rmfile) - 1] = (char) (l + '0');
			}

			if (WriteFiles) {
				fp = fopen (rmfile, "wb");
				if (!fp) {
					FailMess ("Couldn't open file '%s'\n", rmfile);
				}
				if (FileSizes && !DumpMode7)
				{
					UWORD	w = (UWORD) widthrmchars;
					UWORD	h = (UWORD) (smalltiles ? room->Height : (room->Height * 2));

					if (!LittleEndian)
					{
						WordSex (w);
						WordSex (h);
					}

					fwrite (&w, sizeof (w), 1, fp);
					fwrite (&h, sizeof (h), 1, fp);
				}
			}

			for (y = 0; y < room->Height; y++) {
				UBYTE	*mode7char0;
				UBYTE	*mode7char1;

				mode7char0 = CharRow[0];
				mode7char1 = CharRow[1];
				floorplot = ((PlotType *)ActivateXTRA (room->Layers[l]->Plot[y]));
				for (x = 0; x < room->Width; x++) {
					UBYTE		 tileflags;
					UBYTE		 tilesetID;
					UWORD		 tileID;
					TilesetType	*ts;
					TilesetInfo	*tsi;

					if (smalltiles) {
						UWORD	schar;

						tileflags  = floorplot->PlotFlags;
						tilesetID  = floorplot->TilesetID;
						tileID     = floorplot->TileID;
						schar      = BlankChar[TILESET_MODE7];

						if (tilesetID) {
							ts  = map->Tilesets[tilesetID];
							tsi = ts->UserData;
							if (ts->UserType != TILESET_MODE7) {
								WarnMess ("Non MODE 7 tile at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
							} else {
								schar = tsi->ImageChars[tileID];

								if (tileflags) {
									WarnMess ("(%d,%d) No flip or colorsets or priorities allowed with MODE7 Tiles\n", x, y);
								}
							}
						} else {
							schar = 0;
						}
						*mode7char0++ = (UBYTE) (schar + FirstLetter);
					} else {
						ImageTile	im;

						im.TL = BlankChar[TILESET_MODE7];
						im.TR = BlankChar[TILESET_MODE7];
						im.BL = BlankChar[TILESET_MODE7];
						im.BR = BlankChar[TILESET_MODE7];

						tileflags = floorplot->PlotFlags;
						tilesetID = floorplot->TilesetID;
						tileID    = floorplot->TileID;

						if (tilesetID) {
							ts  = map->Tilesets[tilesetID];
							tsi = ts->UserData; 
							if (ts->UserType != TILESET_MODE7) {
								WarnMess ("Non Mode7 tile at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
							} else {
								ImageTile	 origtile;

								if (ts->GridedTilesRoom) {
									/*
									** I'm assuming the grided tiles
									** are 2x2 8x8 tiles.
									*/
									origtile.TL = CompositeTileChar (map, ts, tileID, 0, 0);
									origtile.TR = CompositeTileChar (map, ts, tileID, 1, 0);
									origtile.BL = CompositeTileChar (map, ts, tileID, 0, 1);
									origtile.BR = CompositeTileChar (map, ts, tileID, 1, 1);
								} else {
									origtile = tsi->ImageTiles[tileID];
								}

								if (tileflags) {
									WarnMess ("(%d,%d) No flip or colorsets or priorities allowed with MODE7 Tiles\n", x, y);
								}
								im = origtile;
							}
						}

						*mode7char0 = (UBYTE) (im.TL + FirstLetter);
						mode7char0++;
						*mode7char0 = (UBYTE) (im.TR + FirstLetter);
						mode7char0++;
						*mode7char1 = (UBYTE) (im.BL + FirstLetter);
						mode7char1++;
						*mode7char1 = (UBYTE) (im.BR + FirstLetter);
						mode7char1++;

					}

					floorplot++;
				}
				ReleaseXTRA(room->Layers[l]->Plot[y]);

				if (WriteFiles) {
					if (!DumpMode7) {
							fwrite (CharRow[0], widthrmchars, sizeof (UBYTE), fp);
							if (!smalltiles) {
								fwrite (CharRow[1], widthrmchars, sizeof (UBYTE), fp);
							}
					} else {
						UBYTE	*data;
						short	 byte;
						unsigned cacross;
				
						data    = CharRow[0];
						cacross = widthrmchars;
						if (rowcount) {
							for (byte = 0; byte < 128; byte++) {
								if (cacross) {
									putc (*data, fp);
									data++;
									cacross--;
								} else {
									putc (0, fp);
								}
								if (fontcount >= 0) {
									if (!fontchardata)
									{
										fontcount--;
										fontchardata = sizeof (Mode7Char);
										fontdata     = (UBYTE *)&mode7char;
										CopyChar (fontndx, fontdata, TILESET_MODE7);
										fontndx++;
									}
									putc (*fontdata, fp);
									fontdata++;
									fontchardata--;
								} else {
									putc (0, fp);
								}
							}
							rowcount--;
						}

						if (!smalltiles)
						{
							data    = CharRow[1];
							cacross = widthrmchars;
							if (rowcount) {
								for (byte = 0; byte < 128; byte++) {
									if (cacross) {
										putc (*data, fp);
										data++;
										cacross--;
									} else {
										putc (0, fp);
									}
									if (fontcount >= 0) {
										if (!fontchardata)
										{
											fontcount--;
											fontchardata = sizeof (Mode7Char);
											fontdata     = (UBYTE *)&mode7char;
											CopyChar (fontndx, fontdata, TILESET_MODE7);
											fontndx++;
										}
										putc (*fontdata, fp);
										fontdata++;
										fontchardata--;
									} else {
										putc (0, fp);
									}
								}
								rowcount--;
							}
						}
					}
				}
			}
		}
		free (CharRow[1]);
		free (CharRow[0]);

		if (WriteFiles) {
			if (DumpMode7) {
				while (rowcount) {
					short	byte;

					for (byte = 0; byte < 128; byte++) {
						putc (0, fp);
						if (fontcount >= 0) {
							if (!fontchardata)
							{
								fontcount--;
								fontchardata = sizeof (Mode7Char);
								fontdata     = (UBYTE *)&mode7char;
								CopyChar (fontndx, fontdata, TILESET_MODE7);
								fontndx++;
							}
							putc (*fontdata, fp);
							fontdata++;
							fontchardata--;
						} else {
							putc (0, fp);
						}
					}
					rowcount--;
				}
			}
			fclose (fp);
		}
		}

		WritePalette (rmname, room->ColorInfo.Colors, TILESET_MODE7);
		WriteColorCycle(rmname, room->ColorInfo.Cycles, room->ColorInfo.NumCycles);
	}
	return (!ErrorCount);
} /* WriteMode7Room */


/*********************************************************************
 *
 * WritePicOrParaRoom
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		02/21/94 Monday (GAT) - fix only last layer was closed in picture room
 *
 * SEE ALSO
 *
*/
int WritePicOrParaRoom (MapType *map, RoomType *room)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WritePicOrParaRoom";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		PlotType		*floorplot;
		int				 fh;
		UWORD			*CharRow[2];
		int				 x1;
		int				 y1;
		short			 x;
		short			 y;
		int				 l;
		BOOL			 smalltiles;
		char			 rmfile[EIO_MAXPATH];
		char			 rmname[9];
		unsigned		 widthrowchars;
		int				 writewidth;
		int				 writeheight;
		int				 roomtiletype = (-1);

		/*
		** This is a PICTURE or PARALLAX room
		*/

		strncpy (rmname, room->Name, 8);
		rmname[8] = '\0';

		if (WriteRooms)
		{

		writewidth  = room->Width;
		writeheight = room->Height;

		if (room->UserType == ROOM_PICTURE) {
			BinaryOut = BinFileTable[FILE_RM_PICT];
			EIO_fnmerge (rmfile, OutPath, rmname, ".map");
			if (WriteFiles) {
				printf ("Writing High Quality Screen Room '%s'\n", rmfile);
			}
		} else {
			BinaryOut = BinFileTable[FILE_RM_PARA];
			EIO_fnmerge (rmfile, OutPath, rmname, ".par");
			if (WriteFiles) {
				printf ("Writing Parallax Room '%s'\n", rmfile);
			}
			if (ParallaxWidth) {
				writewidth  = ParallaxWidth;
				writeheight = ParallaxHeight;
			}
		}

		if (room->TileWidth == 8 || BigSNESTiles) {
			smalltiles     = TRUE;
			widthrowchars  = writewidth;
		} else {
			smalltiles     = FALSE;
			if (ParallaxWidth)
			{
				writewidth    /= 2;
				writeheight   /= 2;
			}
			widthrowchars  = writewidth * 2;
		}
		
		CharRow[0] = malloc (widthrowchars * sizeof (UWORD));
		CharRow[1] = malloc (widthrowchars * sizeof (UWORD));
		if (!CharRow[0] || !CharRow[1]) {
			OutOfMemErr ("WriteAllRooms:CharRow\n");
		}

		for (l = 0; l < room->LayerCount; l++) {

			if (room->LayerCount > 1) {
				rmfile[strlen(rmfile) - 1] = (char) (l + '0');
			}

			if (WriteFiles) {
				fh = EIO_WriteOpen (rmfile);
				if (fh == (-1)) {
					FailMess ("Couldn't open file '%s'\n", rmfile);
				}
				if (FileSizes)
				{
					UWORD	w;
					UWORD	h;

					if (room->UserType == ROOM_PICTURE)
					{
						w = (UWORD) (room->Width  * ((smalltiles) ? 1 : 2));
						h = (UWORD) (room->Height * ((smalltiles) ? 1 : 2));

						if (!LittleEndian)
						{
							WordSex (w);
							WordSex (h);
						}
						EIO_Write (fh, &w, sizeof (w));
						EIO_Write (fh, &h, sizeof (h));
					}
					else
					{
						w = (UWORD) ((room->Width  + room->Width  - 1) / writewidth);
						h = (UWORD) ((room->Height + room->Height - 1) / writeheight);

						if (!LittleEndian)
						{
							WordSex (w);
							WordSex (h);
						}
						EIO_Write (fh, &w, sizeof (w));
						EIO_Write (fh, &h, sizeof (h));

						w = (UWORD) (writewidth  * ((smalltiles) ? 1 : 2));
						h = (UWORD) (writeheight * ((smalltiles) ? 1 : 2));

						if (!LittleEndian)
						{
							WordSex (w);
							WordSex (h);
						}
						EIO_Write (fh, &w, sizeof (w));
						EIO_Write (fh, &h, sizeof (h));
					}
				}
			}

			roomtiletype = GetRoomTileType (map, room, l);

			for (y1 = 0; y1 < room->Height; y1 += writeheight) {
				for (x1 = 0; x1 < room->Width; x1 += writewidth) {

					for (y = 0; y < writeheight; y++) {
						UWORD	*segachar0;
						UWORD	*segachar1;

						segachar0 = CharRow[0];
						segachar1 = CharRow[1];
						if (y1 + y < room->Height) {
							floorplot = ((PlotType *)ActivateXTRA (room->Layers[l]->Plot[y1 + y])) + x1;
						}
						for (x = 0; x < writewidth; x++) {
							UBYTE		 tileflags;
							UBYTE		 tilesetID;
							UWORD		 tileID;
							TilesetType	*ts;
							TilesetInfo	*tsi;

							if (smalltiles) {
								UWORD	schar;

								schar = BlankChar[roomtiletype];

								if (y1 + y < room->Height && x1 + x < room->Width) {
									tileflags  = floorplot->PlotFlags;
									tilesetID  = floorplot->TilesetID;
									tileID     = floorplot->TileID;
									if (tilesetID) {
										ts  = map->Tilesets[tilesetID];
										tsi = ts->UserData;
										if (!ImageTypes[ts->UserType]) {
											WarnMess ("Non Image tile at (%d,%d) in image layer in room '%s'\n", x + x1, y + y1, room->Name);
										} else {

											if (ts->UserType != roomtiletype) {
												WarnMess ("Mixed tiles of different types at (%d,%d) in layer %d in room '%s'\n", x + x1, y + y1, l, room->Name);
											}

											if (BigSNESTiles) {
												UWORD	work;

												schar  = tsi->ImageTiles[tileID].TL;
												work   = schar & HDWR_CHAR_MASK;

												work  /= (UWORD) 4;
												work   = (UWORD)
														 (
														  ((work & 0x0007U) * 2) +
														  ((work & 0xFFF8U) * 4)
														 );

												schar  = (UWORD) ((schar & ~(HDWR_CHAR_MASK)) | work);
											} else {
												schar = tsi->ImageChars[tileID];
											}

											if ((tileflags & TUME_XFLIP) && (tileflags & TUME_YFLIP)) {
												schar ^= (HDWR_XFLIP | HDWR_YFLIP);
											} else if (tileflags & TUME_XFLIP) {
												schar ^= HDWR_XFLIP;
											} else if (tileflags & TUME_YFLIP) {
												schar ^= HDWR_YFLIP;
											}

											if (tileflags & TUME_PRIORITY) {
												schar |= HDWR_PRIORITY;
											}

											if (tileflags & TUME_COLORSET_FLAG) {
												UWORD	colorset;

												colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
												schar    = (UWORD) ((schar & (~HDWR_COLORSET_MASK)) | colorset);
											}
										}
									}
								}

								#if __iAPx86__

								*segachar0 = (UWORD) (schar + FirstLetter);
								if (!LittleEndian) {
									WordSex(*segachar0);
								}
								segachar0++;
								#else
								#error Need Code
								#endif

							} else {  // large tiles
								ImageTile	im;

								im.TL = BlankChar[roomtiletype];
								im.TR = BlankChar[roomtiletype];
								im.BL = BlankChar[roomtiletype];
								im.BR = BlankChar[roomtiletype];

								if (y1 + y < room->Height && x1 + x < room->Width) {
									tileflags = floorplot->PlotFlags;
									tilesetID = floorplot->TilesetID;
									tileID    = floorplot->TileID;

									if (tilesetID) {
										ts  = map->Tilesets[tilesetID];
										tsi = ts->UserData; 
										if (!ImageTypes[ts->UserType]) {
											WarnMess ("Non Image tile at (%d,%d) in image layer in room '%s'\n", x + x1, y + y1, room->Name);
										} else {
											ImageTile	 origtile;

											if (ts->UserType != roomtiletype) {
												WarnMess ("Mixed tiles of different types at (%d,%d) in image layer in room '%s'\n", x + x1, y + y1, room->Name);
											}

											if (ts->GridedTilesRoom) {
												/*
												** I'm assuming the grided tiles
												** are 2x2 8x8 tiles.
												*/
												origtile.TL = CompositeTileChar (map, ts, tileID, 0, 0);
												origtile.TR = CompositeTileChar (map, ts, tileID, 1, 0);
												origtile.BL = CompositeTileChar (map, ts, tileID, 0, 1);
												origtile.BR = CompositeTileChar (map, ts, tileID, 1, 1);
											} else {
												origtile = tsi->ImageTiles[tileID];
											}
											if ((tileflags & TUME_XFLIP) && (tileflags & TUME_YFLIP)) {
												im.TL = origtile.BR ^ (HDWR_XFLIP | HDWR_YFLIP);
												im.TR = origtile.BL ^ (HDWR_XFLIP | HDWR_YFLIP);
												im.BL = origtile.TR ^ (HDWR_XFLIP | HDWR_YFLIP);
												im.BR = origtile.TL ^ (HDWR_XFLIP | HDWR_YFLIP);
											} else if (tileflags & TUME_XFLIP) {
												im.TL = origtile.TR ^ HDWR_XFLIP;
												im.TR = origtile.TL ^ HDWR_XFLIP;
												im.BL = origtile.BR ^ HDWR_XFLIP;
												im.BR = origtile.BL ^ HDWR_XFLIP;
											} else if (tileflags & TUME_YFLIP) {
												im.TL = origtile.BL ^ HDWR_YFLIP;
												im.TR = origtile.BR ^ HDWR_YFLIP;
												im.BL = origtile.TL ^ HDWR_YFLIP;
												im.BR = origtile.TR ^ HDWR_YFLIP;
											} else {
												im = origtile;
											}

											if (tileflags & TUME_PRIORITY) {
												im.TL |= HDWR_PRIORITY;
												im.TR |= HDWR_PRIORITY;
												im.BL |= HDWR_PRIORITY;
												im.BR |= HDWR_PRIORITY;
											}

											if (tileflags & TUME_COLORSET_FLAG) {
												UWORD	colorset;

												colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
												im.TL = (UWORD) ((im.TL & (~HDWR_COLORSET_MASK)) | colorset);
												im.TR = (UWORD) ((im.TR & (~HDWR_COLORSET_MASK)) | colorset);
												im.BL = (UWORD) ((im.BL & (~HDWR_COLORSET_MASK)) | colorset);
												im.BR = (UWORD) ((im.BR & (~HDWR_COLORSET_MASK)) | colorset);
											}
										}
									}
								}

								#if __iAPx86__

								*segachar0 = (UWORD) (im.TL + FirstLetter);
								if (!LittleEndian) {
									WordSex(*segachar0);
								}
								segachar0++;
								*segachar0 = (UWORD) (im.TR + FirstLetter);
								if (!LittleEndian) {
									WordSex(*segachar0);
								}
								segachar0++;
								*segachar1 = (UWORD) (im.BL + FirstLetter);
								if (!LittleEndian) {
									WordSex(*segachar1);
								}
								segachar1++;
								*segachar1 = (UWORD) (im.BR + FirstLetter);
								if (!LittleEndian) {
									WordSex(*segachar1);
								}
								segachar1++;

								#else
								#error Need Code
								#endif

							}

							floorplot++;
						}
						if (y1 + y < room->Height) {
							ReleaseXTRA(room->Layers[l]->Plot[y1 + y]);
						}

						if (WriteFiles) {
							EIO_Write (fh, CharRow[0], widthrowchars * sizeof (UWORD));
							if (!smalltiles) {
								EIO_Write (fh, CharRow[1], widthrowchars * sizeof (UWORD));
							}
						}
					}
				}
			}
			if (WriteFiles) {
				EIO_Close (fh);
			}
		}
		free (CharRow[1]);
		free (CharRow[0]);
		}

		WritePalette (rmname, room->ColorInfo.Colors, roomtiletype);
		WriteColorCycle(rmname, room->ColorInfo.Cycles, room->ColorInfo.NumCycles);
	}
	return (!ErrorCount);
} /* WritePicOrParaRoom */


/*********************************************************************
 *
 * WriteTableRoom
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *2.053	08/03/94 Wednesday (dcc) - add support for AddFlipToTBL
 *
 *
 * SEE ALSO
 *
*/
int WriteTableRoom (MapType *map, RoomType *room)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteTableRoom";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		PlotType		*floorplot;
		FILE			*fp;
		int				 x;
		int				 y;
		char			 rmfile[EIO_MAXPATH];
		char			 rmname[9];
		UBYTE			 tileflags;
		UBYTE			 tilesetID;
		UWORD			 tileID;
		TilesetType		*ts;
		TilesetInfo		*tsi;
		short			 count = 0;

		strupr (room->Name);
		strncpy (rmname, room->Name, 8);
		rmname[8] = '\0';

		if (WriteRooms)
		{
		EIO_fnmerge (rmfile, OutPath, rmname, ".tbl");

		if (WriteFiles) {
			printf ("Writing Table Room '%s'\n", rmfile);
		}
		BinaryOut = BinFileTable[FILE_TABLE];

		if (WriteFiles) {
			fp = fopen (rmfile, "w");
			if (!fp) {
				FailMess ("Couldn't open table file '%s'\n", rmfile);
			}

			fprintf (fp,
				";*\n"
				";* %s\n"
				";*\n"
				";* Tagged Char Table\n"
				";*\n"
				,
				EIO_Filename (rmfile)
				);
		}

		for (y = 0; y < room->Height; y++) {
			floorplot = ActivateXTRA (room->Layers[0]->Plot[y]);
			for (x = 0; x < room->Width; x++) {
				tileflags = floorplot->PlotFlags;
				tilesetID = floorplot->TilesetID;
				tileID    = floorplot->TileID;
				floorplot++;
				if (tilesetID) {
					ts  = map->Tilesets[tilesetID];
					tsi = ts->UserData;
					if (!ImageTypes[ts->UserType]) {
						if (ts->UserType != TILESET_CONTOUR) {
							WarnMess ("Non Image tile in table room '%s' at (%d,%d)\n", room->Name, x, y);
						}
					} else {
						if (room->TileWidth == 16) {
							ImageTile	 origtile;

							if (ts->GridedTilesRoom) {
								/*
								** I'm assuming the gridded tiles
								** are 2x2 8x8 tiles.
								*/

								int T, B, L, R;
								if (tileflags & TUME_XFLIP)
								{
									if (tileflags & TUME_YFLIP)
									{
										T = 1;	B = 0;	L = 1;	R = 0;
									}
									else
									{
										T = 0;	B = 1;	L = 1;	R = 0;
									}
								}
								else
								{
									if (tileflags & TUME_YFLIP)
									{
										T = 1;	B = 0;	L = 0;	R = 1;
									}
									else
									{
										T = 0;	B = 1;	L = 0;	R = 1;
									}
								}
								origtile.TL = CompositeTileChar (map, ts, tileID, L, T);
								origtile.TR = CompositeTileChar (map, ts, tileID, R, T);
								origtile.BL = CompositeTileChar (map, ts, tileID, L, B);
								origtile.BR = CompositeTileChar (map, ts, tileID, R, B);
							} else {
								origtile = tsi->ImageTiles[tileID];
							}
							if (AddFlipToTBL)
							{
								origtile.TL &= (HDWR_CHAR_MASK | HDWR_COLORSET_MASK);
								origtile.TR &= (HDWR_CHAR_MASK | HDWR_COLORSET_MASK);
								origtile.BL &= (HDWR_CHAR_MASK | HDWR_COLORSET_MASK);
								origtile.BR &= (HDWR_CHAR_MASK | HDWR_COLORSET_MASK);
							}
							else
							{
								origtile.TL &= HDWR_CHAR_MASK;
								origtile.TR &= HDWR_CHAR_MASK;
								origtile.BL &= HDWR_CHAR_MASK;
								origtile.BR &= HDWR_CHAR_MASK;
							}
							origtile.TL += (UWORD) FirstLetter;
							origtile.TR += (UWORD) FirstLetter;
							origtile.BL += (UWORD) FirstLetter;
							origtile.BR += (UWORD) FirstLetter;
							if (AddFlipToTBL)
							{
								if (tileflags & TUME_XFLIP)
								{
									origtile.TL |= HDWR_XFLIP;
									origtile.TR |= HDWR_XFLIP;
									origtile.BL |= HDWR_XFLIP;
									origtile.BR |= HDWR_XFLIP;
								}
								if (tileflags & TUME_YFLIP)
								{
									origtile.TL |= HDWR_YFLIP;
									origtile.TR |= HDWR_YFLIP;
									origtile.BL |= HDWR_YFLIP;
									origtile.BR |= HDWR_YFLIP;
								}
								if (tileflags & TUME_PRIORITY)
								{
									origtile.TL |= HDWR_PRIORITY;
									origtile.TR |= HDWR_PRIORITY;
									origtile.BL |= HDWR_PRIORITY;
									origtile.BR |= HDWR_PRIORITY;
								}
								if (tileflags & TUME_COLORSET_FLAG)
								{
									UWORD colorset;

									colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
									origtile.TL = (UWORD) ((origtile.TL & (~HDWR_COLORSET_MASK)) | colorset);
									origtile.TR = (UWORD) ((origtile.TR & (~HDWR_COLORSET_MASK)) | colorset);
									origtile.BL = (UWORD) ((origtile.BL & (~HDWR_COLORSET_MASK)) | colorset);
									origtile.BR = (UWORD) ((origtile.BR & (~HDWR_COLORSET_MASK)) | colorset);
								}
							}
							if (WriteFiles) {
								switch (DestHardware)
								{
								case DEST_SNES:
									fprintf (fp, "\tdb\t$%02x,$%02x\t;SNES character number for tag %dtl\n", origtile.TL & 0xFF, origtile.TL >> 8, count);
									fprintf (fp, "\tdb\t$%02x,$%02x\t;SNES character number for tag %dtr\n", origtile.TR & 0xFF, origtile.TR >> 8, count);
									fprintf (fp, "\tdb\t$%02x,$%02x\t;SNES character number for tag %dbl\n", origtile.BL & 0xFF, origtile.BL >> 8, count);
									fprintf (fp, "\tdb\t$%02x,$%02x\t;SNES character number for tag %dbr\n", origtile.BR & 0xFF, origtile.BR >> 8, count);
									break;
								case DEST_GENESIS:
									fprintf (fp, "%sTAGtl%d\tequ\t%d+TOTAL_ASCII\t; TRUE sega character number for tag %dtl\n", room->Name, count, origtile.TL, count);
									fprintf (fp, "%sTAGtr%d\tequ\t%d+TOTAL_ASCII\t; TRUE sega character number for tag %dtr\n", room->Name, count, origtile.TR, count);
									fprintf (fp, "%sTAGbl%d\tequ\t%d+TOTAL_ASCII\t; TRUE sega character number for tag %dbl\n", room->Name, count, origtile.BL, count);
									fprintf (fp, "%sTAGbr%d\tequ\t%d+TOTAL_ASCII\t; TRUE sega character number for tag %dbr\n", room->Name, count, origtile.BR, count);
									break;
								case DEST_GEAR:
									fprintf (fp, "\tdb\t$%02x,$%02x\t;GEAR character number for tag %dtl\n", origtile.TL & 0xFF, origtile.TL >> 8, count);
									fprintf (fp, "\tdb\t$%02x,$%02x\t;GEAR character number for tag %dtr\n", origtile.TR & 0xFF, origtile.TR >> 8, count);
									fprintf (fp, "\tdb\t$%02x,$%02x\t;GEAR character number for tag %dbl\n", origtile.BL & 0xFF, origtile.BL >> 8, count);
									fprintf (fp, "\tdb\t$%02x,$%02x\t;GEAR character number for tag %dbr\n", origtile.BR & 0xFF, origtile.BR >> 8, count);
									break;
								}
							}
							count++;
						} else if (room->TileWidth == 8) {
							UWORD	work;

							work = tsi->ImageChars[tileID];

							if (AddFlipToTBL)
								work &= (HDWR_CHAR_MASK | HDWR_COLORSET_MASK);
							else
								work &= HDWR_CHAR_MASK;

							work += (UWORD) FirstLetter;

							if (AddFlipToTBL)
							{
								if (tileflags & TUME_XFLIP)
									work |= HDWR_XFLIP;
								if (tileflags & TUME_YFLIP)
									work |= HDWR_YFLIP;
								if (tileflags & TUME_PRIORITY)
									work |= HDWR_PRIORITY;
								if (tileflags & TUME_COLORSET_FLAG)
								{
									UWORD colorset;

									colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
									work = (UWORD) ((work & (~HDWR_COLORSET_MASK)) | colorset);
								}
							}
							if (WriteFiles) {
								switch (DestHardware)
								{
								case DEST_SNES:
									fprintf (fp, "\tdb\t$%02x,$%02x\t;SNES character number for tag %d\n", work & 0xFF, work >> 8, count);
									break;
								case DEST_GENESIS:
									fprintf (fp, "%sTAG%d\tequ\t%d+TOTAL_ASCII\t; TRUE sega character number for tag %d\n", room->Name, count, work, count);
									break;
								case DEST_GEAR:
									fprintf (fp, "\tdb\t$%02x,$%02x\t;GEAR character number for tag %d\n", work & 0xFF, work >> 8, count);
									break;
								}
							}
							count++;
						}
					}
				}
			}
			ReleaseXTRA(room->Layers[0]->Plot[y]);
		}

		if (WriteFiles) {
			fclose (fp);
		}

		}
	}
	return (!ErrorCount);
} /* WriteTableRoom */


/*********************************************************************
 *
 * ComputeContour
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
UWORD ComputeContour (
	MapType		*map,
	RoomType	*room,
	int			 x,
	int			 y,
	int			 l,
	PlotType	 con0,
	PlotType	 con1,
	PlotType	 con2,
	PlotType	 con3,
	char		*msg,
	int			 tiletype
)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ComputeContour";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		UBYTE	contsetID;
		UWORD	contileID = 0;

		if (
			(con0.TilesetID && (con0.PlotFlags & (TUME_XFLIP | TUME_YFLIP))) |
			(con1.TilesetID && (con1.PlotFlags & (TUME_XFLIP | TUME_YFLIP))) |
			(con2.TilesetID && (con2.PlotFlags & (TUME_XFLIP | TUME_YFLIP))) |
			(con3.TilesetID && (con3.PlotFlags & (TUME_XFLIP | TUME_YFLIP)))
		   )
		{
			WarnMess ("Flipped %s at %d,%d in room %s layer %d\n", msg, x, y, room->Name, l);
		}

		if (!(con0.TilesetID == con1.TilesetID &&
			  con0.TilesetID == con2.TilesetID &&
			  con0.TilesetID == con3.TilesetID))
		{
			WarnMess ("Mixed tile types at %d,%d in room %s layer %d\n", x, y, room->Name, l);
		}

		contsetID = con0.TilesetID;
		if (contsetID)
		{
			if (!((con0.TileID == con1.TileID - 1) &&
				  (con2.TileID == con3.TileID - 1) &&
				  (con0.TileID == con2.TileID - 32)))
			{
				WarnMess ("Mixed up %s at %d,%d in room %s layer %d\n", msg, x, y, room->Name, l);
			}										

			if (map->Tilesets[contsetID]->UserType != tiletype) {
				WarnMess ("Non %s tile in layer %d at %d,%d in room %s\n", msg, l, x, y, room->Name);
			}
			contileID = (UWORD) (con0.TileID - 1);
			contileID = (UWORD) ((contileID / 64) * 16 + (contileID % 32) / 2);
		}

		return contileID;
	}

} /* ComputeContour */


/*********************************************************************
 *
 * WriteLevel
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
int WriteLevel (MapType *map, RoomType *room, int layer)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteLevel";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (WriteRooms)
	{
		ListType			 objlistX;
		ListType			*objlist = &objlistX;
		STile				 NewImage;
		UWORD				*TileRow;
		PlotType			*floorplot;
		PlotType			*contourplot;
		PlotType			*specialplot;
		PlotType			*objectplot;
		PlotType			*thingplot;
		int					 fh;
		int					 x;
		int					 y;
		int					 tadd;
		BOOL				 smalltiles;
//		int					 imageUserType = 0;
		int					 roomtiletype = (-1);
		char				 rmfile[EIO_MAXPATH];
		char				 rmname[9];
		char				 rmext[5];
		unsigned			 widthrmtiles;
		UWORD				 extrabits;
		int					 objectCount = 0;

		InitList (objlist);

		strncpy (rmname, room->Name, 8);
		rmname[8] = '\0';

		if (!layer)
		{
			strcpy (rmext, ".map");
		}
		else
		{
			sprintf (rmext, ".mp%d", (layer / 4) + 1);
		}
		
		EIO_fnmerge (rmfile, OutPath, rmname, rmext);

		if (WriteFiles) {
			printf ("Writing Room '%s'\n", rmfile);
		}
		BinaryOut = BinFileTable[FILE_RM_LEVEL];

		if (WriteFiles) {
			fh = EIO_WriteOpen (rmfile);
			if (fh == (-1)) {
				FailMess ("Couldn't open file '%s'\n", rmfile);
			}
		}

		if (room->TileWidth == 8) {
			smalltiles    = TRUE;
			tadd          = 2;
			widthrmtiles  = (room->Width / 2);
			if ((room->Height & 0x0001) || (room->Width & 0x0001)) {
				FailMess ("room '%s' has 8x8 tiles and is an odd size\n");
			}
		} else {
			smalltiles    = FALSE;
			tadd          = 1;
			widthrmtiles  = (room->Width);
		}

		if (WriteFiles) {
			if (FileSizes)
			{
				UWORD	w = (UWORD) (room->Width  / tadd);
				UWORD	h = (UWORD) (room->Height / tadd);

				if (!LittleEndian)
				{
					WordSex (w);
					WordSex (h);
				}
				EIO_Write (fh, &w, sizeof (w));
				EIO_Write (fh, &h, sizeof (h));
			}
		}

		
		TileRow = malloc (widthrmtiles * sizeof (UWORD));
		if (!TileRow) {
			OutOfMemErr ("WriteAllRooms:TileRow\n");
		}

		roomtiletype  = GetRoomTileType (map, room, 0);
//		imageUserType = roomtiletype;

		for (y = 0; y < room->Height; y += tadd) {
			UWORD	*segatile;

			segatile = TileRow;
			for (x = 0; x < room->Width; x += tadd) {
				UBYTE		 tileflags;
				UBYTE		 tilesetID;
				UWORD		 tileID;
				UBYTE		 contsetID;
				UWORD		 contileID;
				UWORD		 conbits;
				UBYTE		 specialtsetID;
				UWORD		 specialtileID;
				UBYTE		 objecttsetID;
				UWORD		 objecttileID;
				UBYTE		 thingtsetID;
				UWORD		 thingtileID;
				int			 lyrs = (MakeObjectList) ? 5 : 4;

				TilesetType	*ts;
				TilesetInfo	*tsi;

				conbits       = 0;

				if (smalltiles) {
					int		 x1;
					int		 y1;
					int		 lyr;
					UWORD	*fourchars;
					PlotType plots[5][4][4];

					contsetID     = 0;
					contileID     = 0;
					specialtsetID = 0;
					specialtileID = 0;
					objecttsetID  = 0;
					objecttileID  = 0;
					thingtsetID   = 0;
					thingtileID   = 0;
					fourchars = &NewImage.IM.TL;
					memset (plots, 0, sizeof (plots));

					for (lyr = 0; lyr < lyrs; lyr++)
					{
						for (y1 = 0; y1 < 2; y1++)
						{
							floorplot = ((PlotType *)ActivateXTRA (room->Layers[layer + lyr]->Plot[y + y1])) + x;
							for (x1 = 0; x1 < 2; x1++)
							{
								if (lyr == 0)
								{
									tileflags  = floorplot->PlotFlags;
									tilesetID  = floorplot->TilesetID;
									tileID     = floorplot->TileID;
									*fourchars = BlankChar[roomtiletype];
									if (tilesetID) {
										ts  = map->Tilesets[tilesetID];
										tsi = ts->UserData;
										if (!ImageTypes[ts->UserType]) {
											WarnMess ("Non image tile at (%d,%d) in image layer %d in room '%s'\n", x, y, layer + lyr, room->Name);
										} else {
											UWORD	schar;
											UBYTE	tflags;

											if (ts->UserType != roomtiletype) {
												WarnMess ("Mixed tiles of different types at (%d,%d) in image layer %d in room '%s'\n", x, y, layer + lyr, room->Name);
											}

											schar  = tsi->ImageChars[tileID];
											tflags = tsi->Flags[tileID];
											if (((tflags & CFLG_NOTXSYM) || (tflags & CFLG_NOTYSYM)) &&
												((tileflags & TUME_XFLIP) && (tileflags & TUME_YFLIP))) {
												schar ^= (HDWR_XFLIP | HDWR_YFLIP);
											} else if ((tflags & CFLG_NOTXSYM) && (tileflags & TUME_XFLIP)) {
												schar ^= HDWR_XFLIP;
											} else if ((tflags & CFLG_NOTYSYM) && (tileflags & TUME_YFLIP)) {
												schar ^= HDWR_YFLIP;
											}

											if (tileflags & TUME_PRIORITY) {
												schar |= HDWR_PRIORITY;
											}

											if (tileflags & TUME_COLORSET_FLAG) {
												UWORD	colorset;

												colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
												schar    = (UWORD) ((schar & (~HDWR_COLORSET_MASK)) | colorset);
											}
											*fourchars = schar;
										}
									}
									fourchars++;
								}
								else
								{
									plots[lyr][y1][x1] = *floorplot;
								}
								floorplot++;
							}
							ReleaseXTRA(room->Layers[layer + lyr]->Plot[y + y1]);
						}
					}

					if (WriteContour)
					{
						contileID = ComputeContour (
										map,
										room,
										x,
										y,
										2 + layer,
										plots[1][0][0],
										plots[1][0][1],
										plots[1][1][0],
										plots[1][1][1],
										"contour",
										TILESET_CONTOUR);
					}

					specialtileID = ComputeContour (
									map,
									room,
									x,
									y,
 									3 + layer,
									plots[2][0][0],
									plots[2][0][1],
									plots[2][1][0],
									plots[2][1][1],
									"special",
									TILESET_SPECIAL);

					objecttileID = ComputeContour (
									map,
									room,
									x,
									y,
 									4 + layer,
									plots[3][0][0],
									plots[3][0][1],
									plots[3][1][0],
									plots[3][1][1],
									"object",
									TILESET_OBJECT);

					if (MakeObjectList)
					{
						thingtileID = ComputeContour (
										map,
										room,
										x,
										y,
										5 + layer,
										plots[4][0][0],
										plots[4][0][1],
										plots[4][1][0],
										plots[4][1][1],
										"thing",
										TILESET_THING);
					}

				} else { /* Large Tiles (16x16) */

					floorplot = ((PlotType *)ActivateXTRA (room->Layers[layer]->Plot[y])) + x;
					tileflags = floorplot->PlotFlags;
					tilesetID = floorplot->TilesetID;
					tileID    = floorplot->TileID;
					ReleaseXTRA(room->Layers[layer]->Plot[y]);

					contsetID = 0;
					contileID = 0;
					specialtsetID = 0;
					specialtileID = 0;
					objecttsetID = 0;
					objecttileID = 0;
					thingtsetID = 0;
					thingtileID = 0;

					if (!MergeLayers) {
						if (WriteContour && room->LayerCount > layer + 1) {
							UBYTE contflags;

							contourplot = ((PlotType *)ActivateXTRA (room->Layers[layer + 1]->Plot[y])) + x;
							contsetID   = contourplot->TilesetID;
							contileID   = contourplot->TileID;
							contflags   = contourplot->PlotFlags;

							if (contsetID) {
								if (map->Tilesets[contsetID]->UserType != TILESET_CONTOUR) {
									WarnMess ("Non collision tile at (%d,%d) in contour layer %d in room '%s'\n", x, y, layer + 1, room->Name);
									contsetID = 0;
									contileID = 0;
								} else {
									TilesetType	*cts;
									TilesetInfo	*ctsi;
								
									cts  = map->Tilesets[contsetID];
									ctsi = cts->UserData; 
									contileID = (UWORD) ((contileID - 1) + ctsi->FirstChar);
									if (ContourMode == CONTOUR_GENFLIP)
									{
										contileID = (UWORD) AddContour (contileID, contourplot->PlotFlags);
									}
									else if (ContourMode == CONTOUR_NOFLIP)
									{
										if (contflags & (TUME_XFLIP | TUME_YFLIP))
										{
											WarnMess ("contour tile at (%d,%d,%d) in room '%s' is flipped\n", x, y, layer + 1, room->Name);
										}
									}
									else if (ContourMode == CONTOUR_USEBITS)
									{
										conbits = (UWORD)
												  (
													((contflags & TUME_XFLIP) ? CONTOUR_XFLIPBIT : 0) |
													((contflags & TUME_YFLIP) ? CONTOUR_YFLIPBIT : 0)
												  );
									}
								}
							} else {
								contsetID   = 0;
								contileID   = 0;
							}
							ReleaseXTRA(room->Layers[layer + 1]->Plot[y]);
						} else {
							contsetID   = 0;
							contileID   = 0;
						}

						if (room->LayerCount > layer + 2) {
							specialplot   = ((PlotType *)ActivateXTRA (room->Layers[layer + 2]->Plot[y])) + x;
							specialtsetID = specialplot->TilesetID;
							specialtileID = specialplot->TileID;
							ReleaseXTRA(room->Layers[layer + 2]->Plot[y]);
							if (specialtsetID) {
								if (map->Tilesets[specialtsetID]->UserType != TILESET_SPECIAL) {
									WarnMess ("Non special tile at (%d,%d) in special layer %d in room '%s'\n", x, y, layer + 2, room->Name);
									specialtsetID = 0;
									specialtileID = 0;
								} else {
									specialtileID = (UWORD) (specialtileID - 1);
								}
							} else {
								specialtsetID   = 0;
								specialtileID   = 0;
							}
						} else {
							specialtsetID   = 0;
							specialtileID   = 0;
						}

						if (room->LayerCount > layer + 3) {
							objectplot   = ((PlotType *)ActivateXTRA (room->Layers[layer + 3]->Plot[y])) + x;
							objecttsetID = objectplot->TilesetID;
							objecttileID = objectplot->TileID;
							ReleaseXTRA(room->Layers[layer + 3]->Plot[y]);
							if (objecttsetID) {
								if (map->Tilesets[objecttsetID]->UserType != TILESET_OBJECT) {
									WarnMess ("Non object tile at (%d,%d) in object layer %d in room '%s'\n", x, y, layer + 3, room->Name);
									objecttsetID = 0;
									objecttileID = 0;
								} else {
									objecttileID = (UWORD) (objecttileID - 1);
								}
							} else {
								objecttsetID   = 0;
								objecttileID   = 0;
							}
						} else {
							objecttsetID   = 0;
							objecttileID   = 0;
						}

						if (MakeObjectList && room->LayerCount > layer + 4) {
							thingplot   = ((PlotType *)ActivateXTRA (room->Layers[layer + 4]->Plot[y])) + x;
							thingtsetID = thingplot->TilesetID;
							thingtileID = thingplot->TileID;
							ReleaseXTRA(room->Layers[layer + 4]->Plot[y]);
							if (thingtsetID) {
								if (map->Tilesets[thingtsetID]->UserType != TILESET_THING) {
									WarnMess ("Non thing tile at (%d,%d) in thing layer %d in room '%s'\n", x, y, layer + 4, room->Name);
									thingtsetID = 0;
									thingtileID = 0;
								} else {
									thingtileID = (UWORD) (thingtileID - 1);
								}
							} else {
								thingtsetID   = 0;
								thingtileID   = 0;
							}
						} else {
							thingtsetID   = 0;
							thingtileID   = 0;
						}

					} else { /* MergeLayers */
						UBYTE	worktsetID;
						UWORD	worktileID;
						UBYTE	conttsetIDs[NUM_TILE_TYPES];
						UBYTE	contflags[NUM_TILE_TYPES];
						UWORD	conttileIDs[NUM_TILE_TYPES];
						int		TSetType;
						int		l;

						for (l = 0; l < NUM_TILE_TYPES; l++) {
							conttsetIDs[l] = 0;
							conttileIDs[l] = 0;
						}

						for (l = 1; l < lyrs && (layer + l) < room->LayerCount; l++) {
							contourplot = ((PlotType *)ActivateXTRA (room->Layers[layer + l]->Plot[y])) + x;
							worktsetID  = contourplot->TilesetID;
							worktileID  = contourplot->TileID;
							if (worktsetID) {
								TSetType = map->Tilesets[worktsetID]->UserType;
								if (TSetType == TILESET_IMAGE) {
									WarnMess ("Image tile at (%d,%d) in non image layer %d in room '%s'\n", x, y, layer + l, room->Name);
								} else {
									if (TSetType == TILESET_OBJECT ||
										TSetType == TILESET_CONTOUR ||
										TSetType == TILESET_SPECIAL ||
										TSetType == TILESET_THING
										)
									{
										if (conttsetIDs[TSetType]) {
											WarnMess ("Two tiles of same type on different layers at (%d,%d) in room '%s'\n", x, y, room->Name);
										} else {
											conttsetIDs[TSetType] = worktsetID;
											conttileIDs[TSetType] = (UWORD) (worktileID - 1);
											contflags[TSetType]   = contourplot->PlotFlags;
										}
									}
								}
							}
							ReleaseXTRA(room->Layers[layer + l]->Plot[y]);
						}

						if (WriteContour)
						{
							contsetID     = conttsetIDs[TILESET_CONTOUR];
							contileID     = conttileIDs[TILESET_CONTOUR];
						}
						specialtsetID = conttsetIDs[TILESET_SPECIAL];
						specialtileID = conttileIDs[TILESET_SPECIAL];
						objecttsetID  = conttsetIDs[TILESET_OBJECT];
						objecttileID  = conttileIDs[TILESET_OBJECT];
						thingtsetID  = conttsetIDs[TILESET_THING];
						thingtileID  = conttileIDs[TILESET_THING];

						if (contsetID) {
							TilesetType	*cts;
							TilesetInfo	*ctsi;
						
							cts  = map->Tilesets[contsetID];
							ctsi = cts->UserData; 
							contileID += ctsi->FirstChar;
							if (ContourMode == CONTOUR_GENFLIP)
							{
								contileID = (UWORD) AddContour (contileID, contflags[0]);
							}
							else if (ContourMode == CONTOUR_NOFLIP)
							{
								if (contflags[0] & (TUME_XFLIP | TUME_YFLIP))
								{
									WarnMess ("coutour tile at (%d,%d) in room '%s' is flipped\n", x, y, room->Name);
								}
							}
							else if (ContourMode == CONTOUR_USEBITS)
							{
								conbits = (UWORD)
										  (
											((contflags[0] & TUME_XFLIP) ? CONTOUR_XFLIPBIT : 0) |
											((contflags[0] & TUME_YFLIP) ? CONTOUR_YFLIPBIT : 0)
										  );
							}
						}
					}

					NewImage.IM.TL = BlankChar[roomtiletype];
					NewImage.IM.TR = BlankChar[roomtiletype];
					NewImage.IM.BL = BlankChar[roomtiletype];
					NewImage.IM.BR = BlankChar[roomtiletype];

					extrabits = 0;
					if (!TinyMode) {
						if (tilesetID) {
							ts  = map->Tilesets[tilesetID];
							tsi = ts->UserData; 
							if (!ImageTypes[ts->UserType]) {
								WarnMess ("Non Image tile at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
							} else {
								ImageTile	 origtile;
								UBYTE		 tflags;

								if (ts->UserType != roomtiletype) {
									WarnMess ("Mixed tiles of different types at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
								}

								if (ts->GridedTilesRoom) {
									/*
									** I'm assuming the grided tiles
									** are 2x2 8x8 tiles.
									*/
									origtile.TL = CompositeTileChar (map, ts, tileID, 0, 0);
									origtile.TR = CompositeTileChar (map, ts, tileID, 1, 0);
									origtile.BL = CompositeTileChar (map, ts, tileID, 0, 1);
									origtile.BR = CompositeTileChar (map, ts, tileID, 1, 1);
									tflags   = NULL;
								} else {
									origtile = tsi->ImageTiles[tileID];
									tflags   = tsi->Flags[tileID];
								}

								if (((tflags & (TFLG_XSYM | TFLG_YSYM)) != (TFLG_XSYM | TFLG_YSYM)) &&
									((tileflags & TUME_XFLIP) && (tileflags & TUME_YFLIP))) {
									NewImage.IM.TL = origtile.BR ^ (HDWR_XFLIP | HDWR_YFLIP);
									NewImage.IM.TR = origtile.BL ^ (HDWR_XFLIP | HDWR_YFLIP);
									NewImage.IM.BL = origtile.TR ^ (HDWR_XFLIP | HDWR_YFLIP);
									NewImage.IM.BR = origtile.TL ^ (HDWR_XFLIP | HDWR_YFLIP);
								} else if (!(tflags & TFLG_XSYM) && (tileflags & TUME_XFLIP)) {
									NewImage.IM.TL = origtile.TR ^ HDWR_XFLIP;
									NewImage.IM.TR = origtile.TL ^ HDWR_XFLIP;
									NewImage.IM.BL = origtile.BR ^ HDWR_XFLIP;
									NewImage.IM.BR = origtile.BL ^ HDWR_XFLIP;
								} else if (!(tflags & TFLG_YSYM) && (tileflags & TUME_YFLIP)) {
									NewImage.IM.TL = origtile.BL ^ HDWR_YFLIP;
									NewImage.IM.TR = origtile.BR ^ HDWR_YFLIP;
									NewImage.IM.BL = origtile.TL ^ HDWR_YFLIP;
									NewImage.IM.BR = origtile.TR ^ HDWR_YFLIP;
								} else {
									NewImage.IM = origtile;
								}

								if (tileflags & TUME_PRIORITY) {
									NewImage.IM.TL |= HDWR_PRIORITY;
									NewImage.IM.TR |= HDWR_PRIORITY;
									NewImage.IM.BL |= HDWR_PRIORITY;
									NewImage.IM.BR |= HDWR_PRIORITY;
								}

								if (tileflags & TUME_COLORSET_FLAG) {
									UWORD	colorset;

									colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
									NewImage.IM.TL = (UWORD) ((NewImage.IM.TL & (~HDWR_COLORSET_MASK)) | colorset);
									NewImage.IM.TR = (UWORD) ((NewImage.IM.TR & (~HDWR_COLORSET_MASK)) | colorset);
									NewImage.IM.BL = (UWORD) ((NewImage.IM.BL & (~HDWR_COLORSET_MASK)) | colorset);
									NewImage.IM.BR = (UWORD) ((NewImage.IM.BR & (~HDWR_COLORSET_MASK)) | colorset);
								}
							}
						}
					} else { /* TinyMode */
						if (tilesetID) {
							ts  = map->Tilesets[tilesetID];
							tsi = ts->UserData; 

							if (!ImageTypes[ts->UserType]) {
								WarnMess ("Non Image tile at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
							} else {
								ImageTile	*origtile;
								UWORD		 colorset;

								if (ts->UserType != roomtiletype) {
									WarnMess ("Mixed tiles of different types at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
								}

								origtile = &tsi->ImageTiles[tileID];

								NewImage.IM = *origtile;
							
								if (tileflags & TUME_COLORSET_FLAG) {
									colorset = (UWORD) (((tileflags & TUME_COLORSET_MASK) + FirstColor) << HDWR_COLORSHIFT);
								} else {
									UWORD	c0;
									UWORD	c1;
									UWORD	c2;
									UWORD	c3;

									c0 = (NewImage.IM.TL & HDWR_COLORSET_MASK) ;
									c1 = (NewImage.IM.TR & HDWR_COLORSET_MASK) ;
									c2 = (NewImage.IM.BL & HDWR_COLORSET_MASK) ;
									c3 = (NewImage.IM.BR & HDWR_COLORSET_MASK) ;

									colorset = c0 | c1 | c2 | c3;

									if (!c0) c0 = colorset;
									if (!c1) c1 = colorset;
									if (!c2) c2 = colorset;
									if (!c3) c3 = colorset;

									if ((c0 != c1) ||
										(c0 != c2) ||
										(c0 != c3) ||
										(c1 != c2) ||
										(c1 != c3) ||
										(c2 != c3))
									{
										WarnMess ("Tile uses more than 1 colorset at (%d,%d) in image layer in room '%s'\n", x, y, room->Name);
									}
								
								}

								NewImage.IM.TL &= (UWORD) (~HDWR_COLORSET_MASK);
								NewImage.IM.TR &= (UWORD) (~HDWR_COLORSET_MASK);
								NewImage.IM.BL &= (UWORD) (~HDWR_COLORSET_MASK);
								NewImage.IM.BR &= (UWORD) (~HDWR_COLORSET_MASK);

								extrabits =(UWORD)
										   (
											((tileflags & TUME_XFLIP)    ? HDWR_XFLIP    : 0) |
											((tileflags & TUME_YFLIP)    ? HDWR_YFLIP    : 0) |
											((tileflags & TUME_PRIORITY) ? HDWR_PRIORITY : 0) |
											((tileflags & TUME_PRIORITY) ? HDWR_PRIORITY : 0) |
											colorset
										   );
							}
						}
					}
				}

				/*
				** because of bug in tUME with tileID = 0;
				*/
				if (contileID     == 0xFFFFU) contileID = 0;
				if (specialtileID == 0xFFFFU) specialtileID = 0;
				if (objecttileID  == 0xFFFFU) objecttileID = 0;

				NewImage.IM.TL += (UWORD) FirstLetter;
				NewImage.IM.TR += (UWORD) FirstLetter;
				NewImage.IM.BL += (UWORD) FirstLetter;
				NewImage.IM.BR += (UWORD) FirstLetter;
				NewImage.CT.Contour = (UWORD) ((contileID * ContourSize) | conbits);
				NewImage.CT.Special = specialtileID;
				NewImage.CT.Object  = objecttileID;

				if (MakeObjectList && thingtileID)
				{
					AddObject (objlist, x, y, thingtileID, room->UserNumber, &objectCount);
				}

				{
					STile		*stile;
					STileGroup	*stgrp;
					UWORD		 c;
					int			 t;
					BOOL		 same;

					same  = FALSE;
					c     = 0;
					stgrp = Head (STileList);
					while (!EndOfList(stgrp)) {
						stile = ActivateXTRA (stgrp->STiles);

						if (!objecttileID) {
							for (t = 0; t < stgrp->TileCount; t++) {
								if (!memcmp (stile, &NewImage, sizeof (STile))) {
									same = TRUE;
									ReleaseXTRA(stgrp->STiles);
									break;
								}
								c++;
								stile++;
							}
						} else {
							t = stgrp->TileCount;
						}
						ReleaseXTRA(stgrp->STiles);

						if (same || (t == stgrp->TileCount && stgrp->TileCount != MAX_GROUP_STILES)) {
							break;
						} else {
							stgrp = Next (stgrp);
						}
					}

					if (objecttileID) {
						c    = (UWORD) NumTiles;
						same = FALSE;
					}

					#if __iAPx86__

					if (!TinyMode) {
						c *= (UWORD) 8;
						if (!LittleEndian) {
							WordSex (c);
						}
						*segatile++ = c;
					} else {
						c = c | extrabits;
						if (!LittleEndian) {
							WordSex (c);
						}
						*segatile++ = c;
					}

					#else
					#error Need Code
					#endif

					if (!same) {
						if (EndOfList (stgrp) || t == MAX_GROUP_STILES) {
							stgrp = CreateNode (sizeof (STileGroup), NULL);
							if (!stgrp) {
								OutOfMemErr ("OOM: STileGroup\n");
							}
							stgrp->STiles = AllocXTRA (GROUP_STILE_SIZE);
							if (!stgrp->STiles) {
								OutOfMemErr ("OOM: STileGroup (2)\n");
							}
							AddTail (STileList, stgrp);
						}
						stile = ActivateXTRA (stgrp->STiles);
						stile[stgrp->TileCount] = NewImage;
						UpdateXTRA (stgrp->STiles);
						stgrp->TileCount++;
//						NumBlocks[imageUserType]++;
						NumTiles++;
						if (DebugSwitch) {
							printf ("NumTiles = %d\r", NumTiles);
						}
					}
				}
			}

			if (WriteFiles) {
				EIO_Write (fh, TileRow, widthrmtiles * sizeof (UWORD));
			}
		}
		if (WriteFiles) {
			EIO_Close (fh);
		}

		MinMemFree ();

		free (TileRow);

		if (MakeObjectList && WriteFiles && WriteRooms)
		{
			ListType		 NewListX;
			ListType		*NewList = &NewListX;
			MapObjType		*mot;
			MapObjType		*tom;
			UWORD			*xpos;
			UWORD			*ypos;
			UWORD			*objs;
			UWORD			*yndxs;
			int				 i;
			FILE			*fp;

			printf ("Writing %d Objects\n", objectCount);

			EIO_fnmerge (rmfile, OutPath, rmname, ".otb");

			BinaryOut = BinFileTable[FILE_OBJECTTBL];
			fp = fopen (rmfile, (BinaryOut ? "wb" : "w"));
			if (!fp) {
				FailMess ("Couldn't open file '%s'\n", rmfile);
			}

			if (objectCount) {

				InitList (NewList);

				xpos  = malloc (objectCount * sizeof (UWORD));
				ypos  = malloc (objectCount * sizeof (UWORD));
				objs  = malloc (objectCount * sizeof (UWORD));
				yndxs = malloc (objectCount * sizeof (UWORD));

				if (!xpos | !ypos || !objs || !yndxs) {
					OutOfMemErr ("WriteWorld: Object Tables\n");
				}

				for (i = 0; i < objectCount; i++) {
					mot = RemHead (objlist);
					if (!mot) {
						break;
					}
					mot->ObjectNum = (UWORD) i;
					xpos[i]  = mot->X;
					ypos[i]  = mot->Y;
					objs[i]  = mot->Object;

					tom = Head (NewList);
					if (!room->UserNumber)
					{
						for (;;) {
							if (EndOfList (tom)) {
								InsertBefore (tom, mot);
								break;
							}
							if (tom->Y == mot->Y) {
								if (mot->X < tom->X) {
									InsertBefore (tom, mot);
									break;
								}
							}
							if (mot->Y < tom->Y) {
								InsertBefore (tom, mot);
								break;
							}
							tom = Next (tom);
						}
					}
					else
					{
						for (;;) {
							if (EndOfList (tom)) {
								InsertBefore (tom, mot);
								break;
							}
							if (tom->X == mot->X) {
								if (mot->Y < tom->Y) {
									InsertBefore (tom, mot);
									break;
								}
							}
							if (mot->X < tom->X) {
								InsertBefore (tom, mot);
								break;
							}
							tom = Next (tom);
						}
					}
				}

				mot = Head (NewList);
				for (i = 0; i < objectCount; i++) {
					yndxs[i] = mot->ObjectNum;
					mot = Next (mot);
				}
			}

			if (FileSizes)
			{
				UWORD	count;

				count = (UWORD) objectCount;
				if (!LittleEndian)
				{
					WordSex (count);
				}

				if (BinaryOut)
				{
					fwrite (&count, sizeof (count), 1, fp);
				}
				else
				{
					fprintf (fp,
						";*\n"
						";* Objects for room '%s'\n"
						";*\n"
						"\t%s\t%d\n"
						"\n"
						,
						room->Name,
						"OBJECTCOUNT",
						objectCount
					);
				}
			}

			if (objectCount) {
				for (i = 0; i < objectCount; i++)
				{
					UWORD	data[4];
					int		ndx;

				//	if (room->Width > room->Height)
				//	{
						ndx = i;
				//	}
				//	else
				//	{
				//		ndx = yndxs[i];
				//	}

					data[0] = xpos[ndx];
					data[1] = ypos[ndx];
					data[2] = objs[ndx];
					data[3] = yndxs[ndx];

					if (BinaryOut)
					{
						if (!LittleEndian)
						{
							WordSex (data[0]);
							WordSex (data[1]);
							WordSex (data[2]);
							WordSex (data[3]);
						}

						fwrite (data, sizeof (data), 1, fp);
					}
					else
					{
						fprintf (fp,
							"\t%s\t$%04x,$%04x,$%04x,$%04x\n"
							,
							"OBJECTENTRY",
							data[0],
							data[1],
							data[2],
							data[3]
						);
					}
				}
			}

			if (objectCount) {
				free (yndxs);
				free (objs);
				free (ypos);
				free (xpos);
				EmptyList (NewList);
			}

			fclose (fp);
		}
		EmptyList (objlist);

		strupr (room->Name);
		WritePalette (rmname, room->ColorInfo.Colors, roomtiletype);
		WriteColorCycle(rmname, room->ColorInfo.Cycles, room->ColorInfo.NumCycles);
	}
	return (!ErrorCount);
} /* WriteLevel */


/*********************************************************************
 *
 * WriteLevelRoom
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *2.052	04/10/94 (dcc) - add check for 4 or 8 layers.
 *2.055	09/29/94 (dcc) - add check for 5 layers (if making objects).
 *
 * SEE ALSO
 *
*/
int WriteLevelRoom (MapType *map, RoomType *room)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteLevelRoom";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (WriteRooms)
	{
		if (MakeObjectList)
		{
			if (room->LayerCount != 5)
			{
				fprintf(stdout,
						"Error: level room '%s' must have 5 layers.\n",
						room->Name);
				ErrorCount++;
				return FALSE;
			}
		}
		else
		{
			if (room->LayerCount != 4 && room->LayerCount != 8)
			{
				fprintf(stdout,
						"Error: level room '%s' must have 4 or 8 layers.\n",
						room->Name);
				ErrorCount++;
				return FALSE;
			}
		}
		if (WriteLevel (map, room, 0))
		{
			if (!MakeObjectList && room->LayerCount > 4)
			{
				WriteLevel (map, room, 4);
			}
		}
	}
	return (!ErrorCount);
} /* WriteLevelRoom */


/*********************************************************************
 *
 * WriteAllRooms
 *
 * PURPOSE
 *		Write all Level Rooms
 *		Write all Parallex/Picture Rooms
 *		Write all Table Rooms
 *
 *		While doing this, create BLOCK/FLOOR table for all Level Rooms
 *		in map.
 *
 * INPUT
 *		map		: pointer to a MapType.
 *
 * RETURN VALUE
 *		FALSE = failure.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
BOOL WriteAllRooms (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "WriteAllRooms";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		RoomType	**roomptr;
		RoomType	*room;

		roomptr = map->Rooms + 1;
		while (*roomptr) {
			room = *roomptr;

			/*
			** Skip Composite Tileset Rooms
			*/
			if (room->CompositeTSet) {
			} else {

				if (!room->Name || !strlen(room->Name)) {
					FailMess ("Room size (%d, %d, %d) has no name!\n", room->Width, room->Height, room->LayerCount);
				}

				switch (room->UserType)
				{
				case ROOM_MODE7:
					WriteMode7Room (map, room);
					break;
				case ROOM_PICTURE:
				case ROOM_PARALLAX:
					WritePicOrParaRoom (map, room);
					break;
				case ROOM_TABLE:
					WriteTableRoom (map, room);
					break;
				case ROOM_LEVEL:
					WriteLevelRoom (map, room);
					break;
				}

			}

			roomptr++;
		}

		/*
		**
		** Write out global tables.
		**
		*/

		MinMemFree ();

		if (WriteRooms && WriteFiles && NumTiles) {
			FILE	*fp;
			char	 rmfile[EIO_MAXPATH];

			EIO_fnmerge (rmfile, OutPath, EIO_Name(MapName), ".blk");

			printf ("Writing Block Table '%s'\n", rmfile);
			BinaryOut = BinFileTable[FILE_BLOCK];

			if (!BinaryOut) {

				fp = fopen (rmfile, "w");
				if (!fp) {
					FailMess ("Couldn't open file '%s'\n", rmfile);
				}

				fprintf (fp,
					";*\n"
					";* %s\n"
					";*\n"
					";* Block Table generated by tP16\n"
					";*\n"
					";* <%3d> : No.Tiles\n"
					";*\n",
					rmfile,
					NumTiles
					);

				if (LittleEndian) {
					#if __iAPx86__
					StartDataStatements ("\tdb\t","$%02x",",", 8, DATAST_NOLINEINFO | DATAST_ASMLINENO);
					#else
					#error Need Code
					#endif
				} else {
					StartWordStatements ("\tdc.w\t","$%04x",",", 4, DATAST_NOLINEINFO | DATAST_ASMLINENO);
				}

				{
					STileGroup	*stgrp;
					STile		*stile;
					int			 t;

					stgrp = Head (STileList);
					while (!EndOfList (stgrp)) {
						stile = ActivateXTRA (stgrp->STiles);

						for (t = 0; t < stgrp->TileCount; t++) {
							if (LittleEndian) {
								#if __iAPx86__
								WriteDataStatements (fp, (UBYTE *)&stile->IM, 8);
								#else
								#error Need Code
								#endif
							} else {
								WriteWordStatements (fp, (UWORD *)&stile->IM, 4);
							}
							stile++;
						}
						ReleaseXTRA(stgrp->STiles);
						stgrp = Next (stgrp);
					}
				}

				if (LittleEndian) {
					#if __iAPx86__
					EndDataStatements (fp);
					#else
					#error Need Code
					#endif
				} else {
					EndWordStatements (fp);
				}

				fclose (fp);
			} else { /* Binary Out */
				fp = fopen (rmfile, "wb");
				if (!fp) {
					FailMess ("Couldn't open file '%s'\n", rmfile);
				}

				{
					STileGroup	*stgrp;
					STile		*stile;
					int			 t;

					stgrp = Head (STileList);
					while (!EndOfList (stgrp)) {
						stile = ActivateXTRA (stgrp->STiles);

						for (t = 0; t < stgrp->TileCount; t++) {
							UBYTE	data[8];
							if (LittleEndian) {
								data[0] = (UBYTE) (stile->IM.TL & 0xFF);
								data[1] = (UBYTE) (stile->IM.TL >> 8);
								data[2] = (UBYTE) (stile->IM.TR & 0xFF);
								data[3] = (UBYTE) (stile->IM.TR >> 8);
								data[4] = (UBYTE) (stile->IM.BL & 0xFF);
								data[5] = (UBYTE) (stile->IM.BL >> 8);
								data[6] = (UBYTE) (stile->IM.BR & 0xFF);
								data[7] = (UBYTE) (stile->IM.BR >> 8);
							} else {
								data[0] = (UBYTE) (stile->IM.TL >> 8);
								data[1] = (UBYTE) (stile->IM.TL & 0xFF);
								data[2] = (UBYTE) (stile->IM.TR >> 8);
								data[3] = (UBYTE) (stile->IM.TR & 0xFF);
								data[4] = (UBYTE) (stile->IM.BL >> 8);
								data[5] = (UBYTE) (stile->IM.BL & 0xFF);
								data[6] = (UBYTE) (stile->IM.BR >> 8);
								data[7] = (UBYTE) (stile->IM.BR & 0xFF);
							}
							fwrite (data, 8, 1, fp);
							stile++;
						}
						ReleaseXTRA(stgrp->STiles);
						stgrp = Next (stgrp);
					}
				}

				fclose (fp);
			}

		}

		printf ("Num 16x16 (2x2-8x8) tiles = %d\n", NumTiles);

		if (WriteRooms && WriteFiles && NumTiles) {
			FILE	*fp;
			char	 rmfile[EIO_MAXPATH];

			EIO_fnmerge (rmfile, OutPath, EIO_Name(MapName), ".flr");

			BinaryOut = BinFileTable[FILE_FLOOR];

			printf ("Writing Floor Table '%s'\n", rmfile);

			if (!BinaryOut) {
				fp = fopen (rmfile, "w");
				if (!fp) {
					FailMess ("Couldn't open file '%s'\n", rmfile);
				}

				fprintf (fp,
					";*\n"
					";* %s\n"
					";*\n"
					";* Floor Table generated by tP16\n"
					";*\n"
					";* <%3d> : No.Tiles\n"
					";*\n",
					rmfile,
					NumTiles
					);

				{
					STileGroup	*stgrp;
					STile		*stile;
 					int			 t;
					int			 count = 0;

					stgrp = Head (STileList);
					while (!EndOfList (stgrp)) {
						stile = ActivateXTRA (stgrp->STiles);

						for (t = 0; t < stgrp->TileCount; t++) {
							if (LittleEndian) {
								if (WriteContour)
								{
									fprintf (fp,"\tdb\t$%02x,$%02x,$%02x,$%02x\t; %5d\n",
										stile->CT.Contour & 0xFF,
										stile->CT.Contour >> 8,
										stile->CT.Special,
										stile->CT.Object,
										count);
								}
								else
								{
									fprintf (fp,"\tdb\t$%02x,$%02x\t; %5d\n",
										stile->CT.Special,
										stile->CT.Object,
										count);
								}
							} else {
								if (WriteContour)
								{
									fprintf (fp,"\tdc.b\t$%02x,$%02x,$%02x,$%02x\t; %5d\n",
										stile->CT.Contour >> 8,
										stile->CT.Contour & 0xFF,
										stile->CT.Special,
										stile->CT.Object,
										count);
								}
								else
								{
									fprintf (fp,"\tdc.b\t$%02x,$%02x\t; %5d\n",
										stile->CT.Special,
										stile->CT.Object,
										count);
								}
							}
							count++;
							stile++;
						}
						ReleaseXTRA(stgrp->STiles);
						stgrp = Next (stgrp);
					}
				}

				fclose (fp);
			} else { /* Binary Out */

				fp = fopen (rmfile, "wb");
				if (!fp) {
					FailMess ("Couldn't open file '%s'\n", rmfile);
				}

				{
					STileGroup	*stgrp;
					STile		*stile;
					int			 t;

					stgrp = Head (STileList);
					while (!EndOfList (stgrp)) {
						stile = ActivateXTRA (stgrp->STiles);

						for (t = 0; t < stgrp->TileCount; t++) {
							UBYTE	data[4];
							if (LittleEndian) {
								data[0] = (UBYTE) (stile->CT.Contour & 0xFF);
								data[1] = (UBYTE) (stile->CT.Contour >> 8);
								data[2] = stile->CT.Special;
								data[3] = stile->CT.Object;
							} else {
								data[0] = (UBYTE) (stile->CT.Contour >> 8);
								data[1] = (UBYTE) (stile->CT.Contour & 0xFF);
								data[2] = stile->CT.Special;
								data[3] = stile->CT.Object;
							}

							if (WriteContour)
							{
								fwrite (data, 4, 1, fp);
							}
							else
							{
								fwrite (&data[2], 2, 1, fp);
							}
							stile++;
						}
						ReleaseXTRA(stgrp->STiles);
						stgrp = Next (stgrp);
					}
				}

				fclose (fp);
			}

			if (ContourMode == CONTOUR_GENFLIP && WriteContour) {
				FILE			*fp;
				char			 tsname[EIO_MAXPATH];
				int				 x;
				int				 y;
				int				 c;
				int				 tset;
				TilesetType		*ts;
				TileDataGroup	*tdg;
				unsigned		 tcount;
				UWORD			 contID;
				UWORD			 conflag;
				UBYTE			*tiledata;
				BOOL			 found;
				unsigned		 tilesize;
				UBYTE			 contour[16];


				/*
				** Open Contour File
				*/
				BinaryOut = BinFileTable[FILE_CONTOUR];
				EIO_fnmerge (tsname, OutPath, EIO_Name(ContourName), ".con");
				printf ("Writing %d Contours to '%s'\n", NumContours, tsname);

				if (BinaryOut) {
					fp = fopen (tsname, "wb");
				} else {
					fp = fopen (tsname, "w");
				}

				if (!fp) {
					FailMess ("Writing Contour Tile: Couldn't open '%s'\n", tsname);
				}

				if (!BinaryOut) {
					switch (DestHardware)
					{
					case DEST_SNES:
						StartDataStatements ("\tHEX\t","%02x","",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
						break;
					case DEST_GENESIS:
						StartDataStatements ("\tdc.b\t","%d",",",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
						break;
					case DEST_GEAR:
						StartDataStatements ("\tdc.b\t","$%02x",",",16,DATAST_NOLINEINFO | DATAST_ASMLINENO);
						break;
					}
				}

				/*
				** For each contour tile found
				*/
				for (c = 0; c < NumContours; c++) {
					contID   = ContourTable[c].ContourID;
					conflag  = (UWORD) (ContourTable[c].Flags & (CONTOUR_XFLIP | CONTOUR_YFLIP));

					/*
					** Find which tileset it's in.
					*/
					tset = 0;
					while (contID > CTilesTable[tset]->TileCount) {
						tset++;
						contID -= CTilesTable[tset]->TileCount;
					}

					ts = CTilesTable[tset];
					if (!ts->TMG.TileCount) {
						FailMess ("No Contour Data save with map!\n");
					}

					if (ts->Width != 16 || ts->Height != 16) {
						FailMess ("Writing Contour Tileset (2): Tileset not 16x16 pixels\n");
					}

					/*
					** Lookup tile's data
					*/
					tilesize = ts->TMG.Width * ts->TMG.Height;
					tcount = contID;
					found  = FALSE;

					tdg = Head (ts->TMG.TileGroupList);
					while (!found) {
						if (tcount < tdg->NumTiles) {
							found     = TRUE;
							tiledata  = ActivateXTRA (tdg->TileData);
							tiledata += tilesize * tcount;
						} else {
							tcount -= tdg->NumTiles;
							tdg = Next (tdg);
						}
					}


					/*
					** Convert tiledata to contour data
					*/
					{
						int		xs,xa,xe;
						int		ys,ya,ye;
						int		ct;
						int		v;

						ct = 0;

						if (!conflag) {
							xs =  0;	xa =  1;	xe = 16;
							ys =  0;	ya =  1;	ye = 16;
						} else if ((conflag & CONTOUR_XFLIP) && (conflag & CONTOUR_YFLIP)) {
							xs = 15;	xa = -1;	xe = -1;
							ys = 15;	ya = -1;	ye = -1;
						} else if (conflag & CONTOUR_XFLIP) {
							xs = 15;	xa = -1;	xe = -1;
							ys =  0;	ya =  1;	ye = 16;
						} else {
							xs =  0;	xa =  1;	xe = 16;
							ys = 15;	ya = -1;	ye = -1;
						}

						for (x = xs; x != xe; x += xa) {
							for (y = ys; y != ye; y += ya) {
								if (tiledata[x + y * ts->TMG.Width]) {
									break;
								}
							}
							if (conflag & CONTOUR_YFLIP) {
								v = 15 - y;
							} else {
								v = y;
							}
							if (v < 16) {
								contour[ct] = (UBYTE) (v + 1);
							} else {
								contour[ct] = 0;
							}
							ct++;
						}
					}
					ReleaseXTRA(tdg->TileData);

					if (BinaryOut) {
						fwrite (contour, 16, 1, fp);
					} else {
						WriteDataStatements (fp, contour, 16);
					}
				}

				if (!BinaryOut) {
					EndDataStatements (fp);
				}
				fclose (fp);
	
			}

		}


		return (!ErrorCount);
	}
} /* WriteAllRooms */


/******************************** TEMPLATE ********************************/

#define ARG_A		(newargs[ 0])
#define ARG_B		(newargs[ 1])
#define ARG_C		(newargs[ 2])
#define ARG_D		(newargs[ 3])
#define ARG_E		(newargs[ 4]) 
#define ARG_F		(newargs[ 5])
#define ARG_G		(newargs[ 6])
#define ARG_H		(newargs[ 7])
#define ARG_J		(newargs[ 8])
#define ARG_K		(newargs[ 9])
#define ARG_L		(newargs[10])
#define ARG_M		(newargs[11])
#define ARG_O		(newargs[12])
#define ARG_P		(newargs[13])
#define ARG_Q		(newargs[14])
#define ARG_R		(newargs[15])
#define ARG_S		(newargs[16])
#define ARG_T		(newargs[17])
#define ARG_t		(newargs[18])
#define	ARG_U		(newargs[19])
#define	ARG_V		(newargs[20])
#define ARG_W		(newargs[21])
#define ARG_X		(newargs[22])
#define ARG_Z		(newargs[23])
#define ARG_2		(newargs[24])
#define ARG_7		(newargs[25])
#define ARG_HELP	(newargs[26])
#define ARG_MAPNAME	(newargs[27])
#define ARG_IGNORE	(newargs[28])

#define PROG	"tP16"

char Usage[] = "Usage: " PROG " {-<switches>} <MAPNAME>\n";

ArgSpec Template[] = {
	{ CHRKEYWORD_ARG,	"a",	"\t-a<firstcolorset> First Colorset to use\n", },
	{ CHRTOGGLE_ARG,	"b",	"\tb - Binary output for all files\n", },
	{ CHRTOGGLE_ARG,	"c",	"\tc + Write/Generate Contour\n", },
	{ CHRTOGGLE_ARG,	"d",	"\td - Debugging ON\n", },
	{ CHRKEYWORD_ARG,	"e",	"\t-e<contourType> Contour Type\n"
								"\t      0 = cannot flip contours (default)\n"
								"\t      1 = generate flipped contours\n"
								"\t      2 = use flip bits in .FLR file\n", },
	{ CHRKEYWORD_ARG,	"f",	"\t-f<firstchar> First Char to use\n"
							    "\t     This is invalid of tiles of multiple types\n"
								"\t     are used as in 4color tiles and 16color tiles\n", },
	{ CHRTOGGLE_ARG,	"g",	"\tg - Write Room Width/Height and table sizes into files\n", },
	{ CHRTOGGLE_ARG,	"h",	"\th - Generate Data for SNES 16x16 tiles (TOTAL KLUDGE)\n", },
	{ CHRKEYWORD_ARG,	"j",	"\t-j<WxH> Write Parallax rooms as WxH 8x8 character screens.\n"
								"\t       (e.g. -j32x32)\n", },
	{ CHRTOGGLE_ARG,	"k",	"\tk - Generate only characters used in rooms\n", },
	{ CHRKEYWORD_ARG,	"l",	"\t-l<anim modes>\n"
								"\t      -TOP = Just Top\n"
								"\t      +MRG = Merge Like Frames\n"
								"\t      +CMP = Compress Animation\n"
								"\t      -ZRO = Zero out anim chars from font\n"
								"\t      -INT = Interleave Frames\n", },
	{ CHRTOGGLE_ARG,	"m",	"\tm + Merge top layers\n", },
	{ CHRKEYWORD_ARG,	"o",	"\t-o<binlist>\n"
								"\t      0 = Palette File\n"
								"\t      1 = Coutour File\n"
								"\t      2 = Font File\n"
								"\t      3 = Mode7 Font File\n"
								"\t      4 = Table Room File\n"
								"\t      5 = Block Table File\n"
								"\t      6 = Mode7 Room File\n"
								"\t      7 = Level Room File\n"
								"\t      8 = Parallax Room File\n"
								"\t      9 = Picture Room File\n"
								"\t     10 = Floor File\n"
								"\t     11 = Animation Macro File\n"
								"\t     12 = Thing Table File\n"
								"\t     (def = -o0,1,2b,3b,4,5,6b,7b,8b,9b,10,11,12)\n", },
	{ CHRTOGGLE_ARG,	"p",	"\tp + Pack Font\n", },
	{ CHRKEYWORD_ARG,	"q",	"\tq<output directory>\n", },
	{ CHRKEYWORD_ARG,	"r",	"\t-r<mem> Memory to skip in Font\n"
							    "\t     Similar to -f except works with multiple tile\n"
								"\t     types.  Rounded up to 16 byte boundry\n", },
	{ CHRKEYWORD_ARG,	"s",	"\t-s<target>\n"
								"\t     SNES = Super Nintendo\n"
								"\t     SEGA = Sega Genesis (default)\n"
								"\t     GEAR = Game Gear\n", },
	{ CHRTOGGLE_ARG,	"T",	"\tT - Add colorset & flip info to .TBL files\n", },
	{ CHRTOGGLE_ARG,	"t",	"\tt - Tiny mode (Pack Color, Flip and Pri Bits)\n", },
	{ CHRTOGGLE_ARG,	"u",	"\tu - Make Object Lists\n", },
	{ CHRKEYWORD_ARG,	"v",	"\t-v<contourStyle>\n"
								"\t     0 = 16 bytes elevation (default)\n"
								"\t     1 = Word Color, 16by2 elevation/size\n", },
	{ CHRTOGGLE_ARG,	"w",	"\tw + Write files\n", },
	{ CHRTOGGLE_ARG,	"x",	"\tx + Write tiles\n", },
	{ CHRTOGGLE_ARG,	"z",	"\tz + Write rooms\n", },
	{ CHRTOGGLE_ARG,	"2",	"\t2 - 2Bit (4Color Tiles) use colors 0-3, 16-19, 32-35... instead\n"
								"\t    of colors (0-3, 4-7, 8-11...)\n", },
	{ CHRTOGGLE_ARG,	"7",	"\t7 - All Mode7 Rooms generate 32K with imbedded font\n", },
	{ CHRSWITCH_ARG,	"?",	"", },
	{ STANDARD_ARG|REQUIRED_ARG|MULTI_ARG, "MAPNAME", "", },
	{ STANDARD_ARG|MULTI_ARG,	 "IGNORE",	"\tIGNORE colors.  Default 'IGNORE 0 1'\n", },
	{ 0, NULL, NULL, },
};

/*********************************************************************
 *
 * main
 *
 * PURPOSE
 *		Parse command line and make call to DoConversion.
 *
*/
int main (
	int		argc,
	char	**argv
)
{
	BOOL	 result   = FALSE;
	char	**newargs;
	MapType	*map = NULL;

	MinMemFree ();
	printf (PROG " Version %s  Copyright (c) Echidna (this work is unpublished)\n", Version);
	newargs = argparse (argc, argv, Template);
	if (!newargs || SWITCH_VALUE(ARG_HELP)) {
		printf ("%s\n", GlobalErrMsg);
		printarghelp (Usage, Template);
		return (EXIT_FAILURE);
	}

	InitList (STileList);

	{
		short	i;

		for (i = 0; i < NUM_TILE_TYPES; i++)
		{
			CharList[i] = &CharListX[i];
			InitList (CharList[i]);
		}
	}

	OutPath = ARG_Q;

	GET_TOGGLE_SWITCH (BinaryOut,      ARG_B);
	GET_TOGGLE_SWITCH (WriteContour,   ARG_C);
	GET_TOGGLE_SWITCH (DebugSwitch,    ARG_D);
	GET_TOGGLE_SWITCH (WriteRooms,     ARG_Z);
	GET_TOGGLE_SWITCH (WriteTiles,     ARG_X);
	GET_TOGGLE_SWITCH (WriteFiles,     ARG_W);
	GET_TOGGLE_SWITCH (PackFont,       ARG_P);
	GET_TOGGLE_SWITCH (AddFlipToTBL,   ARG_T);
	GET_TOGGLE_SWITCH (TinyMode,       ARG_t);
	GET_TOGGLE_SWITCH (MergeLayers,    ARG_M);
	GET_TOGGLE_SWITCH (FileSizes,      ARG_G);
	GET_TOGGLE_SWITCH (Other2BitMode,  ARG_2);
	GET_TOGGLE_SWITCH (DumpMode7,      ARG_7);
	GET_TOGGLE_SWITCH (BigSNESTiles,   ARG_H);
	GET_TOGGLE_SWITCH (OnlyUsedChars,  ARG_K);
	GET_TOGGLE_SWITCH (MakeObjectList, ARG_U);

	if (!WriteTiles) WriteRooms = FALSE;

	if (BigSNESTiles) {
		PackFont = FALSE;
	}

	if (BinaryOut) {
		short	i;

		for (i = 0; i < FILE_TYPES; i++) {
			BinFileTable[i] = TRUE;
		}
	}

	if (ARG_O) {
		char	*s;
		char	 c;
		int		 val;
		BOOL	 bin;

		s   = ARG_O;
		val = 0;
		bin = FALSE;
		do {
			c = *s++;
			if (isdigit (c)) {
				val = val * 10 + (c - '0');
			} else if (toupper (c) == 'B') {
				bin = TRUE;
			} else if (!c || c == ',') {
				if (val < 0 || val >= FILE_TYPES) {
					FailMess ("-o binlist value (%d) out of range\n", val);
				}
				BinFileTable[val] = bin;
				val = 0;
				bin = FALSE;
			}
		} while (c);
	}

	if (ARG_S)
	{
		char	*s		= ARG_S;
		short	 done	= FALSE;
		ULONG	 id		= 0;

#define Arg4ID(a,b,c,d)	((((ULONG)(a))<<24)|(((ULONG)(b))<<16)|(((ULONG)(c))<<8)|((ULONG)(d)))

		while (!done)
		{
			if (!*s)
			{
				if (id)
				{

					switch (id)
					{
					case Arg4ID('S','N','E','S'):	DestHardware  = DEST_SNES;		break;
					case Arg4ID('S','E','G','A'):	DestHardware  = DEST_GENESIS;	break;
					case Arg4ID('G','E','A','R'):	DestHardware  = DEST_GEAR;		break;
//					case Arg4ID('G','B','O','Y'):	DestHardware  = DEST_GBOY;		break;
					default:
						ErrMess ("Bad -S parameter\n");
						break;
					}
					
					id = 0;
				}
				if (!*s)
				{
					done = TRUE;
				}
			}
			if (isalpha (*s))
			{
				id = (id << 8) | toupper (*s);
			}
			s++;
		}
	}

	switch (DestHardware)
	{
	case DEST_SNES:
		HDWR_XFLIP			= SNES_XFLIP;
		HDWR_YFLIP			= SNES_YFLIP;
		HDWR_PRIORITY		= SNES_PRIORITY;
		HDWR_COLORSET_MASK	= SNES_COLORSET_MASK;
		HDWR_CHAR_MASK		= SNES_CHAR_MASK;
		HDWR_MAX_COLORSET	= SNES_MAX_COLORSET;
		HDWR_COLORSHIFT		= SNES_COLORSHIFT;
		HDWR_MAX_COLORSET	= SNES_MAX_COLORSET;
		LittleEndian        = TRUE;
		break;
	case DEST_GENESIS:
		HDWR_XFLIP			= SEGA_XFLIP;
		HDWR_YFLIP			= SEGA_YFLIP;
		HDWR_PRIORITY		= SEGA_PRIORITY;
		HDWR_COLORSET_MASK	= SEGA_COLORSET_MASK;
		HDWR_CHAR_MASK		= SEGA_CHAR_MASK;
		HDWR_MAX_COLORSET	= SEGA_MAX_COLORSET;
		HDWR_COLORSHIFT		= SEGA_COLORSHIFT;
		HDWR_MAX_COLORSET	= SEGA_MAX_COLORSET;
		LittleEndian        = FALSE;
		break;
	case DEST_GEAR:
		HDWR_XFLIP			= GEAR_XFLIP;
		HDWR_YFLIP			= GEAR_YFLIP;
		HDWR_PRIORITY		= GEAR_PRIORITY;
		HDWR_COLORSET_MASK	= GEAR_COLORSET_MASK;
		HDWR_CHAR_MASK		= GEAR_CHAR_MASK;
		HDWR_MAX_COLORSET	= GEAR_MAX_COLORSET;
		HDWR_COLORSHIFT		= GEAR_COLORSHIFT;
		HDWR_MAX_COLORSET	= GEAR_MAX_COLORSET;
		LittleEndian        = TRUE;
		break;
	}

	if (ARG_V) { ContourStyle = eggs_atos (ARG_V); }
	if (ARG_E) { ContourMode = eggs_atos (ARG_E); }
	if (ARG_F) { FirstLetter = eggs_atos (ARG_F); }
	if (ARG_A) { FirstColor  = eggs_atos (ARG_A); }
	if (ARG_R)
	{
		FirstChar[TILESET_2BITS] = (UWORD) (
			(eggs_atos (ARG_R) + sizeof (Color4Char) - 1) / 
			sizeof (Color4Char) );
	}

	switch (ContourStyle)
	{
	case 0:
		break;
	case 1:
		ContourSize = 34;
		break;
	default:
		break;
	}

	if (ARG_L) { ParseAnimArgs (ARG_L); }

	if (ARG_J) {
		char	dump[20];
		sscanf (ARG_J, "%d%[-xX,]%d", &ParallaxWidth, dump, &ParallaxHeight);

		if (ParallaxWidth < 1 || ParallaxHeight < 1) {
			FailMess ("Invalid ParallaxWidth %d or ParallaxHeight %d\n", ParallaxWidth, ParallaxHeight);
		}

		printf ("Using ParallaxWidth/Height %dx%d\n", ParallaxWidth, ParallaxHeight);
	}


	if (ARG_IGNORE) {
		char	**args;

		args = MULTI_ARGLIST(ARG_IGNORE);
		if (*args) {
			IgnoreColors[0] = 0;
			IgnoreColors[1] = 1;
			while (*args) {
				IgnoreColors[eggs_atos(*args)] = 1;
				args++;
			}
		}
	}

	//
	// load all requested maps
	//
	{
		char	**tfiles;

		tfiles           = MULTI_ARGLIST(ARG_MAPNAME);

		MapName = *tfiles;
		strupr (MapName);

		while (*tfiles) {
			map = AppendtUMEMapTMGC (map, *tfiles);
			if (!map) {
				break;
			}
			tfiles++;
		}
	}
	if (map) {
		result = ScanTiles (map);
		if (result) {
			result = WriteAllRooms (map);
			if (result) {
				result = WriteTileData (map);
			}			
		}
	}

	PrintMinMemFree();

	if (ErrorCount) {
		fprintf (stdout, "Errors: %d\n", ErrorCount);
		result = FALSE;
	}

	if (WarnCount) {
		fprintf (stdout, "Warnings: %d\n", WarnCount);
	}

	if (GlobalErr) {
		fprintf (stdout, "Error: %s\n", GlobalErrMsg);
		result = FALSE;
	}

	return (result ? EXIT_SUCCESS : EXIT_FAILURE);
}

